//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "GLUPTaskRunner.h"
#import "II2AppDelegate.h"
#import "II2ToolFinder.h"
#import "II2Preferences.h"
#import "GWFifoReader.h"
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <sys/sysctl.h>

static char rcsid[] = "$Id: GLUPTaskRunner.m,v 1.6 2004/12/04 09:41:16 gctwnl Exp $";

@implementation GLUPTaskRunner

+ (NSString *)fixedLaunchPath
{
    NSString *result = [[NSBundle mainBundle] pathForResource:@"gluprunner" ofType:nil];
    if (!result) {
	NSLog( @"Serious error: gluprunner missing!");
    }
    return result;
}

//- (id) retain
//{
//    NSLog(@"before -retain: %u  - %@", [self retainCount], self);
//    return [super retain];
//}
//
//- (void) release
//{
//    NSLog(@"before -release: %u - %@", [self retainCount], self);
//    [super release];
//}

- (id)initWithOwner:(id <GLUPTaskOwnerProtocol>)owner reportsPID:(GLUPTaskRunnerReadPidStyle)reportsPID arguments:(NSArray *)arguments authenticator:(GWAuthenticator *)authenticator
{
    self = [super init];
    if (self) {
	__owner = owner;
	__reportsPID = reportsPID;
	__gluprunner = [[GLUPTaskRunner fixedLaunchPath] retain];
	__arguments = [[NSMutableArray arrayWithArray:arguments] retain];
	__displayRange = NSMakeRange( 0, [__arguments count]);
	if ([[__arguments objectAtIndex:0] isEqualToString:[II2TOOLFINDER pathForTool:@"/usr/bin/env"]] && ([__arguments count] > 1)) {
	    if ([ii2Prefs mainLogLevel] > 7) NSLog( @"Removing %@ from argument list (display only)", [II2TOOLFINDER pathForTool:@"/usr/bin/env"]);
	    __displayRange.location += 1;
	    __displayRange.length -= 1;
	}
	__properties = [[NSMutableDictionary alloc] init];
	__authenticator = authenticator; // Doesn't need to be retained as it is owned by package objects
	__environment = [[NSMutableDictionary alloc] initWithDictionary:[[NSProcessInfo processInfo] environment]];
	__dependentTasks = [[NSMutableArray alloc] init];

	__result = -1;

	if (!__authenticator) {
	    // Created, even if it is not used, so we can connect before launch
	    __nonAuthTask = [[NSTask alloc] init];
	    [__nonAuthTask setLaunchPath: __gluprunner];
	}
    }
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"Init GLUPTaskRunner: %@ %u", [self description], [self retainCount]);
    return self;
}

- (void)__DelayedCreationOfErrorPipe
{
    if (__nonAuthTask && !__errorPipeCreated) {
	[__nonAuthTask setStandardError: [NSPipe pipe]];
	if ([ii2Prefs mainLogLevel] > 6) NSLog( @"%@ ErrPipe: R:%u W:%u OPENED", self, [[[__nonAuthTask standardError] fileHandleForReading] fileDescriptor], [[[__nonAuthTask standardError] fileHandleForWriting] fileDescriptor]);
	__errorPipeCreated = YES;
    }
}

- (void)__DelayedCreationOfOutputPipe
{
    if (__nonAuthTask && !__outputPipeCreated) {
	[__nonAuthTask setStandardOutput:[NSPipe pipe]];
	if ([ii2Prefs mainLogLevel] > 6) NSLog( @"%@ OutPipe: R:%u W:%u OPENED", self, [[[__nonAuthTask standardOutput] fileHandleForReading] fileDescriptor], [[[__nonAuthTask standardOutput] fileHandleForWriting] fileDescriptor]);
	__outputPipeCreated = YES;
    }
}

- (void)__DelayedCreationOfInputPipe
{
    if (__nonAuthTask && !__inputPipeCreated) {
	[__nonAuthTask setStandardInput:[NSPipe pipe]];
	if ([ii2Prefs mainLogLevel] > 6) NSLog( @"%@ InPipe: R:%u W:%u OPENED", self, [[[__nonAuthTask standardInput] fileHandleForReading] fileDescriptor], [[[__nonAuthTask standardInput] fileHandleForWriting] fileDescriptor]);
	__inputPipeCreated = YES;
    }
}

//- (void)__closeInputPipe
//{
//    if (__nonAuthTask) {
//	if (__inputPipeCreated) {
//	    [[[__nonAuthTask standardInput] fileHandleForReading] closeFile];
//	}
//    }
//}

- (void)dealloc
{
    if ([ii2Prefs mainLogLevel] > 8) fprintf( stderr, "Dealloc GLUPTaskRunner %s retain: %u", [[self description] cString], [self retainCount]);

    [__gluprunner release]; __gluprunner = nil;
    [__arguments release]; __arguments = nil;
    [__environment release]; __environment = nil;
    __authenticator = nil;
    [__directoryPath release]; __directoryPath = nil;
    if (!__keepTmpDirOnExit) {
	[DEFFM removeFileAtPath:__tmpDirectoryPath handler:nil];
    }
    [__tmpDirectoryPath release]; __tmpDirectoryPath = nil;
    [__properties release]; __properties = nil;
    [__outData release]; __outData = nil;
    [__errData release]; __errData = nil;
    [___authTaskInOutHandle release]; ___authTaskInOutHandle = nil;
    [___taskInHandle release]; ___taskInHandle = nil;
    [___taskOutHandle release]; ___taskOutHandle = nil;
    [___taskErrHandle release]; ___taskErrHandle = nil;
    [__nonAuthTask release]; __nonAuthTask = nil;
    [__dependentTasks release]; __dependentTasks = nil;
    [__attributedDisplayName release]; __attributedDisplayName = nil;
    [__displayName release]; __displayName = nil;
    [__reportName release]; __reportName = nil;
    // Just to be sure:
    if (__stderrFifoName) {
	[DEFFM removeFileAtPath:__stderrFifoName handler:nil];
    }
    if (__pidFifoName) {
	[DEFFM removeFileAtPath:__pidFifoName handler:nil];
    }
    if (__envFifoName) {
	[DEFFM removeFileAtPath:__envFifoName handler:nil];
    }
    [__pidFifoName release]; __pidFifoName = nil;
    [__envFifoName release]; __envFifoName = nil;
    [__stderrFifoName release]; __stderrFifoName = nil;
    [super dealloc];
}

- (BOOL) isRunning
{
    int nrOfWaits = 0;
    if (__waitCounter > 0) {
	return NO;
    }
    while ((__reportsPID != GLUPTaskRunnerNoPid) && (__pid == 0)) {
	// We wait untill the subprocess has reported its pid. If it never does, time out and report NO
	// This works. Better with a lock?
	usleep( 100);
	nrOfWaits++;
	if (nrOfWaits > 100) {
	    NSLog( @"Problem: while waiting for a subprocess PID to arrive we timed out in the isRunning check for task %@", [self description]);
	    return NO;
	}
    }
    if (__pid) {
	struct kinfo_proc *kpt;
	size_t size = sizeof(struct kinfo_proc);
	int mib[] = {
	    CTL_KERN, KERN_PROC, KERN_PROC_PID, __pid
	};
	kpt = malloc( sizeof(struct kinfo_proc)); // large enough. size is either 1 or 0 entries when we ask for a single pid
	if (sysctl( mib, 4, kpt, &size, NULL, 0) == -1) {
	    NSLog( @"Sysctl error: %s", strerror(errno));
	    return NO;
	}
	else {
	    free( kpt);
	    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"isRunning says: %u for %u", size, __pid);
	    return (size == sizeof(struct kinfo_proc));
	}
    }
    return NO;
}


- (void)setEnvironmentValue:(NSString *)value forKey:(NSString *)key
{
    [__environment setObject:value forKey:key];
}

- (void)setEnvironment: (NSDictionary *)anEnvironment
{
    if (__environment) {
	[__environment release]; __environment = nil;
    }
    if (anEnvironment) {
	__environment = [[NSMutableDictionary dictionaryWithDictionary:anEnvironment] retain];
    }
}

- (void)setProperties: (NSDictionary *)myProperties
{
    if (__properties) {
	[__properties release]; __properties = nil;
	__expectedOutputSize = 0;
    }
    if (myProperties) {
	__properties = [[NSMutableDictionary dictionaryWithDictionary:myProperties] retain];
	__expectedOutputSize = [[myProperties objectForKey:@"expectedoutputsize"] intValue];
	__expectedOutputIsErrData = [[myProperties objectForKey:@"iserrdata"] isEqualToString:@"YES"];
    }
}

- (void)setErrView: (NSTextView *)aView
{
    __errView = aView;
}

- (void)setOutView: (NSTextView *)aView
{
    if (__outTask) {
	NSLog( @"Cannot use output handler for stdout when task is connected");
	return;
    }
    __outView = aView;
}

- (void)setOutData: (NSMutableData *)aData
{
    if (__outTask) {
	NSLog( @"Cannot use output data object for stdout when task is connected");
	return;
    }
    if (__outData) {
	[__outData release]; __outData = nil;
    }
    __outData = [aData retain];
}

- (void)setErrData: (NSMutableData *)aData
{
    if (__errData) {
	[__errData release]; __errData = nil;
    }
    __errData = [aData retain];
}

- (void)setDirectoryPath: (NSString *)aDirectoryPath
{
    if (__directoryPath) {
	[__directoryPath release]; __directoryPath = nil;
    }
    __directoryPath = [[NSString stringWithString:aDirectoryPath] retain];
}

- (void)useTmpDir: (NSString *)aDirectoryPath cleanOnFinish:(BOOL)cleanOnFinish
{
    if (__tmpDirectoryPath) {
	NSLog( @"Illegal second call to useTmpDir:");
	return;
    }
    __tmpDirectoryPath = [[NSString stringWithString:aDirectoryPath] retain];
    __keepTmpDirOnExit = !cleanOnFinish;
}

- (void)makeTmpDir
{
    if (__tmpDirectoryPath) {
	[DEFFM createDirectoryAtPath:__tmpDirectoryPath attributes:nil];
    }
}

- (void)setOutSizeLimit: (size_t)limit
{
    if (__outTask) {
	NSLog( @"Cannot use output data limit for stdout when task is connected");
	return;
    }
    __outSizeLimit = limit;
    __checkOutSize = YES;
}


- (NSDictionary *)properties
{
    return __properties;
}

- (id)propertyForKey:(id)key
{
    return [__properties objectForKey:key];
}

- (void)setProperty:(id)property forKey:(id)key
{
    [__properties setObject:property forKey:(id)key];
}

- (void)setTaskSet:(II2TaskSet *)set
{
    __taskSet = set;
}


- (BOOL)isAuthTask
{
    return (__authenticator != nil);
}


- (NSString *)identifier
{
    return [NSString stringWithFormat:@"%p", self];
}

- (NSString *)description
{
    if (!__displayName || (__pid && !__displayNameWithPID)) {
	NSMutableArray *newArgList;
	NSString *newFirstArg;
	if (__displayName) {
	    [__displayName release]; __displayName = nil;
	    [__attributedDisplayName release]; __attributedDisplayName = nil;
	}
	newArgList = [NSMutableArray arrayWithArray:[__arguments subarrayWithRange:__displayRange]];
	newFirstArg = [[newArgList objectAtIndex:0] lastPathComponent];
	[newArgList replaceObjectAtIndex:0 withObject:newFirstArg];
	if (__pid) {
	    __displayName = [[NSString stringWithFormat:@"%u: %@", __pid, [newArgList componentsJoinedByString:@" "]] retain];
	    __displayNameWithPID = YES;
	}
	else {
	    __displayName = [[NSString stringWithFormat:@"%@", [newArgList componentsJoinedByString:@" "]] retain];
	}
	if (__displayName) {
	    if ([self isAuthTask]) {
		__attributedDisplayName = [[NSAttributedString alloc] initWithString:__displayName attributes:[self signalAttributes]];
	    }
	    else {
		__attributedDisplayName = [[NSAttributedString alloc] initWithString:__displayName attributes:[self normalAttributes]];
	    }
	}
    }
    return __displayName;
}

- (NSAttributedString *)attributedDisplayName
{
    // I am calling [self description] here (and discard the returned result) because the description changes as soon as the pid is detected, and the attributed version follows the nonattributed version.
    [self description];
    return __attributedDisplayName;
}

- (id)progress
{
    if (__waitCounter) {
	if ([ii2Prefs mainLogLevel] > 7) NSLog( @"Progress report waiting (%@)", [self description]);
	return [NSString stringWithFormat:@"Waiting... (%u)", __waitCounter];
    }
    if ([self isRunning]) {
	if ([ii2Prefs mainLogLevel] > 7) NSLog( @"Progress report running (%@)", [self description]);
	return @"Running...";
    }
    if ([ii2Prefs mainLogLevel] > 7) NSLog( @"Progress report terminated (%@)", [self description]);
    return @"Terminated.";
}

- (void) getInputFrom:(NSPipe *)pipe
{
    [self __DelayedCreationOfInputPipe];
    [__nonAuthTask setStandardInput:pipe];
}

- (void) connectOutputTo: (GLUPTaskRunner *)task
{
    if ([task isAuthTask] || [self isAuthTask])
    {
	NSLog( @"Connection from or to authenticated tasks not allowed");
	return;
    }
    if (__outData || __outView) {
	NSLog( @"Connection of a task that sends output to data object or view is not allowed");
	return;
    }
    __outTask = task;
    // Connect input of outTask to outPut of this task
    [self __DelayedCreationOfOutputPipe];
    [__outTask getInputFrom:[__nonAuthTask standardOutput]];
}

- (void) makeFiFo: (NSString*)fifoName
{
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"makeFiFo: %s", [fifoName fileSystemRepresentation]);
    if (mkfifo( [fifoName fileSystemRepresentation], 0600) != 0) {
	NSLog( @"Creating %s failed (%s)", [fifoName fileSystemRepresentation], strerror( errno));
	return;
    }
}

- (void) readPidFifo: (NSString*)fifoName
{
    // This method is run in its own thread. Call with
    // [NSThread detachNewThreadSelector: selector(readPidFifo:) toTarget:thisTaskRunnerObject withObject:fifoName]
    int pidFifoFd;

    // autorelease pool
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

    pidFifoFd = open( [fifoName fileSystemRepresentation], O_RDONLY);
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"%@ readPidFifo: %s (%u)", self, [fifoName fileSystemRepresentation], pidFifoFd);
    if (pidFifoFd == -1) {
	NSLog( @"Opening %s failed (%s)", [fifoName fileSystemRepresentation], strerror( errno));
	[pool release]; return;
    }
    if (read( pidFifoFd, &__pid, sizeof( pid_t)) == -1) {
	NSLog( @"Pid reading from %s failed (%s)", [fifoName fileSystemRepresentation], strerror( errno));
    }
    close( pidFifoFd);
    unlink( [fifoName fileSystemRepresentation]);
    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"Received Fifo PID = %u", __pid);
    [pool release];
    return;
}

- (void) writeEnvFifo: (NSString*)fifoName
{
    int envFifoFd;
    const char *description;
    NSMutableString *descString;
    NSEnumerator *dictEnumerator;
    NSString *envKey;
    // This method is run in its own thread. Call with
    // [NSThread detachNewThreadSelector: selector(readPidFifo:) toTarget:thisTaskRunnerObject withObject:fifoName]
    
    // autorelease pool
    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];

    envFifoFd = open( [fifoName fileSystemRepresentation], O_WRONLY);
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"%@ writeEnvFifo: %s %u OPENED", self, [fifoName fileSystemRepresentation], envFifoFd);
    if (envFifoFd == -1) {
	NSLog( @"Opening %s failed (%s)", [fifoName fileSystemRepresentation], strerror( errno));
	[pool release]; return;
    }
    descString = [[[NSMutableString alloc] init] autorelease];
    dictEnumerator = [__environment keyEnumerator];
    while ((envKey = [dictEnumerator nextObject])) {
	[descString appendFormat:@"%@=%@\n", envKey, [__environment objectForKey:envKey]];
	if ([ii2Prefs mainLogLevel] > 6) NSLog( @"ENV: %@=%@", envKey, [__environment objectForKey:envKey]);
    }
    description = [descString lossyCString];
    if (description) {
	unsigned desclen = strlen( description);
	char lenbuf[32];

	sprintf( lenbuf, "%u\n", desclen);
	if (write( envFifoFd, lenbuf, strlen( lenbuf)) < strlen( lenbuf)) {
	    NSLog( @"Writing length to %s failed (%s)", [fifoName fileSystemRepresentation], strerror( errno));
	    close( envFifoFd);
	    [pool release]; return;
	}
	if (write( envFifoFd, description, desclen) < desclen) {
	    NSLog( @"Writing env to %s failed (%s)", [fifoName fileSystemRepresentation], strerror( errno));
	    close( envFifoFd);
	    [pool release]; return;
	}
    }
    close( envFifoFd);
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"%@ writeEnvFifo: %s %u CLOSED", self, [fifoName fileSystemRepresentation], envFifoFd);
    [pool release];

    return;
}

// Handling data which is the output (stdout) of the subprocess
- (void) handleProcessData: (NSData *)withData orNotification: (NSNotification *)aNotification forView:(NSTextView *)aView isErrData:(BOOL)isErrData
{
    NSData *data;
    size_t length = 0;

    if ([ii2Prefs mainLogLevel] > 7) NSLog( @"handleProcessData: data= %p, notification = %p, errData = %u", withData, aNotification, isErrData);
    if (aNotification  && !withData) {
	data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];
	length = [data length];
	if ([ii2Prefs mainLogLevel] > 8) NSLog( @"Notification data item for %@ = %p (%u)", [self description], data, length);
	if (!(__expectedOutputIsErrData ^ isErrData)) { // Another one right below! If this is changed, change that one too!
	    __currentOutputSize += length;
	    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"Reported outdata %d %d %d", __currentOutputSize, __reportedOutputSize, __reportIncrementTreshold);
	    if (__reporter && (length == 0 || ((__currentOutputSize - __reportedOutputSize) >= __reportIncrementTreshold))) {
		[__reporter updateReportWithDataFromTask:self download:NO];
	    }
	}
    }
    else if (withData && !aNotification) {
	data = withData;
	length = [data length];
	if ([ii2Prefs mainLogLevel] > 8) NSLog( @"isData data item for %@ = %p (%u)", [self description], data, length);
	if (!(__expectedOutputIsErrData ^ isErrData)) { // Another one right above! If this is changed, change that one too!
	    __currentOutputSize += length;
	    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"Reported outdata %d %d %d", __currentOutputSize, __reportedOutputSize, __reportIncrementTreshold);
	    if (__reporter && (length == 0 || ((__currentOutputSize - __reportedOutputSize) >= __reportIncrementTreshold))) {
		[__reporter updateReportWithDataFromTask:self download:NO];
	    }
	}
    }
    else if (withData && aNotification) {
	// Error
	NSLog( @"Program error: cannot handle concurrent notification and data");
	return;
    }
    else {
	// both nil, stop task
	if ([ii2Prefs mainLogLevel] > 5) NSLog( @"handleProcessData: stopping task (double 0)");
    }
    if ([ii2Prefs mainLogLevel] > 8) NSLog(@" Data for proc: %@ (size = %u)", [self description], length);

    if (length) {
	if (isErrData) {
	    if (aView) {
		// [aView appendOutput: [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease]];
		[aView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease] attributes:([self isAuthTask] ? [self fixedSignalAttributes] : [self fixedNormalAttributes])] autorelease]];
	    }
	    if (__errData) {
		[__errData appendData:data];
	    }
	    if (aNotification) {
		[[aNotification object] readInBackgroundAndNotify];
	    }
	}
	else {
	    BOOL stoptask = NO;
	    if (__checkOutSize) {
		if ((__outSize + length) >= __outSizeLimit) {
		    const void *bytes = [data bytes];
		    data = [NSData dataWithBytes:bytes length:(MAX((__outSizeLimit - __outSize),__outSizeLimit))];
		    stoptask = YES;
		}
		else {
		    __outSize += length;
		}
	    }
	    else {
		__outSize += length;
	    }
	    if (__outData) {
		[__outData appendData:data];
	    }
	    if (aView) {
		[aView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease] attributes:([self isAuthTask] ? [self fixedSignalAttributes] : [self fixedNormalAttributes])] autorelease]];
		// [aView appendOutput: [[[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] autorelease]];
	    }

	    if (stoptask) {
		// We've had enough, abort task
		[self abortReportFailure:NO];
	    }
	    else if (aNotification) {
		[[aNotification object] readInBackgroundAndNotify];
	    }
	}
    }
    else {
	// zero size makes me assume process has died while it is in fact only closed, this is an incorrect assumption for tasks that write to another task with an NSPipe on pre-10.1 systems
	if ([ii2Prefs mainLogLevel] > 7) NSLog( @"Zero length data. %@", [self description]);
	if (isErrData) {
	    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object: __taskErrHandle];
	}
	else {
	    // NSLog( @"Received zero length data for %@", [self description]);
	    [self terminate];
	}
    }
}

- (void) getOutData: (NSNotification *)aNotification
{
    [self handleProcessData:nil orNotification:aNotification forView:__outView isErrData:NO];
}

- (void) getErrData: (NSNotification *)aNotification
{
    [self handleProcessData:nil orNotification:aNotification forView:__errView isErrData:YES];
}

- (void)doErrDataHandler: (NSData *)data
{
    [self handleProcessData:data orNotification:nil forView:__errView isErrData:YES];
}

- (void)nsTaskDidEnd:(NSNotification *)aNotification {
    NSTask *task = [aNotification object];
    if (task == __nonAuthTask) {
	// NSLog( @"Received task did end notification for %@", [self description]);
	[self terminate];
    }
}

- (pid_t)pid
{
    return __pid;
}

- (void)__reportTermination
{
    int i;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"__reportTermination on %@", [self description]);
    
    // Signal owner (if any) about finished task
    [__owner glupTaskDidFinish:self withSuccess:(__result == 0)];
    
    // Remove from task set (releases)
    [__taskSet ii2TaskDidEnd:self withSuccess:(__result == 0)];
    
    //Signal dependent tasks
    for (i=[__dependentTasks count]-1; i>=0; i--) {
        if (__result == 0) {
            [[__dependentTasks objectAtIndex:i] endWait];
        }
        else {
            [[__dependentTasks objectAtIndex:i] abortWait];
        }
    }
}

- (void)runWithSynchronicity:(BOOL)synchronicity
{
    int taskInOutfd;

    if (__outTask && synchronicity) {
	NSLog( @"Will not run task synchronously when it is connected to another task (deadlock)");
	return;
    }
    __synchronous = synchronicity;
    [__owner glupTaskWillStart:self];
    
    if (__Skip) {
        // A skipped task is succesful non-execution
        __result = 0;
        [self __reportTermination];
        return;
    }

    if (__environment) {
	__envFifoName = [[NSString stringWithFormat:@"/tmp/.ii2E.%u.%u", getpid(), random()] retain];
	[self makeFiFo:__envFifoName];
	[__arguments insertObject:@"-E" atIndex:0];
	[__arguments insertObject:__envFifoName atIndex:1];
	__displayRange.location += 2;
    }

    if (__directoryPath) {
	[__arguments insertObject:@"-d" atIndex:0];
	[__arguments insertObject:__directoryPath atIndex:1];
	__displayRange.location += 2;
    }

    [self makeTmpDir];
    
    if (__authenticator) {
	const char **args;
	NSEnumerator *enumerator;
	NSString *arg;
	int argcount = 0;
	FILE *commpipe = 0;
	OSStatus err = 0;

	if (![__authenticator setPrivileged:YES]) {
	    // do nothing, authentication failed
	    NSLog( @"Authentication failed on running a subtask");
	    [self abort];
	    [self release];
	    return;
	}

	// Stderr reading
	__stderrFifoName = [[NSString stringWithFormat:@"/tmp/.ii2e.%u.%u", getpid(), random()] retain];
	{
	    NSPort *port1;
	    NSPort *port2;
	    NSArray *portAndFifoNameArray;

	    port1 = [NSPort port];
	    port2 = [NSPort port];

	    __kitConnection = [[NSConnection alloc] initWithReceivePort:port1
						      sendPort:port2];
	    [__kitConnection setRootObject:self];

	    /* Ports switched here. */
	    portAndFifoNameArray = [NSArray arrayWithObjects:port2, port1, __stderrFifoName, nil];

	    [NSThread detachNewThreadSelector:@selector(connectWithPortsAndFifoName:)
			      toTarget:[GWFifoReader class] withObject:portAndFifoNameArray];
	}
	[__arguments insertObject:@"-e" atIndex:0];
	[__arguments insertObject:__stderrFifoName atIndex:1];
	__displayRange.location += 2;

	if (__reportsPID == GLUPTaskRunnerFifoPid) {
	    __pidFifoName = [[NSString stringWithFormat:@"/tmp/.ii2p.%u.%u", getpid(), random()] retain];
	    [self makeFiFo:__pidFifoName];
	    [__arguments insertObject:@"-p" atIndex:0];
	    [__arguments insertObject:__pidFifoName atIndex:1];
	    __displayRange.location += 2;
	    if ([ii2Prefs mainLogLevel] > 7) NSLog( @"Detaching thread to read pidFifo: %s", [__pidFifoName fileSystemRepresentation]);
	    [NSThread detachNewThreadSelector: @selector(readPidFifo:) toTarget:self withObject:__pidFifoName];
	}

	args = calloc( [__arguments count] + 1, sizeof( char *));
	enumerator = [__arguments objectEnumerator];

	// NSLog( "RUN: %p %@", [__authenticator authorizationRef], __arguments);
	while ((arg = [enumerator nextObject])) {
	    args[argcount++] = [arg lossyCString];
	}
	__displayRange.length = [__arguments count] - __displayRange.location;
	if ([ii2Prefs mainLogLevel] > 7) NSLog( @"WILL perform TRUE authenticated launch of %@", [self description]);
	err = AuthorizationExecuteWithPrivileges( [__authenticator authorizationRef], [__gluprunner fileSystemRepresentation], 0, (char * const *)args, &commpipe);
	free( args);
	if (err != errAuthorizationSuccess) {
	    // Cleanup and return
	    return;
	}
	taskInOutfd = fileno(commpipe);
	___authTaskInOutHandle = [[NSFileHandle alloc] initWithFileDescriptor:taskInOutfd closeOnDealloc:YES];
	__taskOutHandle = [___authTaskInOutHandle retain];
	__taskInHandle = [___authTaskInOutHandle retain];
	if (!__synchronous) {
	    if (!__outTask && __taskOutHandle) {
		// NSLog( @"Adding notification for %p", self);
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(getOutData:) name: NSFileHandleReadCompletionNotification object: __taskOutHandle];
		[__taskOutHandle readInBackgroundAndNotify];
	    }
	}
    }
    else {
	[__arguments insertObject:@"-q" atIndex:0];
	__displayRange.location += 1;
	[__nonAuthTask setArguments: __arguments];

	// [self __DelayedCreationOfInputPipe];
	[self __DelayedCreationOfOutputPipe];
	[self __DelayedCreationOfErrorPipe];
	if (__outputPipeCreated) {
	    ___taskOutHandle = [[[__nonAuthTask standardOutput] fileHandleForReading] retain];
	}
	else {
	    ___taskOutHandle = [[__nonAuthTask standardOutput] retain];
	}
	__taskOutHandle = ___taskOutHandle;
	if (__errorPipeCreated) {
	    ___taskErrHandle = [[[__nonAuthTask standardError] fileHandleForReading] retain];
	}
	else {
	    ___taskErrHandle = [[__nonAuthTask standardError] retain];
	}
	__taskErrHandle = ___taskErrHandle;
	if (__inputPipeCreated) {
	    ___taskInHandle = [[[__nonAuthTask standardInput] fileHandleForWriting] retain];
	}
	else {
	    ___taskInHandle = [[__nonAuthTask standardInput] retain];
	}
	__taskInHandle = ___taskInHandle;
	__displayRange.length = [__arguments count] - __displayRange.location;
	if (!__synchronous) {
	    if (__outTask) {
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(nsTaskDidEnd:) name:NSTaskDidTerminateNotification object:nil];
	    }
	    else if (__taskOutHandle) {
		// NSLog( @"Adding notification for %p", self);
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(getOutData:) name: NSFileHandleReadCompletionNotification object: __taskOutHandle];
		[__taskOutHandle readInBackgroundAndNotify];
	    }
	    if (__taskErrHandle) {
		// NSLog( @"Adding notification for %p", self);
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(getErrData:) name: NSFileHandleReadCompletionNotification object: __taskErrHandle];
		[__taskErrHandle readInBackgroundAndNotify];
	    }
	}
	if ([ii2Prefs mainLogLevel] > 7) NSLog( @"WILL perform TRUE launch of %@", [self description]);
	[__nonAuthTask launch];
	__pid = [__nonAuthTask processIdentifier];
    }
    [__taskSet refreshDisplay:self];

    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"%@ launch InOut: %u, In: %u, Out: %u, Err: %u", self, [___authTaskInOutHandle retainCount], [___taskInHandle retainCount], [___taskOutHandle retainCount], [___taskErrHandle retainCount]);

    if (__environment) {
	if ([ii2Prefs mainLogLevel] > 8) NSLog( @"Detaching thread for %@ to write envFifo: %s", [self description], [__envFifoName fileSystemRepresentation]);
	[NSThread detachNewThreadSelector: @selector(writeEnvFifo:) toTarget:self withObject:__envFifoName];
    }

    if (__authenticator && __reportsPID == GLUPTaskRunnerStdoutPid) {
	char errBuf[2048]; //, bad, but it is big enough for the first output lin eof gluprunner
	const char *bufPt;

	// get first line from gluprunner and parse pid
        // Assumes the first line of output is less than 2047 characters long and is of the form
	//	Comment = <pid>
        // where <pid> is the pid of the process
	bufPt = errBuf;
	while ((bufPt < (errBuf+2048)) && read( taskInOutfd, (void *)bufPt, 1) == 1 && *bufPt != '\n') {
	    bufPt++;
	}
	if (bufPt < (errBuf + 2048)) {
	    bufPt = strrchr( errBuf, '=');
	    if (bufPt) {
		bufPt++;
		if (sscanf( bufPt, "%u", &__pid) == 1) {
		    // No number
		    __reportsPID = GLUPTaskRunnerNoPid;
		}
	    }
	    else {
		// No '=' character
		__reportsPID = GLUPTaskRunnerNoPid;
	    }
	}
	else {
	    // No \n within 2047 characters
	    __reportsPID = GLUPTaskRunnerNoPid;
	}
    }
    if ([ii2Prefs mainLogLevel] > 5) NSLog( @"Running: %@ (retainCount = %u)", [self description], [self retainCount]);

    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"%@ launch InOut: %u, In: %u, Out: %u, Err: %u", self, [___authTaskInOutHandle retainCount], [___taskInHandle retainCount], [___taskOutHandle retainCount], [___taskErrHandle retainCount]);

    if (__synchronous) {
	NSData *data;

	while ((data = [__taskOutHandle /*readDataOfLength:64*/availableData]) && [data length] != 0) {
	    [self handleProcessData:data orNotification:nil forView:__outView isErrData:NO];
	}
	if (data) {
	    // length was 0, task ended
	    [self terminate];
	}
	else {
	    NSLog( @"Data failure for %@", [self description]);
	}
    }
}

- (void)wait
{
    if ([ii2Prefs mainLogLevel] > 5) NSLog( @"wait GLUPTaskRunner: %u: %@ (%u)", __pid, [self description], __waitCounter);
    __waitCounter++;
}

- (void)endWait
{
    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"endWait GLUPTaskRunner: %u: %@ (%u)", __pid, [self description], __waitCounter);
    if (__waitCounter > 0) {
	__waitCounter--;
    }
    [self launch];
}

- (void)abortWait
{
    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"abortWait GLUPTaskRunner: %u: %@ (%u)", __pid, [self description], __waitCounter);
    if (!__invalidated) {
	__invalidated = YES;
	[__dependentTasks makeObjectsPerformSelector:@selector(abortWait)];
    }
    if (__waitCounter == 0) {
	[__owner glupTaskDidFinish:self withSuccess:NO];
    }
    if (__waitCounter > 0) {
	__waitCounter--;
    }
    if (__waitCounter == 0) {
	[__taskSet ii2TaskDidEnd:self withSuccess:NO];
	[self release];
    }
}

- (void)addDependentTask:(id <II2TaskDependencyProtocol>)task
{
    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"%@ will wait for %@", [task description], [self description]);
    [__dependentTasks addObject:task];
    [task wait];
}


- (NSArray *)dependentTasks
{
    return __dependentTasks;
}

- (void)addDependentTasks:(NSArray *)tasks
{
    if (!tasks) {
	NSLog( @"Weird error, nil-tasks array in GLUPTaskRunner addDependentTasks");
    }
    if ([tasks count]) {
	[__dependentTasks addObjectsFromArray:tasks];
	[tasks makeObjectsPerformSelector:@selector(wait)];
    }
}

- (void)run
{
    if (__waitCounter == 0) {
	if (!__invalidated) {
	    [self runWithSynchronicity:YES];
	}
	else {
	    [__taskSet ii2TaskDidEnd:self withSuccess:NO];
	    [self release];
	}
    }
}

- (void)launch
{
    if (__waitCounter == 0) {
	if (!__invalidated) {
	    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"Launching: %@ (retain = %u)", [self description], [self retainCount]);
	    [self runWithSynchronicity:NO];
	}
	else {
	    [__taskSet ii2TaskDidEnd:self withSuccess:NO];
	    [self release];
	}
    }
}

- (void)__abort
{
    // We are aborting, kill process group of subtask (if pid is known)
    if (__pid) {
	NSString *pidString = [[NSNumber numberWithUnsignedInt:__pid] stringValue];
	if ([ii2Prefs mainLogLevel] > 4) NSLog( @"Trying to kill still running process \"%d\"", __pid);
	if (__authenticator) {
	    NSString *command = [[NSBundle mainBundle] pathForResource:@"killpg" ofType:nil];
	    if (command && pidString) {
		const char *args[5] = {0,0,0,0,0};
		args[0] = [command fileSystemRepresentation];
		args[1] = "-15";
		args[2] = [pidString lossyCString];

		AuthorizationExecuteWithPrivileges( [__authenticator authorizationRef],
				      [__gluprunner fileSystemRepresentation],
				      0, (char * const *)args, 0);
	    }
	}
	else {
	    if ([__nonAuthTask isRunning]) [__nonAuthTask terminate];
	}
    }
}

- (void)abort
{
    [self abortReportFailure:YES];
}

- (void)abortReportFailure:(BOOL)failure
{
    // NSLog( @"abortReportFailure: \"%d\"", failure);
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [self __abort];
    if (!failure) {
	__result = 0;	
    }
    else {
	[self setProperty:@"YES" forKey:@"aborted"];
    }
    [self __reportTermination];
}

- (void)terminate
{
    int status;
    // May be called when task is running and when it is not.
    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"Terminating: %@ %u (size = %u) (retain = %u)", [self description], __pid, __outSize, [self retainCount]);

    [[NSNotificationCenter defaultCenter] removeObserver:self];

    if (__pid) {
	// Wait for process to finish, get result
	if (__authenticator) {
	    if (waitpid( __pid, &status,0) == -1) {
		NSLog(@"Waitpid failed for pid %u", __pid);
		__result = -1;
	    }
	    else {
		if (WIFEXITED(status)) {
		    if ([ii2Prefs mainLogLevel] > 4) NSLog(@"Normal exit for subprocess %@", [self description]);
		}
		else {
		    if ([ii2Prefs mainLogLevel] > 0) NSLog(@"Abnormal exit for subprocess %@", [self description]);
		}
		__result = WEXITSTATUS( status);
	    }
	    [__kitConnection invalidate];
	    [__kitConnection release]; __kitConnection = nil;
	}
	else {
	    [__nonAuthTask waitUntilExit];
	    __result = [__nonAuthTask terminationStatus];
	}
    }
    
    [self __reportTermination];
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"Terminating. Will release: %@ (retain = %u)", [self description], [self retainCount]);
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"### The result output was:\n\tExpected: %d\n\tCurrent: %d\n\tReported: %d\n\tErrData: %d", __expectedOutputSize, __currentOutputSize, __reportedOutputSize, __expectedOutputIsErrData);
    [self release];
}

- (int)result
{
    return __result;
}

- (BOOL)skip
{
    return __Skip;
}

- (void)setSkip: (BOOL)skip
{
    __Skip = skip;
}

- (void)setReportTarget:(id <II2TaskStatusReporterProtocol>)reporter
{
    __reporter = reporter;
}

- (void)setReportIncrementTreshold:(int)treshold
{
    __reportIncrementTreshold = (treshold >= (__expectedOutputSize/2) ? (__expectedOutputSize/2) : treshold);
}

- (NSString *)typeOfTask
{
    if (!__reportName) {
	NSString *partNumber = [self propertyForKey:@"archivepartnumber"];
	if (partNumber) {
	    __reportName = [[NSString stringWithFormat:@"Unarchiving part %@", partNumber] retain];
	}
	else {
	    __reportName = [[NSString stringWithString:@"Running a Subprocess"] retain];
	}
    }
    return __reportName;
}

- (int)sizeOfTaskWhenCompleted
{
    return __expectedOutputSize;
}

- (int)currentSizeOfTask
{
    return __currentOutputSize;
}

- (int)sizeOfTaskSinceLastReported
{
    int increment = (__currentOutputSize - __reportedOutputSize);
    __reportedOutputSize = __currentOutputSize;
    return increment;
}

- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
}

@end

/*
 * $Log: GLUPTaskRunner.m,v $
 * Revision 1.6  2004/12/04 09:41:16  gctwnl
 * v 2.68
 *
 * Revision 1.5  2004/10/15 14:12:58  gctwnl
 * 2.68
 *
 * Revision 1.4  2004/04/25 13:13:14  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.3  2004/03/19 23:35:11  gctwnl
 * Version 2.55.0. Has a renewed interface to support simple operations.
 * Supports progress bar control from the i-Package.
 *
 * Revision 1.2  2004/02/02 20:15:27  gctwnl
 * Fixed loss of keyed objects in package window (spinning progress). Some small fixes in logging and debug messages.
 *
 * Revision 1.1.1.1  2004/02/01 17:17:30  gctwnl
 * Initial import.
 *
 * Revision 3.12  2004/02/01 17:04:37  gerben
 * Currently investigating a file descriptor leak (lsof shows this)
 *
 * Revision 3.11  2004/01/27 12:57:39  gerben
 * 2.44.0
 *
 * Revision 3.10  2004/01/15 01:00:06  gerben
 * 2.40.0
 *
 * Revision 3.9  2004/01/14 12:56:45  gerben
 * Version 2.39.0
 *
 * Revision 3.8  2004/01/05 23:47:15  gerben
 * *** empty log message ***
 *
 * Revision 3.7  2004/01/05 15:31:33  gerben
 * *** empty log message ***
 *
 * Revision 3.6  2003/11/09 13:02:07  gerben
 * 2.31.0
 *
 * Revision 3.5  2003/11/08 23:37:43  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.4  2003/09/18 22:40:54  gerben
 * 2.26.0 Fragmented archives
 *
 * Revision 3.3  2002/12/13 21:08:40  gerben
 * Release. Version 2.0.0
 *
 * Revision 3.2  2002/12/10 14:24:47  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:22  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:49  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.42  2002/11/04 11:54:11  gerben
 * i-Directory selection added to Known Packages
 *
 * Revision 1.41  2002/10/27 15:39:44  gerben
 * Tried to remove BSD-package dependency
 *
 * Revision 1.40  2002/10/22 22:20:58  gerben
 * Small changes
 *
 * Revision 1.39  2002/10/22 12:52:21  gerben
 * Added stderr handling to authenticated tasks.
 *
 * Revision 1.38  2002/10/21 20:16:54  gerben
 * Closing in on RC1 target
 *
 * Revision 1.37  2002/10/16 21:28:22  gerben
 * A complete new rewrite of taskss, processes and stuff.
 *
 * Revision 1.36  2002/10/16 08:37:19  gerben
 * Changed log levels. Small fixes.
 *
 * Revision 1.35  2002/10/15 14:31:44  gerben
 * Last fixes. No-network access. Later creation of downloaded file.
 *
 * Revision 1.34  2002/10/14 12:34:19  gerben
 * Removed dead code
 *
 * Revision 1.33  2002/10/12 10:54:39  gerben
 * A lot of changes, mostly when producing output/feedback.
 *
 * Revision 1.32  2002/10/10 00:51:24  gerben
 * Many changes. Committing a stable state.
 *
 * Revision 1.31  2002/10/09 14:22:23  gerben
 * Removed outputHandler from GLUPTaskRunner
 *
 * Revision 1.30  2002/10/09 14:16:12  gerben
 * Changed as many %s/lossyCString uses to %@/fileSystemRepresentation/UTF8String
 *
 * Revision 1.29  2002/10/09 10:53:40  gerben
 * Moved textview output to NSTexTView category.
 *
 * Revision 1.28  2002/10/08 22:25:03  gerben
 * GPG checking complete. Output of parallel tasks will not mingle.
 *
 * Revision 1.27  2002/10/08 21:34:29  gerben
 * Working GPG signature checks, shared GPG cache for II2.
 *
 * Revision 1.26  2002/10/07 20:28:06  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.25  2002/10/07 08:35:57  gerben
 * Aborting a task is now understood and reported correctly.
 *
 * Revision 1.24  2002/10/06 21:05:07  gerben
 * Progress Indicator for active tasks, hidden when there are no processes.
 *
 * Revision 1.23  2002/10/06 11:14:38  gerben
 * Changed displaying process information to a more friendly format.
 *
 * Revision 1.22  2002/10/04 11:47:41  gerben
 * Given that terminate does not abort a process anymore it must be safe to terminate (wait for exit) when data size is 0.
 *
 * Revision 1.21  2002/10/03 18:15:22  gerben
 * A typo (bz instead of bz2) prevented the unarchive of anything.
 *
 * Revision 1.20  2002/10/03 13:38:04  gerben
 * Removed addChildWindow as child passes close to parent. Window can only close when there are no running tasks.
 *
 * Revision 1.19  2002/10/02 22:38:31  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */