//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "GWFifoReader.h"
#import "GLUPTaskRunner.h"
#import "II2Preferences.h"
#include <unistd.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>


@implementation GWFifoReader
- (void) dealloc
{
    [__fifoHandle closeFile];
    [__fifoHandle release];
    if (unlink( [__fifoName fileSystemRepresentation]) == -1) {
	fprintf( stderr, "Unable to remove %s (%s)", [__fifoName cString], strerror( errno));
    }
    [__fifoName release];
    [__serverConnection invalidate];
    [__serverConnection release]; __serverConnection = nil;
    [__notificationCenter release];
    [super dealloc];
}

- (int) makeAndOpenFifo:(NSString*)fifoName
{
    int fifoFd;

    // if ([ii2Prefs mainLogLevel] > 8) NSLog( @"makeFiFo: %s", [fifoName fileSystemRepresentation]);
    if (mkfifo( [fifoName fileSystemRepresentation], 0600) != 0) {
	NSLog( @"Creating fifo %s failed (%s)", [fifoName fileSystemRepresentation], strerror( errno));
	return -1;
    }
    fifoFd = open( [fifoName fileSystemRepresentation], O_RDONLY);
    if (fifoFd == -1) {
	NSLog( @"Opening %s failed (%s)", [fifoName fileSystemRepresentation], strerror( errno));
    }
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"GWFifoReader makeAndOpenFifo: %s (%u)", [__fifoName fileSystemRepresentation], [__fifoHandle fileDescriptor]);
    return fifoFd;
}

- (id)initWithFifoName:(NSString *)fifoName
{
    int fifoFd;

    self = [super init];
    if ((fifoFd = [self makeAndOpenFifo:fifoName]) == -1) {
	[self release];
	return nil;
    }
    __fifoName = [fifoName retain];
    __fifoHandle = [[NSFileHandle alloc] initWithFileDescriptor:fifoFd];
    return self;
}

- (void) getOutData: (NSNotification *)aNotification
{
    NSData *data;

    data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];
    if (data && [data length]) {
	[(id)[__serverConnection rootProxy] doErrDataHandler:data];
	[[aNotification object] readInBackgroundAndNotify];
    }
    else {
	[self release];
    }
}


- (void)readAndNotifyWithServerConnection:(NSConnection *)serverConnection
{
    NSData *data;

    __serverConnection = [serverConnection retain];

    // Asynchronous, does not work:
    // __notificationCenter = [[NSNotificationCenter alloc] init];
    if (__notificationCenter) {
	[__notificationCenter addObserver:self selector:@selector(getOutData:) name: NSFileHandleReadCompletionNotification object: __fifoHandle];
	[__fifoHandle readInBackgroundAndNotify];
	return;
    }

    // Synchronous for the time being
    while ((data = [__fifoHandle /* readDataOfLength:64 */availableData]) && [data length] != 0) {
	[(id)[serverConnection rootProxy] doErrDataHandler:data];
    }
    [self release];
}

+ (void)connectWithPortsAndFifoName:(NSArray *)portAndFifoNameArray
{
    NSAutoreleasePool *pool;
    NSConnection *serverConnection;
    GWFifoReader *serverObject;

    pool = [[NSAutoreleasePool alloc] init];

    serverConnection = [NSConnection
            connectionWithReceivePort:[portAndFifoNameArray objectAtIndex:0]
			     sendPort:[portAndFifoNameArray objectAtIndex:1]];
    serverObject = [[self alloc] initWithFifoName:[portAndFifoNameArray objectAtIndex:2]];
    if (serverObject) {
	[serverObject readAndNotifyWithServerConnection:serverConnection];
//	[(id)[serverConnection rootProxy] setServer:serverObject];
//	[serverObject release];
//	[[NSRunLoop currentRunLoop] run];
    }
    [pool release];

    return;
}

@end
