//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "GWProgressCell.h"
#import "II2AppDelegate.h"
#import "II2Preferences.h"


static char rcsid[] = "$Id: GWProgressCell.m,v 1.2 2004/04/25 13:13:21 gctwnl Exp $";

@implementation GWProgressCell
- (void) dealloc
{
    [__offScreenWindow release]; __offScreenWindow = nil;
    [__offScreenProgressIndicator release]; __offScreenProgressIndicator = nil;
    [super dealloc];
}

- (id) init
{
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"Progress bar: Init");
    self = [super init];
    __offScreenWindow = [[NSWindow alloc] initWithContentRect:NSMakeRect(0,0,200,200) styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:NO];
    __offScreenProgressIndicator = [[NSProgressIndicator alloc] initWithFrame:NSMakeRect(0,0,80,14)];
    [__offScreenProgressIndicator setIndeterminate: NO];
    [__offScreenProgressIndicator setMinValue:0.0];
    [__offScreenProgressIndicator setMaxValue:100.0];
    [[__offScreenWindow contentView] addSubview:__offScreenProgressIndicator];
    return self;
}

-(void)setObjectValue:(id)object
{
    double doubleValue  = [object doubleValue];
    if(doubleValue < 0) {
	//NSLog( @"Progress bar: Indeterminate");
        [__offScreenProgressIndicator setIndeterminate: YES];
        [__offScreenProgressIndicator setDoubleValue: 0.0];
    }   else    {
	if ([ii2Prefs mainLogLevel] > 8) NSLog( @"Progress bar: Determinate %f", doubleValue);
        [__offScreenProgressIndicator setIndeterminate: NO];
        [__offScreenProgressIndicator setDoubleValue: doubleValue];
    }
    // if ([ii2Prefs mainLogLevel] > 7) NSLog( @"PB: min: %f max:%f value: %f", [__offScreenProgressIndicator minValue], [__offScreenProgressIndicator maxValue], [__offScreenProgressIndicator doubleValue]);
}

- (void)drawWithFrame:(NSRect)cellFrame inView:(NSView *)controlView
{
    NSBitmapImageRep    *rep = 0;
    NSImage             *image  = [[NSImage alloc] initWithSize:cellFrame.size];

    // NSLog( @"GWProgressCell draw: bar");
    [__offScreenWindow setContentSize: cellFrame.size];
    [__offScreenProgressIndicator setFrameSize: cellFrame.size];
    [[__offScreenWindow  contentView] lockFocus];
    rep = [[NSBitmapImageRep alloc] initWithFocusedViewRect: NSOffsetRect(cellFrame, -cellFrame.origin.x, -cellFrame.origin.y)];
    // if ([ii2Prefs mainLogLevel] > 7) NSLog( @"image = %p, rep = %p", image, rep);
    [[__offScreenWindow  contentView] unlockFocus];

    [image addRepresentation: rep];
    [image compositeToPoint: NSMakePoint( cellFrame.origin.x, cellFrame.origin.y + cellFrame.size.height) operation:NSCompositeCopy];
    [image release];
    [rep release];
    // sleep(1);
}

- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
}

@end

/*
 * $Log: GWProgressCell.m,v $
 * Revision 1.2  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.1.1.1  2004/02/01 17:18:15  gctwnl
 * Initial import.
 *
 * Revision 3.3  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.2  2002/12/10 14:24:49  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:22  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:50  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.11  2002/10/16 08:37:19  gerben
 * Changed log levels. Small fixes.
 *
 * Revision 1.10  2002/10/07 20:28:07  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.9  2002/10/03 13:38:04  gerben
 * Removed addChildWindow as child passes close to parent. Window can only close when there are no running tasks.
 *
 * Revision 1.8  2002/10/02 22:38:31  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */