//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2ActionTask.h"
#import "II2AppDelegate.h"
#import "II2Preferences.h"


static char rcsid[] = "$Id: II2ActionTask.m,v 1.6 2004/04/30 12:18:06 gctwnl Exp $";

@implementation II2ActionTask
- (void)dealloc
{
    if ([ii2Prefs mainLogLevel] > 8) fprintf( stderr, "Dealloc II2ActionTask: %s %u", [[self description] cString], [self retainCount]);
    [__selectorString release]; __selectorString = nil;
    [__dependentTasks release]; __dependentTasks = nil;
    [super dealloc];
}

- (id)initWithTarget:(id)target owner:(id <II2ActionTaskOwnerProtocol>)owner selector:(NSString *)selector andObject:(id)object
{
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"initWithTarget II2ActionTask: %p", self);
    self = [super init];
    __owner = owner;
    __target = target;
    if (selector) {
        __selectorString = [[NSString stringWithString:selector] retain];
    }
    else {
        __selectorString = nil;
    }
    __object = object;
    __dependentTasks = [[NSMutableArray alloc] init];
    
    return self;
}

- (id)initWithTarget:(id)target owner:(id <II2ActionTaskOwnerProtocol>)owner selector:(NSString *)selector andObject:(id)object andObject:(id)object2
{
    self = [self initWithTarget:target owner:owner selector:selector andObject:object];
    __object2 = object2;
    return self;
}

- (NSAttributedString *)attributedDisplayName
{
    NSString *text = [self description];

    return [[[NSAttributedString alloc] initWithString:text attributes:[self infoAttributes]] autorelease];
}

- (NSString *)description
{
    if (!__selectorString) {
        return @"NoOp Task";
    }
    return __selectorString;
}

- (id)progress
{
    if (__waitCounter) {
	return [NSString stringWithFormat:@"Waiting... (%u)", __waitCounter];
    }
    return @"Executing";
}

- (void)setTaskSet:(II2TaskSet *)set
{
    __taskSet = set;
}

- (void)addDependentTask:(id <II2TaskDependencyProtocol>)task
{
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"%@ will wait for %@", [task description], [self description]);
    [__dependentTasks addObject:task];
    [task wait];
}

- (NSArray *)dependentTasks
{
    return __dependentTasks;
}

- (void)addDependentTasks:(NSArray *)tasks
{
    if (!tasks) {
	NSLog( @"Weird error, nil-tasks array in II2ActionTask addDependentTasks");
    }
    if ([tasks count]) {
	[__dependentTasks addObjectsFromArray:tasks];
	[tasks makeObjectsPerformSelector:@selector(wait)];
    }
}

- (void)wait
{
    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"wait II2ActionTask: %@ (%u)", [self description], __waitCounter);
    __waitCounter++;
}

- (void)endWait
{
    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"endWait II2ActionTask: %@ (%u)", [self description], __waitCounter);
    if (__waitCounter > 0) {
	__waitCounter--;
    }
    [self launch];
}

- (void)abortWait
{
    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"abortWait II2ActionTask: %@ (%u)", [self description], __waitCounter);
    if (!__invalidated) {
	__invalidated = YES;
	[__dependentTasks makeObjectsPerformSelector:@selector(abortWait)];
    }
    if (__waitCounter > 0) {
	__waitCounter--;
    }
    if (__waitCounter == 0) {
	[__taskSet ii2TaskDidEnd:self withSuccess:NO];
	[self release];
    }
}

- (void)launch
{
    if (__waitCounter == 0) {
	if (!__invalidated) {
	    int i;
	    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"WILL perform TRUE launch of %@", [self description]);
            if (__target) {
                if (__object2) {
                    [__target performSelector:NSSelectorFromString( __selectorString) withObject:__object withObject:__object2];
                }
                else {
                    [__target performSelector:NSSelectorFromString( __selectorString) withObject:__object];
                }
            }
	    [__taskSet ii2TaskDidEnd:self withSuccess:YES];
	    [__owner actionTaskDidFinish:self withSuccess:YES];
	    //Signal dependent tasks
	    for (i=0; i<[__dependentTasks count]; i++) {
		[[__dependentTasks objectAtIndex:i] endWait];
	    }
	}
	else {
	    [__dependentTasks makeObjectsPerformSelector:@selector(abortWait)];
	    [__taskSet ii2TaskDidEnd:self withSuccess:NO];
	    [__owner actionTaskDidFinish:self withSuccess:NO];
	}
	[self release];
    }
}

- (void)abort
{
    [self terminate];
}

- (void)terminate
{
    if (__waitCounter == 0) {
	NSLog( @"II2ActionTask: This should not happen...");
    }
    else {
	__invalidated = YES;
    }
}

- (int)result
{
    NSLog( @"II2ActionTask:result should never be called!");
    return 0;
}

- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
}

@end

/*
 * $Log: II2ActionTask.m,v $
 * Revision 1.6  2004/04/30 12:18:06  gctwnl
 * 2.62.0
 *
 * Revision 1.5  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.4  2004/03/21 16:52:21  gctwnl
 * 2.58.0
 *
 * Revision 1.3  2004/03/19 23:35:11  gctwnl
 * Version 2.55.0. Has a renewed interface to support simple operations.
 * Supports progress bar control from the i-Package.
 *
 * Revision 1.2  2004/02/02 20:15:27  gctwnl
 * Fixed loss of keyed objects in package window (spinning progress). Some small fixes in logging and debug messages.
 *
 * Revision 1.1.1.1  2004/02/01 17:18:19  gctwnl
 * Initial import.
 *
 * Revision 3.8  2004/01/27 12:57:39  gerben
 * 2.44.0
 *
 * Revision 3.7  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.6  2004/01/05 23:47:15  gerben
 * *** empty log message ***
 *
 * Revision 3.5  2004/01/05 15:31:34  gerben
 * *** empty log message ***
 *
 * Revision 3.4  2003/11/08 23:38:01  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.3  2003/09/18 22:40:54  gerben
 * 2.26.0 Fragmented archives
 *
 * Revision 3.2  2002/12/10 14:24:49  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:22  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:50  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.21  2002/10/22 12:52:21  gerben
 * Added stderr handling to authenticated tasks.
 *
 * Revision 1.20  2002/10/16 21:28:22  gerben
 * A complete new rewrite of taskss, processes and stuff.
 *
 * Revision 1.19  2002/10/16 08:37:19  gerben
 * Changed log levels. Small fixes.
 *
 * Revision 1.18  2002/10/12 10:54:39  gerben
 * A lot of changes, mostly when producing output/feedback.
 *
 * Revision 1.17  2002/10/09 14:16:12  gerben
 * Changed as many %s/lossyCString uses to %@/fileSystemRepresentation/UTF8String
 *
 * Revision 1.16  2002/10/07 20:28:07  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.15  2002/10/03 18:15:22  gerben
 * A typo (bz instead of bz2) prevented the unarchive of anything.
 *
 * Revision 1.14  2002/10/03 13:38:04  gerben
 * Removed addChildWindow as child passes close to parent. Window can only close when there are no running tasks.
 *
 * Revision 1.13  2002/10/02 22:38:31  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */