//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import <Cocoa/Cocoa.h>
#import "II2DictDictionary.h"

#define DEFFM [NSFileManager defaultManager]
#define STDUD [NSUserDefaults standardUserDefaults]
#define PROCINFO [NSProcessInfo processInfo]
#define PROCENV [PROCINFO environment]
#define HOMEDIR [PROCENV objectForKey:@"HOME"]
#define UNIQUESTRING [PROCINFO globallyUniqueString]

#if !defined( NSAppKitVersionNumber10_1)
	// Compiling on a pre-10.2 system
#	define NSAppKitVersionNumber10_1 620
#endif

// #define RETAINCOUNTING 1
// #define RETAINCOUNT(x) NSLog( @"Retain %s for %@ = %u", #x, [x description], [x retainCount]);
// or
#define RETAINCOUNTING 0
#define RETAINCOUNT(x)

@interface II2AppDelegate : NSObject
{
    IBOutlet id			startupPanel;
    IBOutlet NSTextField	*startupPanelVersionTextField;
    IBOutlet NSTextField	*startupPanelBuildDateTextField;
    IBOutlet id			downloadSheet;
    
    IBOutlet id                 viewPropertiesMenuItem;
    IBOutlet id                 viewOutputMenuItem;
    IBOutlet id                 viewContentsMenuItem;
    IBOutlet id                 viewActivityMenuItem;
    IBOutlet id                 viewReportMenuItem;
}

- (NSMenuItem *)propertiesMenuItem;
- (NSMenuItem *)outputMenuItem;
- (NSMenuItem *)contentsMenuItem;
- (NSMenuItem *)activityMenuItem;
- (NSMenuItem *)reportMenuItem;

- (BOOL)applicationShouldOpenUntitledFile:(NSApplication *)sender;
- (void)applicationWillFinishLaunching:(NSNotification *)aNotification;
- (void)applicationDidFinishLaunching:(NSNotification *)aNotification;

- (IBAction)openPreferences:(id)sender;

- (IBAction)openKnownPackages:(id)sender;
- (IBAction)checkPackageUpdates:(id)sender;

- (IBAction)makeADonation:(id)sender;

@end


@interface NSObject (xmlConverter)
- (NSData *)xmlPropertyListData;
@end

@interface NSObject (ii2Attributes)
- (NSFont *)fixedFont;
- (NSDictionary *)fixedNormalAttributes;
- (NSDictionary *)fixedSignalAttributes;
- (NSDictionary *)normalAttributes;
- (NSDictionary *)signalAttributes;
- (NSDictionary *)infoAttributes;
@end

@interface NSTextView (ii2NSTextViewAdditions)
- (void)scrollToVisible:(NSTextView *)view;
- (void)resetParagraphStyleAndClear;
- (void)appendOutput:(NSString *)output;
- (void)appendAttributedOutput:(NSAttributedString *)output;
@end

extern NSData *urlDataFromURL( NSURL *url);
extern NSString *displayStringFromTimestampString( NSString *stamp);
extern NSString *gwBuildStamp();
extern NSString *gwBuildStampDisplay();

/*
 * $Log: II2AppDelegate.h,v $
 * Revision 1.4  2004/10/15 14:13:06  gctwnl
 * 2.68
 *
 * Revision 1.3  2004/10/11 20:16:24  gctwnl
 * Version 2.66
 *
 * Revision 1.2  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.1.1.1  2004/02/01 17:18:20  gctwnl
 * Initial import.
 *
 * Revision 3.8  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.7  2003/11/08 23:38:01  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.6  2003/05/18 15:53:13  gerben
 * Display absolute strings for relative URL's in i-Directory
 *
 * Revision 3.5  2003/05/18 13:35:16  gerben
 * Version 2.21.0. Improved i-Directory handling.
 *
 * Revision 3.4  2003/04/03 22:27:56  gerben
 * 2.17.0. Donation menu item.
 *
 * Revision 3.3  2003/02/06 21:04:12  gerben
 * Version 2.3.0
 *
 * Revision 3.2  2002/12/10 14:24:49  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:22  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:50  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.28  2002/11/04 12:10:06  gerben
 * i-Directory selection added to Known Packages, continued
 *
 * Revision 1.27  2002/11/04 11:54:11  gerben
 * i-Directory selection added to Known Packages
 *
 * Revision 1.26  2002/10/31 14:26:57  gerben
 * Moved authentication setting to package window
 *
 * Revision 1.25  2002/10/27 21:04:20  gerben
 * Changed Known Packages window and handling
 *
 * Revision 1.24  2002/10/25 19:27:22  gerben
 * Several small changes re known packages window. Implemented developer build stamp in main bundle.
 *
 * Revision 1.23  2002/10/25 14:13:29  gerben
 * Download sheet while downloading i-Directory
 *
 * Revision 1.22  2002/10/25 13:47:58  gerben
 * New Known Packages Window
 *
 * Revision 1.21  2002/10/17 18:49:00  gerben
 * Cache/NoCache handling on downloads. Some other changes.
 *
 * Revision 1.20  2002/10/17 10:18:59  gerben
 * Splash screen. MD5 check on opening remote package.
 *
 * Revision 1.19  2002/10/16 21:28:22  gerben
 * A complete new rewrite of taskss, processes and stuff.
 *
 * Revision 1.18  2002/10/15 21:14:21  gerben
 * A less klunky known packages menu
 *
 * Revision 1.17  2002/10/15 14:31:44  gerben
 * Last fixes. No-network access. Later creation of downloaded file.
 *
 * Revision 1.16  2002/10/14 12:42:56  gerben
 * Some name changes
 *
 * Revision 1.15  2002/10/13 10:57:22  gerben
 * Saving stable state before some additions and rewrites
 *
 * Revision 1.14  2002/10/12 10:54:38  gerben
 * A lot of changes, mostly when producing output/feedback.
 *
 * Revision 1.13  2002/10/10 00:51:24  gerben
 * Many changes. Committing a stable state.
 *
 * Revision 1.12  2002/10/09 10:53:40  gerben
 * Moved textview output to NSTexTView category.
 *
 * Revision 1.11  2002/10/08 21:34:28  gerben
 * Working GPG signature checks, shared GPG cache for II2.
 *
 * Revision 1.10  2002/10/07 20:28:06  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.9  2002/10/06 10:19:21  gerben
 * Wrote workaround for indentation problem (indentation stays around even when view is cleared).
 *
 * Revision 1.8  2002/10/02 22:38:30  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */