//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2AppDelegate.h"
#import "II2DocumentController.h"
#import "II2WindowController.h"
#import "II2Package.h"
#import "II2Preferences.h"
#import <objc/objc-class.h>
#import <Foundation/NSRunLoop.h>
#import <ExceptionHandling/NSExceptionHandler.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/resource.h>

static char rcsid[] = "$Id: II2AppDelegate.m,v 1.7 2004/10/15 14:13:06 gctwnl Exp $";

@implementation II2AppDelegate

- (BOOL)applicationShouldOpenUntitledFile:(NSApplication *)sender
{
    return NO;
}

- (void)applicationWillFinishLaunching:(NSNotification *)aNotification
{
}

- (IBAction)makeADonation:(id)sender
{
    NSURL *url = [NSURL URLWithString:@"https://www.paypal.com/xclick/business=paypal%40rna.nl&no_shipping=2&item_name=i-Installer"];
    if (url) [[NSWorkspace sharedWorkspace] openURL:url];
}


- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
    
}


- (IBAction)checkPackageUpdates:(id)sender
{
    int result = NSAlertDefaultReturn;
    
    if (![ii2Prefs fastNetwork]) {
	result = NSRunAlertPanel(@"Warning: Might be slow!", @"Do you want to check all i-Packages in your default save directory for the availability of upgrades? Since this optionally involves a lot of network traffic it may take a long time. You will get another panel when checking is complete.", @"Now", @"Later", nil);
    }
    if (result == NSAlertDefaultReturn) {
	[[NSDocumentController sharedDocumentController] openUpdatedLocalPackages:[ii2Prefs defaultSaveDir]];
    }
}

- (int)translateOldReceipts: (NSString *)dir
{
    int nrOfTranslates = 0;
    int i;
    NSArray *contents = [DEFFM directoryContentsAtPath:dir];

    for (i=0; i<[contents count]; i++) {
	BOOL isDir;
	NSString *file = [contents objectAtIndex:i];
	if ([file hasPrefix:@"."] && [file hasSuffix:@".ii2"] && [DEFFM fileExistsAtPath:[NSString stringWithFormat:@"%@/%@", dir, file] isDirectory:&isDir] && !isDir) {
	    NSString *oldStyleFullPath = [NSString stringWithFormat:@"%@/%@", dir, file];
	    NSDictionary *oldStyleDict = [NSDictionary dictionaryWithContentsOfFile:oldStyleFullPath];
	    NSString *newStyleFullPath = [NSString stringWithFormat:@"%@/%@/%@.ii2receipt", [ii2Prefs ii2LibDir], RECEIPTSSUBDIR, [[file stringByDeletingPathExtension] substringFromIndex:1]];
	    NSMutableDictionary *newStyleDict = [NSMutableDictionary dictionary];
	    NSString *customFullPath = [NSString stringWithFormat:@"%@/%@/Custom.plist", [ii2Prefs defaultSaveDir], [file substringFromIndex:1]];
	    NSMutableDictionary *customDict = [NSMutableDictionary dictionaryWithContentsOfFile:customFullPath];
	    NSDictionary *oldStyleAttrs = [DEFFM fileAttributesAtPath:oldStyleFullPath traverseLink:YES];
	    time_t oldModTime = [[oldStyleAttrs objectForKey:NSFileModificationDate] timeIntervalSince1970];
	    
	    // NSLog( @"Found %@/%@ (%@)", dir, file, [[file stringByDeletingPathExtension] substringFromIndex:1]);
	    if (oldStyleDict) {
		id moveObj;
		NSLog( @"Translating %@ and %@ into %@", oldStyleFullPath, (customDict ? customFullPath : @"(none)"), newStyleFullPath);
		if ((moveObj = [oldStyleDict objectForKey:@"timestamp"]) != nil) {
		    [newStyleDict setObject:moveObj forKey:@"timestamp"];
		}
		if ((moveObj = [customDict objectForKey:@"configuredselectorsets"]) != nil) {
		    [newStyleDict setObject:moveObj forKey:@"configuredselectorsets"];
		    [customDict removeObjectForKey:@"configuredselectorsets"];
		}
		else if ((moveObj = [oldStyleDict objectForKey:@"configuredselectorsets"]) != nil) {
		    [newStyleDict setObject:moveObj forKey:@"configuredselectorsets"];
		}
		if ((moveObj = [customDict objectForKey:@"installedselectorsets"]) != nil) {
		    [newStyleDict setObject:moveObj forKey:@"installedselectorsets"];
		    [customDict removeObjectForKey:@"installedselectorsets"];
		}
		else if ((moveObj = [oldStyleDict objectForKey:@"installedselectorsets"]) != nil) {
		    [newStyleDict setObject:moveObj forKey:@"installedselectorsets"];
		}
	    }
	    else {
		// Try old style time stamp
		NSString *oldtimestamp = [NSString stringWithContentsOfFile:oldStyleFullPath];
		if (oldtimestamp) {
		    [newStyleDict setObject:oldtimestamp forKey:@"timestamp"];
		}
	    }
	    [newStyleDict setObject:[NSString stringWithCString:getlogin()] forKey:@"writtenby"];
	    [newStyleDict setObject:[NSString stringWithFormat:@"%d", oldModTime] forKey:@"writtenat"];
	    [newStyleDict setObject:[[NSString stringWithFormat:@"%@/%@", [ii2Prefs defaultSaveDir], [file substringFromIndex:1]] stringByAbbreviatingWithTildeInPath] forKey:@"writtenfrom"];
	    [newStyleDict setObject:[NSString stringWithString:dir] forKey:@"install_location"];
	    [newStyleDict writeToFile:newStyleFullPath atomically:NO];
	    nrOfTranslates++;

	    [customDict setObject:[NSString stringWithCString:getlogin()] forKey:@"writtenby"];
	    [customDict setObject:[NSString stringWithFormat:@"%d", time( NULL)] forKey:@"writtenat"];
	    [customDict writeToFile:customFullPath atomically:NO];
	}
    }
    return nrOfTranslates;
}

- (void)assertReceiptsDirectory
{
    BOOL isDir;
    NSString *ii2LibDir = [ii2Prefs ii2LibDir];
    NSString *ii2ReceiptsDir = [NSString stringWithFormat: @"%@/%@", ii2LibDir, RECEIPTSSUBDIR];

    if ([DEFFM fileExistsAtPath:ii2ReceiptsDir isDirectory:&isDir] && isDir) {
	// The receipts library already exists.
	return;
    }
    [DEFFM createDirectoryAtPath:ii2LibDir attributes:[NSDictionary dictionaryWithObjectsAndKeys: [NSNumber numberWithUnsignedInt:0775], NSFilePosixPermissions, nil]];
    if (![DEFFM createDirectoryAtPath:ii2ReceiptsDir attributes:[NSDictionary dictionaryWithObjectsAndKeys: [NSNumber numberWithUnsignedInt:0777], NSFilePosixPermissions, nil]]) {
	NSLog( @"Unable to create Receipts Directory %@", ii2ReceiptsDir);
	NSRunAlertPanel(@"Notification", [NSString stringWithFormat:@"i-Installer was not able to create the receipts directory %@. You need to run i-Installer at least once from an account with administrator rights for tis directory to be created. i-Installer will not be able to track info on installed packages without this directory.", [ii2Prefs ii2LibDir]], @"OK", nil, nil);
    }
    else {
	int nrOfTranslates = 0;
	// These were default locations when the switch was made:
	nrOfTranslates += [self translateOldReceipts:@"/usr/local/teTeX/share/texmf.local"];
	nrOfTranslates += [self translateOldReceipts:@"/usr/local/teTeX"];
	nrOfTranslates += [self translateOldReceipts:@"/usr/local"];
	nrOfTranslates += [self translateOldReceipts:@"/usr/X11R6"];
	nrOfTranslates += [self translateOldReceipts:@"/"];
	nrOfTranslates += [self translateOldReceipts:@"/Applications/Utilities"];
	if (nrOfTranslates > 8) {
	    NSRunAlertPanel(@"Notification", [NSString stringWithFormat:@"i-Installer's Receipts mechanism has changed. I have translated all old-style receipts created by previous versions of i-Installer into new style receipts in the standard i-Installer receipts directory %@ (from known default install locations only). See the console for the receipts that were found.\n\nNote that if you installed i-Packages in non-default locations the original receipts probably have not been translated. This is not serious. It just means i-Installer will tell you erroneously in the Pkg Properties tab that a package has not been installed.", [ii2Prefs ii2LibDir]], @"OK", nil, nil);
	}
    }
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    NSArray *documents = [[NSDocumentController sharedDocumentController] documents];
    mode_t old_umask;
    struct rlimit rlp;
    NSDate *appBuildDate = [NSDate dateWithTimeIntervalSince1970:(NSTimeInterval)[gwBuildStamp() intValue]];
    NSString *versionString = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"];
    [startupPanelVersionTextField setStringValue:[NSString stringWithFormat:@"Version: %@", versionString]];
    [startupPanelBuildDateTextField setStringValue:[NSString stringWithFormat:@"Build: %@", [appBuildDate descriptionWithCalendarFormat:@"%d %b %Y, %H:%M:%S %Z" timeZone:[NSTimeZone timeZoneForSecondsFromGMT:0] locale:nil]]];
    
    if ([ii2Prefs showSplashAtStartup] && ![documents count]) {
	[startupPanel makeKeyAndOrderFront:self];
    }
    old_umask = umask( 022);
    if (old_umask != 022) {
	NSLog( @"Non-standard umask: %#o. ", old_umask);
    }
    else {
	if ([ii2Prefs mainLogLevel] > 6) {
	    NSLog( @"Umask: %#o", old_umask);
	}
    }
    umask( old_umask);
    
    if (!getrlimit(RLIMIT_NOFILE, &rlp)) {
	struct rlimit new_rlimit;
	if ([ii2Prefs mainLogLevel] > 6) {
	    NSLog( @"Max file limit. Current %u, max: %u", rlp.rlim_cur, rlp.rlim_max);
	}
	new_rlimit.rlim_cur = 4096;
	new_rlimit.rlim_max = 4096;
	if (setrlimit( RLIMIT_NOFILE, &new_rlimit)) {
	    NSLog( @"Max file limit not set.");
	}
	if (!getrlimit(RLIMIT_NOFILE, &rlp)) {
	    if ([ii2Prefs mainLogLevel] > 6) {
		NSLog( @"Max file limit. Current %u, max: %u", rlp.rlim_cur, rlp.rlim_max);
	    }
	}
    }
    else {
	NSLog( @"Max file limit not retreived.");
    }
    
    [self assertReceiptsDirectory];
}

- (void)awakeFromNib
{
    [[NSExceptionHandler defaultExceptionHandler] setExceptionHandlingMask: 1023];
    [NSApp setDelegate:self];
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
    return NSTerminateNow;
//    return NSTerminateLater;
//    [NSApp replyToApplicationShouldTerminate:YES];
}

- (IBAction)openPreferences:(id)sender
{
    II2Preferences *prefs = ii2Prefs;
    if (prefs != nil) {
	[prefs showPreferences];
    }
}

- (IBAction)openKnownPackages:(id)sender
{
    II2DocumentController *dC = [II2DocumentController sharedDocumentController];
    [dC openDocumentWithContentsOfURL:[NSURL URLWithString:[ii2Prefs defaultII2DDirectory]] display:YES];
}

- (BOOL)validateMenuItem:(NSMenuItem *)anItem
{
    // NSLog( @"%@ Validating %@", [[self class] description], [anItem description]);

    if ([ii2Prefs networkAccessDisabled] && ([anItem tag] == 88)) {
	return NO;
    }
    return YES;
}

- (NSMenuItem *)propertiesMenuItem
{
    return viewPropertiesMenuItem;
}

- (NSMenuItem *)outputMenuItem
{
    NSLog( @"%@ returning %@", [[self class] description], viewOutputMenuItem);
    return viewOutputMenuItem;
}

- (NSMenuItem *)contentsMenuItem
{
    return viewContentsMenuItem;
}

- (NSMenuItem *)activityMenuItem
{
    return viewActivityMenuItem;
}

- (NSMenuItem *)reportMenuItem
{
    return viewReportMenuItem;
}

- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
}

@end

@implementation NSObject (xmlConverter)
- (NSData *)xmlPropertyListData
{
    return [(NSData *)CFPropertyListCreateXMLData(kCFAllocatorDefault, (CFPropertyListRef)self) autorelease];
}
@end

@implementation NSObject (ii2Attributes)
static NSDictionary 		*__normalAttributes = nil;
static NSDictionary 		*__signalAttributes = nil;
static NSDictionary 		*__infoAttributes = nil;
static NSFont 			*__pStyleFixedFont = nil;
static NSFont 			*__pStyleUserFont = nil;
static NSDictionary	 	*__pStyleFixedNormalAttr = nil;
static NSDictionary	 	*__pStyleFixedSignalAttr = nil;
static NSMutableParagraphStyle	*__pStyleLeftAlignment = nil;

- (NSMutableParagraphStyle *)fixedAlignment
{
    if (!__pStyleLeftAlignment) {
	__pStyleLeftAlignment = [[NSMutableParagraphStyle alloc] init];
	[__pStyleLeftAlignment setAlignment:NSLeftTextAlignment];
    }
    return __pStyleLeftAlignment;
}

- (NSFont *)fixedFont
{
    if (!__pStyleFixedFont) {
	__pStyleFixedFont = [[NSFont userFixedPitchFontOfSize:0.0] retain];
    }
    return __pStyleFixedFont;
}

- (NSFont *)userFont
{
    if (!__pStyleUserFont) {
	__pStyleUserFont = [[NSFont userFontOfSize:0.0] retain];
    }
    return __pStyleUserFont;
}

- (NSDictionary *)fixedNormalAttributes
{
    if (!__pStyleFixedNormalAttr) {
	__pStyleFixedNormalAttr = [[NSDictionary dictionaryWithObjectsAndKeys:[NSColor blackColor], NSForegroundColorAttributeName, [self fixedAlignment], NSParagraphStyleAttributeName, [self fixedFont], NSFontAttributeName, nil] retain];
    }
    return __pStyleFixedNormalAttr;
}

- (NSDictionary *)fixedSignalAttributes
{
    if (!__pStyleFixedSignalAttr) {
	__pStyleFixedSignalAttr = [[NSDictionary dictionaryWithObjectsAndKeys:[NSColor redColor], NSForegroundColorAttributeName, [self fixedAlignment], NSParagraphStyleAttributeName, [self fixedFont], NSFontAttributeName, nil] retain];
    }
    return __pStyleFixedSignalAttr;
}

- (NSDictionary *)normalAttributes
{
    if (!__normalAttributes) {
	__normalAttributes = [[NSDictionary dictionaryWithObjectsAndKeys:[NSColor blackColor], NSForegroundColorAttributeName, [self fixedAlignment], NSParagraphStyleAttributeName, [self userFont], NSFontAttributeName, nil] retain];
    }
    return __normalAttributes;
}

- (NSDictionary *)signalAttributes
{
    if (!__signalAttributes) {
	__signalAttributes = [[NSDictionary dictionaryWithObjectsAndKeys:[NSColor redColor], NSForegroundColorAttributeName, [self fixedAlignment], NSParagraphStyleAttributeName, [self userFont], NSFontAttributeName, nil] retain];
    }
    return __signalAttributes;
}

- (NSDictionary *)infoAttributes
{
    if (!__infoAttributes) {
	__infoAttributes = [[NSDictionary dictionaryWithObjectsAndKeys:[NSColor orangeColor], NSForegroundColorAttributeName, [self fixedAlignment], NSParagraphStyleAttributeName, [self userFont], NSFontAttributeName, nil] retain];
    }
    return __infoAttributes;
}
@end

@implementation NSTextView (ii2NSTextViewAdditions)
- (void)appendAttributedOutput:(NSAttributedString *)output
{
    [[self textStorage] appendAttributedString:output];
    // setup a selector to be called the next time through the event loop to scroll
    // the view to the just pasted text.  We don't want to scroll right now,
    // because of a bug in Mac OS X version 10.1 that causes scrolling in the context
    // of a text storage update to starve the app of events
    if (floor( NSAppKitVersionNumber) <= NSAppKitVersionNumber10_1) {
	// Running on a 10.0 - 10.1.x system
	[self performSelector:@selector(scrollToVisible:) withObject:self afterDelay:0.0];
    }
    else {
	[self scrollToVisible:self];
    }
}

- (void)resetParagraphStyleAndClear
{
    NSParagraphStyle *paraStyle = [NSParagraphStyle defaultParagraphStyle];

    [self setString:@" "];
    [[self textStorage] addAttribute:NSParagraphStyleAttributeName value:paraStyle range:NSMakeRange(0, [[self string] length])];
    [self setString:@""];
    [self setFont:[self fixedFont]];
}

- (void)appendOutput:(NSString *)output
{
    [self appendAttributedOutput: [[[NSAttributedString alloc] initWithString: output attributes:[self fixedNormalAttributes]] autorelease]];
}

- (void)scrollToVisible:(NSTextView *)view
{
    [view scrollRangeToVisible:NSMakeRange([[view string] length], 0)];
}
@end

static NSString *__gwBuildStamp = nil;

NSString *gwBuildStamp()
{
    if (!__gwBuildStamp) {
	NSString *stampPath = [[NSBundle mainBundle] pathForResource:@"gwbuild" ofType:@"stamp"];
	__gwBuildStamp = [[NSString alloc] initWithContentsOfFile:stampPath];
	if (!__gwBuildStamp) {
	    __gwBuildStamp = @"0";
	}
    }
    return __gwBuildStamp;
}

NSString *gwBuildStampDisplay()
{
    return displayStringFromTimestampString( gwBuildStamp());
}

extern NSString *displayStringFromTimestampString( NSString *stamp)
{
    if (stamp) {
	return [[NSDate dateWithTimeIntervalSince1970:[[NSNumber numberWithLong:[stamp intValue]] doubleValue]] descriptionWithCalendarFormat:@"%Y/%m/%d %H:%M:%S" timeZone:[NSTimeZone timeZoneWithAbbreviation:@"GMT"] locale:nil];
    }
    else {
	return @"?";
    }
}

extern NSData *urlDataFromURL( NSURL *url)
{
    NSData *urlData = nil;

    if (url) {
	NSURLHandle *urlHandle = [[url URLHandleUsingCache:![ii2Prefs cacheDisabled]] retain];

	if (urlHandle) {
	    if ([[url scheme] hasPrefix:@"http"]) {
		[urlHandle writeProperty:@"i-Installer/2.0" forKey:@"User-Agent"];
		if ([ii2Prefs cacheDisabled]) {
		    [urlHandle writeProperty:@"no-cache" forKey:@"Cache-Control"];
		}
	    }

	    urlData = [[urlHandle loadInForeground] retain];
	    [urlHandle release];
	}
    }

    return [urlData autorelease];
}

// Not needed now that I have found _setIncludeNewFolderButton:...
#if 0
@implementation NSSavePanel (SwizzlingForFunAndProfit)
// From Mac OS X Dev by Luke Adamson  luke@frontbase.com
static IMP oldItemHitIMP = NULL;
static IMP oldGotoFavoriteIMP = NULL;

// Lifted from OmniBase.framework with minor modifications
IMP OBReplaceMethodImplementation( Class aClass, SEL oldSelector, IMP newImp)
{
    struct objc_method *thisMethod;
    IMP oldImp = NULL;
    extern void _objc_flush_caches(Class);

    if ((thisMethod = class_getInstanceMethod(aClass, oldSelector))) {
	oldImp = thisMethod->method_imp;

	// Replace the method in place
	thisMethod->method_imp = newImp;

	// Flush the method cache
	_objc_flush_caches(aClass);
    }

    return oldImp;
}

// Lifted from OmniBase.framework with minor modifications
IMP OBReplaceMethodImplementationWithSelector( Class aClass, SEL oldSelector, SEL newSelector)
{
    struct objc_method *newMethod;

    newMethod = class_getInstanceMethod( aClass, newSelector);
    return OBReplaceMethodImplementation( aClass, oldSelector, newMethod->method_imp);
}

// Swizzle some methods
+ (void)load;
{
    oldItemHitIMP = OBReplaceMethodImplementationWithSelector([self class], @selector(_itemHit:), @selector(_replacedItemHit:));
    oldGotoFavoriteIMP = OBReplaceMethodImplementationWithSelector([self class], @selector(_gotoFavorite:), @selector(_replacedGotoFavorite:));
}

// This method will get called when the user selects an item from the browser.  Be sure to call the original implementation (as is done in this example), otherwise NSSavePanel will stop working.  If you are going to go this route, I would recommend having this method call -[self delegate] and call an extended delegate method of your own design. Retrieving the path information from the "item" argument is left as an exercise for the reader.

- (id)_replacedItemHit:(id)item;
{
    NSLog(@"Item class: %@, item: %@", [item class], item);
    return oldItemHitIMP(self, @selector(_itemHit:), item);
}

// Same as above, but with Favorites.
- (void)_replacedGotoFavorite:(id)favorite;
{
    NSLog(@"Favorite class: %@, favorite: %@", [favorite class],
	  favorite);
    oldGotoFavoriteIMP(self, @selector(_gotoFavorite:), favorite);
}

@end
#endif

/*
 * $Log: II2AppDelegate.m,v $
 * Revision 1.7  2004/10/15 14:13:06  gctwnl
 * 2.68
 *
 * Revision 1.6  2004/10/11 20:16:24  gctwnl
 * Version 2.66
 *
 * Revision 1.5  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.4  2004/03/22 22:48:44  gctwnl
 * 2.59.0
 *
 * Revision 1.3  2004/03/21 16:52:21  gctwnl
 * 2.58.0
 *
 * Revision 1.2  2004/02/27 23:24:24  gctwnl
 * 2.52.0 entry after a long period of CVS problems
 *
 * Revision 1.1.1.1  2004/02/01 17:18:20  gctwnl
 * Initial import.
 *
 * Revision 3.12  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.11  2004/01/05 15:31:34  gerben
 * *** empty log message ***
 *
 * Revision 3.10  2003/11/09 21:43:06  gerben
 * 2.32.0
 *
 * Revision 3.9  2003/11/08 23:38:02  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.8  2003/05/18 15:53:13  gerben
 * Display absolute strings for relative URL's in i-Directory
 *
 * Revision 3.7  2003/05/18 13:35:16  gerben
 * Version 2.21.0. Improved i-Directory handling.
 *
 * Revision 3.6  2003/04/03 22:27:56  gerben
 * 2.17.0. Donation menu item.
 *
 * Revision 3.5  2003/04/02 23:22:49  gerben
 * Set umask to something reasonable
 *
 * Revision 3.4  2003/02/09 16:53:51  gerben
 * 2.4.0 bugfix release
 *
 * Revision 3.3  2003/02/06 21:04:13  gerben
 * Version 2.3.0
 *
 * Revision 3.2  2002/12/10 14:24:50  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:23  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:50  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.39  2002/11/24 23:09:27  gerben
 * Tom's changes and remarks mainly
 *
 * Revision 1.38  2002/11/04 12:10:06  gerben
 * i-Directory selection added to Known Packages, continued
 *
 * Revision 1.37  2002/11/04 11:54:11  gerben
 * i-Directory selection added to Known Packages
 *
 * Revision 1.36  2002/10/31 14:26:57  gerben
 * Moved authentication setting to package window
 *
 * Revision 1.35  2002/10/29 09:36:22  gerben
 * RC2
 *
 * Revision 1.34  2002/10/28 21:49:46  gerben
 * A lot of changes wrt known packages.
 *
 * Revision 1.33  2002/10/27 21:06:19  gerben
 * Changed Known Packages window and handling
 *
 * Revision 1.32  2002/10/25 19:27:22  gerben
 * Several small changes re known packages window. Implemented developer build stamp in main bundle.
 *
 * Revision 1.31  2002/10/25 17:40:09  gerben
 * Fixed editable and double click in known packages table view
 *
 * Revision 1.30  2002/10/25 14:13:29  gerben
 * Download sheet while downloading i-Directory
 *
 * Revision 1.29  2002/10/25 13:47:59  gerben
 * New Known Packages Window
 *
 * Revision 1.28  2002/10/24 23:13:40  gerben
 * No load of menu at first menu access
 *
 * Revision 1.27  2002/10/24 13:03:31  gerben
 * Disable menu items that can affect downloads and availability of files when package is active.
 *
 * Revision 1.26  2002/10/17 20:36:03  gerben
 * Reversed default for caching
 *
 * Revision 1.25  2002/10/17 18:49:00  gerben
 * Cache/NoCache handling on downloads. Some other changes.
 *
 * Revision 1.24  2002/10/17 10:18:59  gerben
 * Splash screen. MD5 check on opening remote package.
 *
 * Revision 1.23  2002/10/16 21:28:22  gerben
 * A complete new rewrite of taskss, processes and stuff.
 *
 * Revision 1.22  2002/10/15 21:14:21  gerben
 * A less klunky known packages menu
 *
 * Revision 1.21  2002/10/15 14:31:44  gerben
 * Last fixes. No-network access. Later creation of downloaded file.
 *
 * Revision 1.20  2002/10/14 12:42:56  gerben
 * Some name changes
 *
 * Revision 1.19  2002/10/14 12:28:21  gerben
 * Some extra progress messages
 *
 * Revision 1.18  2002/10/13 22:22:27  gerben
 * Re-created file menu so validating menu items work. Changed code for inspect and fatten to use generic task array starter.
 *
 * Revision 1.17  2002/10/13 21:20:15  gerben
 * Fixed a serious bug where the task rewrite was a bit release-happy
 *
 * Revision 1.16  2002/10/13 10:57:22  gerben
 * Saving stable state before some additions and rewrites
 *
 * Revision 1.15  2002/10/12 10:54:38  gerben
 * A lot of changes, mostly when producing output/feedback.
 *
 * Revision 1.14  2002/10/10 00:51:24  gerben
 * Many changes. Committing a stable state.
 *
 * Revision 1.13  2002/10/09 10:53:40  gerben
 * Moved textview output to NSTexTView category.
 *
 * Revision 1.12  2002/10/08 22:25:02  gerben
 * GPG checking complete. Output of parallel tasks will not mingle.
 *
 * Revision 1.11  2002/10/07 20:28:06  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.10  2002/10/06 10:19:21  gerben
 * Wrote workaround for indentation problem (indentation stays around even when view is cleared).
 *
 * Revision 1.9  2002/10/03 13:38:04  gerben
 * Removed addChildWindow as child passes close to parent. Window can only close when there are no running tasks.
 *
 * Revision 1.8  2002/10/02 22:38:30  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */