//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2DictDictionary.h"
#import "II2AppDelegate.h"
#import "II2Preferences.h"

static char rcsid[] = "$Id: II2DictDictionary.m,v 1.2 2004/04/25 13:13:21 gctwnl Exp $";

static NSComparisonResult stringSort(NSString *string1, NSString *string2, void *context);
static NSComparisonResult dictSort(NSDictionary *dict1, NSDictionary *dict2, void *context);

@implementation II2DictDictionary
- (void)dealloc
{
    [__loadedIDirectories release]; __loadedIDirectories = nil;
    [__iPackages release]; __iPackages = nil;
    [__iPackageFileNames release]; __iPackageFileNames = nil;
    [super dealloc];
}

- init
{
    self = [super init];
    if (self) {
	__loadedIDirectories = [[NSMutableDictionary alloc] init];
	__iPackages = [[NSMutableDictionary alloc] init];
	__iPackageFileNames = [[NSMutableDictionary alloc] init];
	__maxDescendLevel = 3;
    }
    return self;
}

- (void)setMaxDescendLevel: (int)level
{
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"ii2DictDictionary: setMaxLevel: %d", level);
    __maxDescendLevel = level;
}

- (void)addDictURLString: (NSString *)urlString atLevel:(int)level
{
    if (!urlString) {
	NSLog( @"Nil urlString. No i-Directory addition");
	return;
    }
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"ii2DictDictionary: addDictURLString: %@ atLevel %d", urlString, level);
    if (floor( NSAppKitVersionNumber) <= NSAppKitVersionNumber10_1) {
	// Running on a 10.0 - 10.1.x system
	if ([urlString hasPrefix:@"ftp"]) {
	    NSLog( @"FTP dictionaries not supported on Mac OS X version below 10.2");
	    return;
	}
    }
    if (level <= __maxDescendLevel) {
	NSURL *dictURL = [NSURL URLWithString:urlString];
	if (dictURL) {
	    // NSPropertyListFormat format;
	    NSString *error = nil;
	    NSData *urlData = urlDataFromURL( dictURL);
	    NSDictionary *tmpDict;

	    if (!urlData) {
		NSLog( @"DictDict: No Data received for URL: %@", [dictURL absoluteString]);
		return;
	    }
	    tmpDict = (NSDictionary *)CFPropertyListCreateFromXMLData( NULL, (CFDataRef)urlData, kCFPropertyListImmutable, (CFStringRef *)&error);
	    // tmpDict = [NSPropertyListSerialization propertyListFromData:urlData mutabilityOption:kCFPropertyListImmutable format:&format errorDescription:&error]; // 10.2 only...
	    if (error) {
		NSLog( @"DictDict: propertylist parse error: %@", error);
		[error release];
	    }

	    if (tmpDict) {
		NSDictionary *pkgDict = [tmpDict objectForKey:@"i-Packages"];
		NSDictionary *dictDict = [tmpDict objectForKey:@"i-Directories"];
		//if (![__loadedIDirectories count]) {
		//    // First entry.
		//    [__loadedIDirectories setObject:@"Current root directory" forKey:urlString];
		//}
		if (pkgDict) {
		    NSEnumerator *enumerator = [pkgDict keyEnumerator];
		    NSString *pkgURL;

		    while ((pkgURL = [enumerator nextObject])) {
			NSString *pkgName = [pkgDict objectForKey:pkgURL];
			NSString *pkgFileName = [[pkgURL componentsSeparatedByString:@"/"] lastObject];
			NSMutableDictionary *urlFoundDictionary = [__iPackages objectForKey:pkgName];
			if (floor( NSAppKitVersionNumber) <= NSAppKitVersionNumber10_1) {
			    // Running on a 10.0 - 10.1.x system
			    if ([pkgURL hasPrefix:@"ftp"]) {
				NSLog( @"FTP packages not supported on Mac OS X version below 10.2");
				continue;
			    }
			}
			if ([ii2Prefs noUniqueII2DirectoryNames] || ![__iPackageFileNames objectForKey:pkgFileName]) {
			    if (!urlFoundDictionary) {
				[__iPackages setObject:[NSMutableDictionary dictionaryWithObject:pkgName forKey:pkgURL] forKey:pkgName];
			    }
			    else {
				[urlFoundDictionary setObject:pkgName forKey:pkgURL];
			    }
			    [__iPackageFileNames setObject:@"YES" forKey:pkgFileName];
			}
		    }
		}
		else {
		    if ([ii2Prefs mainLogLevel] > 2) NSLog( @"ii2DictDictionary: addDictURLString: no i-Package URLs in dictionary found at %@", urlString);
		}
		if (dictDict) {
		    NSEnumerator *enumerator = [dictDict keyEnumerator];
		    NSString *key;

		    while ((key = [enumerator nextObject])) {
			if (![__loadedIDirectories objectForKey:key]) {
			    [__loadedIDirectories setObject:[dictDict objectForKey:key] forKey:key];
			    [self addDictURLString:key atLevel:(level+1)];
			}
		    }
		}
		else {
		    if ([ii2Prefs mainLogLevel] > 2) NSLog( @"ii2DictDictionary: addDictURLString: no i-Dictionary URLs in dictionary found at %@", urlString);
		}
	    }
	    else {
		if ([ii2Prefs mainLogLevel] > 1) NSLog( @"ii2DictDictionary: addDictURLString: no dictionary found at %@", urlString);
	    }
	}
	else {
	   if ([ii2Prefs mainLogLevel] > 0)  NSLog( @"ii2DictDictionary: addDictURLString: invalid URL String: %@", urlString);
	}
    }
}

- (NSArray *)namesAndURLs
{
    NSMutableArray *result = [[[NSMutableArray alloc] init] autorelease];
    NSEnumerator *nameEnumerator = [__iPackages keyEnumerator];
    NSString *pkgName;

    while ((pkgName = [nameEnumerator nextObject])) {
	NSDictionary *urlDict = [__iPackages objectForKey:pkgName];
	NSEnumerator *urlEnumerator = [urlDict keyEnumerator];
	NSString *pkgURL = [urlEnumerator nextObject];
	if (pkgURL) {
	    [result addObject:[NSDictionary dictionaryWithObjectsAndKeys: pkgName, @"name", pkgURL, @"url", nil]];
	}
	if ([ii2Prefs noUniqueII2DirectoryNames]) {
	    while ((pkgURL = [urlEnumerator nextObject])) {
		[result addObject:[NSDictionary dictionaryWithObjectsAndKeys: pkgName, @"name", pkgURL, @"url", nil]];
	    }
	}
    }
    [result sortUsingFunction:dictSort context:nil];
    return result;
}

- (NSArray *)directoryNamesAndURLs
{
    NSMutableArray *result = [[[NSMutableArray alloc] init] autorelease];
    NSEnumerator *nameEnumerator = [__loadedIDirectories keyEnumerator];
    NSString *dirURL;

    while ((dirURL = [nameEnumerator nextObject])) {
	NSString *dirName = [__loadedIDirectories objectForKey:dirURL];
	[result addObject:[NSDictionary dictionaryWithObjectsAndKeys: dirName, @"name", dirURL, @"url", nil]];
    }
    [result sortUsingFunction:dictSort context:nil];
    return result;
}

- (NSArray *)displayNames
{
    NSMutableArray *result = [[[NSMutableArray alloc] init] autorelease];
    NSEnumerator *nameEnumerator = [__iPackages keyEnumerator];
    NSString *pkgName;

    while ((pkgName = [nameEnumerator nextObject])) {
	NSDictionary *urlDict = [__iPackages objectForKey:pkgName];
	if ([urlDict count] == 1) {
	    [result addObject:pkgName];
	}
	else {
	    NSEnumerator *urlEnumerator = [urlDict keyEnumerator];
	    NSString *pkgURL;
	    while ((pkgURL = [urlEnumerator nextObject])) {
		[result addObject:[NSString stringWithFormat:@"%@ | %@", pkgName, pkgURL]];
	    }
	}
    }
    [result sortUsingFunction:stringSort context:nil];

    return result;
}

- (NSString *)urlForName:(NSString *)displayName
{
    NSRange barRange = [displayName rangeOfString:@"|" options:NSBackwardsSearch];
    if (!NSEqualRanges( barRange, NSMakeRange( NSNotFound, 0))) {
	NSRange urlRange = NSMakeRange( barRange.location + 2, [displayName length] - barRange.location - 2);
	NSString *urlString = [displayName substringWithRange:urlRange];
	return urlString;
    }
    else {
	NSArray *urlStringArray = [[__iPackages objectForKey:displayName] allKeys];
	if ([urlStringArray count] != 1) {
	    NSLog( @"Program error: unexpected duplicate urls for package name");
	}
	return [urlStringArray objectAtIndex:0];
    }
}


//- (NSDictionary *)allPackages
//{
//    return __iPackages;
//}

- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
}

@end

static NSComparisonResult stringSort(NSString *string1, NSString *string2, void *context)
{
    return [string1 caseInsensitiveCompare:string2];
}

static NSComparisonResult dictSort(NSDictionary *dict1, NSDictionary *dict2, void *context)
{
    return [[dict1 objectForKey:@"name"] caseInsensitiveCompare:[dict2 objectForKey:@"name"]];
}


/*
 * $Log: II2DictDictionary.m,v $
 * Revision 1.2  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.1.1.1  2004/02/01 17:18:20  gctwnl
 * Initial import.
 *
 * Revision 3.6  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.5  2004/01/05 15:31:34  gerben
 * *** empty log message ***
 *
 * Revision 3.4  2003/02/09 16:53:51  gerben
 * 2.4.0 bugfix release
 *
 * Revision 3.3  2003/01/04 02:04:43  gerben
 * Version 2.2.0
 *
 * Revision 3.2  2002/12/10 14:24:50  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:23  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:50  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.24  2002/11/24 10:32:02  gerben
 * Inactive code option to show root iDirectory in iDirectory list.
 *
 * Revision 1.23  2002/11/04 11:54:11  gerben
 * i-Directory selection added to Known Packages
 *
 * Revision 1.22  2002/10/31 14:26:57  gerben
 * Moved authentication setting to package window
 *
 * Revision 1.21  2002/10/28 21:49:45  gerben
 * A lot of changes wrt known packages.
 *
 * Revision 1.20  2002/10/25 13:47:58  gerben
 * New Known Packages Window
 *
 * Revision 1.19  2002/10/17 20:19:26  gerben
 * Changes wrt disabled network access
 *
 * Revision 1.18  2002/10/17 18:49:00  gerben
 * Cache/NoCache handling on downloads. Some other changes.
 *
 * Revision 1.17  2002/10/16 21:28:22  gerben
 * A complete new rewrite of taskss, processes and stuff.
 *
 * Revision 1.16  2002/10/16 08:37:18  gerben
 * Changed log levels. Small fixes.
 *
 * Revision 1.15  2002/10/15 21:14:21  gerben
 * A less klunky known packages menu
 *
 * Revision 1.14  2002/10/09 14:16:12  gerben
 * Changed as many %s/lossyCString uses to %@/fileSystemRepresentation/UTF8String
 *
 * Revision 1.13  2002/10/07 20:28:05  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.12  2002/10/03 13:38:04  gerben
 * Removed addChildWindow as child passes close to parent. Window can only close when there are no running tasks.
 *
 * Revision 1.11  2002/10/02 22:38:30  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */