//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2Preferences.h"
#import "II2AppDelegate.h"
#import "II2Directory.h"

NSString *II2DirectoryType = @"i-Package-v2-dir";

static char rcsid[] = "$Id: II2Directory.m,v 1.2 2004/04/25 13:13:21 gctwnl Exp $";

static NSComparisonResult stringSort(NSString *string1, NSString *string2, void *context);
static NSComparisonResult dictSort(NSDictionary *dict1, NSDictionary *dict2, void *context);

@implementation II2Directory

- (void)dealloc
{
    [__loadedIDirectories release]; __loadedIDirectories = nil;
    [__iPackages release]; __iPackages = nil;
    [__iPackageNotes release]; __iPackageNotes = nil;
    [__iPackageFileNames release]; __iPackageFileNames = nil;
    [super dealloc];
}

- (id) init
{
    self = [super init];
    __loadedIDirectories = [[NSMutableDictionary alloc] init];
    __iPackages = [[NSMutableDictionary alloc] init];
    __iPackageNotes = [[NSMutableDictionary alloc] init];
    __iPackageFileNames = [[NSMutableDictionary alloc] init];
    __maxDescendLevel = [ii2Prefs maxII2DDescendLevel];
    return self;
}

- (id)initWithContentsOfURL:(NSURL *)aURL ofType:(NSString *)docType
{
    self = [super initWithContentsOfURL:aURL ofType:docType];
    __startURLString = [[NSString stringWithString:[aURL absoluteString]] retain];
    return self;
}

- (id)initWithContentsOfFile:(NSString *)fileName ofType:(NSString *)docType
{
    return [self initWithContentsOfURL:[NSURL fileURLWithPath:fileName] ofType:docType];
}

- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
    // NSLog( @"II2P: windowControllerDidLoadNib");
    [super windowControllerDidLoadNib:aController];
    // Add any code here that need to be executed once the windowController has loaded the document's window.
    [__cachedWindowController showWindow:nil];
}

- (II2DirectoryWindowController *)windowController
{
    return __cachedWindowController;
}
- (void)makeWindowControllers {
    // [super makeWindowControllers];
    __cachedWindowController = [[II2DirectoryWindowController allocWithZone:[self zone]] initWithWindowNibName:@"II2KnownPackages"];
    [self addWindowController:__cachedWindowController];
}

- (NSString *)startURLString
{
    return __startURLString;
}

- (void)setStartURLString: (NSString *)newStartURLString
{
    [__startURLString release];
    __startURLString = [[NSString stringWithString:newStartURLString] retain];
}

- (void) reload
{
    [__loadedIDirectories removeAllObjects];
    [__iPackages removeAllObjects];
    [__iPackageNotes removeAllObjects];
    [__iPackageFileNames removeAllObjects];
    [self addDictURLString: __startURLString atLevel:1];
}

- (NSData *)dataRepresentationOfType:(NSString *)type {
    // Implement to provide a persistent data representation of your document OR remove this and implement the file-wrapper or file path based save methods.
    return nil;
}

- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)type {
    // Implement to load a persistent data representation of your document OR remove this and implement the file-wrapper or file path based load methods.
    return YES;
}

- (void)setMaxDescendLevel: (int)level
{
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"ii2DictDictionary: setMaxLevel: %d", level);
    __maxDescendLevel = level;
}

- (void)addDictURLString: (NSString *)urlString atLevel:(int)level
{
    if (!urlString) {
	NSLog( @"Nil urlString. No i-Directory addition");
	return;
    }
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"ii2DictDictionary: addDictURLString: %@ atLevel %d", urlString, level);
    if (floor( NSAppKitVersionNumber) <= NSAppKitVersionNumber10_1) {
	// Running on a 10.0 - 10.1.x system
	if ([urlString hasPrefix:@"ftp"]) {
	    NSLog( @"FTP dictionaries not supported on Mac OS X version below 10.2");
	    return;
	}
    }
    if (level <= __maxDescendLevel) {
	NSURL *dictURL = [NSURL URLWithString:urlString];
	if (dictURL) {
	    // NSPropertyListFormat format;
	    NSString *error = nil;
	    NSData *urlData = urlDataFromURL( dictURL);
	    NSDictionary *tmpDict;

	    if (!urlData) {
		return;
	    }
	    tmpDict = (NSDictionary *)CFPropertyListCreateFromXMLData( NULL, (CFDataRef)urlData, kCFPropertyListImmutable, (CFStringRef *)&error);
	    // tmpDict = [NSPropertyListSerialization propertyListFromData:urlData mutabilityOption:kCFPropertyListImmutable format:&format errorDescription:&error]; // 10.2 only...
	    if (error) {
		NSLog( @"DictDict: propertylist parse error: %@", error);
		[error release];
	    }

	    if (tmpDict) {
		NSDictionary *pkgDict = [tmpDict objectForKey:@"i-Packages"];
		NSDictionary *dictDict = [tmpDict objectForKey:@"i-Directories"];
		//if (![__loadedIDirectories count]) {
		//    // First entry.
		//    [__loadedIDirectories setObject:@"Current root directory" forKey:urlString];
		//}
		if (pkgDict) {
		    NSEnumerator *enumerator = [pkgDict keyEnumerator];
		    NSString *pkgURL;

		    while ((pkgURL = [enumerator nextObject])) {
			if ([pkgURL hasSuffix:@"-note"]) {
			    continue;
			}
			NSString *pkgName = [pkgDict objectForKey:pkgURL];
			NSString *pkgNote = [pkgDict objectForKey:[pkgURL stringByAppendingString:@"-note"]];
			NSString *pkgFileName = [[pkgURL componentsSeparatedByString:@"/"] lastObject];
			NSMutableDictionary *urlFoundDictionary = [__iPackages objectForKey:pkgName];
			if (floor( NSAppKitVersionNumber) <= NSAppKitVersionNumber10_1) {
			    // Running on a 10.0 - 10.1.x system
			    if ([pkgURL hasPrefix:@"ftp"]) {
				NSLog( @"FTP packages not supported on Mac OS X version below 10.2");
				continue;
			    }
			}
			if ([ii2Prefs noUniqueII2DirectoryNames] || ![__iPackageFileNames objectForKey:pkgFileName]) {
			    NSURL *stdURL = [[NSURL URLWithString:pkgURL relativeToURL:dictURL] standardizedURL];
			    if (!urlFoundDictionary) {
				[__iPackages setObject:[NSMutableDictionary dictionaryWithObject:pkgName forKey:[stdURL absoluteString]] forKey:pkgName];
				if (pkgNote) {
				    [__iPackageNotes setObject:pkgNote forKey:pkgName];
				}
			    }
			    else {
				[urlFoundDictionary setObject:pkgName forKey:[stdURL absoluteString]];
			    }
			    [__iPackageFileNames setObject:@"YES" forKey:pkgFileName];
			}
		    }
		}
		else {
		    if ([ii2Prefs mainLogLevel] > 2) NSLog( @"ii2DictDictionary: addDictURLString: no i-Package URLs in dictionary found at %@", urlString);
		}
		if (dictDict) {
		    NSEnumerator *enumerator = [dictDict keyEnumerator];
		    NSString *key;

		    while ((key = [enumerator nextObject])) {
			if (![__loadedIDirectories objectForKey:key]) {
			    [__loadedIDirectories setObject:[dictDict objectForKey:key] forKey:key];
			    [self addDictURLString:key atLevel:(level+1)];
			}
		    }
		}
		else {
		    if ([ii2Prefs mainLogLevel] > 2) NSLog( @"ii2DictDictionary: addDictURLString: no i-Dictionary URLs in dictionary found at %@", urlString);
		}
	    }
	    else {
		if ([ii2Prefs mainLogLevel] > 1) NSLog( @"ii2DictDictionary: addDictURLString: no dictionary found at %@", urlString);
	    }
	}
	else {
	    if ([ii2Prefs mainLogLevel] > 0)  NSLog( @"ii2DictDictionary: addDictURLString: invalid URL String: %@", urlString);
	}
    }
}

- (NSArray *)namesAndURLs
{
    NSMutableArray *result = [[[NSMutableArray alloc] init] autorelease];
    NSEnumerator *nameEnumerator = [__iPackages keyEnumerator];
    NSString *pkgName;

    while ((pkgName = [nameEnumerator nextObject])) {
	NSDictionary *urlDict = [__iPackages objectForKey:pkgName];
	NSEnumerator *urlEnumerator = [urlDict keyEnumerator];
	NSString *pkgURL = [urlEnumerator nextObject];
	NSString *pkgNote = [__iPackageNotes objectForKey:pkgName];
	if (pkgURL) {
	    [result addObject:[NSDictionary dictionaryWithObjectsAndKeys: pkgName, @"name", pkgURL, @"url", pkgNote, @"note", nil]];
	}
	if ([ii2Prefs noUniqueII2DirectoryNames]) {
	    while ((pkgURL = [urlEnumerator nextObject])) {
		[result addObject:[NSDictionary dictionaryWithObjectsAndKeys: pkgName, @"name", pkgURL, @"url", pkgNote, @"note", nil]];
	    }
	}
    }
    [result sortUsingFunction:dictSort context:nil];
    return result;
}

- (NSArray *)directoryNamesAndURLs
{
    NSMutableArray *result = [[[NSMutableArray alloc] init] autorelease];
    NSEnumerator *nameEnumerator = [__loadedIDirectories keyEnumerator];
    NSString *dirURL;

    while ((dirURL = [nameEnumerator nextObject])) {
	NSString *dirName = [__loadedIDirectories objectForKey:dirURL];
	[result addObject:[NSDictionary dictionaryWithObjectsAndKeys: dirName, @"name", dirURL, @"url", nil]];
    }
    [result sortUsingFunction:dictSort context:nil];
    return result;
}

- (NSArray *)displayNames
{
    NSMutableArray *result = [[[NSMutableArray alloc] init] autorelease];
    NSEnumerator *nameEnumerator = [__iPackages keyEnumerator];
    NSString *pkgName;

    while ((pkgName = [nameEnumerator nextObject])) {
	NSDictionary *urlDict = [__iPackages objectForKey:pkgName];
	if ([urlDict count] == 1) {
	    [result addObject:pkgName];
	}
	else {
	    NSEnumerator *urlEnumerator = [urlDict keyEnumerator];
	    NSString *pkgURL;
	    while ((pkgURL = [urlEnumerator nextObject])) {
		[result addObject:[NSString stringWithFormat:@"%@ | %@", pkgName, pkgURL]];
	    }
	}
    }
    [result sortUsingFunction:stringSort context:nil];

    return result;
}

- (NSString *)urlForName:(NSString *)displayName
{
    NSRange barRange = [displayName rangeOfString:@"|" options:NSBackwardsSearch];
    if (!NSEqualRanges( barRange, NSMakeRange( NSNotFound, 0))) {
	NSRange urlRange = NSMakeRange( barRange.location + 2, [displayName length] - barRange.location - 2);
	NSString *urlString = [displayName substringWithRange:urlRange];
	return urlString;
    }
    else {
	NSArray *urlStringArray = [[__iPackages objectForKey:displayName] allKeys];
	if ([urlStringArray count] != 1) {
	    NSLog( @"Program error: unexpected duplicate urls for package name");
	}
	return [urlStringArray objectAtIndex:0];
    }
}


- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
}

@end

static NSComparisonResult stringSort(NSString *string1, NSString *string2, void *context)
{
    return [string1 caseInsensitiveCompare:string2];
}

static NSComparisonResult dictSort(NSDictionary *dict1, NSDictionary *dict2, void *context)
{
    return [[dict1 objectForKey:@"name"] caseInsensitiveCompare:[dict2 objectForKey:@"name"]];
}
