//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2DirectoryWindowController.h"
#import "II2DocumentController.h"
#import "II2Directory.h"
#import "II2Preferences.h"
#import "II2AppDelegate.h"

static char rcsid[] = "$Id: II2DirectoryWindowController.m,v 1.4 2004/05/08 15:31:06 gctwnl Exp $";

static int columnSort(id s1, id s2, void *context)
{
    NSArray *columns = [(NSTableView *)context tableColumns];
    int columnIndex = [(NSTableView *)context selectedColumn];
    NSTableColumn *column = [columns objectAtIndex:columnIndex];
    NSString *identifier = [column identifier];
    int firstCompare = [(NSString *)[s1 objectForKey:identifier] caseInsensitiveCompare:(NSString *)[s2 objectForKey:identifier]];
    if (firstCompare == NSOrderedSame) {
	identifier = [[columns objectAtIndex:(1-columnIndex)] identifier];
	return [(NSString *)[s1 objectForKey:identifier] caseInsensitiveCompare:(NSString *)[s2 objectForKey:identifier]];
    }
    return firstCompare;
}


@implementation II2DirectoryWindowController

- (id)initWithWindowNibName:(NSString *)windowNibName
{
    self = [super initWithWindowNibName:windowNibName];
    if (self) {
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(preferencesChanged:) name: @"II2PreferencesChangedNotification" object: nil];
    }
    return self;
}

- (void)awakeFromNib
{
    if ([[self superclass] instancesRespondToSelector:@selector(awakeFromNib)]) {
        [super awakeFromNib];
    }
    [knownDirectoryTableView setDoubleAction:@selector( openIDirectoryInNewWindow:)];
    [knownPackageTableView setDoubleAction:@selector( openIPackage:)];
    
    [knownDirectoryTableView setAction:@selector( clickAction:)];
    [knownPackageTableView setAction:@selector( clickAction:)];
    
    [knownDirectoryTableView setTarget:self];
    [knownPackageTableView setTarget:self];

    [knownPackageTableView setDelegate:self];
    [knownDirectoryTableView setDelegate:self];

    [knownPackageTableView setDataSource:self];
    [knownDirectoryTableView setDataSource:self];
    [[self window] makeKeyAndOrderFront:self];
    [self reloadIDirectory:nil];
    // [knownPackageTableView addToolTipRect:[knownPackageTableView frame] owner:self userData:nil];

}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    if (aTableView == knownPackageTableView) {
	return [__packageTableSource count];
    }
    else {
	return [__directoryTableSource count];
    }
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
    if (aTableView == knownPackageTableView) {
	return [[__packageTableSource objectAtIndex:rowIndex] objectForKey:[aTableColumn identifier]];
    }
    else {
	return [[__directoryTableSource objectAtIndex:rowIndex] objectForKey:[aTableColumn identifier]];
    }
}

- (void)sortTableSourceWithColumn:(NSTableColumn *)column
{
    NSArray *newArray = [__packageTableSource sortedArrayUsingFunction:columnSort context:knownPackageTableView];
    [__packageTableSource release];
    __packageTableSource = [newArray retain];
    [knownPackageTableView reloadData];
    [knownPackageTableView deselectAll:nil];
}

- (IBAction)clickAction:(id)sender
{
    // NSLog( @"clickAction %d %d", [sender clickedColumn], [sender clickedRow]);
    if (sender == knownPackageTableView) {
	if ([sender clickedRow] == -1) {
	    // Clicked a column header
	    [self sortTableSourceWithColumn:[[sender tableColumns] objectAtIndex:[sender clickedColumn]]];
	}
    }
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
    NSTableView *tableViewChanged = [aNotification object];
    if (tableViewChanged == knownPackageTableView) {
	NSEnumerator *rowEnumerator = [knownPackageTableView selectedRowEnumerator];
	NSArray *allObjects = [rowEnumerator allObjects];
	if ([allObjects count] == 1) {
	    int rowIndex = [knownPackageTableView selectedRow];
	    if ((rowIndex >=0) && (rowIndex < [__packageTableSource count])) {
		NSString *pkgURLString = [[__packageTableSource objectAtIndex:rowIndex] objectForKey:@"url"];
		[currentIPackageTextField setStringValue:pkgURLString];
		[showInfoButton setEnabled:YES];
	    }
	}
	else if ([allObjects count] < 1) {
	    [currentIPackageTextField setStringValue:@"(No i-Package selected)"];
	    [showInfoButton setEnabled:NO];
	}
	else {
	    [currentIPackageTextField setStringValue:@"(Multiple i-Packages selected)"];
	    [showInfoButton setEnabled:NO];
	}
	[iPackageReadmeTextView resetParagraphStyleAndClear];
    }
}

- (IBAction)openIPackage:(id)sender
{
    II2DocumentController *docController = (II2DocumentController *)[NSDocumentController sharedDocumentController];
    NSEnumerator *rowEnumerator = [knownPackageTableView selectedRowEnumerator];
    NSNumber *rowNumber;
    NSMutableDictionary *checkDict = [[NSMutableDictionary alloc] init];

    while ((rowNumber = [rowEnumerator nextObject]) != nil) {
	int rowIndex = [rowNumber intValue];
	if ((rowIndex >= 0) && (rowIndex < [__packageTableSource count])) {
	    II2Directory *directory = (II2Directory *)[self document];
	    NSString *urlString = [[__packageTableSource objectAtIndex:rowIndex] objectForKey:@"url"];
	    NSURL *dictURL = [NSURL URLWithString:[directory startURLString]];
	    if ([checkDict objectForKey:[urlString lastPathComponent]]) {
		NSLog( @"Naughty, naughty! Trying to open two copies of the same package at once. What do you think I am: someone to protect you against exceptional silliness?");
	    }
	    else {
		NSURL *url = [[NSURL URLWithString:urlString relativeToURL:dictURL] standardizedURL];
		if (url) {
		    [checkDict setObject:@"check" forKey:[urlString lastPathComponent]];
		    [docController openDocumentWithContentsOfURL:url display:YES];
		}
	    }
	}
    }
    [checkDict release];
}

- (void)sheetDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{

}

- (void)redisplay
{
    II2Directory *directory = (II2Directory *)[self document];
    int i;

    [knownPackageTableView reloadData];
    [knownDirectoryTableView reloadData];
    for (i=0; i<[__directoryTableSource count]; i++) {
	if ([[directory startURLString] isEqualToString:[[__directoryTableSource objectAtIndex:i] objectForKey:@"url"]]) {
	    [knownDirectoryTableView selectRow:i byExtendingSelection:NO];
	    break;
	}
    }
    if (![__packageTableSource count]) {
	if ([[self window] isVisible]) {
	    NSBeginAlertSheet( @"Empty i-Directory", @"OK", nil, nil, [self window], self, @selector(sheetDidEnd:returnCode:contextInfo:), nil, nil, @"Downloading ended with an empty directory, the site may be offline or your URL in Preferences may be wrong.");
	}
    }
}

- (void)_doReloadKnownPackages
{
    II2Directory *directory = (II2Directory *)[self document];

    if (__packageTableSource) {
	[__packageTableSource release]; __packageTableSource = nil;
    }
    if (__directoryTableSource) {
	[__directoryTableSource release]; __directoryTableSource = nil;
    }
    [currentIDirectoryTextField setStringValue:@""];
    if (![ii2Prefs networkAccessDisabled]) {
	[iDirectoryWindowMessageTextField setStringValue:@"Downloading..."];
	[iDirectoryWindowMessageTextField displayIfNeeded];
	[directory setMaxDescendLevel:[ii2Prefs maxII2DDescendLevel]];
	[directory reload];

	__packageTableSource = [[directory namesAndURLs] retain];
	__directoryTableSource = [[directory directoryNamesAndURLs] retain];
	[iDirectoryWindowMessageTextField setStringValue:@""];
	[iDirectoryWindowMessageTextField displayIfNeeded];
    }
    else {
	[iDirectoryWindowMessageTextField setStringValue:@"Network access disabled"];
    }
    [knownPackageTableView selectColumn:[knownPackageTableView columnWithIdentifier:@"name"] byExtendingSelection:NO];
    [self sortTableSourceWithColumn:[[knownPackageTableView tableColumns] objectAtIndex:[knownPackageTableView columnWithIdentifier:@"name"]]];
    [currentIDirectoryTextField setStringValue:[directory startURLString]];
    [self redisplay];
}

- (IBAction)reloadIDirectory:(id)sender
{
    [self _doReloadKnownPackages];
}

- (void)preferencesChanged:(id)sender
{
    if ([[self window] isVisible]) {
	[self _doReloadKnownPackages];
    }
}

- (IBAction)openIDirectory:(id)sender
{
    II2Directory *directory = (II2Directory *)[self document];
    int rowIndex = [knownDirectoryTableView selectedRow];
    if ((rowIndex >=0) && (rowIndex < [__directoryTableSource count])) {
	[directory setStartURLString:[[__directoryTableSource objectAtIndex:rowIndex] objectForKey:@"url"]];
	[self _doReloadKnownPackages];
	[[self window] setTitle:[[directory startURLString] lastPathComponent]];
    }
}

- (IBAction)openIDirectoryInNewWindow:(id)sender
{
    II2Directory *directory = (II2Directory *)[self document];
    II2DocumentController *docController = (II2DocumentController *)[NSDocumentController sharedDocumentController];
    NSURL *dictURL = [NSURL URLWithString:[directory startURLString]];
    int rowIndex = [knownDirectoryTableView selectedRow];
    if ((rowIndex >=0) && (rowIndex < [__directoryTableSource count])) {
	// Just in case, we already standardize the relative URL's when we are creating the II2Directory
	NSURL *url = [[NSURL URLWithString:[[__directoryTableSource objectAtIndex:rowIndex] objectForKey:@"url"] relativeToURL:dictURL] standardizedURL];
	[docController openDocumentWithContentsOfURL:url display:YES];
    }
}

- (IBAction)showInfoForPackage:(id)sender
{
    int rowIndex = [knownPackageTableView selectedRow];
    if ((rowIndex >=0) && (rowIndex < [__packageTableSource count])) {
	NSString *pkgURLString = [[__packageTableSource objectAtIndex:rowIndex] objectForKey:@"url"];
	NSString *pkgBasename = [[pkgURLString lastPathComponent] stringByDeletingPathExtension];
	NSString *readmeURLString = [NSString stringWithFormat:@"%@/%@.rtf", pkgURLString, pkgBasename];
	NSURL *readmeURL = [NSURL URLWithString:readmeURLString];
	NSData *readmeData = [NSData dataWithContentsOfURL:readmeURL];
	NSAttributedString *readmeString = [[[NSAttributedString alloc] initWithRTF:readmeData documentAttributes:nil] autorelease];
	[iPackageReadmeTextView resetParagraphStyleAndClear];
	[iPackageReadmeTextView appendAttributedOutput:readmeString];
	[iPackageReadmeTextView scrollRangeToVisible:NSMakeRange(0, 0)];
    }
}

- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
}

//- (NSString *)view:(NSView *)view stringForToolTip:(NSToolTipTag)tag point:(NSPoint)point userData:(void *)userData
//{
//    NSEvent *currentEvent = [[self window] currentEvent];
//    NSView *contentView = [[currentEvent window] contentView];
//    NSPoint mouseLocation = [contentView convertPoint:[currentEvent locationInWindow] toView:knownPackageTableView];
//    int col = [knownPackageTableView columnAtPoint:mouseLocation];
//    int row = [knownPackageTableView rowAtPoint:mouseLocation];
//    NSLog( @"Location %d %d", col, row);
//    return @"This is a test tooltip";
//}


@end
