//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import <Cocoa/Cocoa.h>
#import "II2TaskSet.h"

@class II2DownloadTask;

@protocol II2DownloadTaskOwnerProtocol
- (void)downloadTaskDidFinish: (II2DownloadTask *)task withSuccess:(BOOL)success;
- (void)downloadTaskWillStart: (II2DownloadTask *)task;
@end

@interface II2DownloadTask : NSObject <NSURLHandleClient, II2TaskSetMemberProtocol, II2TaskPropertiesProtocol, II2TaskStatusProviderProtocol> {
    NSMutableDictionary 		*__properties;
    NSURLHandle				*__handle;
    NSURL				*__url;
    NSFileHandle			*__fileHandle;
    id <II2DownloadTaskOwnerProtocol>	__owner;
    int					__result;
    NSString				*__fullPath;
    NSString				*__displayName;
    NSString				*__shortDisplayName;
    unsigned				__expectedSize;
    unsigned				__downloadedSize;
    unsigned				__reportedSize;
    II2TaskSet				*__taskSet;
    unsigned int			__waitCounter;
    BOOL				__invalidated;
    BOOL				__ignoreFailure;
    NSMutableArray			*__dependentTasks;
    time_t				__downloadStart;
    time_t				__expectedDownloadFinish;
    time_t				__downloadFinish;
    NSTextView				*__messageView;
    id <II2TaskStatusReporterProtocol> __reporter;
    int					__reportIncrementTreshold;
}

- (II2DownloadTask *)initWithOwner:(id <II2DownloadTaskOwnerProtocol>)owner urlString:(NSString *)anURLString forFile:(NSString *)fullPath ofSize:(unsigned)expectedSize;
- (NSString *)description;
- (NSString *)fullPath;
- (void)launch;
- (void)run;
- (void)terminate;
- (int)result;
- (void)setMessageView:(NSTextView *)outView;
- (time_t)downloadStartTime;
- (time_t)downloadEndTime;
- (unsigned)sizeOfDownload;
- (unsigned)speedOfDownload;

- (void)setReportTarget:(id <II2TaskStatusReporterProtocol>)reporter;
- (NSString *)typeOfTask;
- (int)sizeOfTaskWhenCompleted;
- (int)currentSizeOfTask;

@end


/*
 * $Log: II2DownloadTask.h,v $
 * Revision 1.3  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.2  2004/03/19 23:35:11  gctwnl
 * Version 2.55.0. Has a renewed interface to support simple operations.
 * Supports progress bar control from the i-Package.
 *
 * Revision 1.1.1.1  2004/02/01 17:18:21  gctwnl
 * Initial import.
 *
 * Revision 3.3  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.2  2002/12/10 14:24:51  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:23  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:51  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.13  2002/10/25 20:36:11  gerben
 * Layout changes.
 *
 * Revision 1.12  2002/10/22 14:14:20  gerben
 * Differ between different user actions.
 *
 * Revision 1.11  2002/10/12 10:54:39  gerben
 * A lot of changes, mostly when producing output/feedback.
 *
 * Revision 1.10  2002/10/10 00:51:25  gerben
 * Many changes. Committing a stable state.
 *
 * Revision 1.9  2002/10/09 10:53:41  gerben
 * Moved textview output to NSTexTView category.
 *
 * Revision 1.8  2002/10/07 20:28:06  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.7  2002/10/02 22:38:31  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */