//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2DownloadTask.h"
#import "II2AppDelegate.h"
#import "II2Preferences.h"
#import "GWSizeFormatter.h"

static char rcsid[] = "$Id: II2DownloadTask.m,v 1.5 2004/04/25 13:13:21 gctwnl Exp $";

@implementation II2DownloadTask

- (void)dealloc
{
    if ([ii2Prefs mainLogLevel] > 8) fprintf( stderr, "Dealloc II2DownloadTask: %s %u", [[self description] cString], [self retainCount]);
    [__properties release]; __properties = nil;
    [__handle release] ; __handle = nil;
    [__fileHandle release] ; __fileHandle = nil;
    [__url release] ; __url = nil;
    [__fullPath release]; __fullPath = nil;
    [__displayName release]; __displayName = nil;
    [__shortDisplayName release]; __shortDisplayName = nil;
    [__dependentTasks release]; __dependentTasks = nil;
    [super dealloc];
}

- (BOOL)createFileForDownload
{
    if (![DEFFM createFileAtPath:__fullPath contents:nil attributes:nil]) {
	[__messageView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Unable to create %s for download. Permission problem?\n", [__fullPath fileSystemRepresentation]] attributes:[self signalAttributes]] autorelease]];
	NSLog( @"Cannot create file %s", [__fullPath fileSystemRepresentation]);
	return NO;
    }
    __fileHandle = [[NSFileHandle fileHandleForWritingAtPath:__fullPath] retain];
    if (__fileHandle == nil) {
	NSLog( @"Cannot create file handle for %s", [__fullPath fileSystemRepresentation]);
	return NO;
    }
    return YES;
}

- (II2DownloadTask *)initWithOwner:(id <II2DownloadTaskOwnerProtocol>)owner urlString:(NSString *)anURLString forFile:(NSString *)fullPath ofSize:(unsigned)expectedSize
{
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"init II2DownloadTask: %p", self);
    self = [super init];
    __url = [[NSURL URLWithString:anURLString] retain];
    if (__url == nil) {
	NSLog( @"Cannot create URL object for %@", anURLString);
	return nil;
    }
    __handle = [[__url URLHandleUsingCache:![ii2Prefs cacheDisabled]] retain];
    if (__handle == nil) {
	NSLog( @"Cannot create URL handle for %@", anURLString);
	[__url release]; __url = nil;
	[__fileHandle release]; __fileHandle = nil;
	return nil;
    }
    if ([[__url scheme] hasPrefix:@"http"]) {
	[__handle writeProperty:@"i-Installer/2.0" forKey:@"User-Agent"];
	if ([ii2Prefs cacheDisabled]) {
	    [__handle writeProperty:@"no-cache" forKey:@"Cache-Control"];
	}
    }
    __owner = owner;
    [self setProperties:[NSDictionary dictionaryWithObjectsAndKeys:[fullPath lastPathComponent], @"filename", nil]];
    __fullPath = [[NSString stringWithString:fullPath] retain];
    __expectedSize = expectedSize;
    //__displayName = [[[fullPath lastPathComponent] stringByAppendingString:@" download"] retain];
    __displayName = [[anURLString stringByAppendingString:[NSString stringWithFormat:@" download (%@)", [[GWSizeFormatter sharedFormatter] stringForObjectValue:[NSNumber numberWithUnsignedInt:__expectedSize]]]] retain];
    __shortDisplayName = [[[fullPath lastPathComponent] stringByAppendingString:[NSString stringWithFormat:@" download (%@)", [[GWSizeFormatter sharedFormatter] stringForObjectValue:[NSNumber numberWithUnsignedInt:__expectedSize]]]] retain];
    __result = -1;
    __dependentTasks = [[NSMutableArray alloc] init];
    return self;
}

- (time_t)downloadStartTime
{
    return __downloadStart;
}

- (time_t)downloadEndTime
{
    return __downloadFinish;
}

- (unsigned)sizeOfDownload
{
    return __downloadedSize;
}

- (unsigned)speedOfDownload
{
    unsigned downloadSpeed, downloadTime;
    if (!__downloadStart) {
	return 0;
    }
    downloadTime = (__downloadFinish ? (__downloadFinish - __downloadStart) : 0);
    downloadSpeed = (downloadTime ? (__downloadedSize / downloadTime) : 0x7FFFFFFF);
    // NSLog( @"size: %u time: %u speed: %u", __downloadedSize, downloadTime, downloadSpeed);
    return downloadSpeed;
}

- (void)setMessageView:(NSTextView *)messageView
{
    __messageView = messageView;
}

- (int)result
{
    return __result;
}

- (void)setTaskSet:(II2TaskSet *)set
{
    __taskSet = set;
}

- (void)setProperties: (NSDictionary *)myProperties
{
    if (__properties) {
	[__properties release]; __properties = nil;
    }
    if (myProperties) {
	__properties = [[NSMutableDictionary dictionaryWithDictionary:myProperties] retain];
    }
}

- (NSDictionary *)properties
{
    return __properties;
}

- (id)propertyForKey:(id)key
{
    return [__properties objectForKey:key];
}

- (void)setProperty:(id)property forKey:(id)key
{
    [__properties setObject:property forKey:(id)key];
}

- (NSString *)fullPath
{
    return __fullPath;
}

- (NSAttributedString *)attributedDisplayName
{
    NSString *text = [self description];

    return [[[NSAttributedString alloc] initWithString:text attributes:[self normalAttributes]] autorelease];
}

- (NSString *)description
{
    return __displayName;
}

- (id)progress
{
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"II2DownloadTask (%@) progress", [self description]);
    if (__waitCounter) {
	return [NSString stringWithFormat:@"Waiting... (%u)", __waitCounter];
    }
    switch ([__handle status]) {
	case NSURLHandleNotLoaded:
	    return @"Not Started";
	case NSURLHandleLoadSucceeded:
	    return @"Completed";
	case NSURLHandleLoadInProgress:
	    return [NSString stringWithFormat:@"%.1f%%", 100.0 * (double)__downloadedSize/(double)__expectedSize];
	case NSURLHandleLoadFailed:
	    return @"Failed";
    }
    // NSLog( @"II2DownloadTask progress: returning NSNumber -1.0");
    return [NSNumber numberWithFloat:-1.0];
}

- (void)addDependentTask:(id <II2TaskDependencyProtocol>)task
{
    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"%@ will wait for %@", [task description], [self description]);
    [__dependentTasks addObject:task];
    [task wait];
}


- (NSArray *)dependentTasks
{
    return __dependentTasks;
}

- (void)addDependentTasks:(NSArray *)tasks
{
    if (!tasks) {
	NSLog( @"Weird error, nil-tasks array in II2DownloadTask addDependentTasks");
    }
    if ([tasks count]) {
	[__dependentTasks addObjectsFromArray:tasks];
	[tasks makeObjectsPerformSelector:@selector(wait)];
    }
}

- (void)wait
{
    if ([ii2Prefs mainLogLevel] > 5) NSLog( @"wait II2DownloadTask: %@ (%u)", [self description], __waitCounter);
    __waitCounter++;
}

- (void)endWait
{
    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"endWait II2DownloadTask: %@ (%u)", [self description], __waitCounter);
    if (__waitCounter > 0) {
	__waitCounter--;
    }
    [self launch];
}

- (void)abortWait
{
    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"abortWait II2DownloadTask: %@ (%u)", [self description], __waitCounter);
    if (!__invalidated) {
	__invalidated = YES;
	[__dependentTasks makeObjectsPerformSelector:@selector(abortWait)];
    }
    if (__waitCounter == 0) {
	[__owner downloadTaskDidFinish:self withSuccess:NO];
    }
    if (__waitCounter > 0) {
	__waitCounter--;
    }
    if (__waitCounter == 0) {
	[__taskSet ii2TaskDidEnd:self withSuccess:NO];
	[self release];
    }
}

- (void)launch
{
    if (__waitCounter == 0) {
	if (!__invalidated && [self createFileForDownload]) {
	    [__owner downloadTaskWillStart:self];
	    [__taskSet refreshDisplay:self];
	    [__handle addClient:self]; // RETAINS SELF!!
	    [__handle loadInBackground]; // RETAINS self TOO!!
	}
	else {
	    [__taskSet ii2TaskDidEnd:self withSuccess:NO];
	    [self release];
	}
    }
}

- (void)run
{
    if (__waitCounter == 0) {
	if (!__invalidated && [self createFileForDownload]) {
	    [__handle addClient:self]; // RETAINS SELF!!
	    [__handle loadInForeground];
	}
	else {
	    [__taskSet ii2TaskDidEnd:self withSuccess:NO];
	    [self release];
	}
    }
}

- (void)abort
{
    [self terminate];
}

- (void)terminate
{
    int i;

    if ([ii2Prefs mainLogLevel] > 5) NSLog( @"Task terminate %@: waitCounter == %u", [self description], __waitCounter);
    if (__waitCounter == 0) {
	if ([__handle status] == NSURLHandleLoadInProgress) {
	    if ([ii2Prefs mainLogLevel] > 0) NSLog( @"Task %@: aborting", [self description]);
	    [__handle cancelLoadInBackground];
	    return;
	}
	[__handle removeClient:self]; // RELEASES SELF!!
	// Signal owner (if any) about finished task
	if ([ii2Prefs mainLogLevel] > 8) NSLog( @"Terminate II2DownloadTask: signaling owner");
	[__taskSet ii2TaskDidEnd:self withSuccess:(__ignoreFailure || ([__handle status] == NSURLHandleLoadSucceeded))];
	[__owner downloadTaskDidFinish:self withSuccess:(__ignoreFailure || ([__handle status] == NSURLHandleLoadSucceeded))];
    }
    else {
	__result = -5;
    }
    //Signal dependent tasks
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"Terminate II2DownloadTask: signaling dependants");
    //Signal dependent tasks
    for (i=[__dependentTasks count]-1; i>=0; i--) {
	if (__ignoreFailure || (__result == 0)) {
	    [[__dependentTasks objectAtIndex:i] endWait];
	}
	else {
	    [[__dependentTasks objectAtIndex:i] abortWait];
	}
    }
    [self release];
}

//<NSURLHandleClient> protocol and support:

- (void)URLHandleResourceDidBeginLoading:(NSURLHandle *)sender
{
    if ([ii2Prefs mainLogLevel] > 7) NSLog( @"URLHandleResourceDidBeginLoading %@", [self description]);
    __downloadStart = time( NULL);
    __expectedDownloadFinish = __downloadStart + 3600;
    if (__messageView) {
	[__messageView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@ has started (%@)\n", __displayName, [[GWSizeFormatter sharedFormatter] stringForObjectValue:[NSNumber numberWithUnsignedInt:__expectedSize]]] attributes:[self infoAttributes]] autorelease]];
    }
    [__taskSet refreshDisplay:self];
}

- (void)URLHandle:(NSURLHandle *)sender resourceDataDidBecomeAvailable:(NSData *)newBytes
{
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"resourceDataDidBecomeAvailable: %@ length = %u", [self description], [newBytes length]);
    [__fileHandle writeData:newBytes];
    __downloadedSize += [newBytes length];
    if (__reporter && ((__downloadedSize - __reportedSize) >= __reportIncrementTreshold)) {
	[__reporter updateReportWithDataFromTask:self download:YES];
    }
    if (__messageView && __downloadedSize) {
	time_t timeSpent, now;

	now = time( NULL);
	timeSpent = now - __downloadStart;
	if (timeSpent > 5) {
	    time_t newExpectedDownloadFinish, expectedTotalTime, oldDuration, newDuration;
	    double durationChange;
	    
	    expectedTotalTime = (time_t)((double)timeSpent * (double)__expectedSize / (double)__downloadedSize);
	    newExpectedDownloadFinish = __downloadStart + expectedTotalTime;
	    oldDuration = __expectedDownloadFinish - now;
	    newDuration = newExpectedDownloadFinish - now;
	    durationChange = (double)oldDuration / (double)newDuration;
	    if (ABS(newDuration - oldDuration) > 7 && ((durationChange > 1.2) || (durationChange < 0.8))) {
		NSDate *newDate = [NSDate dateWithTimeIntervalSince1970:newExpectedDownloadFinish];
		__expectedDownloadFinish = newExpectedDownloadFinish;
		[__messageView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@ is expected to finish at %@\n", __displayName, [newDate description]] attributes:[self infoAttributes]] autorelease]];
	    }
	}
    }
    [__taskSet refreshDisplay:self];
}

- (void)URLHandleResourceDidFinishLoading:(NSURLHandle *)sender
{
    __result = 0;
    __downloadFinish = time( NULL);
    if ([ii2Prefs mainLogLevel] > 7) NSLog( @"URLHandleResourceDidFinishLoading %@", [self description]);
    [__fileHandle synchronizeFile];
    [__fileHandle closeFile];
    if (__messageView) {
	[__messageView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@ has completed (%@ at %@ per second)\n", __displayName, [[GWSizeFormatter sharedFormatter] stringForObjectValue:[NSNumber numberWithUnsignedInt:__downloadedSize]], [[GWSizeFormatter sharedFormatter] stringForObjectValue:[NSNumber numberWithUnsignedInt:[self speedOfDownload]]]] attributes:[self infoAttributes]] autorelease]];
    }
    [self terminate];
}

- (void)URLHandleResourceDidCancelLoading:(NSURLHandle *)sender
{
    __result = -2;
    if ([ii2Prefs mainLogLevel] > 7) NSLog( @"URLHandleResourceDidCancelLoading %@", [self description]);
    [__fileHandle closeFile];
    [DEFFM removeFileAtPath: __fullPath handler:nil];
    if (__messageView) {
	[__messageView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@ has been cancelled after %@\n", __displayName, [[GWSizeFormatter sharedFormatter] stringForObjectValue:[NSNumber numberWithUnsignedInt:__downloadedSize]]] attributes:[self infoAttributes]] autorelease]];
    }
    [self terminate];
}

- (void)URLHandle:(NSURLHandle *)sender resourceDidFailLoadingWithReason:(NSString *)reason
{
    __result = -3;
    if ([ii2Prefs mainLogLevel] > 7) NSLog( @"URLHandle: %@ resourceDidFailLoadingWithReason: %@", [self description], reason);
    [__fileHandle closeFile];
    [DEFFM removeFileAtPath: __fullPath handler:nil];
    if (__messageView) {
	[__messageView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@ has failed: %@\n", __displayName, reason] attributes:[self signalAttributes]] autorelease]];
    }
    [self terminate];
}

- (void)setReportTarget:(id <II2TaskStatusReporterProtocol>)reporter
{
    __reporter = reporter;
}

- (void)setReportIncrementTreshold:(int)treshold
{
    __reportIncrementTreshold = (treshold >= (__expectedSize/2) ? (__expectedSize/2) : treshold);
}

- (NSString *)typeOfTask
{
    return __shortDisplayName;
}

- (int)sizeOfTaskWhenCompleted
{
    return __expectedSize;
}

- (int)currentSizeOfTask
{
    return __downloadedSize;
}

- (int)sizeOfTaskSinceLastReported
{
    int increment = __downloadedSize - __reportedSize;
    __reportedSize = __downloadedSize;
    return increment;
}

- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
}

@end

/*
 * $Log: II2DownloadTask.m,v $
 * Revision 1.5  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.4  2004/04/19 22:58:25  gctwnl
 * 2.61.0
 *
 * Revision 1.3  2004/03/19 23:35:11  gctwnl
 * Version 2.55.0. Has a renewed interface to support simple operations.
 * Supports progress bar control from the i-Package.
 *
 * Revision 1.2  2004/02/02 20:15:27  gctwnl
 * Fixed loss of keyed objects in package window (spinning progress). Some small fixes in logging and debug messages.
 *
 * Revision 1.1.1.1  2004/02/01 17:18:22  gctwnl
 * Initial import.
 *
 * Revision 3.5  2004/01/27 12:57:39  gerben
 * 2.44.0
 *
 * Revision 3.4  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.3  2004/01/05 15:31:34  gerben
 * *** empty log message ***
 *
 * Revision 3.2  2002/12/10 14:24:51  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:23  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:51  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.34  2002/10/28 21:49:46  gerben
 * A lot of changes wrt known packages.
 *
 * Revision 1.33  2002/10/27 15:39:44  gerben
 * Tried to remove BSD-package dependency
 *
 * Revision 1.32  2002/10/25 20:36:11  gerben
 * Layout changes.
 *
 * Revision 1.31  2002/10/24 12:49:06  gerben
 * Update refreshes file table. Download reports when remainder of (not total) download time changes significantly.
 *
 * Revision 1.30  2002/10/17 20:36:03  gerben
 * Reversed default for caching
 *
 * Revision 1.29  2002/10/17 18:49:00  gerben
 * Cache/NoCache handling on downloads. Some other changes.
 *
 * Revision 1.28  2002/10/16 22:37:08  gerben
 * A few small changes.
 *
 * Revision 1.27  2002/10/16 21:28:22  gerben
 * A complete new rewrite of taskss, processes and stuff.
 *
 * Revision 1.26  2002/10/16 08:37:19  gerben
 * Changed log levels. Small fixes.
 *
 * Revision 1.25  2002/10/15 14:31:44  gerben
 * Last fixes. No-network access. Later creation of downloaded file.
 *
 * Revision 1.24  2002/10/12 10:54:39  gerben
 * A lot of changes, mostly when producing output/feedback.
 *
 * Revision 1.23  2002/10/10 09:08:47  gerben
 * Fixed bug where internal panel dictionaries were released once too many.
 *
 * Revision 1.22  2002/10/10 00:51:25  gerben
 * Many changes. Committing a stable state.
 *
 * Revision 1.21  2002/10/09 14:16:12  gerben
 * Changed as many %s/lossyCString uses to %@/fileSystemRepresentation/UTF8String
 *
 * Revision 1.20  2002/10/09 10:53:41  gerben
 * Moved textview output to NSTexTView category.
 *
 * Revision 1.19  2002/10/08 21:34:29  gerben
 * Working GPG signature checks, shared GPG cache for II2.
 *
 * Revision 1.18  2002/10/07 20:28:06  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.17  2002/10/03 13:38:04  gerben
 * Removed addChildWindow as child passes close to parent. Window can only close when there are no running tasks.
 *
 * Revision 1.16  2002/10/02 22:38:31  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */