//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import <Cocoa/Cocoa.h>
#import "GWInteractionManagerProtocols.h"
#import "II2WindowController.h"

@interface II2InteractionManager : NSObject <GWInteractionManagerServerProtocol> {
    II2WindowController	*__windowController;
    NSMutableArray	*__panelRequests;
    NSMutableArray 	*__tableDataSource;

    // Outlets for panels
    IBOutlet id		alertSheet;
    IBOutlet id		alertSheetIcon;
    IBOutlet id		alertButtonOK;
    IBOutlet id		alertButtonOther;
    BOOL		alertButtonOtherRemovedFromView;
    IBOutlet id		alertButtonCancel;
    BOOL		alertButtonCancelRemovedFromView;
    IBOutlet id		alertTitleField;
    IBOutlet id		alertInfoField;
    NSRect		__orgAlertSheetFrame;
    NSRect		__orgAlertTitleFieldFrame;
    NSRect		__orgAlertInfoFieldFrame;
    
    IBOutlet id		choiceSheet;
    IBOutlet id		choiceButtonSelect;
    IBOutlet id		choiceButtonDefault;
    BOOL		choiceButtonDefaultRemovedFromView;
    IBOutlet id		choiceButtonCancel;
    BOOL		choiceButtonCancelRemovedFromView;
    IBOutlet id		choiceTitleField;
    IBOutlet id		choiceInfoField;
    IBOutlet id		choicePopupList;
    NSRect		__orgChoiceSheetFrame;
    NSRect		__orgChoiceTitleFieldFrame;
    NSRect		__orgChoiceInfoFieldFrame;
    
    IBOutlet id		progressSheet;
    IBOutlet id		progressButtonAbort;
    IBOutlet id		progressBar;
    IBOutlet id		progressSizeField;
    IBOutlet id		progressTitleField;
    IBOutlet id		progressInfoField;
    NSRect		__orgProgressSheetFrame;
    NSRect		__orgProgressTitleFieldFrame;
    NSRect		__orgProgressInfoFieldFrame;
    
    IBOutlet id		inputSheet;
    IBOutlet id		inputTextField;
    IBOutlet id		inputTitleField;
    IBOutlet id		inputInfoField;
    IBOutlet id		inputButtonOK;
    IBOutlet id		inputButtonCancel;
    BOOL		inputButtonCancelRemovedFromView;
    IBOutlet id		inputButtonDefault;
    BOOL		inputButtonDefaultRemovedFromView;
    NSRect		__orgInputSheetFrame;
    NSRect		__orgInputTitleFieldFrame;
    NSRect		__orgInputInfoFieldFrame;
    
    IBOutlet id		tableSheet;
    IBOutlet id		tableButtonSelect;
    IBOutlet id		tableButtonCancel;
    BOOL		tableButtonCancelRemovedFromView;
    IBOutlet id		tableButtonDefault;
    BOOL		tableButtonDefaultRemovedFromView;
    IBOutlet id		tableTitleField;
    IBOutlet id		tableInfoField;
    IBOutlet id		tableTableView;
    NSTableColumn	*__removedTableColumn;
    NSRect		__orgTableSheetFrame;
    NSRect		__orgTableTitleFieldFrame;
    NSRect		__orgTableInfoFieldFrame;
}

- (id)initWithWindowController:(II2WindowController *)windowController;

- (IBAction)endPanel:(id)sender;
- (void)runPendingSheets;

- (void)startInternalPanel:(NSMutableDictionary *)dictionary;
- (NSProgressIndicator *)progressBar;
- (NSTextField *)inputTextField;
- (NSTextField *)progressSizeField;

- (void)setControlAndSheetSizesForSheet:(NSPanel *)sheet;
- (void)resetControlAndSheetSizesForSheet:(NSPanel *)sheet;
@end

/*
 * $Log: II2InteractionManager.h,v $
 * Revision 1.2  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.1.1.1  2004/02/01 17:18:22  gctwnl
 * Initial import.
 *
 * Revision 3.4  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.3  2003/11/08 23:38:01  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.2  2002/12/10 14:24:51  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:23  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:51  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.10  2002/12/09 13:17:27  gerben
 * Flexible (size) panels for interaction.
 *
 * Revision 1.9  2002/10/13 13:57:47  gerben
 * Implemented a table with checkboxes. The changes for this were largely copied/imitated from the OpenUp project by Scott Anguish
 *
 * Revision 1.8  2002/10/10 00:51:24  gerben
 * Many changes. Committing a stable state.
 *
 * Revision 1.7  2002/10/07 20:28:06  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.6  2002/10/05 19:42:45  gerben
 * Choices with + as first character are selected by default.
 *
 * Revision 1.5  2002/10/02 22:38:31  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */