//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2InteractionManager.h"
#import "GWSizeFormatter.h"
#import "II2Package.h"
#import "II2Preferences.h"
#import "II2DocumentController.h"

#import <unistd.h>

static char rcsid[] = "$Id: II2InteractionManager.m,v 1.3 2004/04/25 13:13:21 gctwnl Exp $";

@implementation II2InteractionManager
- (void) dealloc
{
    [__panelRequests release]; __panelRequests = nil;
    [__tableDataSource release]; __tableDataSource = nil;
    [super dealloc];
}

- (id)initWithWindowController:(II2WindowController *)windowController
{
    self = [super init];
    if (self) {
	__windowController = windowController;
	__panelRequests = [[NSMutableArray alloc] init];
    }
    return self;
}

- (void)awakeFromNib
{
    [progressSizeField setFormatter:[GWSizeFormatter sharedFormatter]];
}

- (NSProgressIndicator *)progressBar
{
    return progressBar;
}

- (NSTextField *)inputTextField
{
    return inputTextField;
}

- (NSTextField *)progressSizeField
{
    return progressSizeField;
}

- (BOOL)lazyNibLoading
{
    if (alertSheet == nil)
    {
	NSTableColumn *checkboxColumn;
	// we need to load the nibs
	if ([NSBundle loadNibNamed:@"II2IMPanels" owner:self] == NO)
	{
	    NSRunAlertPanel(@"Error", @"Could not load II2IMPanels.nib", @"Accept defeat", nil, nil);
	    NSLog( @"Could not load II2IMPanels.nib");
	    [self release];
	    return NO;
	}
	checkboxColumn=[tableTableView tableColumnWithIdentifier:@"check"];
	if (checkboxColumn)
	{
	    NSButtonCell *protoCell;

	    protoCell=[NSButtonCell new];
	    [protoCell setButtonType:NSSwitchButton];
	    [protoCell setImagePosition:NSImageOnly];
	    [protoCell setTitle:@""];
	    [checkboxColumn setDataCell:protoCell];

	    [checkboxColumn setWidth:24];
	    [checkboxColumn setMinWidth:24];
	    [checkboxColumn setMaxWidth:24];
	    [checkboxColumn setResizable:NO];
	}
	[tableTableView setHeaderView:nil];
	[tableTableView setDataSource:self];
	__orgAlertSheetFrame = [alertSheet frame];
	__orgAlertSheetFrame.size.height -= 22; // title bar
	__orgAlertTitleFieldFrame = [alertTitleField frame];
	__orgAlertInfoFieldFrame = [alertInfoField frame];
	__orgChoiceSheetFrame = [choiceSheet frame];
	__orgChoiceSheetFrame.size.height -= 22; // title bar
	__orgChoiceTitleFieldFrame = [choiceTitleField frame];
	__orgChoiceInfoFieldFrame = [choiceInfoField frame];
	__orgTableSheetFrame = [tableSheet frame];
	__orgTableSheetFrame.size.height -= 22; // title bar
	__orgTableTitleFieldFrame = [tableTitleField frame];
	__orgTableInfoFieldFrame = [tableInfoField frame];
	__orgInputSheetFrame = [inputSheet frame];
	__orgInputSheetFrame.size.height -= 22; // title bar
	__orgInputTitleFieldFrame = [inputTitleField frame];
	__orgInputInfoFieldFrame = [inputInfoField frame];
	__orgProgressSheetFrame = [progressSheet frame];
	__orgProgressSheetFrame.size.height -= 22; // title bar
	__orgProgressTitleFieldFrame = [progressTitleField frame];
	__orgProgressInfoFieldFrame = [progressInfoField frame];
    }
    return YES;
}

- (void)clicked:sender
{
    int clickedRow;
    int clickedColumn;

    clickedRow=[tableTableView clickedRow];
    clickedColumn=[tableTableView clickedColumn];

    if (![[[[tableTableView tableColumns] objectAtIndex:clickedColumn] identifier] isEqual:@"check"])
    {
        NSMutableDictionary *theRecord;
        BOOL currentValue;

        theRecord = [__tableDataSource objectAtIndex:clickedRow];
        currentValue=[[theRecord objectForKey:@"check"] boolValue];
        [theRecord setObject:[NSNumber numberWithBool:!currentValue] forKey:@"check"];
        [tableTableView reloadData];
    }
}

- (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
    [[__tableDataSource objectAtIndex:rowIndex] setObject:anObject forKey:[aTableColumn identifier]];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
    id theRecord;

    if (aTableView == tableTableView)
    {
        NSParameterAssert( rowIndex >= 0 && rowIndex < [__tableDataSource count]);
        theRecord = [__tableDataSource objectAtIndex:rowIndex];
        return [theRecord objectForKey:[aTableColumn identifier]];
    }

    return nil;
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    // NSLog( @"tableView:numberOfRowsInTableView: %u", [__tableDataSource count]);
    if (aTableView == tableTableView) {
        return [__tableDataSource count];
    }
    return -1;
}

// return result ot originating internal or external caller
- (void)returnDictionary:(NSDictionary *)dict
{
    NSString *doResultServerName = [dict objectForKey:@"doresultserver"];
    id internalResultHandler = [dict objectForKey:@"resulthandler"];
    SEL internalResultHandlerSelector = NSSelectorFromString( [dict objectForKey:@"resulthandlerselector"]);

    [dict retain];
    if (doResultServerName) {
	// Handle results
	// Send to doresultserver
	NSConnection *returnConnection = [NSConnection connectionWithRegisteredName:doResultServerName host:nil];
	if (returnConnection != nil) {
	    NSDistantObject <GWInteractionManagerClientProtocol> *theProxy = [returnConnection rootProxy];
	    
	    // if we can't find it, bail:
	    if (theProxy != nil) {
		[theProxy setProtocolForProxy:@protocol(GWInteractionManagerClientProtocol)];
		[theProxy panelResult:dict];
	    }
	    else {
		NSLog(@"Couldn't get proxy from Interaction manager DO Client");
	    }
	    // Clean up
	    [returnConnection registerName:nil];
	}
	else {
	    NSLog(@"Couldn't connect to Interaction Manager DO Client");
	}
    }
    else if (internalResultHandler && internalResultHandlerSelector) {
	// Handle results, call callbackmethod
	[internalResultHandler performSelector:internalResultHandlerSelector withObject:dict];
    }
    [dict release];
}

- (IBAction)endPanel:(id)sender
{
    NSMutableDictionary *__currentSheetDictionary = [[__panelRequests objectAtIndex:0] retain];

    [NSApp endSheet: [sender window]];
    [[sender window] orderOut: self];

    [self resetControlAndSheetSizesForSheet:(NSPanel *)[sender window]];

    // Remove from queue (it is released now but it has been retained above, so it still lives)
    [__panelRequests removeObjectAtIndex:0];

    // NSLog( @"end panel: %u", [__currentSheetDictionary retainCount]);
    // NSLog( @"TAG: %d", [sender tag]);
    [__currentSheetDictionary setObject:[NSNumber numberWithInt:[sender tag]] forKey:@"resultButtonTag"];
    [__currentSheetDictionary setObject:[sender title] forKey:@"resultButtonTitle"];
    if ([[__currentSheetDictionary objectForKey:@"type"] isEqualToString:@"Choice"]) {
	[__currentSheetDictionary setObject:[[choicePopupList selectedItem] title] forKey:@"choiceString"];
	[__currentSheetDictionary setObject:[NSNumber numberWithInt:[choicePopupList indexOfSelectedItem]] forKey:@"choiceItemIndex"];
    }
    else if ([[__currentSheetDictionary objectForKey:@"type"] isEqualToString:@"Table"]) {
	NSMutableArray *selections = [[[NSMutableArray alloc] init] autorelease];
	NSMutableArray *numericalselections = [[[NSMutableArray alloc] init] autorelease];
	if ([tableTableView numberOfSelectedRows] > 0) {
	    NSEnumerator *enumerator = [tableTableView selectedRowEnumerator];
	    NSNumber *key;
	    while (key = [enumerator nextObject]) {
		[selections addObject:[[__tableDataSource objectAtIndex:[key unsignedIntValue]] objectForKey:@"info"]];
		[numericalselections addObject:[NSString stringWithFormat:@"%u", [key unsignedIntValue]]];
	    }
	}
	[__currentSheetDictionary setObject:selections forKey:@"selections"];
	[__currentSheetDictionary setObject:numericalselections forKey:@"numericalselections"];
    }
    else if ([[__currentSheetDictionary objectForKey:@"type"] isEqualToString:@"Select"]) {
	NSMutableArray *selections = [[[NSMutableArray alloc] init] autorelease];
	NSMutableArray *numericalselections = [[[NSMutableArray alloc] init] autorelease];
	int i;

	for (i=0; i<[__tableDataSource count]; i++) {
	    NSDictionary *theDict = [__tableDataSource objectAtIndex:i];
	    if ([[theDict objectForKey:@"check"] intValue] != 0) {
		[selections addObject:[@"+" stringByAppendingString:[theDict objectForKey:@"info"]]];
		[numericalselections addObject:[NSString stringWithFormat:@"%u", i]];
	    }
	    else {
		[selections addObject:[@"-" stringByAppendingString:[theDict objectForKey:@"info"]]];
	    }
	}

	[__currentSheetDictionary setObject:selections forKey:@"selections"];
	[__currentSheetDictionary setObject:numericalselections forKey:@"numericalselections"];
    }
    else if ([[__currentSheetDictionary objectForKey:@"type"] isEqualToString:@"Input"]) {
	[__currentSheetDictionary setObject:[inputTextField stringValue] forKey:@"answer"];
    }
    
    // Pass back result
    [self returnDictionary:__currentSheetDictionary];
    
    // Needs to be released
    [__tableDataSource release]; __tableDataSource = nil;

    // This was retained at the start of this method
    // NSLog( @"releasing panel: %u", [__currentSheetDictionary retainCount]);
    [__currentSheetDictionary release];

    // Check for pending panel requests:
    [self runPendingSheets];
}

- (void)startInternalPanel:(NSMutableDictionary *)dictionary
{
    // NSLog( @"start internal panel: %u", [dictionary retainCount]);
    if (dictionary) [__panelRequests addObject:dictionary]; // this retains it, making sure it survives any UI interaction
    [self runPendingSheets];
    return;
}

- (bycopy)selectorSets
{
    II2Package *package = (II2Package *)[__windowController document];
    
    return [package extendedSelectorSetArray];
}

- (oneway void)incrementProgress:(in NSNumber *)increment
{
    [__windowController incrementActionProgressBar:[increment intValue]];
}


- (GWInteractionManagerServerReturnType)startPanel:(NSDictionary *)dictionary
{
    // NSLog( @"start panel: %u", [dictionary retainCount]);
    if ([dictionary objectForKey:@"resulthandler"]) {
	NSLog( @"Access to internal objects not allowed from outside");
	return GWInteractionManagerServerError;
    }
    if (dictionary) {
        NSDictionary *envDict = [dictionary objectForKey:@"environment"];
        if (envDict) {
            NSEnumerator *dictEnumerator = [envDict keyEnumerator];
            NSString * envKey;
            
            while ((envKey = [dictEnumerator nextObject])) {
                NSString *envValue = [envDict objectForKey:envKey];
                // NSLog( @"Adding environment value %@=%@", envKey, envValue);
                if ([envKey hasPrefix:@"!"]) {
                    if ([envKey length] < 2) {
                        NSLog( @"Missing variable after ! character received from script. Ignoring.");
                        return GWInteractionManagerServerError;
                    }
                    [(II2Package *)[__windowController document] unsetEnvironmentValueforKey:[envKey substringFromIndex:1]];
                }
                else {
                    [[__windowController document] setEnvironmentValue:envValue forKey:envKey];
                }
            }
        }
	if (![[__windowController window] attachedSheet]) {
	    [__panelRequests addObject:dictionary];
	    [self runPendingSheets];
	}
	else {
	    if ([ii2Prefs mainLogLevel] > 7) NSLog( @"Window is busy with another sheet. Returning busy.");
	    return GWInteractionManagerServerBusy;
	}
    }
    return GWInteractionManagerServerOK;
}

- (void)setTableDataSource:(NSArray *)choicesArray
{
    NSEnumerator *overArray;
    NSString *eachItem;
    NSMutableArray *newArray;

    newArray=[NSMutableArray array];
    overArray=[choicesArray objectEnumerator];
    while (( eachItem = [overArray nextObject] ))
    {
	BOOL isActive = [eachItem hasPrefix:@"+"];
        NSMutableDictionary *theDict;
	NSString *infoString = eachItem;

	if ([eachItem hasPrefix:@"+"] || [eachItem hasPrefix:@"-"]) {
	    infoString = [eachItem substringFromIndex:1];
	}

        theDict=[NSMutableDictionary dictionary];
        [theDict setObject:[NSNumber numberWithInt:isActive] forKey:@"check"];
        [theDict setObject:infoString forKey:@"info"];
        [newArray addObject:theDict];
    }
    if (__tableDataSource) {
	[__tableDataSource release];
    }
    __tableDataSource = [[NSArray alloc] initWithArray:newArray];
    [tableTableView reloadData];
    // [tableTableView setNeedsDisplay:YES];
    return;
}

- (void)_adjustSizeOfControl:(NSControl *)control minSize:(unsigned int)minSize vertically:(BOOL)vertically
{
    NSRect oldFrame, newFrame;
    NSSize diff;

    oldFrame = [control frame];
    if (vertically) {
	NSCell *cell = [control cell];
	NSRect bounds;
	NSSize newCellSize;

	bounds = [control bounds];
	newCellSize = [cell cellSizeForBounds:bounds];
	// XXX This assumes no scaling (scale factor = 1):
	if (newCellSize.height < minSize) newCellSize.height = minSize;
	[control setFrameSize:newCellSize];
    }
    else {
	// XXX This can get way too wide if string is long enough.
	[control sizeToFit];
    }
    newFrame = [control frame];
    diff.width  = (newFrame.size.width  - oldFrame.size.width);
    diff.height = (newFrame.size.height - oldFrame.size.height);
    if (diff.width < 0) {
	NSSize newSize;

	newSize.width = oldFrame.size.width;
	newSize.height = newFrame.size.height;
	[control setFrameSize:newSize];
	diff.width = 0;
    }
    if (diff.width || diff.height) {
	NSWindow *window = [control window];
	NSRect windowFrame;

	windowFrame = [window frame];
	windowFrame.size.width += diff.width;
	windowFrame.size.height += diff.height;
	[window setFrame:windowFrame display:NO];
    }
}

- (void)setControlAndSheetSizesForSheet:(NSPanel *)sheet
{
    NSTextField *theTitleField = nil;
    NSTextField *theInfoField = nil;
    
    if (sheet == alertSheet) {
	theTitleField = alertTitleField;
	theInfoField = alertInfoField;
    }
    else if (sheet == choiceSheet) {
	theTitleField = choiceTitleField;
	theInfoField = choiceInfoField;
    }
    else if (sheet == inputSheet) {
	theTitleField = inputTitleField;
	theInfoField = inputInfoField;
    }
    else if (sheet == tableSheet) {
	theTitleField = tableTitleField;
	theInfoField = tableInfoField;
    }
    else if (sheet == progressSheet) {
	theTitleField = progressTitleField;
	theInfoField = progressInfoField;
    }
    if (theTitleField && theInfoField) {
	NSRect titleBounds, infoFrame;

	[self _adjustSizeOfControl:theTitleField minSize:0 vertically:NO];
	titleBounds = [theTitleField bounds];
	infoFrame = [theInfoField frame];
	infoFrame.size.width = titleBounds.size.width;
	[theInfoField setFrameSize:infoFrame.size];
	[self _adjustSizeOfControl:theInfoField minSize:40 vertically:YES];
    }
}

- (void)resetControlAndSheetSizesForSheet:(NSPanel *)sheet
{
    NSTextField *theTitleField = nil;
    NSTextField *theInfoField = nil;
    NSRect orgSheetFrame, orgTitleFrame, orgInfoFrame;

    if (sheet == alertSheet) {
	theTitleField = alertTitleField;
	theInfoField = alertInfoField;
	orgSheetFrame = __orgAlertSheetFrame;
	orgTitleFrame = __orgAlertTitleFieldFrame;
	orgInfoFrame = __orgAlertInfoFieldFrame;
    }
    else if (sheet == choiceSheet) {
	theTitleField = choiceTitleField;
	theInfoField = choiceInfoField;
	orgSheetFrame = __orgChoiceSheetFrame;
	orgTitleFrame = __orgChoiceTitleFieldFrame;
	orgInfoFrame = __orgChoiceInfoFieldFrame;
    }
    else if (sheet == inputSheet) {
	theTitleField = inputTitleField;
	theInfoField = inputInfoField;
	orgSheetFrame = __orgInputSheetFrame;
	orgTitleFrame = __orgInputTitleFieldFrame;
	orgInfoFrame = __orgInputInfoFieldFrame;
    }
    else if (sheet == tableSheet) {
	theTitleField = tableTitleField;
	theInfoField = tableInfoField;
	orgSheetFrame = __orgTableSheetFrame;
	orgTitleFrame = __orgTableTitleFieldFrame;
	orgInfoFrame = __orgTableInfoFieldFrame;
    }
    else if (sheet == progressSheet) {
	theTitleField = progressTitleField;
	theInfoField = progressInfoField;
	orgSheetFrame = __orgProgressSheetFrame;
	orgTitleFrame = __orgProgressTitleFieldFrame;
	orgInfoFrame = __orgProgressInfoFieldFrame;
    }
    if (theTitleField && theInfoField) {
	[sheet setFrame:orgSheetFrame display:NO];
	[theTitleField setFrame:orgTitleFrame];
	[theInfoField setFrame:orgInfoFrame];
    }
}

- (void)beginSheet:(NSWindow *)sheet modalForWindow:(NSWindow *)docWindow modalDelegate:(id)modalDelegate didEndSelector:(SEL)didEndSelector contextInfo:(void *)contextInfo
{
    if (![docWindow attachedSheet]) {
	[NSApp beginSheet:sheet modalForWindow:docWindow modalDelegate:modalDelegate didEndSelector:didEndSelector contextInfo:contextInfo];
	[NSApp requestUserAttention:NSCriticalRequest];
    }
    else {
	NSLog( @"Program trouble. There was a sheet present on the window while we were trying to create ours. As a result, the process has failed. This happens for instance when a close window is executed while an interactive process is in execution. In its current version, i-Installer cannot handle this complexity.");
    }
}

- (void)doPanel
{
    NSMutableDictionary *__currentSheetDictionary = [__panelRequests objectAtIndex:0];
    // DO workhorse, depending on information in dictionary, adapts and activates panel
    // TODO: This beginSheet: fails when there is already another sheet. Either all sheets should go through this setup (probably not doable) or this setup should wait for a sheetless window before trying to create one. Or use the windowDidEndSheet notification!
    // NSLog( @"Interactionmanager doPanel");
    if (![self lazyNibLoading]) {
	NSLog( @"Interactionmanager lazyNibLoading failed!");
	return;
    }
    if (__currentSheetDictionary) {
	NSString *type = [__currentSheetDictionary objectForKey:@"type"];
	if ([ii2Prefs mainLogLevel] > 9) NSLog( @"IM sheetdict: %@", __currentSheetDictionary);
	if (type) {
	    NSString *title = [__currentSheetDictionary objectForKey:@"title"];
	    NSString *info = [__currentSheetDictionary objectForKey:@"info"];

	    if ([type isEqualToString:@"SetSelectorSets"]) {
		II2Package *package = (II2Package *)[__windowController document];
		NSArray *selectorSetArray = [__currentSheetDictionary objectForKey:@"selectorsets"];

		[package setSelectorSets:selectorSetArray];
		[__panelRequests removeObjectAtIndex:0];
		[self runPendingSheets];
	    }
	    else if ([type isEqualToString:@"SetConfiguredSelectorSets"]) {
		II2Package *package = (II2Package *)[__windowController document];
		NSArray *selectorSetArray = [__currentSheetDictionary objectForKey:@"selectorsets"];
		int i;
		
		for (i=0; i<[selectorSetArray count]; i++) {
		    [package setSelectorSet:[selectorSetArray objectAtIndex:i] configured:YES];
		}
		
		[__panelRequests removeObjectAtIndex:0];
		[self runPendingSheets];
	    }
	    else if ([type isEqualToString:@"UnsetConfiguredSelectorSets"]) {
		II2Package *package = (II2Package *)[__windowController document];
		NSArray *selectorSetArray = [__currentSheetDictionary objectForKey:@"selectorsets"];
		int i;
		
		for (i=0; i<[selectorSetArray count]; i++) {
		    [package setSelectorSet:[selectorSetArray objectAtIndex:i] configured:NO];
		}
		
		[__panelRequests removeObjectAtIndex:0];
		[self runPendingSheets];
	    }
	    else if ([type isEqualToString:@"SetInstalledSelectorSets"]) {
		II2Package *package = (II2Package *)[__windowController document];
		NSArray *selectorSetArray = [__currentSheetDictionary objectForKey:@"selectorsets"];
		int i;
		
		for (i=0; i<[selectorSetArray count]; i++) {
		    [package setSelectorSet:[selectorSetArray objectAtIndex:i] installed:YES];
		    [package setSelectorSet:[selectorSetArray objectAtIndex:i] configured:NO];
		}
		
		[__panelRequests removeObjectAtIndex:0];
		[self runPendingSheets];
	    }
	    else if ([type isEqualToString:@"UnsetInstalledSelectorSets"]) {
		II2Package *package = (II2Package *)[__windowController document];
		NSArray *selectorSetArray = [__currentSheetDictionary objectForKey:@"selectorsets"];
		int i;
		
		for (i=0; i<[selectorSetArray count]; i++) {
		    [package setSelectorSet:[selectorSetArray objectAtIndex:i] installed:NO];
		    [package setSelectorSet:[selectorSetArray objectAtIndex:i] configured:NO];
		}
		
		[__panelRequests removeObjectAtIndex:0];
		[self runPendingSheets];
	    }
	    else if ([type isEqualToString:@"Open"]) {
		NSString *urlString = [__currentSheetDictionary objectForKey:@"openurl"];
		[__panelRequests removeObjectAtIndex:0];
		if ([urlString hasSuffix:@".ii2"]) {
		    II2DocumentController *dc = (II2DocumentController *)[II2DocumentController sharedDocumentController];
		    NSURL *url = [NSURL URLWithString:urlString];

		    if (url) {
			if ((floor( NSAppKitVersionNumber) <= NSAppKitVersionNumber10_1) && ([[url scheme] isEqualToString:@"ftp"])) {
				NSRunAlertPanel(@"Error", @"FTP not supported by Apple Frameworks in Mac OS X Versions below 10.2. The requested package will not be opened on this system.", @"Failure", nil, nil, nil);
			}
			else {
			    [dc openDocumentWithContentsOfURL:url display:YES];
			}
		    }
		    else {
			NSRunAlertPanel(@"Error", @"Malformed URL. Package asks for other package but URL is broken.", @"Failure", nil, nil, nil);
		    }
		}
		[self runPendingSheets];
	    }
	    else if ([type isEqualToString:@"Alert"]) {
		NSString *okbutton = [__currentSheetDictionary objectForKey:@"okbutton"];
		NSString *cancelbutton = [__currentSheetDictionary objectForKey:@"cancelbutton"];
		NSString *otherbutton = [__currentSheetDictionary objectForKey:@"otherbutton"];

		if (okbutton) [alertButtonOK setTitle:okbutton];
		if (cancelbutton) {
		    [alertButtonCancel setTitle:cancelbutton];
		    if (alertButtonCancelRemovedFromView) {
			[[alertSheet contentView] addSubview:alertButtonCancel];
			[alertButtonCancel release]; // retained by addSubView
			alertButtonCancelRemovedFromView = NO;
		    }
		}
		else {
		    if (!alertButtonCancelRemovedFromView) {
			[[alertButtonCancel retain] removeFromSuperview];
			alertButtonCancelRemovedFromView = YES;
		    }
		}
		if (otherbutton) {
		    [alertButtonOther setTitle:otherbutton];
		    if (alertButtonOtherRemovedFromView) {
			[[alertSheet contentView] addSubview:alertButtonOther];
			[alertButtonOther release]; // retained by addSubView
			alertButtonOtherRemovedFromView = NO;
		    }
		}
		else {
		    if (!alertButtonOtherRemovedFromView) {
			[[alertButtonOther retain] removeFromSuperview];
			alertButtonOtherRemovedFromView = YES;
		    }
		}
		
		if (title) [alertTitleField setStringValue:title];
		if (info) [alertInfoField setStringValue:info];
		[self setControlAndSheetSizesForSheet:alertSheet];

		// if (info) [alertInfoField setString:info];
		// [alertInfoField sizeToFit];
		//NSLog( @"alert info field is vert resizable: %d", [alertInfoField isVerticallyResizable]);
		//NSLog( @"alert info field container tracks textview: %d", [[alertInfoField textContainer] heightTracksTextView]);

		[self beginSheet:alertSheet modalForWindow:[__windowController window] modalDelegate:self didEndSelector:nil contextInfo:__currentSheetDictionary];
	    }
	    else if ([type isEqualToString:@"Choice"]) {
		NSString *selectbutton = [__currentSheetDictionary objectForKey:@"selectbutton"];
		NSString *cancelbutton = [__currentSheetDictionary objectForKey:@"cancelbutton"];
		NSString *otherbutton = [__currentSheetDictionary objectForKey:@"otherbutton"];
		NSArray *choices = [__currentSheetDictionary objectForKey:@"choices"];

		if (!choices) {
		    NSLog( @"Missing choices array in IM dictionary");
		    return;
		}
		if (selectbutton) [choiceButtonSelect setTitle:selectbutton];
		if (cancelbutton) {
		    [choiceButtonCancel setTitle:cancelbutton];
		    if (choiceButtonCancelRemovedFromView) {
			[[choiceSheet contentView] addSubview:choiceButtonCancel];
			[choiceButtonCancel release]; // retained by addSubView
			choiceButtonCancelRemovedFromView = NO;
		    }
		}
		else {
		    if (!choiceButtonCancelRemovedFromView) {
			[[choiceButtonCancel retain] removeFromSuperview];
			choiceButtonCancelRemovedFromView = YES;
		    }
		}
		if (otherbutton) {
		    [choiceButtonDefault setTitle:otherbutton];
		    if (choiceButtonDefaultRemovedFromView) {
			[[choiceSheet contentView] addSubview:choiceButtonDefault];
			[choiceButtonDefault release]; // retained by addSubView
			choiceButtonDefaultRemovedFromView = NO;
		    }
		}
		else {
		    if (!choiceButtonDefaultRemovedFromView) {
			[[choiceButtonDefault retain] removeFromSuperview];
			choiceButtonDefaultRemovedFromView = YES;
		    }
		}

		if (title) [choiceTitleField setStringValue:title];
		if (info) [choiceInfoField setStringValue:info];
		[self setControlAndSheetSizesForSheet:choiceSheet];
		[choicePopupList removeAllItems];
		[choicePopupList addItemsWithTitles:choices];

		[self beginSheet:choiceSheet modalForWindow:[__windowController window] modalDelegate:self didEndSelector:nil contextInfo:__currentSheetDictionary];
	    }
	    else if ([type isEqualToString:@"Input"]) {
		NSString *okbutton = [__currentSheetDictionary objectForKey:@"okbutton"];
		NSString *cancelbutton = [__currentSheetDictionary objectForKey:@"cancelbutton"];
		NSString *otherbutton = [__currentSheetDictionary objectForKey:@"otherbutton"];
		NSString *initialvalue = [__currentSheetDictionary objectForKey:@"initialvalue"];

		if (okbutton) [inputButtonOK setTitle:okbutton];
		if (cancelbutton) {
		    [inputButtonCancel setTitle:cancelbutton];
		    if (inputButtonCancelRemovedFromView) {
			[[inputSheet contentView] addSubview:inputButtonCancel];
			[inputButtonCancel release]; // retained by addSubView
			inputButtonCancelRemovedFromView = NO;
		    }
		}
		else {
		    if (!inputButtonCancelRemovedFromView) {
			[[inputButtonCancel retain] removeFromSuperview];
			inputButtonCancelRemovedFromView = YES;
		    }
		}
		if (otherbutton) {
		    [inputButtonDefault setTitle:otherbutton];
		    if (inputButtonDefaultRemovedFromView) {
			[[inputSheet contentView] addSubview:inputButtonDefault];
			[inputButtonDefault release]; // retained by addSubView
			inputButtonDefaultRemovedFromView = NO;
		    }
		}
		else {
		    if (!inputButtonDefaultRemovedFromView) {
			[[inputButtonDefault retain] removeFromSuperview];
			inputButtonDefaultRemovedFromView = YES;
		    }
		}
		if (title) [inputTitleField setStringValue:title];
		if (info) [inputInfoField setStringValue:info];
		[self setControlAndSheetSizesForSheet:inputSheet];
		if (initialvalue) [inputTextField setStringValue:initialvalue];

		[self beginSheet:inputSheet modalForWindow:[__windowController window] modalDelegate:self didEndSelector:nil contextInfo:__currentSheetDictionary];
	    }
	    else if ([type isEqualToString:@"Table"] || [type isEqualToString:@"Select"]) {
		NSString *selectbutton = [__currentSheetDictionary objectForKey:@"selectbutton"];
		NSString *cancelbutton = [__currentSheetDictionary objectForKey:@"cancelbutton"];
		NSString *otherbutton = [__currentSheetDictionary objectForKey:@"otherbutton"];
		NSArray *choicesArray = [__currentSheetDictionary objectForKey:@"choices"];

		if (!choicesArray) {
		    NSLog( @"Missing choices array in IM dictionary");
		    return;
		}
		if ([type isEqualToString:@"Table"]) {
		    if (!__removedTableColumn) {
			__removedTableColumn = [[tableTableView tableColumnWithIdentifier:@"check"] retain];
			[tableTableView removeTableColumn:__removedTableColumn];
			[tableTableView setAllowsMultipleSelection:YES];
		    }
		    [tableTableView setAllowsMultipleSelection:YES];
		    [tableTableView setAction:nil];
		    [tableTableView setTarget:nil];
		}
		else {
		    if (__removedTableColumn) {
			[tableTableView addTableColumn:__removedTableColumn];
			[__removedTableColumn release]; __removedTableColumn = nil;
			[tableTableView moveColumn:0 toColumn:1];
		    }
		    [tableTableView setAllowsMultipleSelection:NO];
		    [tableTableView setAction:@selector(clicked:)];
		    [tableTableView setTarget:self];
		}
		[self setTableDataSource:choicesArray];
		if ([choicesArray count]) [tableTableView scrollRowToVisible:0];

		if (selectbutton) [tableButtonSelect setTitle:selectbutton];
		if (cancelbutton) {
		    [tableButtonCancel setTitle:cancelbutton];
		    if (tableButtonCancelRemovedFromView) {
			[[tableSheet contentView] addSubview:tableButtonCancel];
			[tableButtonCancel release]; // retained by addSubView
			tableButtonCancelRemovedFromView = NO;
		    }
		}
		else {
		    if (!tableButtonCancelRemovedFromView) {
			[[tableButtonCancel retain] removeFromSuperview];
			tableButtonCancelRemovedFromView = YES;
		    }
		}
		if (otherbutton) {
		    [tableButtonDefault setTitle:otherbutton];
		    if (tableButtonDefaultRemovedFromView) {
			[[tableSheet contentView] addSubview:tableButtonDefault];
			[tableButtonDefault release]; // retained by addSubView
			tableButtonDefaultRemovedFromView = NO;
		    }
		}
		else {
		    if (!tableButtonDefaultRemovedFromView) {
			[[tableButtonDefault retain] removeFromSuperview];
			tableButtonDefaultRemovedFromView = YES;
		    }
		}
		if (title) [tableTitleField setStringValue:title];
		if (info) [tableInfoField setStringValue:info];
		[self setControlAndSheetSizesForSheet:tableSheet];
		// NSLog( @"table info field is vert resizable: %d", [tableInfoField isVerticallyResizable]);

		[self beginSheet:tableSheet modalForWindow:[__windowController window] modalDelegate:self didEndSelector:nil contextInfo:__currentSheetDictionary];
	    }
	    else if ([type isEqualToString:@"Progress"]) {
		NSString *abortbutton = [__currentSheetDictionary objectForKey:@"abortbutton"];
		NSString *size = [__currentSheetDictionary objectForKey:@"size"];

		if (abortbutton) [progressButtonAbort setTitle:abortbutton];
		if (title) [progressTitleField setStringValue:title];
		if (info) [progressInfoField setStringValue:info];
		[self setControlAndSheetSizesForSheet:progressSheet];
		if (size) [progressSizeField setIntValue:[size intValue]];

		[self beginSheet:progressSheet modalForWindow:[__windowController window] modalDelegate:self didEndSelector:nil contextInfo:__currentSheetDictionary];
	    }
	    else {
		// We do nothing, remove this "panel" form the list
		if ([ii2Prefs mainLogLevel] >5) NSLog( @"User interaction without Panel removed from list.");
		[__panelRequests removeObjectAtIndex:0];
	    }
	}
    }
    else {
	NSLog( @"Invalid dictionary for interaction manager panel");
	return;
    }
    return;
}

- (void)runPendingSheets
{
    if ([__panelRequests count]) {
	[self doPanel];
    }
}

- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
}

@end

/*
 * $Log: II2InteractionManager.m,v $
 * Revision 1.3  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.2  2004/03/19 23:35:11  gctwnl
 * Version 2.55.0. Has a renewed interface to support simple operations.
 * Supports progress bar control from the i-Package.
 *
 * Revision 1.1.1.1  2004/02/01 17:18:22  gctwnl
 * Initial import.
 *
 * Revision 3.9  2004/01/30 15:24:23  gerben
 * 2.47.0
 *
 * Revision 3.8  2004/01/23 00:31:16  gerben
 * 2.42.0 release
 *
 * Revision 3.7  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.6  2004/01/05 15:31:34  gerben
 * *** empty log message ***
 *
 * Revision 3.5  2003/11/20 11:56:06  gerben
 * 2.32.0 work in progress
 *
 * Revision 3.4  2003/11/09 21:43:06  gerben
 * 2.32.0
 *
 * Revision 3.3  2003/11/08 23:38:02  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.2  2002/12/10 14:24:52  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:23  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:51  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.28  2002/12/10 08:55:25  gerben
 * II2Package code cleanup, remove of cached view pointers. Added relocatable handling in package dependencies.
 *
 * Revision 1.27  2002/12/09 15:31:43  gerben
 * Small fixes/try outs
 *
 * Revision 1.26  2002/12/09 15:09:00  gerben
 * Report printing
 *
 * Revision 1.25  2002/12/09 13:17:27  gerben
 * Flexible (size) panels for interaction.
 *
 * Revision 1.24  2002/11/22 23:54:47  gerben
 * RC4
 *
 * Revision 1.23  2002/10/21 20:16:53  gerben
 * Closing in on RC1 target
 *
 * Revision 1.22  2002/10/16 08:37:19  gerben
 * Changed log levels. Small fixes.
 *
 * Revision 1.21  2002/10/15 21:14:21  gerben
 * A less klunky known packages menu
 *
 * Revision 1.20  2002/10/14 12:34:19  gerben
 * Removed dead code
 *
 * Revision 1.19  2002/10/13 17:20:28  gerben
 * Go to start of table view
 *
 * Revision 1.18  2002/10/13 13:57:47  gerben
 * Implemented a table with checkboxes. The changes for this were largely copied/imitated from the OpenUp project by Scott Anguish
 *
 * Revision 1.17  2002/10/13 10:57:22  gerben
 * Saving stable state before some additions and rewrites
 *
 * Revision 1.16  2002/10/12 10:54:38  gerben
 * A lot of changes, mostly when producing output/feedback.
 *
 * Revision 1.15  2002/10/10 09:08:47  gerben
 * Fixed bug where internal panel dictionaries were released once too many.
 *
 * Revision 1.14  2002/10/10 00:51:24  gerben
 * Many changes. Committing a stable state.
 *
 * Revision 1.13  2002/10/09 08:38:42  gerben
 * Fixed two 10.1.5 incompatibilities (untested)
 *
 * Revision 1.12  2002/10/07 20:28:06  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.11  2002/10/07 11:52:28  gerben
 * Several fixes related to possible invalid content.
 *
 * Revision 1.10  2002/10/05 20:40:57  gerben
 * Without reloadData and deselectAll a new dataSource works partly with old information.
 *
 * Revision 1.9  2002/10/05 19:42:23  gerben
 * Choices with + as first character are selected by default.
 *
 * Revision 1.8  2002/10/03 13:38:04  gerben
 * Removed addChildWindow as child passes close to parent. Window can only close when there are no running tasks.
 *
 * Revision 1.7  2002/10/02 22:38:31  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */