//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include <sys/types.h>
#include <unistd.h>

#import "II2AppDelegate.h"
#import "II2ToolFinder.h"
#import "II2Preferences.h"
#import "II2Package.h"
#import "II2WindowController.h"
#import "GWAuthenticator.h"
#import "GWSizeFormatter.h"

NSString *II2DocumentType = @"i-Package-v2";

static char rcsid[] = "$Id: II2Package.m,v 1.3 2004/04/25 13:13:21 gctwnl Exp $";

@implementation II2Package

- (void) dealloc
{
    // fprintf( stderr, "II2Package dealloc");
    [__fileTableSource release];	__fileTableSource = nil;
    [__completeFileArray release];	__completeFileArray = nil;
    [__md5Checks release];		__md5Checks = nil;
    [__fileDataTypes release];		__fileDataTypes = nil;
    [__sigChecks release];		__sigChecks = nil;
    [__pkgPlist release];		__pkgPlist = nil;
    [__remotePkgPlist release];		__remotePkgPlist = nil;
    [__installedPkgPlist release];      __installedPkgPlist = nil;
    [__customPkgPlist release];		__customPkgPlist = nil;
    [__dirName release];		__dirName = nil;
    [__dirNameWithSeparator release];	__dirNameWithSeparator = nil;
    [__urlNameWithSeparator release];	__urlNameWithSeparator = nil;
    [__baseName release];		__baseName = nil;
    [__auth release];			__auth = nil;
    [__ii2RunningTaskSet release];	__ii2RunningTaskSet = nil;
    [__cachedWindowController release];	__cachedWindowController = nil;
    [super dealloc];
}

- (id)initWithContentsOfFile:(NSString *)fileName ofType:(NSString *)docType
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"initWithContentsOfFile:%s ofType:%s", [fileName fileSystemRepresentation], [docType fileSystemRepresentation]);
    self = [super initWithContentsOfFile:fileName ofType:docType];
    if (self) {
	if ([self needsAuthentication]) {
	    __auth = [[GWAuthenticator alloc] initWithLaunchPath:[GLUPTaskRunner fixedLaunchPath]];
	}
	__ii2RunningTaskSet = [[II2TaskSet alloc] init];
	__md5Checks = [[NSMutableDictionary alloc] init];
	__sigChecks = [[NSMutableDictionary alloc] init];
	__fileDataTypes = [[NSMutableDictionary alloc] init];
	[self resetEnvironment];
    }
    return self;
}

- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: windowControllerDidLoadNib");
    [super windowControllerDidLoadNib:aController];
    // Add any code here that need to be executed once the windowController has loaded the document's window.
    [__ii2RunningTaskSet setObserver:__cachedWindowController];
    [__ii2RunningTaskSet setHandler:self];
    //[__cachedWindowController showWindow:nil]; 
    [self showInfo];
}

- (II2WindowController *)windowController
{
    return __cachedWindowController;
}


- (void)makeWindowControllers {
    // [super makeWindowControllers];
    __cachedWindowController = [[II2WindowController allocWithZone:[self zone]] initWithWindowNibName:@"II2Package"];
    [self addWindowController:__cachedWindowController];
}

- (NSFileWrapper *)fileWrapperRepresentationOfType:(NSString *)type {
    return nil;
}

- (BOOL)checkStageFileDependencies
{
    NSMutableArray *dependencies = [[[NSMutableArray alloc] init] autorelease];
    NSArray *prepareDependencies = [[__pkgPlist objectForKey:@"dependencies"] objectForKey:@"prepare"];
    NSArray *configureDependencies = [[__pkgPlist objectForKey:@"dependencies"] objectForKey:@"configure"];
    NSArray *removeDependencies = [[__pkgPlist objectForKey:@"dependencies"] objectForKey:@"remove"];
    NSArray *selectorDependencies = [[__pkgPlist objectForKey:@"dependencies"] objectForKey:@"selector"];
    NSArray *selectorsets = [__pkgPlist objectForKey:@"selectorsets"]; // New Style

    int i;
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: checkStageFileDependencies");

    // Old Style
    if (prepareDependencies && [prepareDependencies count]) {
	[dependencies addObjectsFromArray:prepareDependencies];
    }
    if (configureDependencies && [configureDependencies count]) {
	[dependencies addObjectsFromArray:configureDependencies];
    }
    if (removeDependencies && [removeDependencies count]) {
	[dependencies addObjectsFromArray:removeDependencies];
    }
    if (selectorDependencies && [selectorDependencies count]) {
	[dependencies addObjectsFromArray:selectorDependencies];
    }
    
    if (selectorsets) {
	// New Style
	int j;
	
	for (j=0; j<[selectorsets count]; j++) {
	    NSArray *prepareDependencies = [[[selectorsets objectAtIndex:j] objectForKey:@"dependencies"] objectForKey:@"prepare"];
	    NSArray *configureDependencies = [[[selectorsets objectAtIndex:j] objectForKey:@"dependencies"] objectForKey:@"configure"];
	    NSArray *removeDependencies = [[[selectorsets objectAtIndex:j] objectForKey:@"dependencies"] objectForKey:@"remove"];
	    NSArray *selectorDependencies = [[[selectorsets objectAtIndex:j] objectForKey:@"dependencies"] objectForKey:@"selector"];
	    
	    if (prepareDependencies && [prepareDependencies count]) {
		[dependencies addObjectsFromArray:prepareDependencies];
	    }
	    if (configureDependencies && [configureDependencies count]) {
		[dependencies addObjectsFromArray:configureDependencies];
	    }
	    if (removeDependencies && [removeDependencies count]) {
		[dependencies addObjectsFromArray:removeDependencies];
	    }
	    if (selectorDependencies && [selectorDependencies count]) {
		[dependencies addObjectsFromArray:selectorDependencies];
	    }
	}
    }
    
    for (i=0; i<[dependencies count]; i++) {
	NSString *fileName = [dependencies objectAtIndex:i];
	if (![self fileIsPartOfPackage:fileName]) {
	    NSRunAlertPanel(@"Error", @"Package is broken. Could not find file in TOC which is in file dependency list.", @"Accept defeat", nil, nil);
	    return NO;
	}
    }
    return YES;
}

- (BOOL)loadFileWrapperRepresentation:(NSFileWrapper *)wrapper ofType:(NSString *)type
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: loadFileWrapperRepresentation");
    if ([type isEqualToString:II2DocumentType]) {
	NSMutableString *plistname;
	NSMutableString *customplistname;
        NSString *minII2StampString = nil;
	NSNumber *iiVersion;

	// TODO: these may be fragile for changes in [self fileName]
	__baseName = [[[[self fileName] lastPathComponent] stringByDeletingPathExtension] retain];
	__dirName = [[self fileName] retain];
	__dirNameWithSeparator = [[__dirName stringByAppendingString:@"/"] retain];

	plistname = [[[NSMutableString alloc] initWithString:[self fileName]] autorelease];
	[plistname appendFormat:@"/%s.plist", [__baseName fileSystemRepresentation]];
	__pkgPlist = [[NSDictionary dictionaryWithContentsOfFile:plistname] retain];
	if (!__pkgPlist) {
	    NSRunAlertPanel(@"Error", @"This i-Package is broken or damaged. Could not load Table of Contents.", @"Accept defeat", nil, nil);
	    return NO;
	}

	iiVersion = [self pkgIIVersion];
	if ([iiVersion compare:[NSNumber numberWithInt:2]] != NSOrderedSame &&
	    [iiVersion compare:[NSNumber numberWithInt:3]] != NSOrderedSame) {
	    NSRunAlertPanel(@"Error", [NSString stringWithFormat:@"This i-Package has the wrong type (version: %@). Could not load in this version of i-Installer (accepts version: 2, 3).", iiVersion], @"Accept defeat", nil, nil);
	    return NO;
	}
        
        minII2StampString = [self pkgMinII2stamp];
        if (minII2StampString) {
            // Compare with gwbuild.stamp, which contains our II2 timestamp
            NSString *appStampString = [NSString stringWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"gwbuild.stamp" ofType:nil]];
            if (appStampString) {
                NSDecimalNumber *appNumber = [NSDecimalNumber decimalNumberWithString:appStampString];
                NSDecimalNumber *minNumber = [NSDecimalNumber decimalNumberWithString:minII2StampString];
                if ([minNumber compare:appNumber] == NSOrderedDescending) {
                    NSRunAlertPanel(@"Package requires recent i-Installer", @"This i-Package needs a more recent version of i-Installer. Please update i-Installer before opening this i-Package.", @"Accept defeat", nil, nil);
                    return NO;
                }
            }
        }

	customplistname = [[[NSMutableString alloc] initWithString:[self fileName]] autorelease];
	[customplistname appendFormat:@"/Custom.plist"];
	__customPkgPlist = [[NSMutableDictionary dictionaryWithContentsOfFile:customplistname] retain];
	if (!__customPkgPlist) {
	    __customPkgPlist = [[NSMutableDictionary alloc] init];
	}
	
	[self readInstalledPkgPlist:YES]; // If any install, read it in cache dict

	__urlNameWithSeparator = [[[self urlString] stringByAppendingString:@"/"] retain];
        return [self checkStageFileDependencies];
    } else {
	NSLog( @"Wrong file type: %@", type);
        return NO;
    }
}

- (BOOL)canCloseDocument
{
    // Editorial: Apple has declared this method obsolete. I consider that (and some other developments) a part of the complexity-creep that is affecting AppKit (mainly). The massive use of callback methods and such is turning a clean to program system slowly in something resembling X11 of old. Sheets/Panels and this are the most important parts that turn the program into a far more complex one, because so much state has to be preserved while things are handled. Note that none of Apple's own examples use the new method (and the one that needs it uses the obsolete version). For me, the new version and the use of dirtying the document is unacceptable, because I do not have a save function with a save panel, the only function I need to do at close is probably a thinning.
    if ([self active]) {
	NSMutableDictionary *sheetDict = [[[NSMutableDictionary alloc] init] autorelease];
	
	[sheetDict setObject:self forKey:@"resulthandler"];
	[sheetDict setObject:NSStringFromSelector( @selector(endSheetWithDictionary:)) forKey:@"resulthandlerselector"];
	[sheetDict setObject:@"Alert" forKey:@"type"];
	[sheetDict setObject:@"Warning" forKey:@"subtype"];
	[sheetDict setObject:@"Busy" forKey:@"title"];
	[sheetDict setObject:@"There are processes or tasks active. I cannot close while any activity is going on. Please let the processes finish or abort them before trying to close." forKey:@"info"];
	[sheetDict setObject:@"Accept" forKey:@"okbutton"];
	[[__cachedWindowController interactionManager] startInternalPanel:sheetDict];
	return NO;
    }
    if (![self isLocked] && [self isWritable] && [ii2Prefs askForThinAtClose] && ([self sizeOnDisk] > [ii2Prefs thinSize])) {
	NSMutableDictionary *sheetDict = [[[NSMutableDictionary alloc] init] autorelease];
	
	[sheetDict setObject:self forKey:@"resulthandler"];
	[sheetDict setObject:NSStringFromSelector( @selector(endSheetWithDictionary:)) forKey:@"resulthandlerselector"];
	[sheetDict setObject:@"Alert" forKey:@"type"];
	[sheetDict setObject:@"ThinBeforeClose" forKey:@"subtype"];
	[sheetDict setObject:@"Thin -i-Package?" forKey:@"title"];
	[sheetDict setObject:[NSString stringWithFormat:@"Do you want to thin the package to less than %@?", [[GWSizeFormatter sharedFormatter] stringForObjectValue:[NSNumber numberWithUnsignedInt:[ii2Prefs thinSize]]]] forKey:@"info"];
	[sheetDict setObject:@"Yes" forKey:@"okbutton"];
	[sheetDict setObject:@"No" forKey:@"cancelbutton"];
	[[__cachedWindowController interactionManager] startInternalPanel:sheetDict];
	return NO;
    }
    return YES;
}

- (BOOL)validateMenuItem:(NSMenuItem *)anItem
{
    // NSLog( @"%@ Validating %@", [[self class] description], [anItem description]);
    return YES;
}

- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
    //NSLog( @"%s\n", rcsidUserCommunication);
    //NSLog( @"%s\n", rcsidInfo);
    //NSLog( @"%s\n", rcsidTableSource);
    //NSLog( @"%s\n", rcsidActions);
}

@end

BOOL md5StringsAreEqual( NSString *md5String1, NSString *md5String2)
{
    NSArray *parts1 = [md5String1 componentsSeparatedByString:@"="];
    NSMutableString *actual1 = [[[NSMutableString alloc] initWithString:[parts1 lastObject]] autorelease];
    NSArray *parts2 = [md5String2 componentsSeparatedByString:@"="];
    NSMutableString *actual2 = [[[NSMutableString alloc] initWithString:[parts2 lastObject]] autorelease];
    if (![actual1 hasSuffix:@"\n"]) {
	[actual1 appendString:@"\n"];
    }
    if (![actual2 hasSuffix:@"\n"]) {
	[actual2 appendString:@"\n"];
    }
    
    return [actual1 isEqualToString:actual2];
}

static BOOL checkFileMD5AgainstString( NSString *fullPath, NSString *md5String)
{
    NSArray *args = [NSArray arrayWithObjects: [II2TOOLFINDER pathForTool:@"/usr/bin/openssl"], @"md5", fullPath, nil];
    GLUPTaskRunner *md5CheckTask = nil;
    NSMutableData *outData = [[NSMutableData alloc] init];
    NSString *resultString = nil;
    md5CheckTask = [[GLUPTaskRunner alloc] initWithOwner:nil reportsPID:GLUPTaskRunnerFifoPid arguments:args authenticator:nil];
    [md5CheckTask setProperties:[NSDictionary dictionaryWithObjectsAndKeys: @"separateMD5check", @"type", outData, @"outdata", nil]];
    [md5CheckTask setDirectoryPath:[fullPath stringByDeletingLastPathComponent]];
    [md5CheckTask setOutData:outData];
    [md5CheckTask run];

    resultString = [[[NSString alloc] initWithData:outData encoding:NSASCIIStringEncoding] autorelease];
    [outData release];
    return md5StringsAreEqual(md5String, resultString);
}

BOOL checkMD5OfFileAgainstContentsOfFile( NSString *fullPath, NSString *md5FullPath)
{
    NSString *md5String = [NSString stringWithContentsOfFile:md5FullPath];

    return checkFileMD5AgainstString( fullPath, md5String);
}

BOOL checkMD5OfFileAgainstCompanionMD5File( NSString *fullPath)
{
    NSString *md5File = [fullPath stringByAppendingString:@".md5"];

    return checkMD5OfFileAgainstContentsOfFile( fullPath, md5File);
}

BOOL checkMD5OfFileAgainstContentsOfURL( NSString *fullPath, NSString *fullMD5URLString)
{
    NSURL *url = [NSURL URLWithString:fullMD5URLString];
    if (url) {
	NSData *urlData = urlDataFromURL( url);
	if (urlData) {
	    NSString *md5String = [[[NSString alloc] initWithData:urlData encoding:NSUTF8StringEncoding] autorelease];

	    return checkFileMD5AgainstString( fullPath, md5String);
	}
    }
    return NO;
}

/*
 * $Log: II2Package.m,v $
 * Revision 1.3  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.2  2004/02/02 20:15:27  gctwnl
 * Fixed loss of keyed objects in package window (spinning progress). Some small fixes in logging and debug messages.
 *
 * Revision 1.1.1.1  2004/02/01 17:18:23  gctwnl
 * Initial import.
 *
 * Revision 3.14  2004/01/27 22:49:32  gerben
 * 2.45.0
 *
 * Revision 3.13  2004/01/27 12:57:39  gerben
 * 2.44.0
 *
 * Revision 3.12  2004/01/23 00:31:16  gerben
 * 2.42.0 release
 *
 * Revision 3.11  2004/01/15 01:00:06  gerben
 * 2.40.0
 *
 * Revision 3.10  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.9  2004/01/05 15:31:34  gerben
 * *** empty log message ***
 *
 * Revision 3.8  2003/11/09 13:02:07  gerben
 * 2.31.0
 *
 * Revision 3.7  2003/11/08 23:38:01  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.6  2003/08/03 18:52:35  gerben
 * Added ZIP handling. Version 2.25.0.
 *
 * Revision 3.5  2003/05/18 13:35:16  gerben
 * Version 2.21.0. Improved i-Directory handling.
 *
 * Revision 3.4  2003/03/13 14:42:04  gerben
 * 2.11.0. Really fix the permissions problem.
 *
 * Revision 3.3  2003/01/04 15:43:46  gerben
 * 2.2.0 final commit.
 *
 * Revision 3.2  2002/12/10 14:24:52  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:23  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:52  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.39  2002/12/10 08:55:24  gerben
 * II2Package code cleanup, remove of cached view pointers. Added relocatable handling in package dependencies.
 *
 * Revision 1.38  2002/11/30 10:05:54  gerben
 * Rearranged code for II2Package to use catagories instead of code inclusion.
 *
 * Revision 1.37  2002/10/28 21:49:45  gerben
 * A lot of changes wrt known packages.
 *
 * Revision 1.36  2002/10/27 15:39:42  gerben
 * Tried to remove BSD-package dependency
 *
 * Revision 1.35  2002/10/25 20:36:11  gerben
 * Layout changes.
 *
 * Revision 1.34  2002/10/25 19:27:21  gerben
 * Several small changes re known packages window. Implemented developer build stamp in main bundle.
 *
 * Revision 1.33  2002/10/24 13:03:31  gerben
 * Disable menu items that can affect downloads and availability of files when package is active.
 *
 * Revision 1.32  2002/10/17 18:48:59  gerben
 * Cache/NoCache handling on downloads. Some other changes.
 *
 * Revision 1.31  2002/10/16 21:28:21  gerben
 * A complete new rewrite of taskss, processes and stuff.
 *
 * Revision 1.30  2002/10/14 12:15:37  gerben
 * Install stamp file after unarchive.
 *
 * Revision 1.29  2002/10/13 22:22:27  gerben
 * Re-created file menu so validating menu items work. Changed code for inspect and fatten to use generic task array starter.
 *
 * Revision 1.28  2002/10/13 21:20:14  gerben
 * Fixed a serious bug where the task rewrite was a bit release-happy
 *
 * Revision 1.27  2002/10/12 10:54:38  gerben
 * A lot of changes, mostly when producing output/feedback.
 *
 * Revision 1.26  2002/10/10 00:51:24  gerben
 * Many changes. Committing a stable state.
 *
 * Revision 1.25  2002/10/09 14:22:23  gerben
 * Removed outputHandler from GLUPTaskRunner
 *
 * Revision 1.24  2002/10/09 14:16:11  gerben
 * Changed as many %s/lossyCString uses to %@/fileSystemRepresentation/UTF8String
 *
 * Revision 1.23  2002/10/09 13:33:24  gerben
 * Md% comparison that works on 10.1 as well as 10.2
 *
 * Revision 1.22  2002/10/09 10:53:40  gerben
 * Moved textview output to NSTexTView category.
 *
 * Revision 1.21  2002/10/09 08:38:41  gerben
 * Fixed two 10.1.5 incompatibilities (untested)
 *
 * Revision 1.20  2002/10/08 21:34:28  gerben
 * Working GPG signature checks, shared GPG cache for II2.
 *
 * Revision 1.19  2002/10/08 13:20:29  gerben
 * MD5 check on update. Small changes.
 *
 * Revision 1.18  2002/10/07 20:51:30  gerben
 * NSProgressIndicator on package window. Some renaming of functions.
 *
 * Revision 1.17  2002/10/07 20:28:05  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.16  2002/10/07 20:20:03  gerben
 * More md5 stuff and testing of spinning disk progressindicator stuff.
 *
 * Revision 1.15  2002/10/07 19:08:59  gerben
 * Added checking of external MD5's (either URL or file based).
 *
 * Revision 1.14  2002/10/07 11:52:27  gerben
 * Several fixes related to possible invalid content.
 *
 * Revision 1.13  2002/10/03 18:15:21  gerben
 * A typo (bz instead of bz2) prevented the unarchive of anything.
 *
 * Revision 1.12  2002/10/03 13:38:03  gerben
 * Removed addChildWindow as child passes close to parent. Window can only close when there are no running tasks.
 *
 * Revision 1.11  2002/10/02 22:38:30  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */