//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2AppDelegate.h"
#import "II2Preferences.h"
#import "II2Package.h"

#import <sys/types.h>
#import <sys/stat.h>
#import <sys/mount.h>
#import <unistd.h>

static char rcsidInfo[] = "$Id: II2PackageInfo.m,v 1.5 2004/04/25 13:13:21 gctwnl Exp $";

extern BOOL intInArray( int number, NSArray *array)
{
    int i;
    
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: intInArray");
    for (i=0; i<[array count]; i++) {
	int value = [[array objectAtIndex:i] intValue];
	if (number == value) {
	    return YES;
	}
    }
    return NO;
}

@implementation II2Package (II2PackageInfo)

- (void)writeCustomPlistToDisk
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: writeCustomPlistToDisk");
    if ([self isWritable]) {
	// Silently ignore write failure in non-writable packages
	NSMutableString *customplistname = [[[NSMutableString alloc] initWithString:[self fileName]] autorelease];
	[customplistname appendFormat:@"/Custom.plist"];
	[__customPkgPlist setObject:[NSString stringWithCString:getlogin()] forKey:@"writtenby"];
	[__customPkgPlist setObject:[NSString stringWithFormat:@"%d", time( NULL)] forKey:@"writtenat"];
	if (![__customPkgPlist writeToFile:customplistname atomically:YES]) {
	    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Unable to write custom package status to disk.\n"] attributes:[self signalAttributes]] autorelease]];
	}
    }
}

// Returns the selection that [self selectedSelectorSets] produces on known Sets. [self selectedSelectorSets] may be set as a result of user interaction and defines which parts of the package we are interested in for the current action.

- (BOOL)setIdInSelectedSets: (NSString *)knownSetId
{
    int j;
    NSArray *selectorSetArray = [self selectedSelectorSets];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: setIdInSelectedSets %@", knownSetId);
    for (j=0; j<[selectorSetArray count]; j++) {
	NSString *selectedSetId = [selectorSetArray objectAtIndex:j];
	if ([selectedSetId isEqualToString:knownSetId]) {
	    return YES;
	}
    }
    return NO;
}

- (BOOL)setIdInAvailableSets: (NSString *)knownSetId
{
    int j;
    NSArray *selectorSetArray = [self availableSelectorSets];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: setIdInSelectedSets %@ %@", knownSetId, [selectorSetArray description]);
    for (j=0; j<[selectorSetArray count]; j++) {
	NSString *selectedSetId = [selectorSetArray objectAtIndex:j];
	if ([selectedSetId isEqualToString:knownSetId]) {
	    return YES;
	}
    }
    return NO;
}


- (BOOL)setIdInConfiguredSets: (NSString *)knownSetId
{
    int j;
    NSArray *selectorSetArray = [self configuredSelectorSets];
    
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: setIdInConfiguredSets %@", knownSetId);
    for (j=0; j<[selectorSetArray count]; j++) {
	// NSLog( @"setIdInConfiguredSets: Testing %@ against %@", knownSetId, [selectorSetArray objectAtIndex:j]);
	if ([knownSetId isEqualToString:[selectorSetArray objectAtIndex:j]]) {
	    return YES;
	}
    }
    return NO;
}

- (BOOL)setIdInInstalledSets: (NSString *)knownSetId
{
    int j;
    NSArray *selectorSetArray = [self installedSelectorSets];
    
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: setIdInInstalledSets %@", knownSetId);
    for (j=0; j<[selectorSetArray count]; j++) {
	//NSLog( @"setIdInInstalledSets: Testing %@ against %@", knownSetId, [selectorSetArray objectAtIndex:j]);
	if ([knownSetId isEqualToString:[selectorSetArray objectAtIndex:j]]) {
	    // NSLog( @"YES");
	    return YES;
	}
    }
    //NSLog( @"NO");
    return NO;
}

- (NSArray *)selectedKnownSets
{
    NSArray *knownSets = [__pkgPlist objectForKey:@"selectorsets"];
    NSArray *selectorSetArray = [self selectedSelectorSets];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: selectedKnownSets");
    if (!knownSets) {
	return nil;
    }
    if (selectorSetArray) {
	NSMutableArray *result = [[[NSMutableArray alloc] init] autorelease];
	int i;

	// For all known sets, walk [self selectedSelectorSets] and add the set to the result if it is in [self selectedSelectorSets]
	for (i=0; i<[knownSets count]; i++) {
	    NSString *knownSetId = [[knownSets objectAtIndex:i] objectForKey:@"id"];
	    
	    if (!knownSetId) {
		NSLog( @"Id missing in set. Skipping.");
		continue;
	    }
	    if ([self setIdInSelectedSets:knownSetId]) {
		[result addObject:[knownSets objectAtIndex:i]];
	    }
	}
	return result;
    }
    else {
	return knownSets;
    }
}

- (NSArray *)extendedSelectorSetArray
{
    NSArray *knownSets = [__pkgPlist objectForKey:@"selectorsets"];
    NSMutableArray *result = [NSMutableArray array];
    int i;
    
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: extendedSelectorSetArray");
    for (i=0; i<[knownSets count]; i++) {
	NSString *knownSetId = [[knownSets objectAtIndex:i] objectForKey:@"id"];
	NSString *knownSetName = [[knownSets objectAtIndex:i] objectForKey:@"name"];
	NSString *knownSetComment = [[knownSets objectAtIndex:i] objectForKey:@"comment"];
	NSString *selectedString = ([self setIdInSelectedSets:knownSetId] ? @"YES" : @"NO");
	NSString *configuredSet = ([self setIdInConfiguredSets:knownSetId] ? @"YES" : @"NO");
	NSString *installedSet = ([self setIdInInstalledSets:knownSetId] ? @"YES" : @"NO");
	NSString *availableSet = ([self setIdInAvailableSets:knownSetId] ? @"YES" : @"NO");
	NSDictionary *dict;
	if (!knownSetId || !knownSetName || !knownSetComment) {
	    NSLog( @"Id, name or comment missing in set. Skipping.");
	    continue;
	}
	dict = [NSDictionary dictionaryWithObjectsAndKeys: knownSetId, @"id", selectedString, @"selected", knownSetName, @"name", knownSetComment, @"comment", configuredSet, @"configured", installedSet, @"installed", availableSet, @"available", nil];
	if (dict) {
	    // NSLog( @"ESA: %@", [dict description]);
	    [result addObject:dict];
	}
    }
    return result;
}

// Get dependencies for a combination of stage (selector, prepare, configure, remove) and current selector set (if any)
- (NSArray *)dependentFilesForStage:(NSString *)stage filter:(BOOL)filter
{
    NSArray *selectedKnownSets = [self selectedKnownSets];
    NSDictionary *knownDependencies = [__pkgPlist objectForKey:@"dependencies"];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: dependentFilesForStage %@", stage);
    if (selectedKnownSets) {
	NSMutableDictionary *tmpResult = [[[NSMutableDictionary alloc] init] autorelease];
	int i;
	
	for (i=0; i<[selectedKnownSets count]; i++) {
	    NSArray *depArrayOfSet = [[[selectedKnownSets objectAtIndex:i] objectForKey:@"dependencies"] objectForKey:stage];
	    if (depArrayOfSet) {
		int j;
		
		for (j=0; j<[depArrayOfSet count]; j++) {
		    [tmpResult setObject:@"YES" forKey:[depArrayOfSet objectAtIndex:j]];
		}
	    }
	}
	
	return [tmpResult allKeys];
    }
    else if (knownDependencies) {
	// Old style dependencies, not per set but per stage only.
	NSArray *depArray = [[__pkgPlist objectForKey:@"dependencies"] objectForKey:stage];
	if (depArray) {
	    return depArray;
	}
	else {
	    return [NSArray array];
	}
    }
    return nil;
}


// Returns true if the archive part with number i is part of any of the sets selected.
// Returns true if no set was selected (= all sets) or if this package does not have sets
- (BOOL)intInSelectedSets: (int) archivePartNumber
{
    NSString *archivePartString = [NSString stringWithFormat:@"%d", archivePartNumber];
    NSArray *knownSets = [self selectedKnownSets];
    int i;
    
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: intInSelectedSets %d", archivePartNumber);
    if (!knownSets) {
	return YES;
    }
    
    for (i=0; i<[knownSets count]; i++) {
	NSDictionary *set = [knownSets objectAtIndex:i];
	NSDictionary *parts = [set objectForKey:@"parts"];
	NSEnumerator *partEnumerator;
	NSString *key;

	if (!parts) {
	    NSLog( @"Weird package error: has sets without parts.");
	    return YES;
	}
	
	partEnumerator = [parts keyEnumerator];
	while ((key = [partEnumerator nextObject]) != nil) {
	    if ([key isEqualToString:archivePartString]) {
		return YES;
	    }
	}
    }
    return NO;
}

- (void)setSelectorSets:(NSArray *)setArray
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: setSelectorSets");
    [__customPkgPlist setObject:setArray forKey:@"selectedselectorsets"];
    [self writeCustomPlistToDisk];
}

- (void)setSelectorSet:(NSString *)setId configured:(BOOL)configured
{
    NSMutableDictionary *configuredSets;
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: setConfiguredSelectorSet %@ %d", setId, configured);
    configuredSets = [NSMutableDictionary dictionaryWithDictionary:[__installedPkgPlist objectForKey:@"configuredselectorsets"]];
    
    if (!configuredSets) {
	configuredSets = [NSMutableDictionary dictionary];
    }

    if (configured) {
	[configuredSets setObject:@"YES" forKey:setId];
    }
    else {
	[configuredSets removeObjectForKey:setId];
    }
    [__installedPkgPlist setObject:configuredSets forKey:@"configuredselectorsets"];
    [self writeReceipt];
}

- (void)setSelectorSet:(NSString *)setId installed:(BOOL)installed
{
    NSMutableDictionary *installedSets;
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: setInstalledSelectorSet %@ %d", setId, installed);
    installedSets = [NSMutableDictionary dictionaryWithDictionary:[__installedPkgPlist objectForKey:@"installedselectorsets"]];
    
    if (!installedSets) {
	installedSets = [NSMutableDictionary dictionary];
    }
    
    if (installed) {
	[installedSets setObject:@"YES" forKey:setId];
    }
    else {
	[installedSets removeObjectForKey:setId];
    }
    [__installedPkgPlist setObject:installedSets forKey:@"installedselectorsets"];
    [self writeReceipt];
}

- (int) partNumberFromArchivePartFileName: (NSString *)archivePartname
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: partNumberFromArchivePartFileName");
    if ([archivePartname hasSuffix:@".tar.gz"] || [archivePartname hasSuffix:@".tar.bz2"]) {
	NSString *baseFileName = [[archivePartname stringByDeletingPathExtension] stringByDeletingPathExtension];
	
	if ([baseFileName isEqualToString:__baseName]) {
	    return 0;
	}
	else {
	    NSString *nextExt = [baseFileName pathExtension];
	    NSString *scannedExt;
	    NSScanner *scanString = [NSScanner scannerWithString:nextExt];
	    if ([scanString scanCharactersFromSet:[NSCharacterSet decimalDigitCharacterSet] intoString:&scannedExt] && [nextExt isEqualToString:scannedExt]) {
		return [nextExt intValue];
	    }
	}
    }
    return -1;
}

- (BOOL)__pkgPlistKeyExists:(NSString *)keyString custom:(BOOL)custom
{
    NSString *stringFromDict = [(custom?__customPkgPlist:__installedPkgPlist) objectForKey:keyString];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: __pkgPlistKeyExists %@", keyString);
    if (!stringFromDict) {
        stringFromDict = [__pkgPlist objectForKey:keyString];
    }
    return stringFromDict != nil;
}

- (BOOL)__pkgPlistBOOL:(NSString *)keyString custom:(BOOL)custom
{
    NSString *stringFromDict = [(custom?__customPkgPlist:__installedPkgPlist) objectForKey:keyString];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: __pkgPlistBOOL %@", keyString);
    if (!stringFromDict) {
        stringFromDict = [__pkgPlist objectForKey:keyString];
    }
    return [stringFromDict isEqualToString:@"YES"];
}

- (void)__setPlistBOOL:(BOOL)aBool forString:(NSString *)aKey custom:(BOOL)custom
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: __setPlistBOOL %@ %d", aKey, aBool);
    [(custom?__customPkgPlist:__installedPkgPlist) setObject:(aBool? @"YES" : @"NO") forKey:aKey];
    custom ? [self writeCustomPlistToDisk] : [self writeReceipt];
}

- (BOOL)readInstalledPkgPlist:(BOOL)force
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: readInstalledPkgPlist %d", force);
    if (force || !__installedPkgPlist) {
	if (__installedPkgPlist) {
	    [__installedPkgPlist release]; __installedPkgPlist = nil;
	}
	__installedPkgPlist = [[NSMutableDictionary dictionaryWithDictionary:[self installedPackageDictionary]] retain];
    }
    return (__installedPkgPlist != nil);
}

- (BOOL)readRemotePkgPlist:(BOOL)force
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: readRemotePkgPlist %d", force);
    if (force || !__remotePkgPlist) {
	if (__remotePkgPlist) {
	    [__remotePkgPlist release]; __remotePkgPlist = nil;
	}
	__remotePkgPlist = [self foregroundGetURLDictionary:[__baseName stringByAppendingString:@".plist"]];
    }
    return (__remotePkgPlist != nil);
}

- (void)clearChecksForFileSet:(NSString *)fileName
{
    NSArray *partSet = [self allocFilePartsArrayFromFileName:fileName filter:NO];
    int i;
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: clearChecksForFileSet %@", fileName);
    for (i = 0; i < [partSet count]; i++) {
	[self clearChecksForFile:[partSet objectAtIndex:i]];
    }
    [partSet release];
}

- (void)clearChecksForFile:(NSString *)fileName
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: clearChecksForFile %@", fileName);
    [__md5Checks removeObjectForKey:fileName];
    [__sigChecks removeObjectForKey:fileName];
    [__fileDataTypes removeObjectForKey:fileName];
    [__cachedWindowController refreshFileTable]; // noop if file not in that particular table
}

- (NSString *)fileTypeDisplayString:(NSString *)fileName
{
    NSString *result = [__fileDataTypes objectForKey:fileName];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: fileTypeDisplayString %@", fileName);
    if (result) {
	return result;
    }
    return @"?";
}

-(NSString *)fileNameAtIndex:(int)index
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: fileNameAtIndex %d", index);
    if ((index >= 0) && (index < [__fileTableSource count])) {
	return [[__fileTableSource objectAtIndex:index] objectForKey:@"file"];
    }
    return nil;
}

- (NSString *)packageName
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: packageName");
    return [__pkgPlist objectForKey:@"name"];
}

- (BOOL)needsAuthentication
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: needsAuthentication");
    return [self __pkgPlistBOOL:@"adminrequired" custom:YES];
}

- (void)setAuthenticationRequired:(BOOL)required
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: setAuthenticationRequired %d", required);
    [__customPkgPlist setObject:(required? @"YES" : @"NO") forKey:@"adminrequired"];
    [self writeCustomPlistToDisk];
    if (required) {
	[__auth release];
	__auth = [[GWAuthenticator alloc] initWithLaunchPath:[GLUPTaskRunner fixedLaunchPath]];
    }
    else {
	[__auth release];
	__auth = nil;
    }
}

- (BOOL)isLocked
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: isLocked");
    return [self __pkgPlistBOOL:@"lockstatus" custom:YES];
}

- (void)setLockStatus:(BOOL)locked
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: setLockStatus %d", locked);
    [self __setPlistBOOL:locked forString:@"lockstatus" custom:YES];
    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:(locked ? @"Package locked\n" : @"Package unlocked\n")] attributes:[self infoAttributes]] autorelease]];
}


- (unsigned int)totalSize
{
    NSDictionary *fileDict = [__pkgPlist objectForKey:@"files"];
    NSEnumerator *enumerator = [fileDict keyEnumerator];
    NSDictionary *key;
    unsigned int result = 0;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: totalSize");
    while (key = [enumerator nextObject]) {
	result += [[[fileDict objectForKey:key] objectForKey:@"size"] intValue];
    }
    return result;
}

- (NSNumber *)pkgIIVersion
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: pkgIIVersion");
    return [NSNumber numberWithInt:[[__pkgPlist objectForKey:@"iiversion"] intValue]];
}

- (NSNumber *)remoteIIVersion
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: remoteIIVersion");
    if (!__remotePkgPlist) {
	[self readRemotePkgPlist:NO];
    }
    return [NSNumber numberWithInt:[[__remotePkgPlist objectForKey:@"iiversion"] intValue]];
}

- (NSString *)pkgMinII2stamp
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: pkgMinII2stamp");
    return [__pkgPlist objectForKey:@"minii2stamp"];
}

- (NSString *)pkgMinII2stampDisplay
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: pkgMinII2stampDisplay");
    NSString *pkgMinII2stampDisplay = displayStringFromTimestampString( [self pkgMinII2stamp]);
    return pkgMinII2stampDisplay;
}

- (NSString *)pkgTimestamp
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: pkgTimestamp");
    return [__pkgPlist objectForKey:@"timestamp"];
}

- (NSString *)pkgTimestampDisplay
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: pkgTimestampDisplay");
    NSString *pkgTimestampDisplay = displayStringFromTimestampString( [self pkgTimestamp]);
    return pkgTimestampDisplay;
}

- (NSDictionary *)installedPackageDictionary
{
    NSDictionary *result= nil;
    NSString *fullPath = [NSString stringWithFormat:@"%@/%@/%@.ii2receipt", [ii2Prefs ii2LibDir], RECEIPTSSUBDIR, __baseName];
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: installedPackageDictionary %@", fullPath);

    if ([DEFFM fileExistsAtPath:fullPath]) {
	result = [NSDictionary dictionaryWithContentsOfFile:fullPath];
    }
    return result;
}

- (NSString *)installedTimestamp
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: installedTimestamp");
    [self readInstalledPkgPlist:NO];
    if (__installedPkgPlist) {
	return [__installedPkgPlist objectForKey:@"timestamp"];
    }
    else {
	NSString *destinationTimestampPath = [NSString stringWithFormat:@"%s/.%s.ii2", [[self installLocation] fileSystemRepresentation], [__baseName fileSystemRepresentation]];
	NSString *stampString = nil;
	
	if ([DEFFM fileExistsAtPath:destinationTimestampPath]) {
	    stampString = [NSString stringWithContentsOfFile:destinationTimestampPath];
	    if (!stampString) {
		NSLog( @"%@ esists but it is unreadable. Cannot establish old style installation date.", destinationTimestampPath);
	    }
	}
	return stampString;
    }
}

- (NSString *)installedTimestampDisplay
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: installedTimestampDisplay");
    if ([self hasArchive]) {
	NSString *stampString = [self installedTimestamp];
	if (stampString) {
	    NSNumber *stampNumber = [NSNumber numberWithLong:[stampString intValue]];
	    NSNumber *myVersion = [NSNumber numberWithLong:[[self pkgTimestamp] intValue]];
	    if ([stampNumber compare:myVersion] == NSOrderedSame) {
		return @"Same as Self";
	    }
	    else {
		return [@"Different: " stringByAppendingString:displayStringFromTimestampString( stampString)];
	    }

	}
	else {
	    return @"No installation found";
	}
    }
    else {
	return @"NA (archiveless package)";
    }
}

- (NSDictionary *)foregroundGetURLDictionary: (NSString *)fileName
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: foregroundGetURLDictionary %@", fileName);
    if ([ii2Prefs networkAccessDisabled]) {
	if ([ii2Prefs mainLogLevel] > 0) NSLog( @"Unable to get URLDictionary when network access has been disabled.");
	return nil;
    }
    else {
	NSURL *url = [NSURL URLWithString:[self fullURLPathForFileName:fileName]];
	NSData *urlData = urlDataFromURL( url);
	NSDictionary *dictionary;
	NSString *error;

	if (!urlData) {
	    return nil;
	}
	dictionary = (NSDictionary *)CFPropertyListCreateFromXMLData( NULL, (CFDataRef)urlData, kCFPropertyListImmutable, (CFStringRef *)&error);
	
	//TODO release error?

	return dictionary;
    }    
}

- (NSString *)foregroundGetURLString: (NSString *)fileName
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: foregroundGetURLString %@", fileName);
    if ([ii2Prefs networkAccessDisabled]) {
	if ([ii2Prefs mainLogLevel] > 0) NSLog( @"Unable to get URLString when network access has been disabled.");
	return nil;
    }
    else {
	NSURL *url = [NSURL URLWithString:[self fullURLPathForFileName:fileName]];
	NSData *urlData = urlDataFromURL( url);
	NSString *string = [[[NSString alloc] initWithData:urlData encoding:NSUTF8StringEncoding] autorelease];

	return string;
    }
}

- (NSString *)remoteUpdateTimestamp
{
    NSString *updateStatus = [self foregroundGetURLString:[__baseName stringByAppendingString:@".updatestatus"]];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: remoteUpdateTimestamp");
    if (updateStatus) {
	NSArray *updateComponents = [updateStatus componentsSeparatedByString:@":"];
	if ([[updateComponents objectAtIndex:0] isEqualToString:@"STARTUPLOAD"]) {
	    return [updateComponents objectAtIndex:1];
	}
	else {
	    return nil;
	}
    }
    else {
	NSString *startUpdateTimestamp = [self foregroundGetURLString:[__baseName stringByAppendingString:@".startupdate"]];
	if (startUpdateTimestamp && [startUpdateTimestamp isEqualToString:@""]) {
	    return nil;
	}
	return startUpdateTimestamp;
    }
}

- (NSString *)remoteTimestamp
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: remoteTimestamp");
    if (!__remotePkgPlist) {
	[self readRemotePkgPlist:NO];
    }
    return [__remotePkgPlist objectForKey:@"timestamp"];
}

- (NSString *)remoteStatusDisplay
{
    NSString *remoteTimestamp;
    NSString *remoteUpdateTimestamp = [self remoteUpdateTimestamp];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: remoteStatusDisplay");
    remoteTimestamp = [self remoteTimestamp];
    if (remoteTimestamp) {
	NSNumber *remoteVersion;
	NSNumber *myVersion;

	if (remoteUpdateTimestamp) {
	    return [@"Remote is being updated since " stringByAppendingString:displayStringFromTimestampString( remoteUpdateTimestamp)];
	}

	remoteVersion = [NSNumber numberWithLong:[remoteTimestamp intValue]];
	myVersion = [NSNumber numberWithLong:[[__pkgPlist objectForKey:@"timestamp"] intValue]];

	if ([remoteVersion compare:myVersion] == NSOrderedSame) {
	    return @"Same as Self";
	}
	else {
	    return [@"Different: " stringByAppendingString:displayStringFromTimestampString( remoteTimestamp)];
	}
    }
    else {
	return @"Could not get remote timestamp";
    }
}

- (II2TimestampComparisonResult)remoteStatus
{
    NSString *remoteTimestamp = [self remoteTimestamp];
    NSString *remoteUpdateTimestamp = [self remoteUpdateTimestamp];
    NSString *pkgTimestamp = [self pkgTimestamp];
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: remoteStatus");
    if (remoteUpdateTimestamp != nil) {
	return II2IsBeingUpdated;
    }
    else {
	if (remoteTimestamp) {
	    NSNumber *localVersion = [NSNumber numberWithLong:[pkgTimestamp intValue]];
	    NSNumber *remoteVersion = [NSNumber numberWithLong:[remoteTimestamp intValue]];
	    return [localVersion compare:remoteVersion];
	}
	else {
	    return II2IsUnavailable;
	}
    }
}

- (void)returnToDefaults
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: returnToDefaults");
    if (![self isLocked] && [self isWritable]) {
	NSMutableString *customplistname = [[[NSMutableString alloc] initWithString:[self fileName]] autorelease];
	[customplistname appendFormat:@"/Custom.plist"];
	[DEFFM removeFileAtPath:customplistname handler:nil];
	[__customPkgPlist release];
	__customPkgPlist = [[NSMutableDictionary alloc] init];
    }
}

- (BOOL)hasTabViewSettings
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: hasTabViewSettings");
    return [self __pkgPlistBOOL:@"hastabviewsettings" custom:YES];
}

- (BOOL)hideTabForTag:(II2TabTagType)tag
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: hideTabForTag %d", tag);
    if (sizeof( ii2TagNameArray) <= tag) {
        NSLog( @"Program Internal Error: tag >= sizeof tag array");
        return NO;
    }
    NSString *keyString = ii2TagNameArray[tag];
    if ([self __pkgPlistKeyExists:keyString custom:YES]) {
        return [self __pkgPlistBOOL:keyString custom:YES];
    }
    else {
        return [ii2Prefs getTabHideStatusForKey:keyString];
    }
}

- (void)setHideTabForTag:(II2TabTagType)tag toState:(BOOL)state
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: intInSelectedSets %d %d", tag, state);
    if (tag > ii2ReadmeTabTag && tag < ii2NrOfTabs) {
        [self __setPlistBOOL:state forString:ii2TagNameArray[tag] custom:YES];
        [self __setPlistBOOL:YES forString:@"hastabviewsettings" custom:YES];
    }
}

- (NSArray *)availableSelectorSets
{
    // TODO
    // Go over all selector sets. If all of their parts and all of their dependencies are available, add the setId to an array. Return that array.
    NSMutableArray *result = [NSMutableArray array];
    NSArray *knownSets = [__pkgPlist objectForKey:@"selectorsets"];
    int i;

    for (i=0; i<[knownSets count]; i++) {
	NSDictionary *parts = [[knownSets objectAtIndex:i] objectForKey:@"parts"];
	NSDictionary *dependencies = [[knownSets objectAtIndex:i] objectForKey:@"dependencies"];
	NSMutableDictionary *filesToCheck = [NSMutableDictionary dictionary]; // Collect them first
	int j;
	    
	if ([self canBeFattened]) {
	    [result addObject:[[knownSets objectAtIndex:i] objectForKey:@"id"]];
	    continue;
	}
	
	if (parts && [parts count]) {
	    NSEnumerator *partEnumerator = [parts keyEnumerator];
	    NSString *partKey;
	    
	    while ((partKey = [partEnumerator nextObject]) != nil) {
		NSString *partFile;
		
		if ([partKey isEqualToString:@"0"]) {
		    partFile = ([self fileIsPartOfPackage:[__baseName stringByAppendingString:@".tar.bz2"]] ? [__baseName stringByAppendingString:@".tar.bz2"] : [__baseName stringByAppendingString:@".tar.gz"]);
		}
		else {
		    partFile = [self fileIsPartOfPackage:[__baseName stringByAppendingString:@".tar.bz2"]] ? [NSString stringWithFormat:@"%@.%@.tar.bz2", __baseName, partKey] : [NSString stringWithFormat:@"%@.%@.tar.gz", __baseName, partKey];
		}
		[filesToCheck setObject:@"YES" forKey:partFile];
	    }
	}
	
	if (dependencies && [dependencies count]) {
	    NSEnumerator *depEnumerator = [dependencies keyEnumerator];
	    NSString *depKey;
	    
	    while ((depKey = [depEnumerator nextObject]) != nil) {
		NSArray *files = [dependencies objectForKey:depKey];
		if (files && [files count]) {
		    int j;
		    
		    for (j=0; j<[files count]; j++) {
			[filesToCheck setObject:@"YES" forKey:[files objectAtIndex:j]];
		    }
		}
	    }
	}
	
	{
	    NSArray *fileArray = [filesToCheck allKeys];
	    for (j=0; j<[fileArray count]; j++) {
		if (![self fileIsAvailable:[fileArray objectAtIndex:j]]) {
		    break;
		}
	    }
	    if (j == [fileArray count]) {
		// All checked files are available
		[result addObject:[[knownSets objectAtIndex:i] objectForKey:@"id"]];
		continue;
	    }
	}
    }
    return result;
}

- (NSArray *)selectedSelectorSets
{
    NSArray *result = [__customPkgPlist objectForKey:@"selectedselectorsets"];
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: selectedSelectorSets");
    if (!result) {
	result = [__pkgPlist objectForKey:@"selectedselectorsets"];
    }
    return result;
}

- (NSArray *)configuredSelectorSets
{
    NSDictionary *dict = [__installedPkgPlist objectForKey:@"configuredselectorsets"];
    return [dict allKeys];
}

- (NSArray *)installedSelectorSets
{
    NSDictionary *dict = [__installedPkgPlist objectForKey:@"installedselectorsets"];
    return [dict allKeys];
}

- (NSString *)urlString
{
    NSString *stringFromDict = [__customPkgPlist objectForKey:@"default_url"];
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: urlString");
    if (!stringFromDict) {
	stringFromDict = [__pkgPlist objectForKey:@"default_url"];
    }
    return stringFromDict;
}

- (void)setURLString:(NSString *)newURLString
{
    NSURL *url = [NSURL URLWithString:newURLString];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: setURLString %@", newURLString);
    if (url) {
	[__customPkgPlist setObject:newURLString forKey:@"default_url"];
	[self writeCustomPlistToDisk];
	[__urlNameWithSeparator release];
	__urlNameWithSeparator = [[[self urlString] stringByAppendingString:@"/"] retain];
    }
}

- (NSString *)installLocation
{
    NSString *stringFromDict = [__customPkgPlist objectForKey:@"install_location"];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: installLocation");
    if (!stringFromDict) {
	stringFromDict = [__installedPkgPlist objectForKey:@"install_location"];
    }
    if (!stringFromDict) {
	stringFromDict = [__pkgPlist objectForKey:@"install_location"];
    }
    return [stringFromDict stringByStandardizingPath];
}

- (void)setInstallLocation: (NSString*)location
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: setInstallLocation %@", location);
    [__installedPkgPlist setObject:[[location stringByStandardizingPath] stringByAbbreviatingWithTildeInPath] forKey:@"install_location"];
    [__customPkgPlist setObject:[[location stringByStandardizingPath] stringByAbbreviatingWithTildeInPath] forKey:@"install_location"];
    [self writeCustomPlistToDisk];
}

- (BOOL)isInteractive
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: intInSelectedSets");
    return [[__pkgPlist objectForKey:@"interactive"] isEqualToString:@"YES"];
}

- (BOOL)needsResourceForksFixup
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: needsResourceForksFixup");
    return [[__pkgPlist objectForKey:@"needsresourceforkfixup"] isEqualToString:@"YES"];
}

- (BOOL)hasNonStandardFiles
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: hasNonStandardFiles");
    return (([__pkgPlist objectForKey:@"dependencies"] != nil) && ([[__pkgPlist objectForKey:@"dependencies"] count] != 0));
}

- (BOOL)isRelocatable
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: isRelocatable");
    return [[__pkgPlist objectForKey:@"relocatable"] isEqualToString:@"YES"];
}

- (BOOL)hasRemoveScript
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: hasRemoveScript");
    return [self fileIsPartOfPackage:[__baseName stringByAppendingString:@".remove"]];
}

- (BOOL)hasConfigureScript
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: hasConfigureScript");
    return [self fileIsPartOfPackage:[__baseName stringByAppendingString:@".configure"]];
}

- (BOOL)hasSelectorScript
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: hasSelectorScript");
    return [self fileIsPartOfPackage:[__baseName stringByAppendingString:@".selector"]];
}

- (BOOL)hasArchive
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: hasArchive");
    return [self fileIsPartOfPackage:[__baseName stringByAppendingString:@".tar.gz"]] || [self fileIsPartOfPackage:[__baseName stringByAppendingString:@".tar.bz2"]];
}


- (NSString *)baseName
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: baseName");
    return __baseName;
}

-(NSString *)fullPathForFileName:(NSString *)fileName
{
    NSString *tmp = [[__dirNameWithSeparator stringByAppendingString:fileName] stringByExpandingTildeInPath];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: fullPathForFileName %@", fileName);
    return tmp;
}

-(NSString *)fullURLPathForFileName:(NSString *)fileName
{
    NSString *tmp = [__urlNameWithSeparator stringByAppendingString:fileName];
    
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: fullURLPathForFileName %@", fileName);
    return tmp;
}

- (NSString *)fileNameForInspect:(NSString *)fileName withModifiers:(unsigned int)modifiers
{
    NSString *ext = [fileName pathExtension];
    NSString *useFileName = fileName;
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: fileNameForInspect %@ %u", fileName, modifiers);
    if (([fileName hasSuffix:@".tar.gz"] || [fileName hasSuffix:@".tar.bz2"]) && !(modifiers&NSAlternateKeyMask)) {
	NSString *archBaseName = [[fileName stringByDeletingPathExtension] stringByDeletingPathExtension];
	useFileName = [[archBaseName stringByAppendingString:@".l."] stringByAppendingString:ext];
	return useFileName;
    }
    if (([fileName hasSuffix:@".zip"]) && !(modifiers&NSAlternateKeyMask)) {
	NSString *archBaseName = [fileName stringByDeletingPathExtension];
	useFileName = [archBaseName stringByAppendingString:@".l.bz2"];
	return useFileName;
    }
    return useFileName;
}

-(NSMutableArray *)allocFilePartsArrayFromFileName:(NSString *)fileName filter:(BOOL)filter
{
    NSMutableArray *result = [[NSMutableArray alloc] init];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: allocFilePartsArrayFromFileName %@ %d", fileName, filter);
    if ([fileName hasSuffix:@".tar.gz"] || [fileName hasSuffix:@".tar.bz2"]) {
	NSString *compressExt;
	NSString *baseFileName;
	NSString *nextExt;
	NSRange range;

	if (!filter || [self intInSelectedSets: 0]) {
	    [result addObject:fileName];
	}
	
	compressExt = [fileName pathExtension];
	baseFileName = [[fileName stringByDeletingPathExtension] stringByDeletingPathExtension];
	nextExt = [baseFileName pathExtension];
	range = [nextExt rangeOfCharacterFromSet:[NSCharacterSet decimalDigitCharacterSet]];
	if ([nextExt isEqualToString:@""] || (range.length != [nextExt length])) {
	    // No more extensions or not a numerical extension (which is reserved for parts), 
	    if ([self fileIsPartOfPackage:[NSString stringWithFormat:@"%@.1.tar.%@", baseFileName, compressExt]]) {
		// This is a compressed fragmented archive, we are asking for the lead member
		unsigned int partCounter = 1;
		while (TRUE) {
		    NSString *partExt = [NSString stringWithFormat:@".%u.tar.%@", partCounter, compressExt];
		    if ([self fileIsPartOfPackage:[baseFileName stringByAppendingString:partExt]]) {
			if (!filter || [self intInSelectedSets: partCounter]) {
			    [result addObject:[baseFileName stringByAppendingString:partExt]];
			}
		    }
		    else {
			break;
		    }
		    partCounter++;
		}
		return result;
	    }
	}
    }
    else {
	[result addObject:fileName];
    }
    return result;
}

-(BOOL)fileIsAvailable:(NSString *)fileName
{
    NSString *tmp1 = [self fullPathForFileName:fileName];
    NSDictionary *tmp2 = [DEFFM fileAttributesAtPath:tmp1 traverseLink:YES];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: fileIsAvailable %@", fileName);
    if (tmp2 && [[tmp2 objectForKey:NSFileSize] isEqualTo:[NSNumber numberWithUnsignedInt:[self sizeOfFile:fileName]]]) {
	return YES;
    }
    return NO;
}

- (BOOL)fileSetIsAvailable:(NSString *)fileName
{
    int i;
    NSArray *partSet = [self allocFilePartsArrayFromFileName:fileName filter:NO];
    BOOL available = YES;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: fileSetIsAvailable %@", fileName);
    if (available) {
	for (i = 0; i < [partSet count]; i++) {
	    available = available && [self fileIsAvailable:[partSet objectAtIndex:i]];
	}
    }
    [partSet release];
    return available;
}

// Writable
- (BOOL)isWritable
{
    struct statfs statfsbuf;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: isWritable");
    if (!statfs([__dirName fileSystemRepresentation], &statfsbuf)) {
	if (statfsbuf.f_flags & MNT_RDONLY) {
	    return NO;
	}
	else {
	    struct stat statbuf;
	    if (!stat( [__dirName fileSystemRepresentation], &statbuf)) {
		uid_t fileOwner = statbuf.st_uid;
		mode_t fileMode = statbuf.st_mode;
		gid_t fileGroup = statbuf.st_gid;
		uid_t myUid = geteuid();
		gid_t myGid = getegid();
		if (myUid == fileOwner) {
		    return (fileMode & S_IWUSR) != 0;
		}
		else if (myGid == fileGroup) {
		    return (fileMode & S_IWGRP) != 0;
		}
		else {
		    return (fileMode & S_IWOTH) != 0;
		}
	    }
	    else {
		NSLog( @"Unable to get file status information on %s. Assuming \"not writable\".", [__dirName fileSystemRepresentation]);
		return NO;
	    }
	}
    }
    else {
	NSLog( @"Unable to get file system status information on %s. Assuming \"not writable\".", [__dirName fileSystemRepresentation]);
	return NO;
    }
}

    // Can be fattened: unlocked, network not shut down in preferences and ona writable filesystem
- (BOOL)canBeFattened
{
    BOOL isLocked = [self isLocked];
    BOOL networkAccessDisabled = [ii2Prefs networkAccessDisabled];
    BOOL isWritable = [self isWritable];
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: canBeFattened");
    return (!isLocked && !networkAccessDisabled && isWritable);
}

- (unsigned int)sizeOfFile:(NSString *)fileName
{
    unsigned int size;
    NSDictionary *fileDict = [__pkgPlist objectForKey:@"files"];
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: sizeOfFile %@", fileName);
    size = [[[fileDict objectForKey:fileName] objectForKey:@"size"] intValue];
    if (size == 0) size = 1; // Very Dirty Hack?? I forgot why I did this. Probably size=0 signifies someting else.
    return size;
}

- (unsigned int)sizeOnDisk
{
    NSEnumerator *enumerator = [__completeFileArray objectEnumerator];
    NSDictionary *fileDict;
    unsigned int totalSize = 0;
    NSString *plistName = [NSString stringWithFormat:@"%@%@.plist", __dirNameWithSeparator, __baseName];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: sizeOnDisk");
    while ((fileDict = [enumerator nextObject]) != nil) {
	NSString *fileName = [fileDict objectForKey:@"file"];
	if (![plistName isEqualToString:fileName] && [self fileIsAvailable:fileName]) {
	    totalSize += [self sizeOfFile:fileName];
	}
    }
    return totalSize;
}

- (BOOL)packageHasMd5ForFile:(NSString *)fileName
{
    NSDictionary *fileDict = [__pkgPlist objectForKey:@"files"];
    NSDictionary *file = [fileDict objectForKey:fileName];
    NSString *md5 = [file objectForKey:@"md5"];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: packageHasMd5ForFile %@", fileName);
    return (md5 != nil);
}

- (BOOL)packageHasMd5ForFileSet:(NSString *)fileName
{
    NSArray *partSet = [self allocFilePartsArrayFromFileName:fileName filter:NO];
    int i;
    BOOL has = YES;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: packageHasMd5ForFileSet %@", fileName);
    if (has) {
	for (i = 0; i < [partSet count]; i++) {
	    has = has && [self packageHasMd5ForFile:[partSet objectAtIndex:i]];
	}
    }
    [partSet release];
    return has;
}

- (BOOL)packageHasSigForFile:(NSString *)fileName
{
    NSDictionary *fileDict = [__pkgPlist objectForKey:@"files"];
    NSDictionary *file = [fileDict objectForKey:fileName];
    NSString *sig = [file objectForKey:@"sig"];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: packageHasSigForFile %@", fileName);
    return (sig != nil);
}

- (BOOL)packageHasSigForFileSet:(NSString *)fileName
{
    NSArray *partSet = [self allocFilePartsArrayFromFileName:fileName filter:NO];
    int i;
    BOOL has = YES;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: packageHasSigForFileSet %@", fileName);
    if (has) {
	for (i = 0; i < [partSet count]; i++) {
	    has = has && [self packageHasSigForFile:[partSet objectAtIndex:i]];
	}
    }
    [partSet release];
    return has;
}

-(BOOL)fileHasValidMd5:(NSString *)fileName
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: fileHasValidMd5 %@", fileName);
    return [[__md5Checks objectForKey:fileName] isEqualToString:@"YES"];
}

-(BOOL)fileSetHasValidMd5:(NSString *)fileName
{
    NSArray *partSet = [self allocFilePartsArrayFromFileName:fileName filter:NO];
    int i;
    BOOL has = YES;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: fileSetHasValidMd5 %@", fileName);
    if (has) {
	for (i = 0; i < [partSet count]; i++) {
	    has = has && [self fileHasValidMd5:[partSet objectAtIndex:i]];
	}
    }
    [partSet release];
    return has;
}

-(BOOL)fileHasValidSig:(NSString *)fileName
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: fileHasValidSig %@", fileName);
    return [[__sigChecks objectForKey:fileName] isEqualToString:@"YES"];
}

-(BOOL)fileSetHasValidSig:(NSString *)fileName
{
    NSArray *partSet = [self allocFilePartsArrayFromFileName:fileName filter:NO];
    int i;
    BOOL has = YES;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: fileSetHasValidSig %@", fileName);
    if (has) {
	for (i = 0; i < [partSet count]; i++) {
	    has = has && [self fileHasValidSig:[partSet objectAtIndex:i]];
	}
    }
    [partSet release];
    return has;
}

-(BOOL)fileIsPartOfPackage:(NSString *)fileName
{
    //NSLog( @"fileIsPartOfPackage");
    NSDictionary *fileDict = [__pkgPlist objectForKey:@"files"];
    BOOL exists = [fileDict objectForKey:fileName] != nil;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: fileIsPartOfPackage %@", fileName);
    return exists;
    //NSLog( @"fileIsPartOfPackage");
}

- (NSString *)stringFromFileContents:(NSString *)fileName
{
    NSString *path = [self fullPathForFileName:fileName];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: stringFromFileContents %@", fileName);
    if (path && [DEFFM fileExistsAtPath:path]){
	return [NSString stringWithContentsOfFile:path];
    }
    return nil;
}

- (NSData *)dataFromFileContents:(NSString *)fileName
{
    NSString *path = [self fullPathForFileName:fileName];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: dataFromFileContents %@", fileName);
    if (path && [DEFFM fileExistsAtPath:path]){
	return [NSData dataWithContentsOfFile:path];
    }
    return nil;
}

- (void)setEnvironmentValue:(NSString *)value forKey:(NSString *)key
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: setEnvironmentValue %@=%@", key, value);
    [__environment setObject:value forKey:key];
}

- (void)unsetEnvironmentValueforKey:(NSString *)key
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: unsetEnvironmentValueforKey %@", key);
    [__environment removeObjectForKey:key];
}

- (void)resetEnvironment
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: resetEnvironment");
    [__environment release];
    __environment = [[NSMutableDictionary alloc] init];
}

- (void)writeReceipt
{
    NSString *fullPath = [NSString stringWithFormat:@"%@/%@/%@.ii2receipt", [ii2Prefs ii2LibDir], RECEIPTSSUBDIR, __baseName];
    [__installedPkgPlist setObject:[__pkgPlist objectForKey:@"timestamp"] forKey:@"timestamp"];
    [__installedPkgPlist setObject:[NSString stringWithCString:getlogin()] forKey:@"writtenby"];
    [__installedPkgPlist setObject:[NSString stringWithFormat:@"%d", time( NULL)] forKey:@"writtenat"];
    [__installedPkgPlist setObject:[NSString stringWithString:__dirName] forKey:@"writtenfrom"];
    [__installedPkgPlist writeToFile:fullPath atomically:NO];
}

- (void)removeReceipt
{
    NSString *fullPath = [NSString stringWithFormat:@"%@/%@/%@.ii2receipt", [ii2Prefs ii2LibDir], RECEIPTSSUBDIR, __baseName];
    
    [DEFFM removeFileAtPath:fullPath handler:nil];
}

- (void)reportRCSIDInfo
{
    NSLog( @"%s\n", rcsidInfo);
}

@end

/*
 * $Log: II2PackageInfo.m,v $
 * Revision 1.5  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.4  2004/04/19 22:58:25  gctwnl
 * 2.61.0
 *
 * Revision 1.3  2004/03/21 16:52:21  gctwnl
 * 2.58.0
 *
 * Revision 1.2  2004/03/20 22:14:52  gctwnl
 * 2.57.0
 *
 * Revision 1.1.1.1  2004/02/01 17:18:26  gctwnl
 * Initial import.
 *
 * Revision 3.17  2004/01/30 15:24:23  gerben
 * 2.47.0
 *
 * Revision 3.16  2004/01/27 23:24:33  gerben
 * 2.46.0 (+bugfix)
 *
 * Revision 3.15  2004/01/27 23:10:19  gerben
 * 2.46.0
 *
 * Revision 3.14  2004/01/27 22:49:32  gerben
 * 2.45.0
 *
 * Revision 3.13  2004/01/23 00:31:16  gerben
 * 2.42.0 release
 *
 * Revision 3.12  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.11  2004/01/05 15:31:34  gerben
 * *** empty log message ***
 *
 * Revision 3.10  2003/11/20 11:56:06  gerben
 * 2.32.0 work in progress
 *
 * Revision 3.9  2003/11/09 21:43:06  gerben
 * 2.32.0
 *
 * Revision 3.8  2003/11/08 23:38:01  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.7  2003/09/18 22:41:29  gerben
 * 2.26.0 Fragmented archives
 *
 * Revision 3.6  2003/08/03 18:55:17  gerben
 * Added ZIP handling. Version 2.25.0.
 *
 * Revision 3.5  2003/04/02 23:20:47  gerben
 * Support for new style update protocol.
 *
 * Revision 3.4  2003/02/24 09:26:26  gerben
 * No log message.
 *
 * Revision 3.3  2003/02/23 23:29:11  gerben
 * Package Locking
 *
 * Revision 3.2  2002/12/10 14:24:53  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:23  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:52  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.45  2002/12/10 08:55:24  gerben
 * II2Package code cleanup, remove of cached view pointers. Added relocatable handling in package dependencies.
 *
 * Revision 1.44  2002/12/03 08:16:57  gerben
 * New style install stamps .foo.ii2
 *
 * Revision 1.43  2002/12/02 23:24:41  gerben
 * Basic dependency handling
 *
 * Revision 1.42  2002/11/30 10:05:54  gerben
 * Rearranged code for II2Package to use catagories instead of code inclusion.
 *
 * Revision 1.41  2002/11/24 23:09:27  gerben
 * Tom's changes and remarks mainly
 *
 * Revision 1.40  2002/11/07 21:03:31  gerben
 * Several small changes
 *
 * Revision 1.39  2002/11/04 11:54:11  gerben
 * i-Directory selection added to Known Packages
 *
 * Revision 1.38  2002/10/31 14:26:57  gerben
 * Moved authentication setting to package window
 *
 * Revision 1.37  2002/10/30 21:42:22  gerben
 * RC3a
 *
 * Revision 1.36  2002/10/28 21:49:45  gerben
 * A lot of changes wrt known packages.
 *
 * Revision 1.35  2002/10/27 15:39:43  gerben
 * Tried to remove BSD-package dependency
 *
 * Revision 1.34  2002/10/25 20:36:11  gerben
 * Layout changes.
 *
 * Revision 1.33  2002/10/24 20:12:32  gerben
 * Better handling of package name clashes and changed properties
 *
 * Revision 1.32  2002/10/22 14:14:19  gerben
 * Differ between different user actions.
 *
 * Revision 1.31  2002/10/21 20:16:53  gerben
 * Closing in on RC1 target
 *
 * Revision 1.30  2002/10/17 20:19:25  gerben
 * Changes wrt disabled network access
 *
 * Revision 1.29  2002/10/17 18:48:59  gerben
 * Cache/NoCache handling on downloads. Some other changes.
 *
 * Revision 1.28  2002/10/16 21:28:21  gerben
 * A complete new rewrite of taskss, processes and stuff.
 *
 * Revision 1.26  2002/10/15 14:31:43  gerben
 * Last fixes. No-network access. Later creation of downloaded file.
 *
 * Revision 1.25  2002/10/14 12:34:19  gerben
 * Removed dead code
 *
 * Revision 1.24  2002/10/14 12:15:37  gerben
 * Install stamp file after unarchive.
 *
 * Revision 1.23  2002/10/13 21:20:14  gerben
 * Fixed a serious bug where the task rewrite was a bit release-happy
 *
 * Revision 1.22  2002/10/10 00:51:24  gerben
 * Many changes. Committing a stable state.
 *
 * Revision 1.21  2002/10/09 14:16:11  gerben
 * Changed as many %s/lossyCString uses to %@/fileSystemRepresentation/UTF8String
 *
 * Revision 1.20  2002/10/09 10:53:40  gerben
 * Moved textview output to NSTexTView category.
 *
 * Revision 1.19  2002/10/09 08:38:42  gerben
 * Fixed two 10.1.5 incompatibilities (untested)
 *
 * Revision 1.18  2002/10/08 21:49:47  gerben
 * GPG sigs checked when preferences change.
 *
 * Revision 1.17  2002/10/08 21:34:28  gerben
 * Working GPG signature checks, shared GPG cache for II2.
 *
 * Revision 1.16  2002/10/07 20:28:05  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.15  2002/10/07 11:52:27  gerben
 * Several fixes related to possible invalid content.
 *
 * Revision 1.14  2002/10/03 13:38:03  gerben
 * Removed addChildWindow as child passes close to parent. Window can only close when there are no running tasks.
 *
 * Revision 1.13  2002/10/02 22:38:30  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */