//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2AppDelegate.h"
#import "II2Preferences.h"
#import "II2Package.h"

static char rcsidTableSource[] = "$Id: II2PackageTableSource.m,v 1.2 2004/04/25 13:13:21 gctwnl Exp $";

static NSComparisonResult fileSort(NSDictionary *file1, NSDictionary *file2, void *context)
{
    NSString *s1 = [file1 objectForKey:@"displayname"];
    NSString *s2 = [file2 objectForKey:@"displayname"];

    return [s1 caseInsensitiveCompare:s2];
}

@implementation II2Package (II2PackageTableSource)

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    if (aTableView == [__cachedWindowController fileTableView]) {
	return [__fileTableSource count];
    }
    return 0;
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
    if (aTableView == [__cachedWindowController fileTableView]) {
	if ((rowIndex >= 0) && (rowIndex < [__fileTableSource count])) {
	    id theRecord = [__fileTableSource objectAtIndex:rowIndex];
	    NSString *identifier = [aTableColumn identifier];
	    NSString *fileName = [self fileNameAtIndex:rowIndex];
	    if ([identifier isEqualToString:@"available"]) {
		if ([self fileSetIsAvailable:fileName]) {
		    return @"";
		}
		return @"";
	    }
	    if ([identifier isEqualToString:@"md5"]) {
		if ([self fileSetIsAvailable:fileName]) {
		    if ([self fileSetHasValidMd5:fileName]) {
			return @"";
		    }
		    return @"";
		}
		return @"?";

	    }
	    else if ([identifier isEqualToString:@"type"]) {
		return [self fileTypeDisplayString:fileName];
	    }
	    else if ([identifier isEqualToString:@"size"]) {
		if ([fileName hasPrefix:[__baseName stringByAppendingString:@".tar"]] && ([fileName hasSuffix:@".gz"] || [fileName hasSuffix:@".bz2"])) {
		    // The archive. Look for the parts
		    NSMutableArray *partSet = [self allocFilePartsArrayFromFileName:fileName filter:NO];
		    int i;
		    unsigned int totalSize=0;

		    for (i=0; i<[partSet count]; i++) {
			totalSize += [[[[__pkgPlist objectForKey:@"files"] objectForKey:[partSet objectAtIndex:i]] objectForKey:@"size"] intValue];
		    }
		    [partSet release];

		    return [NSNumber numberWithUnsignedInt:totalSize];
		}
		else {
		    return [theRecord objectForKey:identifier];
		}
	    }
	    else if ([identifier isEqualToString:@"sig"]) {
		if ([self fileSetIsAvailable:fileName]) {
		    BOOL result = [self fileSetHasValidSig:fileName];
		    if (result) {
			return @"";
		    }
		    return @"";
		}
		return @"?";
	    }
	    else {
		// fallback
		if ([identifier isEqualToString:@"displayname"]) {
		    id att = [theRecord objectForKey:@"attributeddisplayname"];
		    if (att) return att;
		}
		return [theRecord objectForKey:identifier];
	    }
	}
	return nil;
    }
    return @"! Programmer Error";
}

- (void)setupFileTableSourceAndInfo {
    if (__fileTableSource) {
	[__fileTableSource release];
	__fileTableSource = nil;
    }
    if (__completeFileArray) {
	[__completeFileArray release];
	__completeFileArray = nil;
    }
    if (__pkgPlist) {
	NSDictionary *fileDict = [__pkgPlist objectForKey:@"files"];
	NSEnumerator *enumerator = [fileDict keyEnumerator];
	NSDictionary *key;

	__fileTableSource = [[NSMutableArray arrayWithCapacity:[fileDict count]] retain];
	__completeFileArray = [[NSMutableArray arrayWithCapacity:[fileDict count]] retain];

	while (key = [enumerator nextObject]) {
	    NSMutableDictionary *fileItemCopy = [NSMutableDictionary dictionaryWithDictionary:[fileDict objectForKey:key]];
	    NSString *fileName = [fileItemCopy objectForKey:@"file"];
	    NSString *fileBaseName = [fileName stringByDeletingPathExtension];
	    NSString *ext = [fileName pathExtension];

	    [__completeFileArray addObject:fileItemCopy];

	    // These are hidden from the user
	    if (!([fileName isEqualToString:[__baseName stringByAppendingString:@".dependencies.plist"]] || [fileName hasSuffix:@".l.gz"] || [fileName hasSuffix:@".l.bz2"]|| [fileName isEqualToString:[__baseName stringByAppendingString:@".stamp"]])) {
		if ([fileName hasSuffix:@".tar.gz"] || [fileName hasSuffix:@".tar.bz2"]) {
		    if ([[[fileName stringByDeletingPathExtension] stringByDeletingPathExtension] isEqualToString:__baseName]) {
			// This automatically ignores the .1.tar.bz2, .2.tar.bz2 files
			[fileItemCopy setObject:@"i-Package Archive" forKey:@"displayname"];
			[__fileTableSource addObject:fileItemCopy];
		    }
		}
		else {
		    if ([fileBaseName isEqualToString:__baseName]) {
			if ([ext isEqualToString:@"rtf"]) {
			    [fileItemCopy setObject:@"i-Package README File" forKey:@"displayname"];
			}
			else if ([ext isEqualToString:@"prepare"]) {
			    [fileItemCopy setObject:@"i-Package Preparation Stage" forKey:@"displayname"];
			}
			else if ([ext isEqualToString:@"configure"]) {
			    [fileItemCopy setObject:@"i-Package Configuration Stage" forKey:@"displayname"];
			}
			else if ([ext isEqualToString:@"remove"]) {
			    [fileItemCopy setObject:@"i-Package Removal Stage" forKey:@"displayname"];
			}
			else if ([ext isEqualToString:@"selector"]) {
			    [fileItemCopy setObject:@"i-Package Selector Stage" forKey:@"displayname"];
			}
		    }
		    if (![fileItemCopy objectForKey:@"displayname"]) {
			[fileItemCopy setObject:fileName forKey:@"displayname"];
		    }
		    [__fileTableSource addObject:fileItemCopy];
		}
	    }
	}
	[__fileTableSource sortUsingFunction:fileSort context:nil];
    }
}

- (id)taskDataSource
{
    return __ii2RunningTaskSet;
}

- (void) reportRCSIDTableSource
{
    NSLog( @"%s\n", rcsidTableSource);
}

@end

/*
 * $Log: II2PackageTableSource.m,v $
 * Revision 1.2  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.1.1.1  2004/02/01 17:18:26  gctwnl
 * Initial import.
 *
 * Revision 3.6  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.5  2004/01/05 15:31:34  gerben
 * *** empty log message ***
 *
 * Revision 3.4  2003/09/18 22:40:54  gerben
 * 2.26.0 Fragmented archives
 *
 * Revision 3.3  2003/08/03 18:52:35  gerben
 * Added ZIP handling. Version 2.25.0.
 *
 * Revision 3.2  2002/12/10 14:24:53  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:24  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:52  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.11  2002/12/10 08:55:25  gerben
 * II2Package code cleanup, remove of cached view pointers. Added relocatable handling in package dependencies.
 *
 * Revision 1.10  2002/12/02 23:24:41  gerben
 * Basic dependency handling
 *
 * Revision 1.9  2002/11/30 10:05:54  gerben
 * Rearranged code for II2Package to use catagories instead of code inclusion.
 *
 * Revision 1.8  2002/10/29 09:36:22  gerben
 * RC2
 *
 * Revision 1.7  2002/10/24 12:49:05  gerben
 * Update refreshes file table. Download reports when remainder of (not total) download time changes significantly.
 *
 * Revision 1.6  2002/10/22 22:20:58  gerben
 * Small changes
 *
 * Revision 1.5  2002/10/07 20:28:05  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.4  2002/10/02 22:38:30  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */