//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2AppDelegate.h"
#import "II2Preferences.h"
#import "II2Package.h"
#import "II2DocumentController.h"
#import "GWInteractionManagerProtocols.h"

static char rcsidUserCommunication[] = "$Id: II2PackageUserCommunication.m,v 1.3 2004/04/25 13:13:21 gctwnl Exp $";

@implementation II2Package (II2PackageUserCommunication)

- (void)compareSheet:(II2TimestampComparisonResult)compare forAction:actionString withObject:(id)object
{
    NSMutableDictionary *sheetDict = [[[NSMutableDictionary alloc] init] autorelease];
    NSString *ignoreMessage = @".";
    NSString *updateMessage;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: compareSheet forAction: %@", actionString);
    [sheetDict setObject:self forKey:@"resulthandler"];
    [sheetDict setObject:NSStringFromSelector( @selector(endSheetWithDictionary:)) forKey:@"resulthandlerselector"];
    [sheetDict setObject:@"Alert" forKey:@"type"];
    [sheetDict setObject:@"Compare" forKey:@"subtype"];
    if (actionString) [sheetDict setObject:actionString forKey:@"action"];
    [sheetDict setObject:@"Cancel" forKey:@"okbutton"];
    if ([self canBeFattened]) {
	[sheetDict setObject:@"Update" forKey:@"cancelbutton"];
	updateMessage = @", press Update to update and proceed";
    }
    else {
	updateMessage = @" (updating is currently not possible: read-only package, locked package or network access turned off)";
    }
    if ([actionString isEqualToString:@"fatten"]) {
	NSArray *tasks = [self allocTasksForAction:@"fatten"]; // Only created temporarily for checking, then destroyed properly

	if (![self checkForDownloadsInTaskArray:tasks]) {
	    [sheetDict setObject:@"Check" forKey:@"otherbutton"];
	    ignoreMessage = @" or press Check to only peform the integrity checks on your current package contents.";
	}
	[self destroyTaskArray:tasks];
    }
    else if ([actionString isEqualToString:@"selectandinstall"] || [actionString isEqualToString:@"configure"] || [actionString isEqualToString:@"remove"]) {
	NSArray *tasks = nil;
	BOOL canIgnore = YES;
	
	if ([actionString isEqualToString:@"selectandinstall"]) {
	    if ([self hasSelectorScript]) {
		tasks = [self allocTasksForAction:@"selectandinstall"]; // Only created temporarily for checking, then destroyed properly
	    }
	    else {
		tasks = [self allocTasksForAction:@"trueinstall"]; // Only created temporarily for checking, then destroyed properly
		[sheetDict setObject:@"trueinstall" forKey:@"action"];
	    }
	}
	else {
	    tasks = [self allocTasksForAction:actionString]; // Only created temporarily for checking, then destroyed properly
	    [sheetDict setObject:actionString forKey:@"action"];
	}

	if (tasks) {
	    canIgnore = ![self checkForDownloadsInTaskArray:tasks];
	    [self destroyTaskArray:tasks];
	}
	else {
	    NSLog( @"Weird task allocation error in compareSheet");
	}
	
	if (canIgnore) {
	    [sheetDict setObject:@"Ignore" forKey:@"otherbutton"];
	    if ([actionString isEqualToString:@"selectandinstall"] && [self hasSelectorScript]) {
		ignoreMessage = @" or press Ignore to try to go ahead anyway (not recommended). Note that the current Ignore action only pertains to the availability of files for the selector phase. It is only after the selection has been made that i-Installer knows what is actually needed for an install.\n\nIf after the selection of what to install it turns out that all necessary files are available, installation will commence. If not, the Install will be aborted.";
	    }
	    else {
		ignoreMessage = @" or press Ignore to try to go ahead anyway (not recommended).";
	    }
	}
	else {
	    ignoreMessage = @" (ignoring the remote version and proceeding is not possible because not all parts for this actions are locally available).";
	}
    }
    else if ([actionString isEqualToString:@"inspect"]) {
	[sheetDict setObject:object forKey:@"filetoinspect"];
    }
    else {
	NSLog( @"Unexpected actionString: %@ in compareSheet:", actionString);
	return;
    }
    switch (compare) {
	case II2IsUnavailable:
	    [sheetDict setObject:@"Remote package is unavailable" forKey:@"title"];
	    [sheetDict setObject:[@"The remote package is unavailable (wrong URL or no network connection, maybe?). Press Cancel to cancel the current action" stringByAppendingString:ignoreMessage] forKey:@"info"];
	    [sheetDict removeObjectForKey:@"cancelbutton"];
	    break;
	case II2IsBeingUpdated:
	    [sheetDict setObject:@"Remote package is unavailable" forKey:@"title"];
	    [sheetDict setObject:@"The remote package is currently being updated. Please try again later." forKey:@"info"];
	    [sheetDict removeObjectForKey:@"cancelbutton"];
	    break;
	case II2OrderedAscending:
	    [sheetDict setObject:@"Remote package is more recent" forKey:@"title"];
	    [sheetDict setObject:[NSString stringWithFormat:@"The remote package is newer. Press Cancel to cancel the current action%@%@", updateMessage, ignoreMessage] forKey:@"info"];
	    break;
	case II2OrderedDescending:
	    [sheetDict setObject:@"Remote package is older(!)" forKey:@"title"];
	    [sheetDict setObject:[NSString stringWithFormat:@"The remote package is older (has the URL changed, maybe?).  Press Cancel to cancel the current action%@%@", updateMessage, ignoreMessage] forKey:@"info"];
	    break;
	default:
	    [sheetDict release];
	    NSLog( @"Program error: A compare sheet should never be called when versions are the same");
	    return;
    }

    if ([[__cachedWindowController window] isVisible]) {
	[[__cachedWindowController interactionManager] startInternalPanel:sheetDict];
    }
}

- (void)doUpdateCheckAndAction:(NSString *)actionString withObject:(id)object
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: doUpdateCheckAndAction %@", actionString);
    if (![ii2Prefs networkAccessDisabled]) {
	II2TimestampComparisonResult compare;
	[self readRemotePkgPlist:YES]; // Force read
	compare = [self remoteStatus];
	if (compare != NSOrderedSame) {
	    [self compareSheet:compare forAction:actionString withObject:object];
	    return;
	}
    }
    [self action:actionString];
}

- (void)unimplementedSheet
{
    NSMutableDictionary *sheetDict = [[[NSMutableDictionary alloc] init] autorelease];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: unimplementedSheet");
    [sheetDict setObject:self forKey:@"resulthandler"];
    [sheetDict setObject:NSStringFromSelector( @selector(endSheetWithDictionary:)) forKey:@"resulthandlerselector"];
    [sheetDict setObject:@"Alert" forKey:@"type"];
    [sheetDict setObject:@"Unimplemented" forKey:@"subtype"];
    [sheetDict setObject:@"Unimplemented" forKey:@"title"];
    [sheetDict setObject:@"This part of i-Installer v2 is not yet implemented." forKey:@"info"];
    [sheetDict setObject:@"Too bad" forKey:@"okbutton"];

    [[__cachedWindowController interactionManager] startInternalPanel:sheetDict];
}

- (void)progressSheet
{
    NSMutableDictionary *sheetDict = [[[NSMutableDictionary alloc] init] autorelease];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: progressSheet");
    [sheetDict setObject:self forKey:@"resulthandler"];
    [sheetDict setObject:NSStringFromSelector( @selector(endSheetWithDictionary:)) forKey:@"resulthandlerselector"];
    [sheetDict setObject:@"Progress" forKey:@"type"];
    [sheetDict setObject:@"testprogress" forKey:@"subtype"];
    [sheetDict setObject:@"Testing Progress" forKey:@"title"];
    [sheetDict setObject:@"What a hack!." forKey:@"info"];
    [sheetDict setObject:@"12320934" forKey:@"size"];
    [sheetDict setObject:@"Abort" forKey:@"abortbutton"];

    [[__cachedWindowController interactionManager] startInternalPanel:sheetDict];
}

- (void)endSheetWithDictionary: (NSDictionary *)dictionary
{
    NSNumber *tag = [dictionary objectForKey:@"resultButtonTag"];

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: endSheetWithDictionary %@", [dictionary description]);
    if ([[dictionary objectForKey:@"subtype"] isEqualToString:@"Unimplemented"]) {
	if ([ii2Prefs mainLogLevel] > 7) NSLog( @"Unimplemented. Tag = %d", [tag intValue]);
    }
    else if ([[dictionary objectForKey:@"subtype"] isEqualToString:@"Compare"]) {
	NSString *action = [dictionary objectForKey:@"action"];
	switch ([tag intValue]) {
	    NSArray *tasks;
	    case 1: // Cancel (okbutton)
		[__cachedWindowController autoButtonsEnabled];
		break;
	    case 0: // Update (cancelbutton)
		if ([self update]) {
		    [__cachedWindowController buttonsEnabled:NO];
		    if (action) {
			[[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Starting %@ tasks after update\n", action] attributes:[self infoAttributes]] autorelease]];
			if ([action isEqualToString:@"inspect"]) {
			    [self inspect:[dictionary objectForKey:@"filetoinspect"] withModifiers:0 inView:[__cachedWindowController inspectView]];
			}
			else if ([action isEqualToString:@"fatten"]) {
			    [self fatten];
			}
			else {
			    [self action:action];
			}
		    }
		    else {
			[__cachedWindowController autoButtonsEnabled];
		    }
		}
		else {
		    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"The update failed. Action %@ has been aborted.\n", action] attributes:[self signalAttributes]] autorelease]];
		}
		break;
	    case -1: // Ignore (otherbutton)
		if (action) {
		    // Check if complete for action (no downloads needed), if so, start action
		    NSString *realAction = ([action isEqualToString:@"dependencycheckandinstall"] ? @"trueinstall" : action);
		    tasks = [self allocTasksForAction:realAction];
		    if (tasks) {
			BOOL hasDownloads = [self checkForDownloadsInTaskArray:tasks];
			[self destroyTaskArray:tasks]; // Only built for check, will be rebuilt (could be improved, this)
			if (!hasDownloads) {
			    [self action:realAction];
			    return;
			}
			else {
			    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Cannot ignore. Action %@ requires downloads.\n", action] attributes:[self signalAttributes]] autorelease]];
			}
		    }
		}
		[__cachedWindowController autoButtonsEnabled];
		break;
	}
    }
    else if ([[dictionary objectForKey:@"subtype"] isEqualToString:@"RequiredOthers"]) {
	NSString *action = [dictionary objectForKey:@"action"];
	switch ([tag intValue]) {
	    case 1: // Cancel (okbutton)
		[__cachedWindowController autoButtonsEnabled];
		break;
	    case -1: // Ignore (otherbutton)
		if (action) {
		    [self action:([action isEqualToString:@"dependencycheckandinstall"] ? @"trueinstall" : action)];
		}
	}
    }
    else if ([[dictionary objectForKey:@"subtype"] isEqualToString:@"ThinBeforeClose"]) {
	if ([tag intValue] == 1) {
	    [self thin];
	}
	[[__cachedWindowController window] close];
    }
}

- (NSString *)urlStringForNewPackageName:(NSString *)newPkgBaseName
{
    NSString *newPkgNeighbourPath = [[__dirName stringByDeletingLastPathComponent] stringByAppendingPathComponent:[newPkgBaseName stringByAppendingString:@".ii2"]];
    NSString *newPkgDefaultPath = [[ii2Prefs defaultSaveDir] stringByAppendingPathComponent:[newPkgBaseName stringByAppendingString:@".ii2"]];
    NSURL *pkgURL = [NSURL URLWithString:[self urlString]];
    NSURL *newPkgURL = [[NSURL URLWithString:[NSString stringWithFormat:@"%@.ii2", newPkgBaseName] relativeToURL:pkgURL] standardizedURL];
    BOOL isDir;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: urlStringForNewPackageName");
    // First look in same directory as current package on disk
    if ([DEFFM fileExistsAtPath:newPkgNeighbourPath isDirectory:&isDir]) {
	if (isDir) {
	    return [[NSURL fileURLWithPath:newPkgNeighbourPath] absoluteString];
	}
    }
    // Then look in default save directory
    if ([DEFFM fileExistsAtPath:newPkgDefaultPath isDirectory:&isDir]) {
	if (isDir) {
	    return [[NSURL fileURLWithPath:newPkgDefaultPath] absoluteString];
	}
    }

    // Not on disk, return URL
    if (![ii2Prefs networkAccessDisabled]) {
	return [newPkgURL absoluteString];
    }
    else {
	if ([ii2Prefs mainLogLevel] > 4) NSLog( @"Will not return remote URL for %@ when network access is turned off", newPkgBaseName);
    }
    return nil;
}

- (BOOL)aPackage:(NSString *)pkgBaseName isInstalledInLocation:(NSString *)location
{
    NSString *stampFileName = [NSString stringWithFormat:@"%s/.%s.ii2", [location fileSystemRepresentation], [pkgBaseName fileSystemRepresentation]];
    NSString *oldStampFileName = [NSString stringWithFormat:@"%s/.%s", [location fileSystemRepresentation], [pkgBaseName fileSystemRepresentation]];
    BOOL isDir = YES;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: aPackage:isInstalledInLocation");
    // NSLog( @"Checking %@", stampFileName);
    return ([DEFFM fileExistsAtPath:stampFileName isDirectory:&isDir] && !isDir) || ([DEFFM fileExistsAtPath:oldStampFileName isDirectory:&isDir] && !isDir);
}

- (BOOL)reportPkgDependencies
{
    BOOL result = YES;
    NSArray *deps = [__pkgPlist objectForKey:@"otherfiledependencies"];
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: reportPkgDependencies %@", [deps description]);
    if ([ii2Prefs dependencyCheckType] == ii2DependencyCheckNone) {
	return YES;
    }
    if (deps) {
	int i, j;
	for (i=0; i<[deps count]; i++) {
	    NSMutableDictionary *sheetDict = [[[NSMutableDictionary alloc] init] autorelease];
	    NSMutableArray *availableFiles = [[[NSMutableArray alloc] init] autorelease];
	    NSMutableArray *unavailableFiles = [[[NSMutableArray alloc] init] autorelease];
	    NSDictionary *set = [[deps objectAtIndex:i] objectForKey:@"set"];
	    NSEnumerator *setEnumerator;
	    NSString *setKey;
	    NSString *include = [[deps objectAtIndex:i] objectForKey:@"include"];
	    NSString *optionalString = [[deps objectAtIndex:i] objectForKey:@"optional"];
	    NSArray *selectorSets = [[deps objectAtIndex:i] objectForKey:@"selectorsets"];
	    BOOL optional = [optionalString isEqualToString:@"YES"];
	    NSString *reason = [[deps objectAtIndex:i] objectForKey:@"reason"];

	    if (!set) {
		NSLog( @"Broken dependency. Set %d is missing.", i);
		return YES;
	    }
	    if (!include) {
		NSLog( @"Broken dependency. Include mode %d is missing.", i);
		return YES;
	    }
	    if (!optionalString) {
		NSLog( @"Broken dependency. Optional mode %d is missing.", i);
		return YES;
	    }
	    if (!reason) {
		NSLog( @"Broken dependency. Reason %d is missing.", i);
		return YES;
	    }
	    if (([ii2Prefs dependencyCheckType] != ii2DependencyCheckAll) && [optionalString isEqualToString:@"YES"]) {
		// It is optional (recommended) and we have not set checking to all
		continue;
	    }
	    if (selectorSets) {
		int k;
		for (k=0; k<[selectorSets count]; k++) {
		    NSString *setId = [selectorSets objectAtIndex:k];
		    if ([self setIdInSelectedSets:setId]) {
			break;
		    }
		}
		if (k == [selectorSets count]) {
		    // We did not find a selector set in the otherfiledependencies that is currently selected. Hence, this dependency is not valid for us now and we are going to skip it.
		    continue; // next i
		}
	    }

	    [sheetDict setObject:self forKey:@"resulthandler"];
	    [sheetDict setObject:NSStringFromSelector( @selector(endSheetWithDictionaryForDependency:)) forKey:@"resulthandlerselector"];
	    [sheetDict setObject:@"Alert" forKey:@"type"];
	    [sheetDict setObject:include forKey:@"subtype"];
	    [sheetDict setObject:@"Cancel" forKey:@"cancelbutton"];
	    
	    setEnumerator = [set keyEnumerator];
	    while ((setKey = [setEnumerator nextObject]) != nil) {
		NSDictionary *setItem = [set objectForKey:setKey];
		NSString *file = [setItem objectForKey:@"file"];
		NSString *iPackage = [setItem objectForKey:@"ipackage"]; // If there is an i-Package for this file, otherwise nil
		BOOL isDir;
		if (!file) {
		    NSLog( @"Broken dependency. Set %@ is missing file info.", setKey);
		    return YES;
		}
		if ([DEFFM fileExistsAtPath:file isDirectory:&isDir]) {
		    [availableFiles addObject:[NSArray arrayWithObjects: file, iPackage, nil]];
		}
		else {
		    [unavailableFiles addObject:[NSArray arrayWithObjects: file, iPackage, nil]];
		}
	    }
	    
	    // We now have two arrays: the arry with available files and the array with unavailable files. Now we look at the include mode to see what we need to do.
	    if ([include isEqualToString:@"AND"]) {
		// All files need to be available
		[[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Checking (%@) dependency set %d (all should be available). Reason: %@\n", (optional ? @"recommended" : @"required"), i, reason] attributes:[self infoAttributes]] autorelease]];
		for (j=0; j<[availableFiles count]; j++) {
		    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"## %@ file %@ is available. Good.\n", (optional ? @"Recommended " : @"Required "), [[availableFiles objectAtIndex:j] objectAtIndex:0]] attributes:[self infoAttributes]] autorelease]];
		}
		if ([unavailableFiles count]) {
		    NSMutableArray *packagesToOpen = [[[NSMutableArray alloc] init] autorelease];
		    NSMutableArray *packagesToOpenShortNames = [[[NSMutableArray alloc] init] autorelease];

		    for (j=0; j<[unavailableFiles count]; j++) {
			NSString *newPkgURLString;
			[[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"## %@ file %@ is unavailable.\n", (optional ? @"Recommended " : @"Required "), [[unavailableFiles objectAtIndex:j] objectAtIndex:0]] attributes:[self signalAttributes]] autorelease]];
			if ([[unavailableFiles objectAtIndex:j] count] == 2) {
			    // There was an ipackage link for this missing file
			    newPkgURLString = [self urlStringForNewPackageName:[[unavailableFiles objectAtIndex:j] objectAtIndex:1]];
			    if (newPkgURLString) {
				[packagesToOpen addObject:newPkgURLString];
				[packagesToOpenShortNames addObject:[[unavailableFiles objectAtIndex:j] objectAtIndex:1]];
			    }
			    else {
				[[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Unable to create URL from missing package name.\n"] attributes:[self signalAttributes]] autorelease]];
			    }
			}
			
		    }
		    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Dependency requirements %d not fulfilled as files are missing. The reason for this dependency is \"%@\".\n", i, reason] attributes:[self signalAttributes]] autorelease]];
		    result = result && optional;
		    
		    [sheetDict setObject:@"Package requires other files" forKey:@"title"];
		    if ([packagesToOpen count]) {
			[sheetDict setObject:[NSString stringWithFormat:@"Files required for this package to work are missing (see Message View). The install of the following packages is %@: %@. Do you want to open them so you can install them?", (optional ? @"recommended" : @"required"), [packagesToOpenShortNames componentsJoinedByString:@", "]] forKey:@"info"];
			[sheetDict setObject:@"Open" forKey:@"okbutton"];
			[sheetDict setObject:packagesToOpen forKey:@"openpackages"];
		    }
		    else {
			[sheetDict setObject:[NSString stringWithFormat:@"Files required for this package to work are missing. There are no packages that I know of that contain the missing files. Sorry."] forKey:@"info"];
			[sheetDict setObject:@"Accept" forKey:@"okbutton"];
			[sheetDict removeObjectForKey:@"cancelbutton"];
		    }
		    [[__cachedWindowController interactionManager] startInternalPanel:sheetDict];
		}
		else {
		    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Dependency requirements fulfilled.\n"] attributes:[self infoAttributes]] autorelease]];
		}
	    }
	    else if ([include isEqualToString:@"XOR"]) {
		// Only one should be available
		BOOL fulfilled = ([availableFiles count] == 1);
		if (!fulfilled) {
		    [sheetDict setObject:@"Package needs other package actions" forKey:@"title"];
		    [sheetDict setObject:@"Choice" forKey:@"type"];
		    [sheetDict setObject:@"Select" forKey:@"okbutton"];
		    [sheetDict setObject:@"None" forKey:@"cancelbutton"];
		}
		[[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Checking (%@) dependency set %d (only one should be available). Reason: %@.\n", (optional ? @"recommended" : @"required"), i, reason] attributes:[self infoAttributes]] autorelease]];
		for (j=0; j<[unavailableFiles count]; j++) {
		    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"## File %@ is not available.\n", [[unavailableFiles objectAtIndex:j] objectAtIndex:0]] attributes:(fulfilled ? [self infoAttributes] : [self signalAttributes])] autorelease]];
		}
		for (j=0; j<[availableFiles count]; j++) {
		    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"## File %@ is available.\n", [[availableFiles objectAtIndex:j] objectAtIndex:0]] attributes:(fulfilled ? [self infoAttributes] : [self signalAttributes])] autorelease]];
		}
		if (fulfilled) {
		    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Dependency requirements fulfilled.\n"] attributes:[self infoAttributes]] autorelease]];
		}
		else {
		    NSMutableArray *packagesToOpen = [[[NSMutableArray alloc] init] autorelease];
		    NSMutableArray *packagesToOpenShortNames = [[[NSMutableArray alloc] init] autorelease];
		    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Dependency requirements not fulfilled as one and only one of the checked files should be present.\n"] attributes:[self signalAttributes]] autorelease]];
		    result = result && optional;

		    for (j=0; j<[unavailableFiles count]; j++) {
			NSString *newPkgURLString;
			if ([[unavailableFiles objectAtIndex:j] count] == 2) {
			    // There was an ipackage link for this missing file
			    newPkgURLString = [self urlStringForNewPackageName:[[unavailableFiles objectAtIndex:j] objectAtIndex:1]];
			    if (newPkgURLString) {
				[packagesToOpen addObject:newPkgURLString];
				[packagesToOpenShortNames addObject:[[unavailableFiles objectAtIndex:j] objectAtIndex:1]];
			    }
			    else {
				[[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Unable to create URL from missing package name.\n"] attributes:[self signalAttributes]] autorelease]];
			    }
			}
		    }
		    for (j=0; j<[availableFiles count]; j++) {
			NSString *newPkgURLString;
			if ([[availableFiles objectAtIndex:j] count] == 2) {
			    // There was an ipackage link for this missing file
			    newPkgURLString = [self urlStringForNewPackageName:[[availableFiles objectAtIndex:j] objectAtIndex:1]];
			    if (newPkgURLString) {
				[packagesToOpen addObject:newPkgURLString];
				[packagesToOpenShortNames addObject:[[availableFiles objectAtIndex:j] objectAtIndex:1]];
			    }
			    else {
				[[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Unable to create URL from missing package name.\n"] attributes:[self signalAttributes]] autorelease]];
			    }
			}
		    }
		    if ([packagesToOpen count]) {
			if ([availableFiles count] > 1) {
			    // One of the installed packages should be removed
			    [sheetDict setObject:packagesToOpen forKey:@"openpackages"];
			    if ([packagesToOpen count] > 1) {
				[sheetDict setObject:[NSString stringWithFormat:@"Of the found dependency files just one may be %@installed (see Message View). Which package should be opened for removal?", (optional ? @"optionally " : @"")] forKey:@"info"];
				if (optional) [sheetDict setObject:@"All" forKey:@"otherbutton"];
				[sheetDict setObject:packagesToOpenShortNames forKey:@"choices"];
			    }
			    else {
				[sheetDict setObject:@"Alert" forKey:@"type"];
				[sheetDict setObject:@"Open" forKey:@"okbutton"];
				[sheetDict setObject:@"Cancel" forKey:@"cancelbutton"];
				[sheetDict setObject:[NSString stringWithFormat:@"Of the found dependency files just one may be %@installed (see Message View). Open %@ package for removal?", (optional ? @"optionally " : @""), [packagesToOpenShortNames componentsJoinedByString:@", "]] forKey:@"info"];
			    }
			}
			else if ([availableFiles count] == 0) {
			    // This implies none installed. Choice needed
			    [sheetDict setObject:packagesToOpen forKey:@"openpackages"];
			    if ([packagesToOpen count] > 1) {
				[sheetDict setObject:[NSString stringWithFormat: @"It is %@ that one of the missing dependency files must be installed. Which package should be opened for install?", (optional ? @"recommended" : @"required")] forKey:@"info"];
				[sheetDict setObject:packagesToOpenShortNames forKey:@"choices"];
			    }
			    else {
				[sheetDict setObject:@"Alert" forKey:@"type"];
				[sheetDict setObject:@"Open" forKey:@"okbutton"];
				[sheetDict setObject:@"Cancel" forKey:@"cancelbutton"];
				[sheetDict setObject:[NSString stringWithFormat: @"It is %@ that one of the missing dependency files must be installed. Open %@ package for install?", (optional ? @"recommended" : @"required"), [packagesToOpenShortNames componentsJoinedByString:@", "]] forKey:@"info"];
			    }
			}
			[[__cachedWindowController interactionManager] startInternalPanel:sheetDict];
		    }
		}
	    }
	    else if ([include isEqualToString:@"NOT"]) {
		// No file should be available
		[[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Checking (%@) dependency set %d (none should be available). Reason: %@.\n", (optional ? @"recommended" : @"required"), i, reason] attributes:[self infoAttributes]] autorelease]];
		for (j=0; j<[unavailableFiles count]; j++) {
		    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"## %@ unavailable file %@ is not available. Good.\n", (optional ? @"Recommended " : @"Required "), [[unavailableFiles objectAtIndex:j] objectAtIndex:0]] attributes:[self infoAttributes]] autorelease]];
		}
		if ([availableFiles count]) {
		    NSMutableArray *packagesToOpen = [[[NSMutableArray alloc] init] autorelease];
		    NSMutableArray *packagesToOpenShortNames = [[[NSMutableArray alloc] init] autorelease];

		    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"## %@ unavailable file %@ is available.\n", (optional ? @"Recommended " : @"Required "), [[availableFiles objectAtIndex:j] objectAtIndex:0]] attributes:[self signalAttributes]] autorelease]];
		    for (j=0; j<[availableFiles count]; j++) {
			NSString *newPkgURLString;
			if ([[availableFiles objectAtIndex:j] count] == 2) {
			    // There was an ipackage link for this missing file
			    newPkgURLString = [self urlStringForNewPackageName:[[availableFiles objectAtIndex:j] objectAtIndex:1]];
			    if (newPkgURLString) {
				[packagesToOpen addObject:newPkgURLString];
				[packagesToOpenShortNames addObject:[[unavailableFiles objectAtIndex:j] objectAtIndex:1]];
			    }
			    else {
				[[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Unable to create URL from missing package name.\n"] attributes:[self signalAttributes]] autorelease]];
			    }
			}
		    }
		    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Dependency requirements not fulfilled as files that should be unavailable are present.\n"] attributes:[self signalAttributes]] autorelease]];
		    result = result && optional;
		    [sheetDict setObject:@"Package requires uninstall(s)" forKey:@"title"];
		    if ([packagesToOpen count]) {
			[sheetDict setObject:[NSString stringWithFormat:@"Files required absent for this package to work are available. The uninstall of the following packages is %@: %@. Do you want to open them so you can uninstall them?", (optional ? @"recommended" : @"required"), [packagesToOpenShortNames componentsJoinedByString:@", "]] forKey:@"info"];
			[sheetDict setObject:@"Open" forKey:@"okbutton"];
			[sheetDict setObject:packagesToOpen forKey:@"openpackages"];
		    }
		    else {
			[sheetDict setObject:[NSString stringWithFormat:@"Files required absent for this package to work are available (see Messsage View). There are no packages that I know of that can uninstall these files. Sorry."] forKey:@"info"];
			[sheetDict setObject:@"Accept" forKey:@"okbutton"];
			[sheetDict removeObjectForKey:@"cancelbutton"];
		    }
		    [[__cachedWindowController interactionManager] startInternalPanel:sheetDict];
		}
		else {
		    [[__cachedWindowController messageView] appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Dependency requirements fulfilled.\n"] attributes:[self infoAttributes]] autorelease]];
		}
	    }
	}
    }
    return result;
}

- (void)__openWithURLString:(NSString *)urlString
{
    II2DocumentController *docController = (II2DocumentController *)[NSDocumentController sharedDocumentController];
    NSURL *url = [NSURL URLWithString:urlString]; // TODO = nil when file://???
   
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: __openWithURLString");
    if (url) {
        if ((floor( NSAppKitVersionNumber) <= NSAppKitVersionNumber10_1)  && ([[url scheme] isEqualToString:@"ftp"])) {
		   NSRunAlertPanel(@"Error", @"FTP not supported by Apple Frameworks in Mac OS X Versions below 10.2", @"Use other scheme", nil, nil, nil);
	   return;
	}
	[docController openDocumentWithContentsOfURL:url display:YES];
    }
}

- (void)endSheetWithDictionaryForDependency: (NSDictionary *)dictionary
{
    NSString *subType = [dictionary objectForKey:@"subtype"];
    NSNumber *resultTag = [dictionary objectForKey:@"resultButtonTag"];
    NSArray *openPackages = [dictionary objectForKey:@"openpackages"];
    int index;

    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"II2P: endSheetWithDictionaryForDependency");
    if ((([resultTag intValue] == 1) && !([subType isEqualToString:@"XOR"])) || (([resultTag intValue] == -1) && ([subType isEqualToString:@"XOR"]))) {
	// Open all in array
	for (index = 0; index < [openPackages count]; index++) {
	    NSString *urlString = [openPackages objectAtIndex:index];
	    [self __openWithURLString:urlString];
	}
    }
    else if (([resultTag intValue] == 1) && ([subType isEqualToString:@"XOR"])) {
	// Open single one
	index = [[dictionary objectForKey:@"choiceItemIndex"] intValue];
	NSString *urlString = [openPackages objectAtIndex:index];
	[self __openWithURLString:urlString];
    }
}


- (void) reportRCSIDUserCommunication
{
    NSLog( @"%s\n", rcsidUserCommunication);
}

@end

/*
 * $Log: II2PackageUserCommunication.m,v $
 * Revision 1.3  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.2  2004/02/27 23:24:27  gctwnl
 * 2.52.0 entry after a long period of CVS problems
 *
 * Revision 1.1.1.1  2004/02/01 17:18:26  gctwnl
 * Initial import.
 *
 * Revision 3.14  2004/01/31 13:21:58  gerben
 * 2.48.0
 *
 * Revision 3.13  2004/01/27 22:49:32  gerben
 * 2.45.0
 *
 * Revision 3.12  2004/01/27 12:57:39  gerben
 * 2.44.0
 *
 * Revision 3.11  2004/01/23 00:31:16  gerben
 * 2.42.0 release
 *
 * Revision 3.10  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.9  2004/01/05 15:31:34  gerben
 * *** empty log message ***
 *
 * Revision 3.8  2003/11/08 23:38:01  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.7  2003/02/24 10:10:34  gerben
 * Ignore dependency when there is a problem locating related packages.
 *
 * Revision 3.6  2003/02/24 09:44:25  gerben
 * 2.8.0. Some changes in comparing with remote versions, locking, proceeding, etc.
 *
 * Revision 3.5  2003/02/10 16:03:35  gerben
 * 2.6.0 Fixed horrible crasher bug introduced when fattening.
 *
 * Revision 3.4  2003/01/04 02:04:43  gerben
 * Version 2.2.0
 *
 * Revision 3.3  2002/12/16 13:13:38  gerben
 * Bug fix: The install directory was not created when one chose to install while ignoring the remote availability of another version.
 *
 * Revision 3.2  2002/12/10 14:24:53  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:24  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:52  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.25  2002/12/10 08:55:24  gerben
 * II2Package code cleanup, remove of cached view pointers. Added relocatable handling in package dependencies.
 *
 * Revision 1.24  2002/12/03 20:57:39  gerben
 * A preference for automatic dependency checking
 *
 * Revision 1.23  2002/12/03 08:16:57  gerben
 * New style install stamps .foo.ii2
 *
 * Revision 1.22  2002/12/02 23:24:40  gerben
 * Basic dependency handling
 *
 * Revision 1.21  2002/11/30 10:05:54  gerben
 * Rearranged code for II2Package to use catagories instead of code inclusion.
 *
 * Revision 1.20  2002/11/07 21:03:31  gerben
 * Several small changes
 *
 * Revision 1.19  2002/10/27 12:38:34  gerben
 * Lazy update checking. Update followed by action for inspect and fatten.
 *
 * Revision 1.18  2002/10/25 21:48:57  gerben
 * Message instead of NSLog
 *
 * Revision 1.17  2002/10/25 19:52:35  gerben
 * Changes to update reporting.
 *
 * Revision 1.16  2002/10/21 20:16:52  gerben
 * Closing in on RC1 target
 *
 * Revision 1.15  2002/10/16 21:28:21  gerben
 * A complete new rewrite of taskss, processes and stuff.
 *
 * Revision 1.14  2002/10/16 08:37:18  gerben
 * Changed log levels. Small fixes.
 *
 * Revision 1.13  2002/10/15 14:31:43  gerben
 * Last fixes. No-network access. Later creation of downloaded file.
 *
 * Revision 1.12  2002/10/15 10:31:18  gerben
 * A task array was released while actions were already running. Reason: we started the tasks without authenticating first, so authentication came when we were already started. TODO: this can happen too when authorizations expire!
 *
 * Revision 1.11  2002/10/13 17:20:40  gerben
 * Typo fix
 *
 * Revision 1.10  2002/10/13 13:57:47  gerben
 * Implemented a table with checkboxes. The changes for this were largely copied/imitated from the OpenUp project by Scott Anguish
 *
 * Revision 1.9  2002/10/13 10:57:22  gerben
 * Saving stable state before some additions and rewrites
 *
 * Revision 1.8  2002/10/12 10:54:38  gerben
 * A lot of changes, mostly when producing output/feedback.
 *
 * Revision 1.7  2002/10/10 09:08:47  gerben
 * Fixed bug where internal panel dictionaries were released once too many.
 *
 * Revision 1.6  2002/10/08 13:20:29  gerben
 * MD5 check on update. Small changes.
 *
 * Revision 1.5  2002/10/07 20:28:05  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.4  2002/10/07 11:52:27  gerben
 * Several fixes related to possible invalid content.
 *
 * Revision 1.3  2002/10/02 22:38:30  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */