//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import <Cocoa/Cocoa.h>

// Make sure it is the same name everywhere
#define RECEIPTSSUBDIR @"Receipts"

// Combined window and preferences table. Can't be helped.
typedef enum {
    ii2ReadmeTabTag = 0,
    ii2PropertiesTabTag,
    ii2OutputTabTag,
    ii2ContentsTabTag,
    ii2ActivityTabTag,
    ii2ReportTabTag,
    ii2NrOfTabs,
    ii2BasicTabsTag = 10,
    ii2ExpertTabsTag
} II2TabTagType;

typedef enum {
    ii2DependencyCheckRequired = 0,
    ii2DependencyCheckAll,
    ii2DependencyCheckNone,
    nrOfII2DependencyChecks
} II2DependencyCheckType;

extern id ii2TagNameArray[ii2NrOfTabs];

@interface II2Preferences : NSObject {
    IBOutlet NSPanel			*preferencesPanel;
    IBOutlet NSTextField		*logLevelTextField;
    IBOutlet NSTextField		*thinLevelTextField;
    IBOutlet NSTextField		*maxII2DDescendLevelTextField;
    IBOutlet NSTextField		*saveDirTextField;
    IBOutlet NSTextField		*gpgKeyServerTextField;
    IBOutlet NSTextField		*defaultII2DirectoryTextField;
    IBOutlet NSButton			*noUniqueII2DirectoryNamesButton;
    IBOutlet NSButton			*autoDownloadButton;
    IBOutlet NSButton			*keepInvalidMd5Button;
    IBOutlet NSButton			*showSplashAtStartupButton;
    IBOutlet NSButton			*askForThinButton;
    IBOutlet NSButton			*useGPGButton;
    IBOutlet NSButton			*cacheDisabledButton;
    IBOutlet NSButton			*networkAccessDisabledButton;
    IBOutlet NSButton			*fastNetworkButton;
    IBOutlet NSMatrix			*dependencyMatrix;
    IBOutlet NSPopUpButton		*cronDayPopUpButton;
    IBOutlet NSTextField		*cronHourTextField;
    IBOutlet NSTextField		*cronMinutesTextField;
    IBOutlet NSButton			*cronEnableButton;

    int					__mainLogLevel;
    // 0 = Program error, serious errors, damage
    // 1+ = errors the user should know but can live without
    // 2+ = warnings which are interesting for the user, details of the previous level
    // 4+ = file handling, url handling, details of the previous level
    // 5+ = task handling, process handling, details of the previous level
    // 7+ = details of any of the previous levels
    // 8+ Program flow
    // 9+ Cocoa debug messages
    
    double				__thinSize;
    BOOL				__autoFatten;
    BOOL				__keepInvalidMd5;
    BOOL				__showSplashAtStartup;
    BOOL				__askForThinAtClose;
    BOOL				__useGPG;
    BOOL				__cacheDisabled;
    BOOL				__networkAccessDisabled;
    BOOL				__fastNetwork;
    II2DependencyCheckType		__dependencyCheckType;
    BOOL				__useCron;
    int					__cronDay;
    int					__cronHour;
    int					__cronMinutes;
    NSString				*__defaultSaveDir;
    NSString				*__gpgKeyServer;
    NSString				*__defaultII2DDirectory;
    int					__maxII2DDescendLevel;
    BOOL				__noUniqueII2DirectoryNames;
    
    BOOL                                __hideContentsTab;
    BOOL                                __hideReportTab;
    BOOL                                __hideActivityTab;
    BOOL                                __hideOutputTab;
    BOOL                                __hidePropertiesTab;
    IBOutlet NSButton			*showContentsTabButton;
    IBOutlet NSButton			*showReportTabButton;
    IBOutlet NSButton			*showActivityTabButton;
    IBOutlet NSButton			*showOutputTabButton;
    IBOutlet NSButton			*showPropertiesTabButton;
    BOOL				__dynamicDisplayOfTabs;
    IBOutlet NSButton			*dynamicDisplayOfTabsButton;

    // Defaults without a UI interface
    NSString				*__ii2LibDir;
}

+ (id)sharedInstance;

- (id)init;
- (void)readFromDefaults:(NSUserDefaults *)defaults;

- (IBAction)applyButtonPressed:(id)sender;
- (IBAction)cancelButtonPressed:(id)sender;
- (IBAction)defaultsButtonPressed:(id)sender;
- (IBAction)setSaveDirButtonPressed:(id)sender;
- (IBAction)networkButtonClicked:(id)sender;
- (IBAction)cronButtonClicked:(id)sender;
- (IBAction)basicViewButtonClicked:(id)sender;
- (IBAction)expertViewButtonClicked:(id)sender;

- (void)resetToDefaults;
- (void)showPreferences;

- (int)mainLogLevel;
- (unsigned int)thinSize;
- (int)maxII2DDescendLevel;
- (BOOL)autoFatten;
- (BOOL)noUniqueII2DirectoryNames;
- (BOOL)keepInvalidMd5;
- (BOOL)showSplashAtStartup;
- (BOOL)askForThinAtClose;
- (BOOL)useGPG;
- (BOOL)dynamicDisplayOfTabs;
- (II2DependencyCheckType)dependencyCheckType;
- (BOOL)cacheDisabled;
- (BOOL)networkAccessDisabled;
- (BOOL)fastNetwork;
- (NSString *)ii2LibDir;
- (NSString *)defaultSaveDir;
- (NSString *)defaultII2DDirectory;
- (NSString *)gpgKeyServer;
- (BOOL)getTabHideStatusForKey:(NSString *)keyString;
- (BOOL)getTabHideStatusForTag:(II2TabTagType)tag;

@end

#define ii2Prefs [II2Preferences sharedInstance]


/*
 * $Log: II2Preferences.h,v $
 * Revision 1.5  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.4  2004/03/21 16:52:21  gctwnl
 * 2.58.0
 *
 * Revision 1.3  2004/03/19 23:35:11  gctwnl
 * Version 2.55.0. Has a renewed interface to support simple operations.
 * Supports progress bar control from the i-Package.
 *
 * Revision 1.2  2004/02/27 23:24:27  gctwnl
 * 2.52.0 entry after a long period of CVS problems
 *
 * Revision 1.1.1.1  2004/02/01 17:18:27  gctwnl
 * Initial import.
 *
 * Revision 3.7  2004/01/31 13:21:58  gerben
 * 2.48.0
 *
 * Revision 3.6  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.5  2004/01/05 15:31:34  gerben
 * *** empty log message ***
 *
 * Revision 3.4  2003/11/08 23:38:01  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.3  2003/01/04 02:04:43  gerben
 * Version 2.2.0
 *
 * Revision 3.2  2002/12/10 14:24:54  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:24  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:52  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.19  2002/12/03 20:57:40  gerben
 * A preference for automatic dependency checking
 *
 * Revision 1.18  2002/11/21 22:45:14  gerben
 * RC3e, added checkii2status crontab handling
 *
 * Revision 1.17  2002/10/17 20:36:03  gerben
 * Reversed default for caching
 *
 * Revision 1.16  2002/10/17 18:49:00  gerben
 * Cache/NoCache handling on downloads. Some other changes.
 *
 * Revision 1.15  2002/10/17 10:18:59  gerben
 * Splash screen. MD5 check on opening remote package.
 *
 * Revision 1.14  2002/10/16 08:37:19  gerben
 * Changed log levels. Small fixes.
 *
 * Revision 1.13  2002/10/15 14:31:44  gerben
 * Last fixes. No-network access. Later creation of downloaded file.
 *
 * Revision 1.12  2002/10/14 12:34:20  gerben
 * Removed dead code
 *
 * Revision 1.11  2002/10/07 20:28:07  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.10  2002/10/07 11:52:28  gerben
 * Several fixes related to possible invalid content.
 *
 * Revision 1.9  2002/10/02 22:38:31  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */