//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2Preferences.h"
#import "II2AppDelegate.h"
#import "GWSizeFormatter.h"
#import "GLUPTaskRunner.h"

static char rcsid[] = "$Id: II2Preferences.m,v 1.5 2004/04/25 13:13:21 gctwnl Exp $";

id ii2TagNameArray[] = {
    @"hastabviewsettings", // takes the place of readme in this array so the rest is in the right place
    @"hidepropertiestab",
    @"hideoutputtab",
    @"hidecontentstab",
    @"hideactivitytab",
    @"hidereporttab"
};

@implementation II2Preferences
static II2Preferences *_sharedInstance = nil;

+ (id)sharedInstance
{
    if (_sharedInstance == nil)
    {
	_sharedInstance = [[II2Preferences alloc] init];
    }
    return _sharedInstance;
}

- (id)init
{
    if (_sharedInstance != nil)
    {
        [super dealloc];
        return _sharedInstance;
    }
    _sharedInstance = self;
    if ([STDUD stringForKey:@"hidepropertiestab"] == nil) { // They are not new or they did not exist yet
        if ([STDUD stringForKey:@"loglevel"] != nil) { // They existed, but are out of date
            NSRunAlertPanel(@"Warning: Resetting defaults!", @"An incompatible change between your existing defaults and the new version of the program has been detected. I am regrettably forced to reset the defaults.  Please reapply any changes you have made.", @"Continue", nil, nil);
        }
	[self resetToDefaults];
    }
    [self readFromDefaults:STDUD];
    
    return self;
}

- (void)display
{
    if (preferencesPanel) {
	[autoDownloadButton setState:__autoFatten];
	[keepInvalidMd5Button setState:__keepInvalidMd5];
	[showSplashAtStartupButton setState:__showSplashAtStartup];
	[askForThinButton setState:__askForThinAtClose];
	[useGPGButton setState:__useGPG];
	[noUniqueII2DirectoryNamesButton setState:__noUniqueII2DirectoryNames];
	[cacheDisabledButton setState:__cacheDisabled];
	[networkAccessDisabledButton setState:__networkAccessDisabled];
	[fastNetworkButton setState:__fastNetwork];
	[self networkButtonClicked:nil];
	[saveDirTextField setStringValue:__defaultSaveDir];
	[gpgKeyServerTextField setStringValue:__gpgKeyServer];
	[defaultII2DirectoryTextField setStringValue:__defaultII2DDirectory];
	[logLevelTextField setIntValue:__mainLogLevel];
	[maxII2DDescendLevelTextField setIntValue:__maxII2DDescendLevel];
	[thinLevelTextField setFloatValue:(double)__thinSize];
	[cronEnableButton setState:__useCron];
	[cronHourTextField setFloatValue:(double)__cronHour];
	[cronMinutesTextField setFloatValue:(double)__cronMinutes];
	[cronDayPopUpButton selectItemAtIndex:__cronDay];
	[self cronButtonClicked:nil];
        [showContentsTabButton setState:!__hideContentsTab];
        [showReportTabButton setState:!__hideReportTab];
        [showActivityTabButton setState:!__hideActivityTab];
        [showOutputTabButton setState:!__hideOutputTab];
        [showPropertiesTabButton setState:!__hidePropertiesTab];
	[dynamicDisplayOfTabsButton setState:__dynamicDisplayOfTabs];
	[dependencyMatrix selectCellAtRow:0 column:__dependencyCheckType]; // ordered left to right as the enum above
    }
}

- (void)readFromDefaults:(NSUserDefaults *)defaults
{
    __mainLogLevel = [defaults integerForKey:@"loglevel"];
    __thinSize = [defaults integerForKey:@"thinsize"];
    __maxII2DDescendLevel = [defaults integerForKey:@"maxidirectorydescendlevel"];
    __noUniqueII2DirectoryNames = [defaults boolForKey:@"nouniqueii2dirnames"];
    __autoFatten = [defaults boolForKey:@"autodownload"];
    __keepInvalidMd5 = [defaults boolForKey:@"keepinvalidmd5"];
    __showSplashAtStartup = [defaults boolForKey:@"showsplashatstartup"];
    __askForThinAtClose = [defaults boolForKey:@"askforthinatclose"];
    __useGPG = [defaults boolForKey:@"usegpg"];
    __cacheDisabled = [defaults boolForKey:@"cachedisabled"];
    __networkAccessDisabled = [defaults boolForKey:@"networkaccessdisabled"];
    __fastNetwork = [defaults boolForKey:@"fastnetwork"];
    __useCron = [defaults boolForKey:@"usecron"];
    __cronDay = [defaults integerForKey:@"cronday"];
    __cronHour = [defaults integerForKey:@"cronhour"];
    __cronMinutes = [defaults integerForKey:@"cronminutes"];
    [__defaultSaveDir release];
    __defaultSaveDir = [[defaults stringForKey:@"savedir"] retain];
    [__gpgKeyServer release];
    __gpgKeyServer = [[defaults stringForKey:@"gpgkeyserver"] retain];
    [__defaultII2DDirectory release];
    __defaultII2DDirectory = [[defaults stringForKey:@"default_ii2idirectory"] retain];
    __hideContentsTab = [defaults boolForKey:@"hidecontentstab"];
    __hideReportTab = [defaults boolForKey:@"hidereporttab"];
    __hideActivityTab = [defaults boolForKey:@"hideactivitytab"];
    __hideOutputTab = [defaults boolForKey:@"hideoutputtab"];
    __hidePropertiesTab = [defaults boolForKey:@"hidepropertiestab"];
    __dynamicDisplayOfTabs = [defaults boolForKey:@"dynamicdisplayoftabs"];
    __dependencyCheckType = [defaults boolForKey:@"dependencychecktype"]%nrOfII2DependencyChecks; // Just to be sure...
    [self display];
}

- (void)writeToDefaults
{
    NSArray *cronTaskArgs;
    GLUPTaskRunner *cronTask;
    NSString *setCronCheck = [[NSBundle mainBundle] pathForResource:@"setcroncheck" ofType:nil];

    [STDUD setBool:__autoFatten forKey:@"autodownload"];
    [STDUD setBool:__keepInvalidMd5 forKey:@"keepinvalidmd5"];
    [STDUD setBool:__showSplashAtStartup forKey:@"showsplashatstartup"];
    [STDUD setBool:__askForThinAtClose forKey:@"askforthinatclose"];
    [STDUD setBool:__useGPG forKey:@"usegpg"];
    [STDUD setBool:__noUniqueII2DirectoryNames forKey:@"nouniqueii2dirnames"];
    [STDUD setBool:__cacheDisabled forKey:@"cachedisabled"];
    [STDUD setBool:__networkAccessDisabled forKey:@"networkaccessdisabled"];
    [STDUD setBool:__fastNetwork forKey:@"fastnetwork"];
    [STDUD setObject:__defaultSaveDir forKey:@"savedir"];
    [STDUD setObject:__defaultII2DDirectory forKey:@"default_ii2idirectory"];
    [STDUD setObject:__gpgKeyServer forKey:@"gpgkeyserver"];
    [STDUD setInteger:__mainLogLevel forKey:@"loglevel"];
    [STDUD setInteger:__maxII2DDescendLevel forKey:@"maxidirectorydescendlevel"];
    [STDUD setInteger:__thinSize forKey:@"thinsize"];
    [STDUD setBool:__useCron forKey:@"usecron"];
    [STDUD setInteger:__cronDay forKey:@"cronday"];
    [STDUD setInteger:__cronHour forKey:@"cronhour"];
    [STDUD setInteger:__cronMinutes forKey:@"cronminutes"];
    [STDUD setBool:__hideContentsTab forKey:@"hidecontentstab"];
    [STDUD setBool:__hideReportTab forKey:@"hidereporttab"];
    [STDUD setBool:__hideActivityTab forKey:@"hideactivitytab"];
    [STDUD setBool:__hideOutputTab forKey:@"hideoutputtab"];
    [STDUD setBool:__hidePropertiesTab forKey:@"hidepropertiestab"];
    [STDUD setBool:__dynamicDisplayOfTabs forKey:@"dynamicdisplayoftabs"];
    [STDUD setInteger:(__dependencyCheckType%nrOfII2DependencyChecks) forKey:@"dependencychecktype"];
    [STDUD synchronize];
    if (__useCron) {
	cronTaskArgs = [NSArray arrayWithObjects: @"/usr/bin/perl", setCronCheck, [self defaultSaveDir], [setCronCheck stringByDeletingLastPathComponent], [NSString stringWithFormat:@"%d", __cronDay], [NSString stringWithFormat:@"%d", __cronHour], [NSString stringWithFormat:@"%d", __cronMinutes], nil];
    }
    else {
	cronTaskArgs = [NSArray arrayWithObjects: @"/usr/bin/perl", setCronCheck, nil];
    }
    cronTask = [[GLUPTaskRunner alloc] initWithOwner:nil reportsPID:GLUPTaskRunnerFifoPid arguments:cronTaskArgs authenticator:nil];
    [cronTask run];
}

- (BOOL)_mkdir_minp: (NSString *)path
{
    NSArray *pathComponents = [[path stringByExpandingTildeInPath] pathComponents];
    NSMutableString *checkPath = [NSMutableString stringWithString:[pathComponents objectAtIndex:0]];
    int i;

    if (![checkPath isEqualToString:@"/"]) {
	NSLog( @"%@ is not an absolute or tilde path. Failed to create complete directory path.", path);
	return NO;
    }
    for (i=1; i<[pathComponents count]; i++) {
	BOOL isDir;

	// Do not turn / into //:
	if (![checkPath hasSuffix:@"/"]) {
	    [checkPath appendString:@"/"];
	}
	[checkPath appendString:[pathComponents objectAtIndex:i]];
	if ([DEFFM fileExistsAtPath:checkPath isDirectory:&isDir]) {
	    if (!isDir) {
		NSLog( @"%@ is not a directory. Failed to create complete directory path.", checkPath); 
		return NO;
	    }
	}
	else {
	    if (![DEFFM createDirectoryAtPath:checkPath attributes:nil]) {
		NSLog( @"Cannot create %@. Failed to create complete directory path.", checkPath);
		return NO;
	    }
	}
    }
    return YES;
}

- (BOOL)fileManager:(NSFileManager *)manager shouldProceedAfterError:(NSDictionary *)errorInfo
{
    if ([ii2Prefs mainLogLevel] > 0) NSLog( @"Preferences Filemanager Error: %@", errorInfo);
    NSBeginAlertSheet( @"Error", @"Accept", nil, nil, preferencesPanel, nil, nil, nil, nil, @"I cannot move %@ to %@. Cause: %@.", [errorInfo objectForKey:@"Path"], [errorInfo objectForKey:@"ToPath"], [errorInfo objectForKey:@"Error"]);
    return NO;
}

- (void)fileManager:(NSFileManager *)manager willProcessPath:(NSString *)path
{
    if ([ii2Prefs mainLogLevel] > 5) NSLog( @"Preferences Filemanager will process path: %@", path);
}

- (BOOL)_moveDefaultSaveDir:(NSString *)oldDir toDir:(NSString *)newDir
{
    BOOL returnStatus = YES;
    NSString *absoluteOldPath = [oldDir stringByExpandingTildeInPath];
    NSString *absoluteNewPath = [newDir stringByExpandingTildeInPath];

    if (![absoluteOldPath isEqualToString:absoluteNewPath]) {
	NSArray *documentsArray = [[NSDocumentController sharedDocumentController] documents];
	
	if ([documentsArray count]) {
	    int i;
	    for (i=0; i<[documentsArray count]; i++) {
		NSString *fileType = [[documentsArray objectAtIndex:i] fileType];
		if ([fileType isEqualToString:@"i-Package-v2"]) {
		    NSBeginAlertSheet( @"Error", @"Accept", nil, nil, preferencesPanel, nil, nil, nil, nil, @"I cannot move the default save directory while i-Packages are open. Sorry.");
		    return NO;
		}
	    }
	}
	// Create new dir
	if ([self _mkdir_minp:absoluteNewPath]) {
	    // Move packages from old to new default location
	    NSArray *dirContents = [DEFFM directoryContentsAtPath:absoluteOldPath];
	    int i;
	    BOOL everythingMoved = YES;

	    for (i=0; i<[dirContents count]; i++) {
		NSString *fileName = [dirContents objectAtIndex:i];
		if ([[fileName pathExtension] isEqualToString:@"ii2"] || [fileName isEqualToString:@".DS_Store"]) {
		    NSString *oldBundlePath = [absoluteOldPath stringByAppendingFormat:@"/%@", fileName];
		    NSString *newBundlePath = [absoluteNewPath stringByAppendingFormat:@"/%@", fileName];
		    if (![DEFFM movePath:oldBundlePath toPath:newBundlePath handler:self]) {
			returnStatus = NO;
		    }
		}
		else {
		    NSLog( @"Skipping. %@ will not be moved from %@ to %@. ", fileName, absoluteOldPath, absoluteNewPath);
		    everythingMoved = NO;
		}
	    }
	    if (everythingMoved) {
		[DEFFM removeFileAtPath:absoluteOldPath handler:nil];
	    }
	    else {
		NSLog( @"Directory %@ not empty. Will not be removed. ", absoluteOldPath);
	    }
	}
	else {
	    NSBeginAlertSheet( @"Error", @"Accept", nil, nil, preferencesPanel, nil, nil, nil, nil, @"The directory you entered (%@) is not an existing writable directory, nor can I create it.", newDir);
	    return NO;
	}
    }
    return returnStatus;
}

- (void)resetToDefaults
{
    __mainLogLevel = 0;
    __thinSize = 1024*1024;
    __maxII2DDescendLevel = 1;
    __autoFatten = YES;
    __keepInvalidMd5 = NO;
    __showSplashAtStartup = YES;
    __askForThinAtClose = NO;
    __useGPG = NO;
    __noUniqueII2DirectoryNames = NO;
    __cacheDisabled = YES;
    __networkAccessDisabled = NO;
    __fastNetwork = NO;
    __cronDay = random() % 7;
    __cronHour = random() % 24;
    __cronMinutes = random() % 60;
    __useCron = NO;
    [self _moveDefaultSaveDir:__defaultSaveDir toDir:@"~/Documents/i-Packages"];
    [__defaultSaveDir release];
    __defaultSaveDir = [[NSString stringWithString:@"~/Documents/i-Packages"] retain];
    [__gpgKeyServer release];
    __gpgKeyServer = [[NSString stringWithString:@"pgp.mit.edu"] retain];
    [__defaultII2DDirectory release];
    __defaultII2DDirectory = [[NSString stringWithString:@"http://tug.org/i-packages/iid/gwrelative.iid"] retain];
    __hideContentsTab = YES;
    __hideReportTab = YES;
    __hideActivityTab = YES;
    __hideOutputTab = YES;
    __hidePropertiesTab = NO;
    __dynamicDisplayOfTabs = NO;
    __dependencyCheckType = ii2DependencyCheckRequired;
    __ii2LibDir = @"/Library/i-Installer";

    [self writeToDefaults];
}

- (void)showPreferences
{
    if (preferencesPanel == nil)
    {
	// we need to load the nib
	if ([NSBundle loadNibNamed:@"Defaults" owner:self] == NO)
	{
	    NSRunAlertPanel(@"Error", @"Could not load Defaults.nib", @"Accept defeat", nil, nil);
	    return;
	}
	[thinLevelTextField setFormatter:[GWSizeFormatter sharedFormatter]];
    }
    [self readFromDefaults:STDUD];

    [preferencesPanel makeKeyAndOrderFront:self];
}

- (void)savePreferences
{
    NSString *saveDir;

    __autoFatten = [autoDownloadButton state];
    __keepInvalidMd5 = [keepInvalidMd5Button state];
    __showSplashAtStartup = [showSplashAtStartupButton state];
    __askForThinAtClose = [askForThinButton state];
    __useGPG = [useGPGButton state];
    __noUniqueII2DirectoryNames = [noUniqueII2DirectoryNamesButton state];
    __cacheDisabled = [cacheDisabledButton state];
    __networkAccessDisabled = [networkAccessDisabledButton state];
    __fastNetwork = [fastNetworkButton state];
    __dependencyCheckType = [dependencyMatrix selectedColumn]%nrOfII2DependencyChecks;
	
    saveDir = [saveDirTextField stringValue];
    if ([self _moveDefaultSaveDir:__defaultSaveDir toDir:saveDir]) {
	[__defaultSaveDir release];
	__defaultSaveDir = [[NSString stringWithString:saveDir] retain];
	[STDUD setObject:__defaultSaveDir forKey:@"savedir"];
    }
    else {
	[saveDirTextField setStringValue:[STDUD stringForKey:@"savedir"]];
    }
    __mainLogLevel = [logLevelTextField intValue];
    __thinSize = [thinLevelTextField intValue];
    __maxII2DDescendLevel = [maxII2DDescendLevelTextField intValue];
    if ([NSURL URLWithString:[defaultII2DirectoryTextField stringValue]]) {
	[__defaultII2DDirectory release];
	__defaultII2DDirectory = [[defaultII2DirectoryTextField stringValue] retain];
    }
    [__gpgKeyServer release];
    __gpgKeyServer = [[gpgKeyServerTextField stringValue] retain];

    __useCron = [cronEnableButton state];
    __cronDay = [cronDayPopUpButton indexOfSelectedItem];
    __cronHour = [cronHourTextField intValue];
    __cronMinutes = [cronMinutesTextField intValue];

    __hideContentsTab = ![showContentsTabButton state];
    __hideReportTab = ![showReportTabButton state];
    __hideActivityTab = ![showActivityTabButton state];
    __hideOutputTab = ![showOutputTabButton state];
    __hidePropertiesTab = ![showPropertiesTabButton state];
    __dynamicDisplayOfTabs = [dynamicDisplayOfTabsButton state];

    [self writeToDefaults];
}

- (void)openPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode == NSOKButton) {
	[saveDirTextField setStringValue:[[sheet filenames] objectAtIndex:0]];
    }
}

- (IBAction)basicViewButtonClicked:(id)sender
{
    [showPropertiesTabButton setState:YES];
    [showOutputTabButton setState:NO];
    [showContentsTabButton setState:NO];
    [showActivityTabButton setState:NO];
    [showReportTabButton setState:NO];
}

- (IBAction)expertViewButtonClicked:(id)sender
{
    [showPropertiesTabButton setState:YES];
    [showOutputTabButton setState:YES];
    [showContentsTabButton setState:YES];
    [showActivityTabButton setState:YES];
    [showReportTabButton setState:YES];
}

- (IBAction)networkButtonClicked:(id)sender
{
    [autoDownloadButton setEnabled:![networkAccessDisabledButton state]];
    [useGPGButton setEnabled:![networkAccessDisabledButton state]];
    [cacheDisabledButton setEnabled:![networkAccessDisabledButton state]];
    [gpgKeyServerTextField setEnabled:![networkAccessDisabledButton state]];
    [cronEnableButton setEnabled:![networkAccessDisabledButton state]];
    [fastNetworkButton setEnabled:![networkAccessDisabledButton state]];
    [self cronButtonClicked:nil];
}

- (IBAction)cronButtonClicked:(id)sender
{
    [cronDayPopUpButton setEnabled:[cronEnableButton state]&&![networkAccessDisabledButton state]];
    [cronHourTextField setEnabled:[cronEnableButton state]&&![networkAccessDisabledButton state]];
    [cronMinutesTextField setEnabled:[cronEnableButton state]&&![networkAccessDisabledButton state]];
}

- (IBAction)setSaveDirButtonPressed:(id)sender
{
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];

    [oPanel setAllowsMultipleSelection:NO];
    [oPanel setCanChooseDirectories:YES];
    [oPanel setCanChooseFiles:NO];
    [oPanel setPrompt:@"Set"];
    [oPanel _setIncludeNewFolderButton:YES];

    [oPanel setTitle:@"Set i-Package directory"];
    [oPanel beginSheetForDirectory:[[STDUD stringForKey:@"savedir"] stringByExpandingTildeInPath] file:nil types:nil modalForWindow:preferencesPanel modalDelegate:(id)self didEndSelector:@selector(openPanelDidEnd:returnCode:contextInfo:) contextInfo:nil];
}

- (IBAction)applyButtonPressed:(id)sender
{
    [self savePreferences];
    // [preferencesPanel performClose:self];
    [[NSNotificationCenter defaultCenter] postNotificationName:@"II2PreferencesChangedNotification" object:nil];
}

- (IBAction)cancelButtonPressed:(id)sender
{
    [preferencesPanel performClose:self];
}

- (IBAction)defaultsButtonPressed:(id)sender
{
    [self resetToDefaults];
    [self showPreferences];
    // [preferencesPanel performClose:self];
}

- (int)mainLogLevel
{
    return __mainLogLevel;
}

- (unsigned int)thinSize
{
    return __thinSize;
}

- (int)maxII2DDescendLevel
{
    return __maxII2DDescendLevel;
}

- (BOOL)askForThinAtClose
{
    return __askForThinAtClose;
}

- (BOOL)autoFatten
{
    return __autoFatten;
}

- (BOOL)useGPG
{
    return __useGPG;
}

- (II2DependencyCheckType)dependencyCheckType
{
    return __dependencyCheckType;
}


- (BOOL)dynamicDisplayOfTabs
{
    return __dynamicDisplayOfTabs;
}

- (BOOL)noUniqueII2DirectoryNames
{
    return __noUniqueII2DirectoryNames;
}

- (BOOL)cacheDisabled
{
    return __cacheDisabled;
}

- (BOOL)networkAccessDisabled
{
    return __networkAccessDisabled;
}

- (BOOL)fastNetwork
{
    return __fastNetwork;
}

- (BOOL)keepInvalidMd5
{
    return __keepInvalidMd5;
}

- (BOOL)showSplashAtStartup
{
    return __showSplashAtStartup;
}

- (NSString *)defaultSaveDir
{
    return [__defaultSaveDir stringByExpandingTildeInPath];
}

- (NSString *)ii2LibDir
{
    NSString *libDir = [__ii2LibDir stringByExpandingTildeInPath];
    if (!libDir) {
	libDir = @"/Library/i-Installer";
    }
    return libDir;
}

- (NSString *)defaultII2DDirectory
{
    return __defaultII2DDirectory;
}

- (NSString *)gpgKeyServer
{
    return __gpgKeyServer;
}

- (BOOL)getTabHideStatusForKey:(NSString *)keyString
{
    if ([keyString isEqualToString:@"hidepropertiestab"]) {
        return __hidePropertiesTab;
    }
    else if ([keyString isEqualToString:@"hideoutputtab"]) {
        return __hideOutputTab;
    }
    else if ([keyString isEqualToString:@"hidecontentstab"]) {
        return __hideContentsTab;
    }
    else if ([keyString isEqualToString:@"hideactivitytab"]) {
        return __hideActivityTab;
    }
    else if ([keyString isEqualToString:@"hidereporttab"]) {
        return __hideReportTab;
    }
    return NO;
}

- (BOOL)getTabHideStatusForTag:(II2TabTagType)tag
{
    if (tag > ii2ReadmeTabTag && tag < ii2NrOfTabs) {
        return [self getTabHideStatusForKey:ii2TagNameArray[tag]];
    }
    return NO;
}

- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
}

@end

/*
 * $Log: II2Preferences.m,v $
 * Revision 1.5  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.4  2004/03/21 16:52:21  gctwnl
 * 2.58.0
 *
 * Revision 1.3  2004/03/19 23:35:11  gctwnl
 * Version 2.55.0. Has a renewed interface to support simple operations.
 * Supports progress bar control from the i-Package.
 *
 * Revision 1.2  2004/02/27 23:24:27  gctwnl
 * 2.52.0 entry after a long period of CVS problems
 *
 * Revision 1.1.1.1  2004/02/01 17:18:27  gctwnl
 * Initial import.
 *
 * Revision 3.13  2004/01/31 13:21:58  gerben
 * 2.48.0
 *
 * Revision 3.12  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.11  2004/01/05 15:31:34  gerben
 * *** empty log message ***
 *
 * Revision 3.10  2003/12/05 12:40:52  gerben
 * Fixed .stamp and Custom.plist handling. Changed GPG key server default.
 *
 * Revision 3.9  2003/11/09 21:43:06  gerben
 * 2.32.0
 *
 * Revision 3.8  2003/11/08 23:38:01  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.7  2003/07/17 08:52:27  gerben
 * 2.23.0. Change max descend level. Adapt some Help.
 *
 * Revision 3.6  2003/05/19 08:02:57  gerben
 * No log message.
 *
 * Revision 3.5  2003/02/09 16:53:51  gerben
 * 2.4.0 bugfix release
 *
 * Revision 3.4  2003/01/04 15:43:46  gerben
 * 2.2.0 final commit.
 *
 * Revision 3.3  2003/01/04 02:04:43  gerben
 * Version 2.2.0
 *
 * Revision 3.2  2002/12/10 14:24:54  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:24  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:53  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.26  2002/12/03 20:57:40  gerben
 * A preference for automatic dependency checking
 *
 * Revision 1.25  2002/11/24 23:09:28  gerben
 * Tom's changes and remarks mainly
 *
 * Revision 1.24  2002/11/21 22:45:14  gerben
 * RC3e, added checkii2status crontab handling
 *
 * Revision 1.23  2002/10/30 21:42:23  gerben
 * RC3a
 *
 * Revision 1.22  2002/10/29 09:36:23  gerben
 * RC2
 *
 * Revision 1.21  2002/10/28 21:49:46  gerben
 * A lot of changes wrt known packages.
 *
 * Revision 1.20  2002/10/24 12:49:06  gerben
 * Update refreshes file table. Download reports when remainder of (not total) download time changes significantly.
 *
 * Revision 1.19  2002/10/17 20:36:03  gerben
 * Reversed default for caching
 *
 * Revision 1.18  2002/10/17 18:49:00  gerben
 * Cache/NoCache handling on downloads. Some other changes.
 *
 * Revision 1.17  2002/10/17 10:18:59  gerben
 * Splash screen. MD5 check on opening remote package.
 *
 * Revision 1.16  2002/10/15 14:31:44  gerben
 * Last fixes. No-network access. Later creation of downloaded file.
 *
 * Revision 1.15  2002/10/14 12:34:20  gerben
 * Removed dead code
 *
 * Revision 1.14  2002/10/08 21:34:29  gerben
 * Working GPG signature checks, shared GPG cache for II2.
 *
 * Revision 1.13  2002/10/07 20:28:07  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.12  2002/10/07 11:52:28  gerben
 * Several fixes related to possible invalid content.
 *
 * Revision 1.11  2002/10/03 13:38:04  gerben
 * Removed addChildWindow as child passes close to parent. Window can only close when there are no running tasks.
 *
 * Revision 1.10  2002/10/02 22:38:31  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */