//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2TaskSet.h"
#import "II2AppDelegate.h"
#import "II2Preferences.h"


static char rcsid[] = "$Id: II2TaskSet.m,v 1.2 2004/04/25 13:13:21 gctwnl Exp $";

@implementation II2TaskSet
- (void)dealloc
{
    [__tasks release]; __tasks = nil;
    [super dealloc];
}

- (id)init
{
    self = [super init];
    __tasks = [[NSMutableArray alloc] init];
    return self;
}

- (void)addII2Task:(id <II2TaskSetMemberProtocol>)task
{
    unsigned i, count = [__tasks count];

    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"Received addII2Task: %@ (retain = %u)", [task description], [task retainCount]);
    if (task == nil) {
	NSLog( @"Addition of nil task ignored");
	return;
    }
    for (i=0; i<count; i++) {
	if ((id <II2TaskSetMemberProtocol>)[__tasks objectAtIndex:i] == task) {
	    NSLog( @"Multiple additions of same task not allowed");
	    return;
	}
    }

    [__tasks addObject:task]; // This retains the task
    [task setTaskSet:self];
    if ([ii2Prefs mainLogLevel] > 7) NSLog( @"Added task to set: %@", [task description]);
    [__observer ii2TaskSetDidRefresh:self];
}

- (void)ii2TaskDidEnd:(id <II2TaskSetMemberProtocol>)task withSuccess:(BOOL)success
{
    unsigned i, count = [__tasks count];

    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"Received ii2TaskDidEnd:withSuccess: %@ (retain = %u)", [task description], [task retainCount]);
    for (i=0; i<count; i++) {
	if ((id <II2TaskSetMemberProtocol>)[__tasks objectAtIndex:i] == task) {
	    break;
	}
    }
    if (i < count) {
	[task retain]; // temporarily, since next statement releases it
	[__tasks removeObjectAtIndex:i];
	if ([ii2Prefs mainLogLevel] > 7) NSLog( @"Removed task from set: %@", [task description]);
	[__observer ii2TaskSetDidRefresh:self];
	[task release];
    }
    else {
	NSLog( @"ii2TaskDidEnd: No such task in task set: %@", [task description]);
    }
}

- (unsigned) count
{
    return [__tasks count];
}


- (void)setObserver:(id <II2TaskSetObserverProtocol>)observer
{
    __observer = observer;
}

- (void)setHandler:(id <II2TaskSetHandlerProtocol>)handler
{
    __handler = handler;
}

- (id <II2TaskSetMemberProtocol>)taskAtIndex:(int)row
{
    if ((row >= 0) && (row < [__tasks count])) {
	return [__tasks objectAtIndex:row];
    }
    return nil;
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    //if ([ii2Prefs mainLogLevel] > 10) NSLog( @"Received numberOfRowsInTableView:, returned %d", [__tasks count]);
    return [__tasks count];
}

- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex
{
    if ((rowIndex >= 0) && (rowIndex < [__tasks count])) {
	id <II2TaskSetMemberProtocol> task = [__tasks objectAtIndex:rowIndex];
	NSString *identifier = [aTableColumn identifier];

	//if ([ii2Prefs mainLogLevel] > 10) NSLog( @"Received tableView:objectValueForTableColumn: %@ row: %d", identifier, rowIndex);
	if (task != nil) {
	    if ([identifier isEqualToString:@"process"]) {
		return [task attributedDisplayName];
	    }
	    else if ([identifier isEqualToString:@"progress"]) {
		return [task progress];
	    }
	}
    }
    return @"Task Set Error";
}

- (void)refreshDisplay:(id <II2TaskSetMemberProtocol>)task
{
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"Received II2TaskSet refreshDisplay");
    [__observer ii2TaskSetDidRefresh:self];
}

- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
}

@end

/*
 * $Log: II2TaskSet.m,v $
 * Revision 1.2  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.1.1.1  2004/02/01 17:18:27  gctwnl
 * Initial import.
 *
 * Revision 3.4  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.3  2003/09/18 22:40:54  gerben
 * 2.26.0 Fragmented archives
 *
 * Revision 3.2  2002/12/10 14:24:54  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:24  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:53  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.16  2002/10/29 09:36:23  gerben
 * RC2
 *
 * Revision 1.15  2002/10/16 21:28:22  gerben
 * A complete new rewrite of taskss, processes and stuff.
 *
 * Revision 1.14  2002/10/16 08:37:19  gerben
 * Changed log levels. Small fixes.
 *
 * Revision 1.13  2002/10/12 10:54:39  gerben
 * A lot of changes, mostly when producing output/feedback.
 *
 * Revision 1.12  2002/10/09 14:16:12  gerben
 * Changed as many %s/lossyCString uses to %@/fileSystemRepresentation/UTF8String
 *
 * Revision 1.11  2002/10/07 20:28:06  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.10  2002/10/03 18:15:22  gerben
 * A typo (bz instead of bz2) prevented the unarchive of anything.
 *
 * Revision 1.9  2002/10/03 13:38:04  gerben
 * Removed addChildWindow as child passes close to parent. Window can only close when there are no running tasks.
 *
 * Revision 1.8  2002/10/02 22:38:31  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */