//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2ToolFinder.h"
#import "II2AppDelegate.h"
#import "II2Preferences.h"

static II2ToolFinder *__theToolFinder = nil;

static char rcsid[] = "$Id: II2ToolFinder.m,v 1.2 2004/04/25 13:13:21 gctwnl Exp $";

@implementation II2ToolFinder
+ (II2ToolFinder *)sharedToolFinder
{
    if (__theToolFinder == nil) {
	__theToolFinder = [[II2ToolFinder alloc] init];
    }
    return __theToolFinder;
}

- (void) dealloc
{
    [_toolHash release];
}

- init
{
    self = [super init];
    _toolHash = [[NSMutableDictionary alloc] init];
    return self;
}

- (NSString *)pathForTool:(NSString *)systemTool
{
    NSString *toolString = [_toolHash objectForKey:systemTool];
    BOOL isDir;
    
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"Checking existence of system tool: %@", systemTool);
    if (!toolString) {
	if ([[NSFileManager defaultManager] fileExistsAtPath:systemTool isDirectory:&isDir]) {
	    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"Found system tool: %@", systemTool);
	    [_toolHash setObject:[NSString stringWithString:systemTool] forKey:systemTool];
	    return [NSString stringWithString:systemTool];
	}
	else {
	    NSString *toolName = [systemTool lastPathComponent];
	    NSString *resourceTool = [[NSBundle mainBundle] pathForResource:toolName ofType:nil];
	    if ([[NSFileManager defaultManager] fileExistsAtPath:resourceTool isDirectory:&isDir]) {
		NSLog( @"Warning: %@ not found on your system. Using %@ instead.", systemTool, resourceTool);
		[_toolHash setObject:resourceTool forKey:systemTool];
		return resourceTool;
	    }
	    else {
		NSLog( @"Program error. %@ not found in the app wrapper.", resourceTool);
		return nil;
	    }
	}
    }
    else {
	if ([ii2Prefs mainLogLevel] > 9) NSLog( @"Found (cached) system tool: %@", systemTool);
    }
    return toolString;
}

- (NSDictionary *)cachedTools
{
    return _toolHash;
}

@end
