//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import <Cocoa/Cocoa.h>
#import "GLUPTaskRunner.h"
#import "II2TaskSet.h"
#import "GWInteractionManagerProtocols.h"
#import "II2Preferences.h"

@interface II2WindowController : NSWindowController </*GLUPTaskOutputHandler, */II2TaskSetObserverProtocol, II2TaskStatusReporterProtocol>
{
    IBOutlet NSTextView 		*messageView;

    IBOutlet NSButton 			*lockButton;
    IBOutlet NSButton 			*installButton;
    IBOutlet NSButton 			*configureOnlyButton;
    IBOutlet NSButton 			*removeButton;
    IBOutlet NSProgressIndicator 	*tasksRunningProgressIndicator;

    IBOutlet NSTextField		*downloadsProgressTitle;
    IBOutlet NSProgressIndicator	*downloadsProgressBar;
    IBOutlet NSTextField		*actionsProgressTitle;
    IBOutlet NSProgressIndicator	*actionsProgressBar;
    
    // The main tab view
    IBOutlet NSTabView 			*tabView;
    NSArray                             *tabViewItemsCache;
    BOOL                                __tabViewItemsSettingsStore[ii2NrOfTabs];
    NSTabViewItem                       *__savedSelectedItem;
    
    // Contents Tab
    IBOutlet NSTableView 		*fileTable;
    IBOutlet NSTextView 		*inspectView;
    IBOutlet NSButton 			*inspectButton;
    
    // Process Output Tab
    IBOutlet NSTextView			*outView;
    IBOutlet NSTextView 		*errView;

    // Processes Tab
    IBOutlet NSButton 			*taskmanagerAbortButton;
    IBOutlet NSTableView 		*taskManagerTableView;

    // Properties Tab
    IBOutlet NSTextField		*urlField;
    IBOutlet NSButton 			*setURLButton;
    IBOutlet NSTextField		*sizeField;
    IBOutlet NSTextField		*urlVersionField;
    IBOutlet NSTextField		*packageVersionField;
    IBOutlet NSButton 			*updateButton;
    IBOutlet NSButton 			*checkButton;
    IBOutlet NSTextField		*installedVersionField;
    IBOutlet NSTextField		*installLocationField;
    IBOutlet NSButton 			*setInstallLocationButton;
    IBOutlet id 			authenticationRequiredSwitch;
    IBOutlet NSTextField		*interactiveField;
    IBOutlet NSTextField		*nonStandardFilesField;
    IBOutlet NSTextView 		*signatureView;

    // Readme tab
    IBOutlet NSTextView 		*readmeView;

    // Report tab
    IBOutlet NSTextView			*reportView;

    // SetURL Panel:
    IBOutlet NSPanel 			*setURLPanel;
    IBOutlet NSTextField		*setURLPanelTeXField;

    // Data
    NSMutableArray			*__tableDataSource;

    // Interaction
    NSConnection			*__connection;
    id <GWInteractionManagerInternalProtocol> 		__interactionManager;
    NSString 				*__doServerName;
}

- (void)startDOServer;
- (void)stopDOServer;
- (NSString *)doServerName;
- (id <GWInteractionManagerInternalProtocol>)interactionManager;

- (void)runPendingSheets;

- (void)processesRunning:(BOOL)running;
- (void)updatePropertiesDisplay;
- (IBAction)returnToDefaults:(id)sender;

- (void)refreshFileTable;
- (void)refreshTaskTable;

- (void)modifyTabItemDisplayByTag:(II2TabTagType)tag toState:(BOOL)visible;
- (void)showTab:(II2TabTagType)tag; // Shorthand for set to visible and activate
- (IBAction)modifyTabItemDisplay:(id)sender;
- (void)saveTabViewSettings;
- (void)restoreTabViewSettings;

// Hide or display progress bar
- (void)startDisplayActionProgress:(unsigned int)maxValue;
- (void)startDisplayDownloadProgress:(unsigned int)maxValue;
- (void)showActionProgress:(BOOL)enable;
- (void)showDownloadProgress:(BOOL)enable;

// Increment action bar (used by interaction manager)
- (void)incrementActionProgressBar:(unsigned int)increment;

// II2TaskSetObserverProtocol:
- (void)ii2TaskSetDidRefresh:(II2TaskSet *)sender;

- (void)displayPackage;
- (IBAction)install:(id)sender;
- (IBAction)remove:(id)sender;
- (IBAction)inspect:(id)sender;
- (IBAction)deleteSelectedFile:(id)sender;
- (IBAction)update:(id)sender;
- (IBAction)setInstallLocation:(id)sender;
- (IBAction)setURL:(id)sender;
- (IBAction)setURLSetClicked:(id)sender;
- (IBAction)checkURL:(id)sender;

- (IBAction)fattenPackage:(id)sender;
- (IBAction)thinPackage:(id)sender;
- (IBAction)lockPackage:(id)sender;

- (void)autoButtonsEnabled;
- (void)buttonsEnabled:(BOOL)enabled;
- (void)buttonsEnabledForTaskSet:(II2TaskSet *)taskSet;

- (IBAction)toggleProcessesTab:(id)sender;
- (IBAction)toggleContentsTab:(id)sender;
- (IBAction)togglePropertiesTab:(id)sender;
- (IBAction)toggleOutputTab:(id)sender;
- (IBAction)toggleReportTab:(id)sender;

- (IBAction)abortTask:(id)sender;

- (NSTextView *)readmeView;
- (NSTextView *)messageView;
- (NSTextView *)outView;
- (NSTextView *)errView;
- (NSTextView *)reportView;
- (NSTextView *)inspectView;
- (NSTextView *)signatureView;
- (NSTableView *)taskTableView;
- (NSTableView *)fileTableView;

- (IBAction)ii2PrintReport:(id)sender;
- (IBAction)ii2ShowReport:(id)sender;
- (IBAction)ii2MailReport:(id)sender;

- (BOOL)windowShouldClose:(id)sender;

@end

/*
 * $Log: II2WindowController.h,v $
 * Revision 1.4  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.3  2004/03/19 23:35:11  gctwnl
 * Version 2.55.0. Has a renewed interface to support simple operations.
 * Supports progress bar control from the i-Package.
 *
 * Revision 1.2  2004/03/12 09:50:30  gctwnl
 * Version 2.53.0
 *
 * Revision 1.1.1.1  2004/02/01 17:18:27  gctwnl
 * Initial import.
 *
 * Revision 3.6  2004/01/27 12:57:39  gerben
 * 2.44.0
 *
 * Revision 3.5  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.4  2003/11/08 23:38:01  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.3  2003/02/23 23:29:11  gerben
 * Package Locking
 *
 * Revision 3.2  2002/12/10 14:24:54  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:24  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:53  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.29  2002/12/10 13:50:18  gerben
 * A bit of cleanup. One bugfix (wrong method)
 *
 * Revision 1.28  2002/12/09 15:09:00  gerben
 * Report printing
 *
 * Revision 1.27  2002/12/02 23:24:41  gerben
 * Basic dependency handling
 *
 * Revision 1.26  2002/11/24 23:09:28  gerben
 * Tom's changes and remarks mainly
 *
 * Revision 1.25  2002/10/31 14:26:57  gerben
 * Moved authentication setting to package window
 *
 * Revision 1.24  2002/10/30 21:42:22  gerben
 * RC3a
 *
 * Revision 1.23  2002/10/29 09:36:22  gerben
 * RC2
 *
 * Revision 1.22  2002/10/25 20:36:11  gerben
 * Layout changes.
 *
 * Revision 1.21  2002/10/24 20:12:32  gerben
 * Better handling of package name clashes and changed properties
 *
 * Revision 1.20  2002/10/14 12:34:19  gerben
 * Removed dead code
 *
 * Revision 1.19  2002/10/14 12:15:38  gerben
 * Install stamp file after unarchive.
 *
 * Revision 1.18  2002/10/09 10:53:40  gerben
 * Moved textview output to NSTexTView category.
 *
 * Revision 1.17  2002/10/08 21:34:28  gerben
 * Working GPG signature checks, shared GPG cache for II2.
 *
 * Revision 1.16  2002/10/08 13:20:29  gerben
 * MD5 check on update. Small changes.
 *
 * Revision 1.15  2002/10/07 20:28:06  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.14  2002/10/07 11:52:27  gerben
 * Several fixes related to possible invalid content.
 *
 * Revision 1.13  2002/10/07 09:10:11  gerben
 * Failing authentication re-enables buttons.
 *
 * Revision 1.12  2002/10/06 12:29:57  gerben
 * Progress Indicator for active tasks
 *
 * Revision 1.11  2002/10/06 10:19:21  gerben
 * Wrote workaround for indentation problem (indentation stays around even when view is cleared).
 *
 * Revision 1.10  2002/10/02 22:38:30  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */