//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import "II2WindowController.h"
#import "II2DocumentController.h"
#import "II2InteractionManager.h"
#import "II2Package.h"
#import "II2AppDelegate.h"
#import "II2ToolFinder.h"
#import "GWSizeFormatter.h"
#import "GWProgressCell.h"

#include <sys/types.h>
#include <unistd.h>
#include <string.h>

static char rcsid[] = "$Id: II2WindowController.m,v 1.9 2004/05/01 12:31:34 gctwnl Exp $";

@implementation II2WindowController

- (void)startDOServer
{
    if (__interactionManager && !__connection) {
	NSPort *__port = [[NSPort port] retain];
	__connection = [[NSConnection alloc] initWithReceivePort:__port sendPort:__port];
	if ([__connection registerName:__doServerName] == NO) {
	    /* Handle error. */
	    NSLog( @"Unable to start DO server with name %@", __doServerName);
	} else {
	    if ([ii2Prefs mainLogLevel] > 4) NSLog( @"Started DO server with name %@", __doServerName);
	}
	[__connection setRootObject:__interactionManager];
    }
}

- (void)stopDOServer
{
    if (__interactionManager && __connection) {
	[[__connection receivePort] invalidate]; // Without this the port lives on even if it is released
	if ([ii2Prefs mainLogLevel] > 4) NSLog( @"Invalidated DO server with name %@", __doServerName);
	[[__connection receivePort] release]; // we retained it
	[__connection invalidate]; // good for something?
	[__connection release]; __connection = nil;//same
    }
}

- (NSString *)doServerName
{
    return __doServerName;
}

- (void)dealloc
{
    [self stopDOServer];
    [__doServerName release]; __doServerName = nil;
    [__interactionManager release]; __interactionManager = nil;
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [tabViewItemsCache release]; tabViewItemsCache = nil;
    [super dealloc];
}

- (id)initWithWindowNibName:(NSString *)windowNibName
{
    self = [super initWithWindowNibName:windowNibName];
    if (self) {
	__doServerName = [[NSString stringWithFormat:@"ii2.%d.%u.%u.%p", getpid(), random(), time( NULL), self] retain];
	__interactionManager = [[II2InteractionManager alloc] initWithWindowController:self];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(resignKey:) name: NSWindowDidResignKeyNotification object: nil];
	[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(preferencesChanged:) name: @"II2PreferencesChangedNotification" object: nil];
    }
    return self;
}

- (void)preferencesChanged:(id)sender
{
    [self autoButtonsEnabled];
}

- (id<GWInteractionManagerInternalProtocol>)interactionManager
{
    return __interactionManager;
}

-(void)setInspectButtonStatus:(unsigned int)modifiers
{
    II2Package *package = (II2Package *)[self document];
    //unsigned int modifiers = [[[self window] currentEvent] modifierFlags];
    int index = [fileTable selectedRow];
    NSString *fileFromTable;
    NSString *fileToInspect;
    NSString *basicFileToInspect;
    NSString *altFileToInspect;
    NSString *trueString = @"";
    BOOL isAvailable;
    BOOL mayBeDownloaded;

    if (index < 0) {
	return;
    }

    fileFromTable = [package fileNameAtIndex:index];
    fileToInspect = [package fileNameForInspect:fileFromTable withModifiers:modifiers];
    basicFileToInspect = [package fileNameForInspect:fileFromTable withModifiers:0];
    altFileToInspect = [package fileNameForInspect:fileFromTable withModifiers:NSAlternateKeyMask];
    isAvailable = [package fileSetIsAvailable:fileToInspect];
    mayBeDownloaded = [package canBeFattened] && ([ii2Prefs autoFatten] || (modifiers&NSShiftKeyMask));

    if (![altFileToInspect isEqualToString:basicFileToInspect]) {
	if (modifiers&NSAlternateKeyMask) {
	    trueString = @" (Real)";
	}
	else {
	    trueString = @" (Cache)";
	}
    }
    if (isAvailable) {
	// Available, nontrue
	[inspectButton setTitle:[@"Inspect" stringByAppendingString:trueString]];
	[inspectButton setEnabled:YES];
    }
    else if (mayBeDownloaded) {
	// Not available, downloadable
	[inspectButton setTitle:[@"Download & Inspect" stringByAppendingString:trueString]];
	[inspectButton setEnabled:![package active]];
    }
    else {
	// Not available, not downloadable
	[inspectButton setTitle:@"Unavailable"];
	[inspectButton setEnabled:NO];
    }
}

- (void)normalizeFlaggedButtons
{
    [self setInspectButtonStatus:0];
}

- (void)tableViewSelectionDidChange:(NSNotification *)aNotification
{
    NSTableView *tableViewChanged = [aNotification object];
    if (tableViewChanged == fileTable) {
	NSEvent *currentEvent = [[self window] currentEvent];
	[self flagsChanged:currentEvent];
    }
}


- (void)resignKey:(id)sender
{
    [self setInspectButtonStatus:0];
}

- (void)flagsChanged:(NSEvent *)theEvent
{
    unsigned int modifiers = [theEvent modifierFlags];

    [self setInspectButtonStatus:modifiers];
}

- (void)refreshFileTable {
    NSEvent *currentEvent = [[self window] currentEvent];
    unsigned int modifiers = [currentEvent modifierFlags];
    [fileTable reloadData];
    [self setInspectButtonStatus:modifiers];
}

// Method called on start of execution of actions
- (void)startDisplayActionProgress:(unsigned int)maxValue;
{
    //NSLog( @"startDisplayActionProgress: %u", maxValue);
    if (maxValue > 0) {
	[actionsProgressBar setIndeterminate:NO];
	[actionsProgressBar setMinValue:0.0];
	if (maxValue) [actionsProgressBar setMaxValue:maxValue];
	[actionsProgressBar setDoubleValue:0.0];
    }
    else {
	[actionsProgressBar setIndeterminate:YES];
	[actionsProgressBar startAnimation:nil];
    }
    [self showActionProgress:YES];
}

// Method called on start of execution of downloads
- (void)startDisplayDownloadProgress:(unsigned int)maxValue;
{
    //NSLog( @"startDisplayDownloadProgress: %u", maxValue);
    if (maxValue > 0) {
	[downloadsProgressBar setIndeterminate:NO];
	[downloadsProgressBar setMinValue:0.0];
	if (maxValue) [downloadsProgressBar setMaxValue:maxValue];
	[downloadsProgressBar setDoubleValue:0.0];
    }
    [self showDownloadProgress:(maxValue > 0.0)];
}

// #define NSAppKitVersionNumber10_2_3 663.6 // Not defined when we are running in 10.2 mode.

- (void)showActionProgress:(BOOL)enable
{
    //NSLog( @"showActionProgress enable: %@", (enable ? @"YES" :@"NO"));
    if ([actionsProgressTitle respondsToSelector:@selector(setHidden:)]) {
	// [actionsProgressTitle setHidden:!enable];
	[actionsProgressBar setHidden:!enable];
    }
    else if (!enable) {
	if ([actionsProgressBar isIndeterminate]) {
	    [actionsProgressBar setDisplayedWhenStopped:NO];
	    [actionsProgressBar stopAnimation:nil];
	}
    }
//    else {
//	NSLog( @"showActionProgress enable!");
//    }
}

- (void)showDownloadProgress:(BOOL)enable
{
    // NSLog( @"showDownloadProgress enable: %@", (enable ? @"YES" :@"NO"));
    if ([downloadsProgressTitle respondsToSelector:@selector(setHidden:)]) {
	// [downloadsProgressTitle setHidden:!enable];
	[downloadsProgressBar setHidden:!enable];
    }
    else if (!enable) {
	if ([downloadsProgressBar isIndeterminate]) {
	    [downloadsProgressBar setDisplayedWhenStopped:NO];
	    [downloadsProgressBar stopAnimation:nil];
	}
    }
//    else {
//	NSLog( @"showDownloadProgress enable!");
//    }
}

- (void)refreshTaskTable {
    [taskManagerTableView reloadData];
}

- (void)ii2TaskSetDidRefresh:(II2TaskSet *)sender
{
    if ([ii2Prefs mainLogLevel] > 8) NSLog( @"Received ii2TaskSetDidRefresh");
    [self refreshTaskTable];
    [self buttonsEnabledForTaskSet:sender];
}

- (void)updatePropertiesDisplay
{
    II2Package *package = (II2Package *)[self document];

    [urlField setStringValue:[package urlString]];
    [installLocationField setStringValue:[package installLocation]];
    [packageVersionField setStringValue:[package pkgTimestampDisplay]];
    [installedVersionField setStringValue:[package installedTimestampDisplay]];
    [urlVersionField setStringValue:[package remoteStatusDisplay]];
    [authenticationRequiredSwitch setState:[package needsAuthentication]];
}

- (IBAction)returnToDefaults:(id)sender
{
    II2Package *package = (II2Package *)[self document];
    [package returnToDefaults];
    [self updatePropertiesDisplay];
}

- (void)displayPackage
{
    II2Package *package = (II2Package *)[self document];
    [self synchronizeWindowTitleWithDocumentName];
    [lockButton setState:([package isLocked] ? NSOnState : NSOffState)];
    [sizeField setObjectValue:[NSNumber numberWithInt:[package totalSize]]];
    if ([package isInteractive]) {
	[interactiveField setStringValue:@"YES"];
    }
    else {
	[interactiveField setStringValue:@"NO"];
    }
    if ([package hasNonStandardFiles]) {
	[nonStandardFilesField setStringValue:@"YES"];
    }
    else {
	[nonStandardFilesField setStringValue:@"NO"];
    }
    if ([package hasConfigureScript]) {
	[installButton setTitle:@"Install & Configure"];
	if (![configureOnlyButton superview]) {
	    [[installButton superview] addSubview:configureOnlyButton];
	    [configureOnlyButton release];
	}
    }
    else {
	[installButton setTitle:@"Install"];
	if ([configureOnlyButton superview]) {
	    [[configureOnlyButton retain] removeFromSuperview];
	}
    }
    [self buttonsEnabled:YES];
    [self updatePropertiesDisplay];
    [self refreshFileTable];
    // [self normalizeFlaggedButtons];
}

- (NSTextView *)readmeView
{
    return readmeView;
}

- (NSTextView *)messageView
{
    return messageView;
}

- (NSTextView *)outView
{
    return outView;
}

- (NSTextView *)errView
{
    return errView;
}

- (NSTextView *)reportView
{
    return reportView;
}

- (NSTextView *)inspectView
{
    return inspectView;
}

- (NSTextView *)signatureView
{
    return signatureView;
}

- (NSTableView *)taskTableView
{
    return taskManagerTableView;
}

- (NSTableView *)fileTableView
{
    return fileTable;
}

- (void)scrollToStart:(id)view
{
    [view scrollRangeToVisible:NSMakeRange(0, 0)];
}

- (NSString *)windowTitleForDocumentDisplayName: (NSString *)description
{
    II2Package *package = (II2Package *)[self document];
    return [package packageName];
}

- (IBAction)selectFile:(id)sender
{
    NSEvent *currentEvent = [[self window] currentEvent];
    unsigned int modifiers = [currentEvent modifierFlags];
    [self setInspectButtonStatus:modifiers];
}

- (BOOL)validateMenuItem:(NSMenuItem *)anItem
{
    II2Package *package = (II2Package *)[self document];
    
    //NSLog( @"%@ Validating %@", [[self class] description], [anItem description]);
    
    if (([anItem tag] > ii2ReadmeTabTag) && ([anItem tag] < ii2NrOfTabs)) {
        int hideTab = [tabView indexOfTabViewItemWithIdentifier:[NSString stringWithFormat:@"%d", [anItem tag]]];
        if (hideTab == NSNotFound){
            [anItem setState:NSOffState];
        }
        else {
            [anItem setState:NSOnState];
        }
    }
    if (([anItem tag] == 97) && [package isLocked]) {
        return NO;
    }
    if ((([anItem tag] == 98) || ([anItem tag] == 99)) && ([package isLocked] || [package active])) {
        return NO;
    }
    if (([anItem tag] == 99) && ([package isLocked] || ![[[tabView selectedTabViewItem] identifier] isEqualToString:@"3"])) {
        return NO;
    }
    return YES;
}

- (IBAction)modifyTabItemDisplay:(id)sender
{
    [self modifyTabItemDisplayByTag:[sender tag] toState:![sender state]];
    [self saveTabViewSettings];
}

- (void)showTab:(II2TabTagType)tag // Shorthand for set to visible and activate
{
    [self modifyTabItemDisplayByTag:tag toState:YES];
    [tabView selectTabViewItemWithIdentifier:[NSString stringWithFormat:@"%u", tag]];
}


- (void)modifyTabItemDisplayByTag:(II2TabTagType)tag toState:(BOOL)visible
{
    II2Package *package = (II2Package *)[self document];

    if (tag> ii2ReadmeTabTag && tag < ii2NrOfTabs) {
        int hideTab = [tabView indexOfTabViewItemWithIdentifier:[NSString stringWithFormat:@"%d", tag]];
	if (visible == YES) {
	    if (hideTab == NSNotFound) {
		// Find out which next item is displayed wrt to the item standard at this place.
		NSArray *currentItemsArray = [tabView tabViewItems];
		NSTabViewItem *itemToAdd = [tabViewItemsCache objectAtIndex:tag];
		int i;
		int j = [currentItemsArray count];
		
		for (i=tag; i<[tabViewItemsCache count]; i++) {
		    NSTabViewItem *itemTested = [tabViewItemsCache objectAtIndex:i];
		    // itemTested is the item we want to try to find in the current displayed set
		    
		    for (j=1; j<[currentItemsArray count]; j++) {
			if ([currentItemsArray objectAtIndex:j] == itemTested) {
			    break;
			}
		    }
		    if (j < [currentItemsArray count]) {
			// we did find itemTested in displayed set
			break;
		    }
		}
		
		if (j >= [currentItemsArray count]) {
		    [tabView addTabViewItem:itemToAdd];
		}
		else {
		    [tabView insertTabViewItem:itemToAdd atIndex:j];
		}
		[tabView selectTabViewItem:itemToAdd];
		[package setHideTabForTag:tag toState:NO];
	    }
	    else {
		[tabView selectTabViewItemWithIdentifier:[NSString stringWithFormat:@"%d", tag]];
	    }
	}
        else if (hideTab > ii2ReadmeTabTag && hideTab < ii2NrOfTabs) {
            [tabView removeTabViewItem:[tabViewItemsCache objectAtIndex:tag]];
            [package setHideTabForTag:tag toState:YES];
        }
    }
    else if (tag == ii2BasicTabsTag) {
        // Basic = Readme + Properties only
        [self modifyTabItemDisplayByTag:ii2PropertiesTabTag toState:YES];
        [self modifyTabItemDisplayByTag:ii2OutputTabTag toState:NO];
        [self modifyTabItemDisplayByTag:ii2ContentsTabTag toState:NO];
        [self modifyTabItemDisplayByTag:ii2ActivityTabTag toState:NO];
        [self modifyTabItemDisplayByTag:ii2ReportTabTag toState:NO];
        [tabView selectTabViewItemWithIdentifier:[NSString stringWithFormat:@"%u", ii2ReadmeTabTag]];
    }
    else if (tag == ii2ExpertTabsTag) {
        // Expert = all
        [self modifyTabItemDisplayByTag:ii2PropertiesTabTag toState:YES];
        [self modifyTabItemDisplayByTag:ii2OutputTabTag toState:YES];
        [self modifyTabItemDisplayByTag:ii2ContentsTabTag toState:YES];
        [self modifyTabItemDisplayByTag:ii2ActivityTabTag toState:YES];
        [self modifyTabItemDisplayByTag:ii2ReportTabTag toState:YES];
        [tabView selectTabViewItemWithIdentifier:[NSString stringWithFormat:@"%u", ii2ContentsTabTag]];
    }
}

- (void)saveTabViewSettings
{
    int i;
    
    for (i=ii2ReadmeTabTag; i<ii2NrOfTabs; i++) {
        int hideTab = [tabView indexOfTabViewItemWithIdentifier:[NSString stringWithFormat:@"%d", i]];        
        __tabViewItemsSettingsStore[i] = (hideTab != NSNotFound);
    }
    __savedSelectedItem = [tabView selectedTabViewItem];
}

- (void)restoreTabViewSettings
{
    int i, index;
    
    for (i=ii2ReadmeTabTag; i<ii2NrOfTabs; i++) {
        [self modifyTabItemDisplayByTag:i toState: __tabViewItemsSettingsStore[i]];
    }
    index = [tabView indexOfTabViewItem:__savedSelectedItem];
    if (index != NSNotFound) {
        [tabView selectTabViewItem:__savedSelectedItem];
    }
}

- (void)windowDidLoad
{
    [super windowDidLoad];
    // The following does not work in Mac OS X 10.3
    //if ([tasksRunningProgressIndicator respondsToSelector:@selector( setStyle:)])
    //{
    //    [tasksRunningProgressIndicator setFrameSize:NSMakeSize( 20, 20)];
//	[tasksRunningProgressIndicator setStyle:NSProgressIndicatorSpinningStyle];
//	[tasksRunningProgressIndicator setDisplayedWhenStopped:NO];
//	[self showWindow:nil];
//	[tasksRunningProgressIndicator startAnimation:nil];
//	[tasksRunningProgressIndicator stopAnimation:nil];
//    }
    [fileTable setDoubleAction:@selector( inspect:)];
    [fileTable setAction:@selector( selectFile:)];
    [fileTable setTarget:self];

    if (![ii2Prefs networkAccessDisabled]) {
	II2Package *package = (II2Package *)[self document];
	
	if ([ii2Prefs useGPG]) {
	    [package gpgCheckToC];
	}
    }
}

- (IBAction)authRequiredSwitchToggled:(id)sender
{
    II2Package *package = (II2Package *)[self document];
    [package setAuthenticationRequired:[sender state]];
}

- (void)awakeFromNib
{
    int i;
    II2Package *package = (II2Package *)[self document];
    NSTableColumn *sizeTableColumn;

    if ([[self superclass] instancesRespondToSelector:@selector(awakeFromNib)]) {
        [super awakeFromNib];
    }
    sizeTableColumn = [[fileTable tableColumns] objectAtIndex:[fileTable columnWithIdentifier:@"size"]];
    [[sizeTableColumn dataCell] setFormatter:[GWSizeFormatter sharedFormatter]];
    [sizeField setFormatter:[GWSizeFormatter sharedFormatter]];
    [package setupFileTableSourceAndInfo];
    [fileTable setDataSource:package];
    //[[[taskManagerTableView tableColumns] objectAtIndex:0] setDataCell:[[GWProgressCell alloc] init]];

    [authenticationRequiredSwitch setTarget:self];
    [authenticationRequiredSwitch setAction:@selector(authRequiredSwitchToggled:)];

    [taskManagerTableView setDataSource:[package taskDataSource]];
    [tabView selectTabViewItemWithIdentifier:@"0"];
    // The following call is needed because it is only called automatically when the NSDocument subclass is the nib's owner  (ie if the document implements -windowNibName)
    [self buttonsEnabled:NO];

    tabViewItemsCache = [[NSArray arrayWithArray:[tabView tabViewItems]] retain];
    
    for (i=ii2ReadmeTabTag+1; i<ii2NrOfTabs; i++) {
        if ([package hideTabForTag:i]) {
            [self modifyTabItemDisplayByTag:i toState:NO];
        }
    }
    [self saveTabViewSettings];

    [package windowControllerDidLoadNib:self];
    
    [self displayPackage];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(tableViewSelectionDidChange:) name:NSTableViewSelectionDidChangeNotification object:fileTable];
}

- (BOOL)windowShouldClose:(id)sender
{
    II2Package *package = (II2Package *)[self document];
    if ([package active]) {
	NSLog( @"Can't close a window when processes are still active. A panel is not provided for technical reasons (may damage interactive panel activities)");
	return NO;
    }
    return YES;
}

// Progress indicator support
- (void)processesRunning:(BOOL)running
{
    // NSLog( @"Processes running: %@", (running ? @"YES" :@"NO"));
    if (running) {
	[tasksRunningProgressIndicator startAnimation:nil];
    }
    else {
	[tasksRunningProgressIndicator stopAnimation:nil];
	[self showDownloadProgress:NO];
	[self showActionProgress:NO];
    }
}

// User interface actions and support:

- (IBAction)install:(id)sender
{
    II2Package *package = (II2Package *)[self document];
    NSString *actionString = ((sender == configureOnlyButton) ? @"configure" : @"selectandinstall");

    [self saveTabViewSettings];
    [self buttonsEnabled:NO];
    
    [package doUpdateCheckAndAction:actionString withObject:nil];
}

- (IBAction)remove:(id)sender
{
    II2Package *package = (II2Package *)[self document];

    [self saveTabViewSettings];
    [self buttonsEnabled:NO];

    [package doUpdateCheckAndAction:@"remove" withObject:nil];
}

- (IBAction)update:(id)sender
{
    II2Package *package = (II2Package *)[self document];
    [self buttonsEnabled:NO];
    [package update];
}

-(void)buttonsEnabled:(BOOL)enabled
{
    II2Package *package = (II2Package *)[self document];
    NSEvent *currentEvent = [[self window] currentEvent];
    unsigned int modifiers = [currentEvent modifierFlags];
    
    // NSLog( @"buttonsEnabled enabled:%@",(enabled ? @"YES" : @"NO"));
    
    if (enabled) {
	if ([package hasRemoveScript]) {
	    [removeButton setEnabled:YES];
	}
	if ([package hasConfigureScript]) {
	    [configureOnlyButton setEnabled:YES];
	}
	if ([package hasArchive]) {
	    [installButton setEnabled:YES];
	}
	[inspectButton setEnabled:YES];
	[updateButton setEnabled:([package canBeFattened] && [ii2Prefs autoFatten])];
	[checkButton setEnabled:![ii2Prefs networkAccessDisabled]];
	[lockButton setEnabled:YES];
	[setURLButton setEnabled:![package isLocked]];
	[setInstallLocationButton setEnabled:(![package isLocked] && [package isRelocatable])];
    }
    else {
	[installButton setEnabled:NO];
	[removeButton setEnabled:NO];
	[configureOnlyButton setEnabled:NO];
	[inspectButton setEnabled:NO];
	[updateButton setEnabled:NO];
	[checkButton setEnabled:NO];
	[lockButton setEnabled:NO];
	[setURLButton setEnabled:NO];
	[setInstallLocationButton setEnabled:NO];
    }
    [self processesRunning:!enabled];
    [self setInspectButtonStatus:modifiers];
}

- (void)buttonsEnabledForTaskSet:(II2TaskSet *)taskSet
{
    [self buttonsEnabled:([taskSet count] == 0)];
    //  [self processesRunning:[taskSet count] == 0); // A problem here when th elast tasl of a series is an action task. The problem is that the action task is started by the dying gluptask and thus the gluptask still exists when the action task looks if there still is activity.
}

- (void)autoButtonsEnabled
{
    II2Package *package = (II2Package *)[self document];

    [self buttonsEnabled:![package active]];
}

- (IBAction)inspect:(id)sender
{
    NSEvent *currentEvent = [[self window] currentEvent];
    unsigned int modifiers = [currentEvent modifierFlags];
    II2Package *package = (II2Package *)[self document];

    [self normalizeFlaggedButtons];
    if ([fileTable selectedRow] >= 0) {
	[inspectView setString:@""];
	[self buttonsEnabled:NO];
	[package inspect:[package fileNameAtIndex:[fileTable selectedRow]] withModifiers:modifiers inView:inspectView];
    } 
}

- (IBAction)ii2GPGCheckToC:(id)sender
{
    II2Package *package = (II2Package *)[self document];

    [self buttonsEnabled:NO];
    [package gpgCheckToC];
}

- (IBAction)ii2CheckDependencies:(id)sender
{
    II2Package *package = (II2Package *)[self document];

    [package reportPkgDependencies];
}

- (IBAction)ii2RestoreButtons:(id)sender
{
    [self autoButtonsEnabled];
}

- (IBAction)checkURL:(id)sender
{
    II2Package *package = (II2Package *)[self document];

    [package readRemotePkgPlist:YES]; // Force re-read
    [urlVersionField setStringValue:[package remoteStatusDisplay]];
}

- (IBAction)deleteSelectedFile:(id)sender
{
    II2Package *package = (II2Package *)[self document];

    if ([fileTable selectedRow] >= 0) {
	NSEvent *currentEvent = [[self window] currentEvent];
	unsigned int modifiers = [currentEvent modifierFlags];
	[package deleteFile:[package fileNameAtIndex:[fileTable selectedRow]]];
	[self refreshFileTable];
	[self setInspectButtonStatus:modifiers];
    }
}

- (IBAction)fattenPackage:(id)sender
{
    II2Package *package = (II2Package *)[self document];

    [package doUpdateCheckAndAction:@"fatten" withObject:nil];
}

- (IBAction)thinPackage:(id)sender
{
    II2Package *package = (II2Package *)[self document];

    [package thin];
}

- (IBAction)lockPackage:(id)sender
{
    II2Package *package = (II2Package *)[self document];
    int state = [lockButton state];
    [package setLockStatus:(state == NSOnState)]; // Locked is default
    [self autoButtonsEnabled];
}


- (void)installLocationOpenPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
    II2Package *package = (II2Package *)[self document];
    if (returnCode == NSOKButton) {
	NSString *result = [NSString stringWithString:[[sheet filenames] objectAtIndex:0]];
	char *cString = (char *)[result fileSystemRepresentation];
	char *pt = cString;
	while ((pt = strchr( pt, ':')) != 0) {
	    *pt = '/';
	}
	[package setInstallLocation:[NSString stringWithCString:cString]];
	[installLocationField setStringValue:[package installLocation]];
	[installedVersionField setStringValue:[package installedTimestampDisplay]];
    }
}


- (IBAction)setInstallLocation:(id)sender
{
    II2Package *package = (II2Package *)[self document];
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];

    [oPanel setAllowsMultipleSelection:NO];
    [oPanel setCanChooseDirectories:YES];
    [oPanel setCanChooseFiles:NO];
    [oPanel setPrompt:@"Set"];
    [oPanel _setIncludeNewFolderButton:YES];

    [oPanel setTitle:@"Set Install Location"];
    [oPanel beginSheetForDirectory:[package installLocation] file:nil types:nil modalForWindow:[self window] modalDelegate:(id)self didEndSelector:@selector(installLocationOpenPanelDidEnd:returnCode:contextInfo:) contextInfo:nil];
}

- (IBAction)setURLSetClicked:(id)sender
{
    II2Package *package = (II2Package *)[self document];
    NSString *newURLString = [setURLPanelTeXField stringValue];

    if ([[newURLString lastPathComponent] isEqualTo:[[[self document] fileName] lastPathComponent]]) {
	NSURL *url = [NSURL URLWithString:newURLString];

	if (url) {
	    if (floor( NSAppKitVersionNumber) <= NSAppKitVersionNumber10_1) {
		// Running on a 10.0 - 10.1.x system
		if ([[url scheme] isEqualToString:@"ftp"]) {
		    NSRunAlertPanel(@"Error", @"FTP not supported by Apple Frameworks in Mac OS X Versions below 10.2", @"Use other scheme", nil, nil, nil);
		    return;
		}
	    }
	    [setURLPanel performClose:self];
	    [package setURLString:newURLString];
	    [self checkURL:nil];
	    [self updatePropertiesDisplay];
	}
    }
    else {
	NSRunAlertPanel(@"Error", @"The package name (the last part if the URL) should remain equal.", @"Retry", nil, nil, nil);
    }
}

- (IBAction)setURL:(id)sender
{
    [setURLPanelTeXField setStringValue:[urlField stringValue]];
    [setURLPanel makeKeyAndOrderFront:self];
}

- (IBAction)abortTask:(id)sender
{
    int row = [taskManagerTableView selectedRow];
    II2Package *package = (II2Package *)[self document];

    if (row >= 0) {
	[package abortTaskAtIndex:row];
    }
}

- (void)runPendingSheets
{
    [__interactionManager runPendingSheets];
}

- (IBAction)toggleProcessesTab:(id)sender
{
}

- (IBAction)toggleContentsTab:(id)sender
{
}

- (IBAction)togglePropertiesTab:(id)sender
{
}

- (IBAction)toggleOutputTab:(id)sender
{
}

- (IBAction)toggleReportTab:(id)sender
{
}

- (IBAction)createReport
{
    II2Package *package = (II2Package *)[self document];
    NSArray *envArgs, *hostinfoArgs;
    GLUPTaskRunner *envTask, *hostinfoTask;
    NSData *data;

    [reportView resetParagraphStyleAndClear];

    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Properties of package \"%@\" (%@.ii2):\n\n", [package packageName], [package baseName]] attributes:[self signalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"URL: %@\n", [package urlString]] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Self version: %@\n", [package pkgTimestampDisplay]] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Remote version: %@\n", [package remoteStatusDisplay]] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Installed version: %@\n", [package installedTimestampDisplay]] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Install location: %@\n", [package installLocation]] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Is Relocatable: %@\n", ([package isRelocatable] ? @"YES" : @"NO")] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Needs Authentication: %@\n", ([package needsAuthentication] ? @"YES" : @"NO")] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Needs Resource Forks Fixup: %@\n", ([package needsResourceForksFixup] ? @"YES" : @"NO")] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Is Interactive: %@\n", ([package isInteractive] ? @"YES" : @"NO")] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Has Nonstandard Files: %@\n", ([package hasNonStandardFiles] ? @"YES" : @"NO")] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Has Archive: %@\n", ([package hasArchive] ? @"YES" : @"NO")] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Has Remove Script: %@\n", ([package hasRemoveScript] ? @"YES" : @"NO")] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Has Configure Script: %@\n", ([package hasConfigureScript] ? @"YES" : @"NO")] attributes:[self normalAttributes]] autorelease]];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"\n"] attributes:[self normalAttributes]] autorelease]];

    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"Contents of package \"%@\" output view:\n\n", [package packageName]] attributes:[self signalAttributes]] autorelease]];
    data = [outView RTFFromRange:NSMakeRange(0, [[outView string] length])];
    [reportView replaceCharactersInRange:NSMakeRange([[reportView string] length], 0) withRTF:data];

    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"\n\nContents of package \"%@\" error view:\n\n", [package packageName]] attributes:[self signalAttributes]] autorelease]];
    data = [errView RTFFromRange:NSMakeRange(0, [[errView string] length])];
    [reportView replaceCharactersInRange:NSMakeRange([[reportView string] length], 0) withRTF:data];

    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"\n\nContents of package \"%@\" messages view:\n\n", [package packageName]] attributes:[self signalAttributes]] autorelease]];
    data = [messageView RTFFromRange:NSMakeRange(0, [[messageView string] length])];
    [reportView replaceCharactersInRange:NSMakeRange([[reportView string] length], 0) withRTF:data];
    
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"\n\nContents of package \"%@\" signature view:\n\n", [package packageName]] attributes:[self signalAttributes]] autorelease]];
    data = [signatureView RTFFromRange:NSMakeRange(0, [[signatureView string] length])];
    [reportView replaceCharactersInRange:NSMakeRange([[reportView string] length], 0) withRTF:data];
    
    envArgs = [NSArray arrayWithObjects: [II2TOOLFINDER pathForTool:@"/usr/bin/env"], nil];
    envTask = [[GLUPTaskRunner alloc] initWithOwner:nil reportsPID:GLUPTaskRunnerFifoPid arguments:envArgs authenticator:nil];
    if (!envTask) {
	return;
    }
    [envTask setOutView:reportView];
    [envTask setErrView:reportView];
    [package setCommandEnvironment:envTask withUserDefined:YES];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"\n\nEnvironment for running commands of package \"%@\" (%@.ii2):\n\n", [package packageName], [package baseName]] attributes:[self signalAttributes]] autorelease]];
    [envTask run];

    hostinfoArgs = [NSArray arrayWithObjects: @"/usr/bin/hostinfo", nil];
    hostinfoTask = [[GLUPTaskRunner alloc] initWithOwner:nil reportsPID:GLUPTaskRunnerFifoPid arguments:hostinfoArgs authenticator:nil];
    if (!hostinfoTask) {
	return;
    }
    [hostinfoTask setOutView:reportView];
    [hostinfoTask setErrView:reportView];
    [package setCommandEnvironment:hostinfoTask withUserDefined:NO];
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"\n\nHostinfo for machine:\n\n"] attributes:[self signalAttributes]] autorelease]];
    [hostinfoTask run];
    
    [reportView appendAttributedOutput:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"\n\nTools cache:\n%@\n\n", [[II2TOOLFINDER cachedTools] description]] attributes:[self signalAttributes]] autorelease]];

    [self modifyTabItemDisplayByTag:ii2ReportTabTag toState:YES];
    [tabView selectTabViewItemWithIdentifier:[NSString stringWithFormat:@"%u", ii2ReportTabTag]];
}

- (IBAction)ii2PrintReport:(id)sender
{
    [reportView print:nil];
}

- (IBAction)ii2ShowReport:(id)sender
{
    [self createReport];
}

#define IS_NULL(a) (!a)
#define NOT_NULL(a) (a != 0)

- (NSString *)mailScriptBody:(NSAttributedString *)richBody to:(NSString *)to subject:(NSString *)subject isMIME:(BOOL)isMIME {
    NSString *cc = nil;

    NSMutableString *s = [NSMutableString stringWithCapacity:1000];
    NSString *logoPath = nil;
    NSMutableString *body = [NSMutableString stringWithString:[richBody string]];
    NSString *mailer = [[NSUserDefaults standardUserDefaults] objectForKey:@"Mail"];;

    [body insertString:@"Type your message here\n\n========== REPORT ===========\n\n" atIndex:0];
    [body replaceOccurrencesOfString:@"\"" withString:@"\\\"" options:0 range:NSMakeRange(0, [body length])];

    
    if (IS_NULL(mailer)) mailer = @"Mail";

    // must skip over the image:
    if (isMIME) body = [body substringFromIndex:1];

    [s appendString:[NSString stringWithFormat:@"tell application \"%@\"\n",mailer]];
    [s appendString:@"activate\n"];	// added for powermail? maybe fixes other bug?

    [s appendString:[NSString stringWithFormat:@"set bodyvar to \"%@\"\n",body]];
    [s appendString:[NSString stringWithFormat:@"set addrNameVar to \"%@\"\n",to]];
    [s appendString:[NSString stringWithFormat:@"set addrVar to \"%@\"\n",to]];

    if (NOT_NULL(cc)) {
        [s appendString:[NSString stringWithFormat:@"set ccNameVar to \"%@\"\n",cc]];
        [s appendString:[NSString stringWithFormat:@"set ccVar to \"%@\"\n",cc]];
    }

    [s appendString:[NSString stringWithFormat:@"set subjectvar to \"%@\"\n",subject]];


    if ([mailer isEqualToString:@"PowerMail"]) {
	[s appendString:@"if addrVar = addrNameVar then\nset recip to addrVar\nelse\nset recip to addrNameVar & \" <\" & addrVar & \">\"\nend if\n"];
	[s appendString:@"set theMessage to make new message with properties {subject:subjectvar,recipient:{{recipient type:cc recipient, address:{display name:ccNameVar,address:ccVar}}, recip}, content:bodyvar}\n"];
	[s appendString:@"open theMessage"];
    } else if ([mailer isEqualToString:@"Eudora"]) {
	[s appendString:@"if addrVar = addrNameVar then\nset recip to addrVar\nelse\nset recip to addrNameVar & \" <\" & addrVar & \">\"\nend if\n"];
	[s appendString:@"if ccVar = ccNameVar then\nset ccrecip to ccVar\nelse\nset ccrecip to ccNameVar & \" <\" & ccVar & \">\"\nend if\n"];

	[s appendString:@"make message at end of mailbox \"out\"\n"];
	[s appendString:@"set field \"to\" of message 0 to recip\n"];
	[s appendString:@"set field \"subject\" of message 0 to subjectvar\n"];
	[s appendString:@"set field \"cc\" of message 0 to ccrecip\n"];
	//-- field "" indicates the body of the message
	[s appendString:@"set field \"\" of message 0 to bodyvar\n"];
    }  else if ([mailer isEqualToString:@"Mailsmith 1.5"]) {
        // 2.0 RC 2 july 29 2002
        [s appendString:@"if (addrVar = addrNameVar) or (addrNameVar = \"\") then\n"];
	[s appendString:@"set aRecip to addrVar\n"];
	// process aRecip to see if it contains multiple addresses separated by commas
        [s appendString:@"set recip to {}\n"];
        [s appendString:@"set loc to offset of \",\" in aRecip\n"];
        [s appendString:@"repeat while loc > 0\n"];
        [s appendString:@"set newstr to text 1 thru (loc - 1) of aRecip\n"];
        [s appendString:@"set aRecip to text (loc + 1) thru length of aRecip\n"];
        [s appendString:@"set recip to recip & newstr\n"];
        [s appendString:@"set loc to offset of \",\" in aRecip\n"];
        [s appendString:@"end repeat\n"];
        [s appendString:@"set recip to recip & aRecip\n"];
        [s appendString:@"else\n"];
        [s appendString:[NSString stringWithFormat:@"set aRecip to \"%c\"\" & addrNameVar & \"%c\" <\" & addrVar & \">\"\n",'\\','\\']];

        [s appendString:@"set recip to {aRecip}\n"];
        [s appendString:@"end if\n"];
        [s appendString:@"if (ccVar = ccNameVar) or (ccNameVar = \"\") then\n"];
        [s appendString:@"set aRecip to ccVar\n"];
        // process aRecip to see if it contains multiple addresses separated by commas
        [s appendString:@"set ccrecip to {}\n"];
        [s appendString:@"set loc to offset of \",\" in aRecip\n"];
        [s appendString:@"repeat while loc > 0\n"];
        [s appendString:@"set newstr to text 1 thru (loc - 1) of aRecip\n"];
        [s appendString:@"set aRecip to text (loc + 1) thru length of aRecip\n"];
        [s appendString:@"set ccrecip to ccrecip & newstr\n"];
        [s appendString:@"set loc to offset of \",\" in aRecip\n"];
        [s appendString:@"end repeat\n"];
        [s appendString:@"set ccrecip to ccrecip & aRecip\n"];
        [s appendString:@"else\n"];
        //[s appendString:@"set aRecip to \"\\"\" & ccNameVar & \"\\" <\" & ccVar & \">\"\n"];
        [s appendString:[NSString stringWithFormat:@"set aRecip to \"%c\"\" & ccNameVar & \"%c\" <\" & ccVar & \">\"\n",'\\','\\']];

        [s appendString:@"set ccrecip to {aRecip}\n"];
        [s appendString:@"end if\n"];

        [s appendString:@"make new message window\n"];
        [s appendString:@"tell message window 1\n"];
        [s appendString:@"set subject to subjectvar\n"];
        [s appendString:@"repeat with aRecip in recip\n"];
        [s appendString:@"make new to_recipient at end with properties {address:aRecip}\n"];
        [s appendString:@"end repeat\n"];
        [s appendString:@"repeat with aRecip in ccrecip\n"];
        [s appendString:@"make new cc_recipient at end with properties {address:aRecip}\n"];
        [s appendString:@"end repeat\n"];
        [s appendString:@"set contents to bodyvar\n"];
        [s appendString:@"end tell\n"];

    }  else if ([mailer isEqualToString:@"Entourage"] || [mailer isEqualToString:@"Outlook Express"] ||  [mailer isEqualToString:@"Entourage X"]) {
	[s appendString:@"if addrVar = addrNameVar then\nset recip to addrVar\nelse\nset recip to addrNameVar & \" <\" & addrVar & \">\"\nend if\n"];
	[s appendString:@"if ccVar = ccNameVar then\nset ccrecip to ccVar\nelse\nset ccrecip to ccNameVar & \" <\" & ccVar & \">\"\nend if\n"];
	[s appendString:@"make new draft window with properties {to recipients:recip, subject:subjectvar, content:bodyvar, CC recipients:ccrecip}\n"];

    } else {   // regular Mail.app by Apple

        if (floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_1) {
            [s appendString:@"set composeMessage to (a reference to (make new compose message at beginning of compose messages))\n"];
            [s appendString:@"tell composeMessage\n"];

            [s appendString:@"make new to recipient at beginning of to recipients with properties {address:addrVar, display name:addrNameVar}\n"];
            if (NOT_NULL(cc)) {
		[s appendString:@"make new cc recipient at beginning of cc recipients with properties {address:ccVar, display name:ccNameVar}\n"];
            }
            [s appendString:@"set the subject to subjectvar\n"];

            [s appendString:@"set the content to bodyvar\n"];
            if (isMIME  && NOT_NULL(logoPath) && [[NSFileManager defaultManager] fileExistsAtPath:logoPath]) {
                [s appendString:[NSString stringWithFormat:@"set aFile to \"%@\"\n",logoPath]];
                [s appendString:@"tell content\n"];
                [s appendString:@"make new text attachment with properties {file name:aFile} at before the first word of the first paragraph\n"];
                [s appendString:@"end tell\n"];

            }
            [s appendString:@"end tell\n"];

            [s appendString:@"make new message editor at beginning of message editors\n"];
            [s appendString:@"set compose message of last message editor to composeMessage\n"];
        } else {
            /* 10.2 or later system */
            [s appendString:@"set m to make new outgoing message\n"];
            [s appendString:@"tell m\n"];
            [s appendString:@"make new to recipient at end of to recipients with properties {name:addrNameVar, address:addrVar}\n"];
            if (NOT_NULL(cc)) {
		[s appendString:@"make new cc recipient at end of cc recipients with properties {name:ccNameVar, address:ccVar}\n"];
            }
            [s appendString:@"set subject to subjectvar\n"];

            [s appendString:@"set content to bodyvar\n"];
            if (isMIME  && NOT_NULL(logoPath) && [[NSFileManager defaultManager] fileExistsAtPath:logoPath]) {
                [s appendString:[NSString stringWithFormat:@"set aFile to \"%@\"\n",logoPath]];
                [s appendString:@"tell content\n"];
                [s appendString:@"make new attachment with properties {file name:aFile} at before the first word of the first paragraph\n"];
                [s appendString:@"end tell\n"];

            }
            [s appendString:@"set visible to true\n"];
            [s appendString:@"end tell\n"];
        }


    }
    [s appendString:@"end tell\n"];
    if ([[NSUserDefaults standardUserDefaults] boolForKey:@"LogMailApplescript"])
	NSLog(s);
    return s;
}

- (IBAction)ii2MailReport:(id)sender
{
    NSDictionary *errDict = nil;
    NSString *appleScriptString;
    NSAppleScript *appleScript;
    NSString *subject = [NSString stringWithFormat:@"%@ Report",
	[[NSProcessInfo processInfo] processName]];
    // [self createReport]; No creation, mail only. We have the re-create buttonn now.
    appleScriptString = [self mailScriptBody:[reportView textStorage] to:@"ii2@rna.nl" subject:subject isMIME:NO];
    if ([ii2Prefs mainLogLevel] > 7) NSLog( appleScriptString);
    appleScript = [[NSAppleScript alloc] initWithSource:appleScriptString];
    [appleScript executeAndReturnError:&errDict];
}

- (IBAction)ii2Clear:(id)sender
{
    [outView resetParagraphStyleAndClear];
    [errView resetParagraphStyleAndClear];
}

- (IBAction)ii2ToggleEditMode:(id)sender
{
    [inspectView toggleRuler:(id)sender];
    [inspectView setEditable:[inspectView isRulerVisible]];
    [outView toggleRuler:(id)sender];
    [outView setEditable:[outView isRulerVisible]];
    [errView toggleRuler:(id)sender];
    [errView setEditable:[errView isRulerVisible]];
    [signatureView toggleRuler:(id)sender];
    [signatureView setEditable:[signatureView isRulerVisible]];
    [messageView toggleRuler:(id)sender];
    [messageView setEditable:[signatureView isRulerVisible]];
}

- (void)incrementActionProgressBar:(unsigned int)increment
{
    [actionsProgressBar incrementBy:increment];
}

- (void)updateReportWithDataFromTask:(id <II2TaskStatusProviderProtocol>)task download:(BOOL)download
{
    if ([task sizeOfTaskWhenCompleted] != 0) {
	[(download?downloadsProgressBar:actionsProgressBar) incrementBy:[task sizeOfTaskSinceLastReported]];
    }
}

- (void)setReportMaxDownloadsSize:(int)size
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"setReportMaxDownloadsSize: %d", size);
    if (size) {
	[downloadsProgressBar setIndeterminate:NO];
	[downloadsProgressBar setMinValue:0.0];
	[downloadsProgressBar setMaxValue:size];
    }
    else {
	[downloadsProgressBar setIndeterminate:YES];
    }
}

- (void)setReportMaxActionsSize:(int)size
{
    if ([ii2Prefs mainLogLevel] > 9) NSLog( @"setReportMaxActionsSize: %d", size);
    if (size) {
	[actionsProgressBar setIndeterminate:NO];
	[actionsProgressBar setMinValue:0.0];
	[actionsProgressBar setMaxValue:size];
    }
    else {
	[actionsProgressBar setIndeterminate:YES];
    }
}
- (void) reportRCSID
{
    NSLog( @"%s\n", rcsid);
}

@end

/*
 * $Log: II2WindowController.m,v $
 * Revision 1.9  2004/05/01 12:31:34  gctwnl
 * 2.63.0
 *
 * Revision 1.8  2004/04/30 12:18:06  gctwnl
 * 2.62.0
 *
 * Revision 1.7  2004/04/25 13:13:21  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.6  2004/04/19 22:58:25  gctwnl
 * 2.61.0
 *
 * Revision 1.5  2004/03/22 22:48:44  gctwnl
 * 2.59.0
 *
 * Revision 1.4  2004/03/19 23:35:11  gctwnl
 * Version 2.55.0. Has a renewed interface to support simple operations.
 * Supports progress bar control from the i-Package.
 *
 * Revision 1.3  2004/03/12 09:50:30  gctwnl
 * Version 2.53.0
 *
 * Revision 1.2  2004/02/02 20:15:27  gctwnl
 * Fixed loss of keyed objects in package window (spinning progress). Some small fixes in logging and debug messages.
 *
 * Revision 1.1.1.1  2004/02/01 17:18:28  gctwnl
 * Initial import.
 *
 * Revision 3.20  2004/01/27 22:49:32  gerben
 * 2.45.0
 *
 * Revision 3.19  2004/01/27 12:57:39  gerben
 * 2.44.0
 *
 * Revision 3.18  2004/01/23 08:50:56  gerben
 * 2.43.0
 *
 * Revision 3.17  2004/01/23 00:31:16  gerben
 * 2.42.0 release
 *
 * Revision 3.16  2004/01/15 01:00:06  gerben
 * 2.40.0
 *
 * Revision 3.15  2004/01/14 12:56:46  gerben
 * Version 2.39.0
 *
 * Revision 3.14  2004/01/05 23:47:15  gerben
 * *** empty log message ***
 *
 * Revision 3.13  2004/01/05 15:31:34  gerben
 * *** empty log message ***
 *
 * Revision 3.12  2003/11/08 23:38:01  gerben
 * 2.30.0 release candidate
 *
 * Revision 3.11  2003/09/18 22:40:53  gerben
 * 2.26.0 Fragmented archives
 *
 * Revision 3.10  2003/04/27 20:14:42  gerben
 * 2.19.0
 *
 * Revision 3.9  2003/04/08 07:31:47  gerben
 * ii2@rna.nl instead of iinstaller@rna.nl
 *
 * Revision 3.8  2003/04/03 22:27:55  gerben
 * 2.17.0. Donation menu item.
 *
 * Revision 3.7  2003/04/02 23:27:07  gerben
 * Basic mail report handling
 *
 * Revision 3.6  2003/04/02 23:24:37  gerben
 * Basic mail report handling
 *
 * Revision 3.5  2003/03/13 14:42:04  gerben
 * 2.11.0. Really fix the permissions problem.
 *
 * Revision 3.4  2003/02/23 23:29:11  gerben
 * Package Locking
 *
 * Revision 3.3  2003/01/04 02:04:43  gerben
 * Version 2.2.0
 *
 * Revision 3.2  2002/12/10 14:24:55  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:24  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:53  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.65  2002/12/10 13:50:18  gerben
 * A bit of cleanup. One bugfix (wrong method)
 *
 * Revision 1.64  2002/12/09 15:09:00  gerben
 * Report printing
 *
 * Revision 1.63  2002/12/03 20:57:40  gerben
 * A preference for automatic dependency checking
 *
 * Revision 1.62  2002/12/03 08:16:58  gerben
 * New style install stamps .foo.ii2
 *
 * Revision 1.61  2002/12/02 23:24:41  gerben
 * Basic dependency handling
 *
 * Revision 1.60  2002/11/24 23:09:28  gerben
 * Tom's changes and remarks mainly
 *
 * Revision 1.59  2002/10/31 14:26:58  gerben
 * Moved authentication setting to package window
 *
 * Revision 1.58  2002/10/30 21:42:23  gerben
 * RC3a
 *
 * Revision 1.57  2002/10/29 09:36:23  gerben
 * RC2
 *
 * Revision 1.56  2002/10/27 12:38:34  gerben
 * Lazy update checking. Update followed by action for inspect and fatten.
 *
 * Revision 1.55  2002/10/25 20:36:11  gerben
 * Layout changes.
 *
 * Revision 1.54  2002/10/24 20:12:32  gerben
 * Better handling of package name clashes and changed properties
 *
 * Revision 1.53  2002/10/24 13:03:32  gerben
 * Disable menu items that can affect downloads and availability of files when package is active.
 *
 * Revision 1.52  2002/10/21 20:16:53  gerben
 * Closing in on RC1 target
 *
 * Revision 1.51  2002/10/20 16:15:35  gerben
 * Fixed bug in network disabling
 *
 * Revision 1.50  2002/10/17 20:19:26  gerben
 * Changes wrt disabled network access
 *
 * Revision 1.49  2002/10/16 22:37:08  gerben
 * A few small changes.
 *
 * Revision 1.48  2002/10/16 21:28:22  gerben
 * A complete new rewrite of taskss, processes and stuff.
 *
 * Revision 1.47  2002/10/16 08:37:18  gerben
 * Changed log levels. Small fixes.
 *
 * Revision 1.46  2002/10/14 12:34:19  gerben
 * Removed dead code
 *
 * Revision 1.45  2002/10/14 12:15:38  gerben
 * Install stamp file after unarchive.
 *
 * Revision 1.44  2002/10/13 22:22:27  gerben
 * Re-created file menu so validating menu items work. Changed code for inspect and fatten to use generic task array starter.
 *
 * Revision 1.43  2002/10/13 21:20:15  gerben
 * Fixed a serious bug where the task rewrite was a bit release-happy
 *
 * Revision 1.42  2002/10/13 10:57:22  gerben
 * Saving stable state before some additions and rewrites
 *
 * Revision 1.41  2002/10/12 10:54:38  gerben
 * A lot of changes, mostly when producing output/feedback.
 *
 * Revision 1.40  2002/10/10 00:51:24  gerben
 * Many changes. Committing a stable state.
 *
 * Revision 1.39  2002/10/09 14:16:12  gerben
 * Changed as many %s/lossyCString uses to %@/fileSystemRepresentation/UTF8String
 *
 * Revision 1.38  2002/10/09 10:53:40  gerben
 * Moved textview output to NSTexTView category.
 *
 * Revision 1.37  2002/10/09 08:38:42  gerben
 * Fixed two 10.1.5 incompatibilities (untested)
 *
 * Revision 1.36  2002/10/08 22:25:03  gerben
 * GPG checking complete. Output of parallel tasks will not mingle.
 *
 * Revision 1.35  2002/10/08 21:34:29  gerben
 * Working GPG signature checks, shared GPG cache for II2.
 *
 * Revision 1.34  2002/10/08 13:20:30  gerben
 * MD5 check on update. Small changes.
 *
 * Revision 1.33  2002/10/08 10:03:42  gerben
 * Several small changes, amongst others first responder behaviour.
 *
 * Revision 1.32  2002/10/07 20:51:30  gerben
 * NSProgressIndicator on package window. Some renaming of functions.
 *
 * Revision 1.31  2002/10/07 20:20:04  gerben
 * More md5 stuff and testing of spinning disk progressindicator stuff.
 *
 * Revision 1.30  2002/10/07 11:52:27  gerben
 * Several fixes related to possible invalid content.
 *
 * Revision 1.29  2002/10/07 09:10:11  gerben
 * Failing authentication re-enables buttons.
 *
 * Revision 1.28  2002/10/06 21:05:07  gerben
 * Progress Indicator for active tasks, hidden when there are no processes.
 *
 * Revision 1.27  2002/10/06 12:29:57  gerben
 * Progress Indicator for active tasks
 *
 * Revision 1.26  2002/10/06 10:19:21  gerben
 * Wrote workaround for indentation problem (indentation stays around even when view is cleared).
 *
 * Revision 1.25  2002/10/05 19:40:53  gerben
 * Enable basic printing
 *
 * Revision 1.24  2002/10/03 15:26:28  gerben
 * Do not accept actions on tables without a selected row. Now with only disabling buttons when something is going to happen.
 *
 * Revision 1.23  2002/10/03 15:21:26  gerben
 * Do not accept actions on tables without a selected row.
 *
 * Revision 1.22  2002/10/03 13:38:04  gerben
 * Removed addChildWindow as child passes close to parent. Window can only close when there are no running tasks.
 *
 * Revision 1.21  2002/10/02 22:38:30  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */
