//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import <stdio.h>
#include <unistd.h>

#import "GWSizeFormatter.h"

static NSDictionary *_iiDefaults;


static NSData *urlDataFromURL( NSURL *url)
{
    NSData *urlData = nil;

    if (url) {
	NSURLHandle *urlHandle = [[url URLHandleUsingCache:NO] retain];

	if (urlHandle) {
	    if ([[url scheme] hasPrefix:@"http"]) {
		[urlHandle writeProperty:@"i-Installer/2.0" forKey:@"User-Agent"];
		[urlHandle writeProperty:@"no-cache" forKey:@"Cache-Control"];
	    }

	    urlData = [[urlHandle loadInForeground] retain];
	    [urlHandle release];
	}
    }

    return [urlData autorelease];
}

static NSDictionary *foregroundGetURLDictionary( NSString *fileURL)
{
    NSURL *url = [NSURL URLWithString:fileURL];
    NSData *urlData = urlDataFromURL( url);
    NSDictionary *dictionary;
    NSString *error;

    if (!urlData) {
	return nil;
    }
    dictionary = (NSDictionary *)CFPropertyListCreateFromXMLData( NULL, (CFDataRef)urlData, kCFPropertyListImmutable, (CFStringRef *)&error);

    return dictionary;
}

static NSString *foregroundGetURLString(NSString * fileName)
{
    NSURL *url = [NSURL URLWithString:fileName];
    NSData *urlData = urlDataFromURL( url);
    NSString *string;

    if (!urlData) {
	return nil;
    }
    string = [[[NSString alloc] initWithData:urlData encoding:NSUTF8StringEncoding] autorelease];

    return string;
}



int main( int argc, char *argv[])
{
    int i;
    NSAutoreleasePool *pool =[[NSAutoreleasePool alloc] init];
    NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    // NSDictionary *tmp =
    _iiDefaults = [[defaults persistentDomainForName:@"nl.rna.v2.i-Installer"] retain];

    if (![_iiDefaults objectForKey:@"networkaccessdisabled"]) {
	NSLog( @"Missing i-Installer preferences. i-Installer should be run at least once before you can check");
	return 1;
    }
    if (CFBooleanGetValue( (CFBooleanRef)[_iiDefaults objectForKey:@"networkaccessdisabled"])) {
	NSLog( @"Cannot check remote packages when network setting is set to disabled in i-Installer preferences");
	return 1;
    }
    for (i=1; i<argc; i++) {
	NSString *pkgName = [[NSString stringWithCString:argv[i]] stringByExpandingTildeInPath];
	NSString *plistName, *customPlistName, *pkgBaseName;
	NSDictionary *plistDict, *customPlistDict, *remotePlistDict;
	NSString *remoteLocationString, *remotePlistURLString, *remoteUpdateStampString, *remoteUpdateStampContents;
	NSNumber *localTimestamp, *remoteTimestamp;
	
	if (![pkgName hasSuffix:@".ii2"]) {
	    NSLog( @"%@ is not an i-Package. Wrong extension. Skipping...", pkgName);
	    continue;
	}
	pkgBaseName = [[[pkgName lastPathComponent] stringByDeletingPathExtension] retain];
	plistName = [NSString stringWithFormat:@"%@/%@.plist", pkgName, pkgBaseName];
	plistDict = [[NSDictionary dictionaryWithContentsOfFile:plistName] retain];
	if (!plistDict) {
	    NSLog( @"%@ not found. Missing package or missing Table of Contents. Skipping...", plistName);
	    continue;
	}
	customPlistName = [[pkgName stringByAppendingString:@"/Custom.plist"] retain];
	customPlistDict = [[NSDictionary dictionaryWithContentsOfFile:customPlistName] retain];

	remoteLocationString = [customPlistDict objectForKey:@"default_url"];
	if (!remoteLocationString) {
	    remoteLocationString = [plistDict objectForKey:@"default_url"];
	}
	if (!remoteLocationString) {
	    NSLog( @"%@ is not an i-Package. Missing URL in Table of Contents. Skipping...", pkgName);
	    continue;
	}
	remoteUpdateStampString = [NSString stringWithFormat:@"%@/%@.startupdate", remoteLocationString, pkgBaseName];
	remoteUpdateStampContents = foregroundGetURLString( remoteUpdateStampString);
	if (remoteUpdateStampContents) {
	    printf( "The remote location for %s is currently being updated\n", [[pkgName stringByAbbreviatingWithTildeInPath] UTF8String]);
	    continue;
	}
	
	remotePlistURLString = [NSString stringWithFormat:@"%@/%@.plist", remoteLocationString, pkgBaseName];
	remotePlistDict = foregroundGetURLDictionary( remotePlistURLString);
	if (!remotePlistDict) {
	    NSLog( @"%@ is not an i-Package. Missing Table of Contents. Skipping...", remoteLocationString);
	    continue;
	}
	localTimestamp = [NSNumber numberWithLong:[[plistDict objectForKey:@"timestamp"] intValue]];
	remoteTimestamp = [NSNumber numberWithLong:[[remotePlistDict objectForKey:@"timestamp"] intValue]];
	if ([localTimestamp compare:remoteTimestamp] != NSOrderedSame) {
	    NSString *timediff = ([localTimestamp compare:remoteTimestamp] == NSOrderedAscending ? @"newer" :@"older");
	    unsigned int changedSize = 0;
	    NSString *formattedSize;
	    NSDictionary *localFilesDict = [plistDict objectForKey:@"files"];
	    NSDictionary *remoteFilesDict = [remotePlistDict objectForKey:@"files"];

	    NSEnumerator *enumerator = [remoteFilesDict keyEnumerator];
	    NSString *remoteFileKey;

	    while ((remoteFileKey = [enumerator nextObject])) {
		NSDictionary *localFileDict = [localFilesDict objectForKey:remoteFileKey];
		if (!localFileDict) {
		    // New file in package
		    changedSize += [[[remoteFilesDict objectForKey:remoteFileKey] objectForKey:@"size"] intValue];
		}
		else {
		    // File in both local and remote
		    NSString *localmd5 = [localFileDict objectForKey:@"md5"];
		    NSString *remotemd5 = [[remoteFilesDict objectForKey:remoteFileKey] objectForKey:@"md5"];
		    if (![localmd5 isEqualToString:remotemd5]) {
			changedSize += [[[remoteFilesDict objectForKey:remoteFileKey] objectForKey:@"size"] intValue];
		    }
		}
	    }
	    formattedSize = [[GWSizeFormatter sharedFormatter] stringForObjectValue:[NSNumber numberWithUnsignedInt:changedSize]];
		
	    printf( "There is a %s version available for %s (%s changed)\n", [timediff UTF8String], [[pkgName stringByAbbreviatingWithTildeInPath] UTF8String], [formattedSize UTF8String]);
	}
	
    }
    [pool release];
    return 0;
}
