//  Copyright (c) 2002-2004, Gerben Wierda, R&A
//  All rights reserved.
//
//  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
//
//  Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
//  Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
//
//  Neither the name of R&A nor the names of its contributors may be used to endorse or promote products derived from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>
#import <stdio.h>
#include <unistd.h>
#import "GWInteractionManagerProtocols.h"

static char rcsid[] = "$Id: doalerter.m,v 1.3 2004/04/25 13:13:22 gctwnl Exp $";

// Tool for two-way asynchronous DO communication to i-Installer.app v2

static BOOL shouldKeepRunning = YES;
static BOOL verbose = NO;
static NSRunLoop *runLoop;
static NSConnection *_connection;
static const char *progname;

typedef enum {
    ResultOK = 0,
    GenericError = 128,
    NoResultButtonError,
    NoDOPanelError,
    NoConnectionError,
    NoReturnHandlerError,
    IllegalArgumentError,
    NoSetError
} ReturnStatus;

static ReturnStatus returnStatus; // Contains panel result or 

BOOL remotePanel( NSDictionary *dict, NSString *serverName) {
    NSConnection *connection = [NSConnection connectionWithRegisteredName:serverName host:nil];
    if (connection != nil) {
	NSDistantObject <GWInteractionManagerServerProtocol> *theProxy = [connection rootProxy];
	if (verbose) NSLog(@"Received connection %@ to remote program", serverName);
	if (theProxy != nil) {
	    [theProxy setProtocolForProxy:@protocol(GWInteractionManagerServerProtocol)];
	    // NSLog( @"*** Performing %@", [dict objectForKey:@"type"]);
	    if ([[dict objectForKey:@"type"] isEqualToString:@"IncrementProgress"]) {
		[theProxy incrementProgress:[dict objectForKey:@"incrementvalue"]];
	    }
	    else if ([[dict objectForKey:@"type"] isEqualToString:@"GetSelectorSets"]) {
		NSArray *extendedSets = [theProxy selectorSets];
		if (extendedSets) {
		    returnStatus = ResultOK;
		    if ([dict objectForKey:@"verbose"] && [[dict objectForKey:@"verbose"] isEqualToString:@"YES"]) {
			int i;
			
			for (i=0; i<[extendedSets count]; i++) {
			    NSDictionary *set = [extendedSets objectAtIndex:i];
			    NSString *idString = [set objectForKey:@"id"];
			    NSString *nameString = [set objectForKey:@"name"];
			    NSString *commentString = [set objectForKey:@"comment"];
			    NSString *selectedString = [set objectForKey:@"selected"];
			    NSString *configuredString = [set objectForKey:@"configured"];
			    NSString *installedString = [set objectForKey:@"installed"];
			    NSString *availableString = [set objectForKey:@"available"];
			    
			    fprintf( stdout, "%s\n", [idString UTF8String]); fflush( stdout);
			    fprintf( stdout, "%s\n", [nameString UTF8String]); fflush( stdout);
			    fprintf( stdout, "%s\n", [commentString UTF8String]); fflush( stdout);
			    fprintf( stdout, "%s\n", [selectedString UTF8String]); fflush( stdout);
			    fprintf( stdout, "%s\n", [configuredString UTF8String]); fflush( stdout);
			    fprintf( stdout, "%s\n", [installedString UTF8String]); fflush( stdout);
			    fprintf( stdout, "%s\n", [availableString UTF8String]); fflush( stdout);
			}
		    }
		    else {
			NSMutableString *result = [[[NSMutableString alloc] init] autorelease];
			int i;
			
			for (i=0; i<[extendedSets count]; i++) {
			    NSDictionary *set = [extendedSets objectAtIndex:i];
			    NSString *idString = [set objectForKey:@"id"];
			    NSString *selectedString = [set objectForKey:@"selected"];
			    if ([result length] != 0) {
				[result appendString:@"|"];
			    }
			    if ([selectedString isEqualToString:@"YES"]) {
				[result appendString:@"+"];
			    }
			    else {
				[result appendString:@"-"];
			    }
			    [result appendString:idString];
			}
			fprintf( stdout, "%s\n", [result UTF8String]); fflush( stdout);
		    }
		}
		else {
		    returnStatus = NoSetError;
		}
	    }
	    else {
		GWInteractionManagerServerReturnType result;
		while ((result = [theProxy startPanel:dict]) == GWInteractionManagerServerBusy) {
		    if (verbose) NSLog( @"i-Installer reported busy on a request for a panel. Pausing 3 seconds and retrying.");
		    sleep( 3);
		}
		if (result == GWInteractionManagerServerError) {
		    return NO;
		}
	    }
	    return YES;
	}
	else {
	    NSLog(@"Couldn't get Panel start result from remote program");
	}
	[connection registerName:nil];
    }
    else {
	NSLog(@"Couldn't get connection %@ to remote program", serverName);
    }
    return NO;
}

@interface DOPanelResultHandler: NSObject <GWInteractionManagerClientProtocol>{
    // NSConnection *_connection;
}
@end

@implementation DOPanelResultHandler
- (oneway void)panelResult:(in NSDictionary *)dictionary
{
    [[_connection receivePort] invalidate];
    [_connection invalidate];

    if (verbose) NSLog( @"Result for panel received");

    if ([dictionary objectForKey:@"resultButtonTag"]) {
	// Apple uses:
	//	1 for OK
	//	0 for Cancel
	//	-1 for Other
	//	-2 for Panel problem
	// This we keep to in the app ( we could have changed the tags, but we keep to defaults there) 

	// We return:
	//	0 for OK
	//	1 for Cancel
	//	2 for Other
	//	3 for Panel problem
	returnStatus = -[[dictionary objectForKey:@"resultButtonTag"] intValue] + 1;
	// So that OK becomes 0 and we translate Cocoa defaults to unix-like defaults.
    }
    else {
	NSLog( @"Returned dictionary does not contain result Button tag");
	returnStatus = NoResultButtonError;
	return;
    }
    if ([[dictionary objectForKey:@"type"] isEqualToString:@"Choice"]) {
	if ([[dictionary objectForKey:@"resultButtonTag"] intValue] == 1) {
	    if ([dictionary objectForKey:@"verbose"] && [[dictionary objectForKey:@"verbose"] isEqualToString:@"YES"]) {
		fprintf( stdout, "%s\n", [[dictionary objectForKey:@"choiceString"] UTF8String]);
	    }
	    else {
		fprintf( stdout, "%d\n", [[dictionary objectForKey:@"choiceItemIndex"] intValue]);
	    }
	}
	else {
	    fprintf( stdout, "\n"); fflush( stdout);
	}
    }
    else if ([[dictionary objectForKey:@"type"] isEqualToString:@"Table"] || [[dictionary objectForKey:@"type"] isEqualToString:@"Select"]) {
	if ([[dictionary objectForKey:@"resultButtonTag"] intValue] == 1) {
	    if ([dictionary objectForKey:@"verbose"] && [[dictionary objectForKey:@"verbose"] isEqualToString:@"YES"]) {
		fprintf( stdout, "%s\n", [[[dictionary objectForKey:@"selections"] componentsJoinedByString:[dictionary objectForKey:@"separator"]] UTF8String]);
	    }
	    else {
		fprintf( stdout, "%s\n", [[[dictionary objectForKey:@"numericalselections"] componentsJoinedByString:[dictionary objectForKey:@"separator"]] UTF8String]);
	    }
	}
	else {
	    fprintf( stdout, "\n"); fflush( stdout);
	}
    }
    else if ([[dictionary objectForKey:@"type"] isEqualToString:@"Alert"]) {
	if ([dictionary objectForKey:@"verbose"] && [[dictionary objectForKey:@"verbose"] isEqualToString:@"YES"]) {
	    fprintf( stdout, "%s\n", [[dictionary objectForKey:@"resultButtonTitle"] UTF8String]); fflush( stdout);
	}
	else {
	    fprintf( stdout, "%d\n", [[dictionary objectForKey:@"resultButtonTag"] intValue]); fflush( stdout);
	}
    }
    else if ([[dictionary objectForKey:@"type"] isEqualToString:@"Input"]) {
	fprintf( stdout, "%s\n", [[dictionary objectForKey:@"answer"] UTF8String]); fflush( stdout);
    }
    else {
	NSLog( @"Broken result dictionary, unknown type\n");
    }
    shouldKeepRunning = NO;
    return;
}
@end

void Usage()
{
    NSLog( @"%s\nUsage: %s -t (Alert|Choice|Table|Input|Environment|SetSelectorSets|GetSelectorSets|SetConfiguredSelectorSets|UnsetConfiguredSelectorSets|SetInstalledSelectorSets|UnsetInstalledSelectorSets|IncrementProgress) -s DOserverPort -S choicesseparator -C \"choice1|choice2|choice3\" -T titlestring -I infomessage -1 okselectbuttontext -2 otherbuttontext -3 cancelbuttontext -v verbose -V verbose-result\n", rcsid, progname);
}

int main( int argc, char *argv[])
{
    int ch;
    int necessaryArgumentCount = 0; // Weak check
    //
    // getopt handling:
    // -t <type>
    //		<type> = Alert, Table, etc
    // -s <server>
    //		connect to DOPanel server <server> on localhost
    // -T <title>
    //		Use <title> for panel title (Alert, Choice, Table, Input)
    // -u <url>
    //		Use <url> for type = Open
    // -I <info>
    //		Use <info> for panel info
    // -i <initialvalue>
    //		Use <initialvalue> for initial value if applicable
    // -C <choices>
    //		Use the list of choices as choices for the choice panel ("-t choice" only)
    // -E foo=bar
    //		Add foo=bar to the environment (-t Environment only)
    //		When !foo it means remove foo from the environment
    //		Current limitation: no = allowed in value
    // -P parts
    //		Send list of selector set numbers to app
    // -p number
    //		Increment progress indicator by number
    // -S <separator>
    //		Use <separator> to split the argument of -C in parts (default: '|')
    // -1 <string>
    //		Use <string> as the OK/Select button title
    // -2 <string>
    //		Present 'other' button and use <string> as the 'other' button title
    // -3 <string>
    //		Present 'cancel' button and use <string> as the 'cancel' button title
    // -v
    //		Verbose output of progress
    // -V
    //		Verbose output of result
    
    NSAutoreleasePool *pool =[[NSAutoreleasePool alloc] init];
    NSMutableDictionary *dict = [[NSMutableDictionary alloc] init];
    NSString *serverName;
    NSString *separator = @"|";
    runLoop = [NSRunLoop currentRunLoop];
    // runLoop = [[NSRunLoop alloc] init];
    progname = argv[0];
    BOOL waitForReply = YES;

    while ((ch = getopt( argc, argv, "u:vVt:s:S:T:I:i:C:1:2:3:E:P:p:")) != -1) {
	switch (ch) {
	    case 'V':
		[dict setObject:@"YES" forKey:@"verbose"];
		break;
	    case 'v':
		verbose = YES;
		break;
	    case 't':
		[dict setObject:[NSString stringWithCString:optarg] forKey:@"type"];
		necessaryArgumentCount++;
                if ([[dict objectForKey:@"type"] isEqualToString:@"SetSelectorSets"] || [[dict objectForKey:@"type"] isEqualToString:@"GetSelectorSets"] || [[dict objectForKey:@"type"] isEqualToString:@"SetConfiguredSelectorSets"] || [[dict objectForKey:@"type"] isEqualToString:@"UnsetConfiguredSelectorSets"] || [[dict objectForKey:@"type"] isEqualToString:@"SetInstalledSelectorSets"] || [[dict objectForKey:@"type"] isEqualToString:@"UnsetInstalledSelectorSets"] || [[dict objectForKey:@"type"] isEqualToString:@"IncrementProgress"]) {
                    waitForReply = NO;
                }
		else if ([[dict objectForKey:@"type"] isEqualToString:@"Environment"]) {
                    [dict setObject:[[NSMutableDictionary alloc] init] forKey:@"environment"];
                    waitForReply = NO;
                }
                else if ([[dict objectForKey:@"type"] isEqualToString:@"Open"]) {
                    waitForReply = NO;
                }
		break;
	    case 's':
		serverName = [NSString stringWithCString:optarg];
		necessaryArgumentCount++;
		break;
	    case 'T':
		[dict setObject:[NSString stringWithCString:optarg] forKey:@"title"];
		break;
	    case 'u':
		[dict setObject:[NSString stringWithCString:optarg] forKey:@"openurl"];
		necessaryArgumentCount++;
		break;
	    case 'I':
		[dict setObject:[NSString stringWithCString:optarg] forKey:@"info"];
		break;
	    case 'i':
		[dict setObject:[NSString stringWithCString:optarg] forKey:@"initialvalue"];
		break;
	    case '1':
		[dict setObject:[NSString stringWithCString:optarg] forKey:@"okbutton"];
		[dict setObject:[NSString stringWithCString:optarg] forKey:@"selectbutton"];
		break;
	    case '2':
		[dict setObject:[NSString stringWithCString:optarg] forKey:@"otherbutton"];
		break;
	    case '3':
		[dict setObject:[NSString stringWithCString:optarg] forKey:@"cancelbutton"];
		break;
	    case 'S':
		separator = [NSString stringWithCString:optarg];
		break;
	    case 'C':
		[dict setObject:[[NSString stringWithCString:optarg] componentsSeparatedByString:separator] forKey:@"choices"];
		necessaryArgumentCount++;
		break;
	    case 'p':
		[dict setObject:[NSNumber numberWithInt:[[NSString stringWithCString:optarg] intValue]] forKey:@"incrementvalue"];
		break;
	    case 'P':
		[dict setObject:[[NSString stringWithCString:optarg] componentsSeparatedByString:separator] forKey:@"selectorsets"];
		necessaryArgumentCount++;
		break;
	    case 'E':
		{
		    NSString *envSetString = [NSString stringWithCString:optarg];
		    NSArray *envSetArray = [envSetString componentsSeparatedByString:@"="];
		    if ([envSetArray count] == 2) {
			[[dict objectForKey:@"environment"] setObject:[envSetArray objectAtIndex:1] forKey:[envSetArray objectAtIndex:0]];
		    }
		    else {
			NSLog( @"%s: illegal environment setting %@\n", progname, envSetString);
		    }
		}
                break;
	    default:
		NSLog( @"%s: illegal option\n", progname);
		Usage();
		exit( GenericError);
	}
    }
    argc -= (optind-1);
    argv += (optind-1);

    [dict setObject:separator forKey:@"separator"];

    if ((necessaryArgumentCount == 2 && 
	 ([[dict objectForKey:@"type"] isEqualToString:@"Alert"] || [[dict objectForKey:@"type"] isEqualToString:@"Environment"] || [[dict objectForKey:@"type"] isEqualToString:@"GetSelectorSets"] || [[dict objectForKey:@"type"] isEqualToString:@"IncrementProgress"])) || 
	(necessaryArgumentCount == 3 && ([[dict objectForKey:@"type"] isEqualToString:@"Choice"] || [[dict objectForKey:@"type"] isEqualToString:@"Table"] || [[dict objectForKey:@"type"] isEqualToString:@"Select"] || [[dict objectForKey:@"type"] isEqualToString:@"Open"] || [[dict objectForKey:@"type"] isEqualToString:@"SetSelectorSets"] || [[dict objectForKey:@"type"] isEqualToString:@"SetConfiguredSelectorSets"] || [[dict objectForKey:@"type"] isEqualToString:@"UnsetConfiguredSelectorSets"] || [[dict objectForKey:@"type"] isEqualToString:@"SetInstalledSelectorSets"] || [[dict objectForKey:@"type"] isEqualToString:@"UnsetInstalledSelectorSets"]))) {
	DOPanelResultHandler *handler = nil;
	
	// Create unique server name
	NSString *resultServerName = [[NSString stringWithFormat:@"ii2.%d.%u.%u.%s", getpid(), random(), time( NULL), progname] retain];
	
	// Start receiving DOPanel server
        handler = [[DOPanelResultHandler alloc] init];
        // If succesful startup, send message to remote object which will then send
        // the result back asynchronously to our tool
        if (handler) {
            [dict setObject:resultServerName forKey:@"doresultserver"];
	    if (remotePanel( dict, serverName)) {
		if (waitForReply) {
		    // Open/Environment/SetSelectorSets are one way streets
		    NSPort *port = [[NSPort port] retain];
		    _connection = [[NSConnection alloc] initWithReceivePort:port sendPort:port];
		    if ([_connection registerName:resultServerName] == NO) {
			/* Handle error. */
			NSLog( @"Unable to start DOPanel server with name %@", resultServerName);
			returnStatus = NoDOPanelError;
		    } else {
			if (verbose) NSLog( @"Started DOPanel server with name %@", resultServerName);
			[_connection setRootObject:handler];
			while (shouldKeepRunning && [runLoop runMode:NSDefaultRunLoopMode beforeDate:[NSDate distantFuture]]);
		    }
		    [port release]; // has been invalidated already in DOPanelResultHandler
		    [_connection release]; // has been invalidated already too
		}
	    }
	    else {
		NSLog( @"Unable to connect to panel-providing application");
		returnStatus = NoConnectionError;
	    }
	}
        else {
            NSLog( @"Eek! (NSObject alloc init failed...");
            returnStatus = NoReturnHandlerError;
        }
    }
    else {
	NSLog( @"Illegal argument combination (%d)", necessaryArgumentCount);
	returnStatus = IllegalArgumentError;
    }
    
    [pool release];
    // NSLog( @"Exiting with status %d", returnStatus);
    exit( returnStatus);
}

/*
 * $Log: doalerter.m,v $
 * Revision 1.3  2004/04/25 13:13:22  gctwnl
 * Forced commit of all .h and .m files of version 2.61.0
 *
 * Revision 1.2  2004/03/19 23:35:11  gctwnl
 * Version 2.55.0. Has a renewed interface to support simple operations.
 * Supports progress bar control from the i-Package.
 *
 * Revision 1.1.1.1  2004/02/01 17:17:28  gctwnl
 * Initial import.
 *
 * Revision 3.7  2004/01/27 22:49:32  gerben
 * 2.45.0
 *
 * Revision 3.6  2004/01/23 00:31:16  gerben
 * 2.42.0 release
 *
 * Revision 3.5  2004/01/14 12:56:47  gerben
 * Version 2.39.0
 *
 * Revision 3.4  2004/01/05 15:31:35  gerben
 * *** empty log message ***
 *
 * Revision 3.3  2003/11/08 23:31:26  gerben
 * 2.30.0 release version
 *
 * Revision 3.2  2002/12/10 14:24:55  gerben
 * Set sticky tag 3.2
 *
 * Revision 3.0  2002/12/10 14:04:24  gerben
 * Upping all files with a sticky tag of 1.99
 *
 * Revision 1.99  2002/12/10 13:52:53  gerben
 * Set all revision numbers to 1.99
 *
 * Revision 1.11  2002/11/22 23:54:48  gerben
 * RC4
 *
 * Revision 1.10  2002/10/21 20:16:54  gerben
 * Closing in on RC1 target
 *
 * Revision 1.9  2002/10/13 13:57:48  gerben
 * Implemented a table with checkboxes. The changes for this were largely copied/imitated from the OpenUp project by Scott Anguish
 *
 * Revision 1.8  2002/10/12 10:54:39  gerben
 * A lot of changes, mostly when producing output/feedback.
 *
 * Revision 1.7  2002/10/09 14:16:12  gerben
 * Changed as many %s/lossyCString uses to %@/fileSystemRepresentation/UTF8String
 *
 * Revision 1.6  2002/10/07 20:28:07  gerben
 * Removed circular CVS Log entry in C Comments at the end of files
 *
 * Revision 1.5  2002/10/02 22:38:31  gerben
 * Added CVS Log in C Comment at the end of every file
 *
 */
